/*
 *  O_XV.c:	Implementation of Xv video access object.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     20 May 93 - Andy Goodrich
 *       - Initial coding
 *
 *  Copyright(c) 1993, Andrew C. Goodrich
 *
 * $Log:   /plxhp/dave/pvcsrc/O_XV/archives/O_XV.c_v  $
 * 
 *    Rev 1.5   09 Sep 1996 09:51:18   dave
 * Andy Goodrich: Added output support and augmentation of status
 *                to include current state of digitizers and encoder.
 * Revision 1.5  1995/08/31 12:51:35  dist
 * Andy Goodrich: Added output support and augmentation of status
 *                to include current state of digitizers and encoder.
 *
 * Revision 1.4  95/08/29  15:30:59  15:30:59  dist ()
 * Andy Goodrich: Added terminate method to clean up an O_XV object when
 *                it is no longer needed.
 * 
 * Revision 1.3  95/07/13  12:09:13  12:09:13  dist ()
 * Andy Goodrich: Fixed unterminated comment in method_xv_get_attribute 
 *                parameter list.
 * 
 * Revision 1.2  95/07/13  12:07:00  12:07:00  dist ()
 * Andy Goodrich: Added code to properly handle resizing and output.
 * 
 * Revision 1.1  95/06/07  16:18:43  16:18:43  dist ()
 * Initial revision
 * 
 * Revision 1.1  95/05/16  10:09:33  10:09:33  dist ()
 * Initial revision
 * 
 */
#include <sys/types.h>			/* Standard types for this C environment. */
#include <stdio.h>				/* Stream input/output declarations. */
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <Xvlib.h>
#include "Xvddx.h"
#include "O_XV.h"

/*----------------------------------------------------------------------------*
 |"method_xv_create_window"
 |
 |  Create a window to place video in for this Xv access object.
 |
 |    = window id on success, 0 on failure.
 *----------------------------------------------------------------------------*/
static Window method_xv_create_window( p_me, visual_id, depth )
	O_XV*	p_me;		/* -> object instance using. */
	int		visual_id;	/* =  visual id to use or -1 to default. */
	int		depth;		/* =  depth desired or -1 to default. */
{
	XColor			scolor;
	XColor			ecolor;	
	XEvent			event;		   /* Information about the last X event. */
	XGCValues		gc_attr;	   /* Graphics context creation attributes. */
	int			    i_format;	   /* # of format now accessing. */
	int			    n_formats;	   /* # of formats available for this adaptor */
	Visual			*p_def_visual; /* Default visual for screen. */
	XvFormat	    *p_format;	   /* Format record now accessing. */
	XVisualInfo	    *p_vis_info;   /* Information returned from visual search */
	XVisualInfo		vis_template;  /* Search template for finding visual. */
	int				rc;			   /* Get visual info return code. */
	XSetWindowAttributes  w_attr;  /* Window creation attributes. */

	n_formats = p_me->p_adaptors->num_formats;


	/* SELECT DEPTH OF VISUAL: */

	p_format = p_me->p_adaptor->formats;
	if ( depth < 0 )
	{
		depth = p_format->depth;
	}
	else
	{
		for ( i_format = 0; i_format < n_formats; i_format++, p_format++ )
		{
			if ( p_format->depth == depth ) break;
		}
	}
	if ( depth < 0 )
	{
		fprintf(stderr, "O_XV_Window: unable to find a visual with depth %d\n",
			depth);
		return 0;
	}

	/*
     *  SELECT ID (TYPE) OF VISUAL:
     */
	p_format = p_me->p_adaptor->formats;
	if ( visual_id < 0 )
	{
		visual_id = p_format->visual_id;
	}
	else
	{
		for ( i_format = 0; i_format < n_formats; i_format++, p_format++ )
		{
			if ( p_format->visual_id == visual_id && p_format->depth == depth ) 
			{
				break;
			}
		}
	}
	if ( visual_id < 0 )
	{
		fprintf(stderr,"O_XV_Window: unable to find visual id %x of depth %d\n",
			visual_id, depth);
		return 0;
	}

	/* 
     *  USE VISUAL TO DETERMINE WHAT COLOR MAP TO USE FOR OUR WINDOW. ALLOCATE
     *  A BACKGROUND COLOR, AND SET UP OUR WINDOW CREATION ATTRIBUTES.
     */
    vis_template.visualid = visual_id;
    p_vis_info=XGetVisualInfo(p_me->display, VisualIDMask, &vis_template, &rc);
    if ( !p_vis_info )
    {
        fprintf(stderr, "O_XV_Window: visual %x not found\n", visual_id);
        return 0;
    }
    p_me->p_visual = p_vis_info->visual;
	p_def_visual = XDefaultVisual(p_me->display, p_me->screen);
	p_me->color_map = ( p_me->p_visual->visualid == p_def_visual->visualid ) ?
		XDefaultColormap(p_me->display, p_me->screen) :
		XCreateColormap(p_me->display, p_me->w_root, p_me->p_visual, AllocNone);
	XAllocNamedColor(p_me->display, p_me->color_map, "black", &scolor, &ecolor);

    w_attr.colormap = p_me->color_map;
    w_attr.background_pixel = scolor.pixel;
    w_attr.event_mask = KeyPressMask | VisibilityChangeMask | ExposureMask;
    w_attr.border_pixel = scolor.pixel;
	w_attr.backing_store = NotUseful;

	/* 
	 *  CREATE AND MAP THE WINDOW. SETUP UP XV SELECTION NOTIFY ON IT.
     */
    p_me->w_main = XCreateWindow(p_me->display, p_me->w_root, 0, 0, 
		p_me->win_width, p_me->win_height, 0,
	    depth, InputOutput, p_me->p_visual,
	    CWBackingStore | CWColormap | CWBackPixel | CWEventMask | CWBorderPixel,
		&w_attr
    );
    XMapWindow(p_me->display, p_me->w_main);
	for(;;)
	{
		XNextEvent(p_me->display, &event);
		if ( event.type = VisibilityNotify ) break;
	}


	/* 
     *  CREATE A GRAPHICS CONTEXT FOR OUR MAIN WINDOW AND PUT UP VIDEO.
     */
  	gc_attr.foreground = 0xffffff;
  	p_me->gc = XCreateGC(p_me->display, p_me->w_main, GCForeground, &gc_attr);

	return p_me->w_main;
}

/*----------------------------------------------------------------------------*
 |"method_xv_dump"
 |
 |  Dump the contents of this object to a file descriptor.
 |
 |
 *----------------------------------------------------------------------------*/
static void method_xv_dump( p_me, fd )
	O_XV*	p_me;		/* -> object instance using. */
	FILE*	fd;		/* -> file descriptor to dump to. */
{
	fprintf(fd, "\nDump of xv access object at address %08x\n", p_me);
	fprintf(fd, "  xv: version = %d revision = %d major_op = %d\n",
		p_me->xv_version, p_me->xv_revision, p_me->xv_major_op);
	fprintf(fd, "      error_base = %d event_base = %d\n",
		p_me->xv_error_base, p_me->xv_event_base);
	fprintf(fd, "  a_brightness = %08x\n", p_me->a_brightness);
	fprintf(fd, "  a_contrast = %08x\n", p_me->a_contrast);
	fprintf(fd, "  a_encoding = %08x\n", p_me->a_encoding);
	fprintf(fd, "  a_hue = %08x\n", p_me->a_hue);
	fprintf(fd, "  a_saturation = %08x\n", p_me->a_saturation);
	fprintf(fd, "  n_adaptors = %d p_adaptors = %08x\n", 
		p_me->n_adaptors, p_me->p_adaptors);
	fprintf(fd, "  n_encodings = %d p_encodings = %08x\n", 
		p_me->n_encodings, p_me->p_encodings);
	fprintf(fd, "  port_id = %08x p_visual = %08x\n", 
		p_me->port_id, p_me->p_visual);
	fprintf(fd, "  display: %08x w_root = %08x w_main = %08x gc = %08x \n",
		p_me->display, p_me->w_root, p_me->w_main, p_me->gc);
	fprintf(fd, "  sig_width = %d sig_height = %d\n", 
		p_me->sig_width, p_me->sig_height);
	fprintf(fd, "  win_width = %d win_height = %d\n", 
		p_me->win_width, p_me->win_height);
}

/*----------------------------------------------------------------------------*
 |"method_xv_get_attribute"
 |
 |  Get an attribute associated with an O_XV object's video.
 |
 |    = return code from XvGetPortAttribute.
 *----------------------------------------------------------------------------*/
static int method_xv_get_attribute( p_me, atom, value )
	O_XV*	p_me;		/* -> object instance using. */
	Atom	atom;		/* =  atom for attribute to set. */
	int*	value;		/* =  value to set attribute to. */
{
	return XvGetPortAttribute(p_me->display, p_me->port_id, atom, value);
}

/*----------------------------------------------------------------------------*
 |"method_xv_set_attribute"
 |
 |  Set an attribute associated with an O_XV object's video. If the attribute
 |  is zoom get new encoding values.
 |
 |    = return code from XvSetPortAttribute.
 *----------------------------------------------------------------------------*/
static int method_xv_set_attribute( p_me, atom, value )
	O_XV*	p_me;		/* -> object instance using. */
	Atom	atom;		/* =  atom for attribute to set.
	int		value;		/* =  value to set attribute to. */
{
	int		rc;			/* Return code from XvSetPortAttribute. */


	rc = XvSetPortAttribute(p_me->display, p_me->port_id, atom, value);

	if ( atom == p_me->a_zoom )
	{
		free((char*)p_me->p_encodings);
		XvQueryEncodings(p_me->display, p_me->port_id, 
			&p_me->n_encodings, &p_me->p_encodings);
		p_me->sig_width=p_me->p_encodings->width; 
		p_me->sig_height=p_me->p_encodings->height;
		p_me->win_width=p_me->p_encodings->width; 
		p_me->win_height=p_me->p_encodings->height;
	}
	return rc;
}

/*----------------------------------------------------------------------------*
 |"method_xv_source"
 |
 |  Set the source associated with an O_XV object's video.
 |
 |    = 1 on success, 0 on failure
 *----------------------------------------------------------------------------*/
static int method_xv_source( p_me, dmsd, timing, type, instance)
	O_XV*	p_me;		/* -> object instance using. */
	int		dmsd;		/* =  dmsd utilizing. */
	int		timing;		/* =  timing of video source (from Xvddx.h). */
	int		type;		/* =  type of video source (from Xvddx.h). */
	int		instance;	/* =  number of video source of this type. */
{
	int				i_encoding;		/* # of encoding currently testing. */
	XvEncodingInfo	*p_encoding;	/* Encoding currently testing. */
	u_int			search_id;		/* Encoded search id for video source. */

	search_id = timing | type | (dmsd << 4) | instance;
	p_encoding = p_me->p_encodings;
	for ( i_encoding = 0; i_encoding < p_me->n_encodings; i_encoding++ )
	{
		if ( p_encoding->encoding_id == search_id )
		{
			XvSetPortAttribute(p_me->display, p_me->port_id, p_me->a_encoding, 
				search_id);		
			p_me->sig_width = p_encoding->width;	
			p_me->sig_height = p_encoding->height;	
			p_me->win_width = p_encoding->width;	
			p_me->win_height = p_encoding->height;	
			return 1;
		}
		p_encoding++;
	}
	fprintf(stderr, "Could not find encoding %x\n", search_id);
	return 0;
}
			
/*----------------------------------------------------------------------------*
 |"method_xv_status"
 |
 |  Return status associated with an O_XV object's port.
 |
 |    = status
 *----------------------------------------------------------------------------*/
static int method_xv_status( p_me )
	O_XV*	p_me;		/* -> object instance using. */
{
	int		status;		/* Port status to return. */

	XvGetPortAttribute(p_me->display, p_me->port_id, p_me->a_status, &status);
	if ( p_me->video_in_on ) status |= O_XV_Status_Video_In;
	if ( p_me->video_out_on ) status |= O_XV_Status_Video_Out;
	return status;
}


/*----------------------------------------------------------------------------*
 |"method_xv_terminate"
 |
 |  Terminate use of this O_XV object.
 *----------------------------------------------------------------------------*/
static void method_xv_terminate( p_me )
	O_XV*	p_me;		/* -> object instance using. */
{
	free((char*)p_me->p_adaptors);
	free((char*)p_me);
}


/*----------------------------------------------------------------------------*
 |"method_xv_use_window"
 |
 |  Specify the window to use for this video with this Xv access object.
 *----------------------------------------------------------------------------*/
static void method_xv_use_window( p_me, window )
	O_XV*	p_me;		/* -> object instance using. */
	Window  window;		/* =  window to use for video. */
{
	XGCValues		gc_attr;	   /* Graphics context creation attributes. */

    p_me->w_main = window;
  	gc_attr.foreground = 0xffffff;
  	p_me->gc = XCreateGC(p_me->display, p_me->w_main, GCForeground, &gc_attr);
}


/*----------------------------------------------------------------------------*
 |"method_xv_video_off"
 |
 |  Stop video in the window associated with this O_XV object.
 |
 |    = status
 *----------------------------------------------------------------------------*/
static void method_xv_video_off( p_me )
	O_XV*	p_me;		/* -> object instance using. */
{
	XvStopVideo(p_me->display, p_me->port_id, p_me->w_main);
	p_me->video_in_on = 0;
	p_me->video_out_on = 0;
}


/*----------------------------------------------------------------------------*
 |"method_xv_video_on"
 |
 |  Start video in the window associated with this O_XV object.
 |
 |    = status
 *----------------------------------------------------------------------------*/
static void method_xv_video_on( p_me )
	O_XV*	p_me;		/* -> object instance using. */
{
	XvPutVideo(p_me->display, p_me->port_id, p_me->w_main, p_me->gc, 
		0, 0, p_me->sig_width, p_me->sig_height, 
		0, 0, p_me->win_width, p_me->win_height
	);
	p_me->video_out_on = 0;
	p_me->video_in_on = 1;
}


/*----------------------------------------------------------------------------*
 |"method_xv_video_output"
 |
 |  Start video in the window associated with this O_XV object.
 |
 |    = status
 *----------------------------------------------------------------------------*/
static void method_xv_video_output( p_me )
	O_XV*	p_me;		/* -> object instance using. */
{
	XvGetVideo(p_me->display, p_me->port_id, p_me->w_main, p_me->gc, 
		0, 0, p_me->sig_width, p_me->sig_height, 
		0, 0, p_me->win_width, p_me->win_height
	);
	p_me->video_in_on = 0;
	p_me->video_out_on = 1;
}


static Tv_XV O_XV_transfer_vector = {
	method_xv_create_window,
	method_xv_dump,
	method_xv_get_attribute,
	method_xv_set_attribute,
	method_xv_source,
	method_xv_status,
	method_xv_terminate,
	method_xv_use_window,
	method_xv_video_off,
	method_xv_video_on,
	method_xv_video_output
};

/*----------------------------------------------------------------------------*
 |"O_XV_O_XV"
 |
 |  Create an Xv video extension access object for a specific adaptor and
 |  port of a display.
 |
 |    -> object on success, 0 on failure.
 *----------------------------------------------------------------------------*/
O_XV *O_XV_O_XV( p_me, display, i_adapt, i_port )
	O_XV*	 p_me;		/* -> storage for object creating, or 0. */
	Display* display;	/* -> display to open Xv port on. */
	int		 i_adapt;	/* =  # of adaptor to use (0..m). */
	int		 i_port;	/* =  number of port on adaptor to use (0..n). */
{
	int		status;		/* Status from function call. */

	/*
     *  CREATE OUR OBJECT AND FILL IN THE PARAMETERS WHICH WERE PASSED TO US.
	 */
	if ( !p_me )
	{
		if ( !(p_me = (O_XV *)malloc(sizeof(O_XV))) )
		{
			fprintf(stderr, "\n O_XV_O_XV: couldn't allocate storage\n");
			return 0;
		}
	}
	
    p_me->display = display;
	p_me->i_adaptor = i_adapt;
	p_me->i_port = i_port;
	p_me->tv = &O_XV_transfer_vector;
	p_me->video_in_on = 0;
	p_me->video_out_on = 0;



	/*  MAKE SURE Xv VIDEO EXTENSION IS PRESENT ON THIS DISPLAY, AND THAT
     *  OUR ADAPTOR AND PORT EXIST.
     */
	p_me->w_root = XDefaultRootWindow(p_me->display);
	p_me->screen = XDefaultScreen(p_me->display);
	p_me->w_main = 0;

	XSynchronize(p_me->display, True);

	status = XvQueryExtension(p_me->display, 
		&p_me->xv_version, &p_me->xv_revision, &p_me->xv_major_op, 
		&p_me->xv_event_base, &p_me->xv_error_base);

	if (status != Success)
	{
		fprintf(stderr, "O_XV_O_XV: no Xv extension on this display.\n");
		free(p_me);
        return 0;
	}
	  
	XvQueryAdaptors(p_me->display, p_me->w_root, &p_me->n_adaptors, 
		&p_me->p_adaptors);

	if ( i_adapt >= p_me->n_adaptors )
	{
		fprintf(stderr, "O_XV_O_XV: there are %d adaptors you wanted # %d\n",
			p_me->n_adaptors, i_adapt);
		free(p_me);
        return 0;
	}
	p_me->p_adaptor = &p_me->p_adaptors[i_adapt];

	if ( i_port >= p_me->p_adaptor->num_ports )
	{
		fprintf(stderr, "O_XV_O_XV: %d ports on adaptor %d you wanted # %d\n",
			p_me->p_adaptor->num_ports, i_adapt, i_port);
		free(p_me);
        return 0;
	}
 
	p_me->port_id = p_me->p_adaptor->base_id + i_port;

	/*
     *  SET UP ACCESS TO PORT ATTRIBUTES. PICK UP ENCODINGS FOR THIS PORT 
	 *  AND DEFAULT THE VIDEO SOURCE TO THE FIRST LISTED ENCODING. 
     */
    p_me->a_brightness = XInternAtom(p_me->display, "XV_BRIGHTNESS", False);
	p_me->a_channel =    XInternAtom(p_me->display, "XV_PLX_CHANNEL", False);
    p_me->a_contrast =   XInternAtom(p_me->display, "XV_CONTRAST", False);
	p_me->a_encoding =   XInternAtom(p_me->display, "XV_ENCODING", False);
    p_me->a_hue =        XInternAtom(p_me->display, "XV_HUE", False);
    p_me->a_saturation = XInternAtom(p_me->display, "XV_SATURATION", False);
	p_me->a_status =     XInternAtom(p_me->display, "XV_STATUS", False);
	p_me->a_zoom   =     XInternAtom(p_me->display, "XV_PLX_ZOOM", False);

	XvQueryEncodings(p_me->display, p_me->port_id, 
		&p_me->n_encodings, &p_me->p_encodings);
    p_me->sig_width=p_me->p_encodings->width; 
	p_me->sig_height=p_me->p_encodings->height;
    p_me->win_width=p_me->p_encodings->width; 
	p_me->win_height=p_me->p_encodings->height;

	return p_me;
}

