/*
 *  menus.c:	Create a menu from a set of signal definitions.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     19 Apr 95 - Andy Goodrich
 *       - Initial coding
 *
 * $Log:   /plxhp/dave/pvcsrc/O_XV/archives/encodings.c_v  $
 * 
 *    Rev 1.4   09 Sep 1996 09:52:44   dave
 * Andy Goodrich: Added code to support the head of an input or output
 *                list as a menu item in the list rather than the
 *                pointer to the list as a sub-menu.
 * Revision 1.4  1995/08/31 12:50:01  dist
 * Andy Goodrich: Added code to support the head of an input or output
 *                list as a menu item in the list rather than the
 *                pointer to the list as a sub-menu.
 *
 * Revision 1.3  95/07/13  12:05:18  12:05:18  dist ()
 * Andy Goodrich: Fixed output menu to display in order NTSC, PAL.
 * 
 * Revision 1.2  95/07/12  13:50:18  13:50:18  dist ()
 * Andy Goodrich: Added build_outputs_list function.
 * 
 * Revision 1.1  95/06/01  16:47:51  16:47:51  dist ()
 * Initial revision
 * 
 * Revision 1.1  95/05/16  10:10:21  10:10:21  dist ()
 * Initial revision
 * 
 *
 *  Copyright(c) 1995, Parallax Graphics, Inc.
 */
#include <sys/types.h>			/* Standard types for this C environment. */
#include <stdio.h>				/* Stream input/output declarations. */
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <Xvlib.h>
#include <Xm/DialogS.h>
#include <Xm/Separator.h>
#include <Xm/PushB.h>
#include <Xm/CascadeB.h>
#include <Xm/RowColumn.h>
#include <Xm/ToggleB.h>
#include <Xm/Label.h>
#include <Xm/Form.h>
#include <Xm/PanedW.h>
#include <Xm/PushBG.h>
#include <Xm/DrawingA.h>
#include <Xm/SashP.h>
#include "Xvddx_PGI.h"
#include "support.h"

typedef struct _CB_Encoding {
    int channel;
    int format;
    int instance;
    int standard;
} CB_Encoding;

static char* decoder_labels[] = {
	"Decoder 1",
	"Decoder 2",
	"Decoder 3",
	"Decoder 4",
	"Decoder 5",
	"Decoder 6",
	"Decoder 7",
	"Decoder 8",
	"Decoder 9",
	"Decoder 10"
};


static char* instance_labels[] = {
	"1",
	"2",
	"3",
	"4",
	"5",
	"6",
	"7",
	"8",
	"9",
	"10"
};


/*---------------------------------------------------------------------------*
 |"build_inputs_list"
 |
 |	Build a digitizer selection menu given the available encodings. Note we 
 |  process the encodings in reverse order since we build the menu linked
 |  list by adding entries on the head, not the tail of the list.
 *----------------------------------------------------------------------------*/
Menu_Linked_E* 
	build_inputs_list(title, cb_head, cb_rtn, n_encodings, p_encodings)
	char*			title;			/* -> title for menu or null. */
	void		    (*cb_head)();   /* -> callback routine for head. */
	void		    (*cb_rtn)();    /* -> callback routine to use. */
    int				n_encodings;	/* =  # of input signals. */
    XvEncodingInfo*	p_encodings;	/* -> input signals. */
{
	int				dmsd;		/* Digitizer for current encoding. */
	int				format;		/* Video format of input. */
	int				i_digit;	/* # of digitizer now accessing. */
	int			  	i_encode;	/* # of encoding now examining. */
	int				instance;	/* Video format instance of input. */
	Menu_Linked_E*		p_digit;	/* Digitizer menu item now accessing. */
	Menu_Linked_E*		p_format;	/* Format menu item now accessing. */
    Menu_Linked_E*	    p_head;		/* Head of linked list of menu items. */
	Menu_Linked_E*		p_instance;	/* Format instance menu item now accessing. */
	Menu_Linked_E*		p_timing;	/* VIdeo timing menu item now accessing. */
	int				timing;		/* Video timing of input. */

	p_head = (Menu_Linked_E*)malloc(sizeof(Menu_Linked_E));
	p_head->p_next = 0;
	p_head->mnemonic = 0;
	p_head->misc = 0;
	p_head->name = title;
	p_head->cb_data = 0;
	p_head->cb_func = cb_head;
	p_head->sub_menu = 0;
	p_head->sub_menu_title = 0;
	p_head->widget = 0;

	for ( i_encode = n_encodings-1; i_encode >= 0; --i_encode )
	{

		/* LOOK UP DIGITIZER */

		dmsd = (p_encodings[i_encode].encoding_id & XV_PLX_DMSD) >> 4;
		for ( p_digit = p_head->sub_menu; p_digit; p_digit = p_digit->p_next )
		{
			if ( p_digit->misc == dmsd ) break;
		}
		if ( !p_digit )
		{
			p_digit = (Menu_Linked_E*)malloc(sizeof(Menu_Linked_E));
			p_digit->p_next = p_head->sub_menu;
			p_head->sub_menu = p_digit;
			p_digit->mnemonic = 0;
			p_digit->misc = dmsd;
			p_digit->cb_data = 0;
			p_digit->cb_func = 0;
			p_digit->sub_menu = 0;
			p_digit->sub_menu_title = 0;
			p_digit->widget = 0;
		}

	
		/* LOOK UP FORMAT */

		format = p_encodings[i_encode].encoding_id & XV_SOURCE_TYPE;
		for ( p_format=p_digit->sub_menu; p_format; p_format=p_format->p_next )
		{
			if ( p_format->misc == format ) break;
		}
		if ( !p_format )
		{
			p_format = (Menu_Linked_E*)malloc(sizeof(Menu_Linked_E));
			p_format->p_next = p_digit->sub_menu;
			p_digit->sub_menu = p_format;
			p_format->mnemonic = 0;
			p_format->misc = format;
			p_format->cb_data = 0;
			p_format->cb_func = 0;
			p_format->sub_menu = 0;
			p_format->sub_menu_title = 0;
			p_format->widget = 0;
			switch ( format )
			{
			  case XV_COMPOSITE:	p_format->name = "COMPOSITE";	break;
			  case XV_YC:			p_format->name = "YC";			break;
			}
		}


		/* LOOK UP INSTANCE NUMBER. */

		instance = p_encodings[i_encode].encoding_id & XV_PLX_INPUT;
		for ( p_instance=p_format->sub_menu; p_instance; 
				p_instance=p_instance->p_next )
		{
			if ( p_instance->misc == instance ) break;
		}
		if ( !p_instance )
		{
			p_instance = (Menu_Linked_E*)malloc(sizeof(Menu_Linked_E));
			p_instance->p_next = p_format->sub_menu;
			p_format->sub_menu = p_instance;
			p_instance->mnemonic = 0;
			p_instance->misc = instance;
			p_instance->cb_data = 0;
			p_instance->cb_func = 0;
			p_instance->sub_menu = 0;
			p_instance->sub_menu_title = 0;
			p_instance->widget = 0;
			p_instance->name = instance_labels[instance];
		}


		/* SET UP TIMING (THIS IS LEAF OF TREE.) */

		p_timing = (Menu_Linked_E*)malloc(sizeof(Menu_Linked_E));
		p_timing->p_next = p_instance->sub_menu;
		p_instance->sub_menu = p_timing;
		p_timing->mnemonic = 0;
		p_timing->misc = 0;
		p_timing->cb_data = &p_encodings[i_encode];
		p_timing->cb_func = cb_rtn;
		p_timing->sub_menu = 0;
		p_timing->sub_menu_title = 0;
		p_timing->widget = 0;
		switch (p_encodings[i_encode].encoding_id & XV_SOURCE_TIMING)
		{
		  case XV_NTSC:  p_timing->name = "NTSC";  break;
		  case XV_PAL:   p_timing->name = "PAL";   break;
		  case XV_SECAM: p_timing->name = "SECAM"; break;
		}
	}

	/* FILL IN DIGITIZER LABELS. */

	for(p_digit=p_head->sub_menu, i_digit=0; p_digit; 
		p_digit = p_digit->p_next, i_digit++)
	{
		p_digit->name = decoder_labels[i_digit];
	}


	/*  IF THE HEAD OF THE LIST HAS A CALL-BACK FUNCTION WE ARE A SUB-MENU.
	 *
	 *  So turn the p_head entry in to the first element of the sub-menu
	 *  rather than a menu item pointing to a sub-menu.
	 */
	if ( cb_head )
	{
		p_head->p_next = p_head->sub_menu;
		p_head->sub_menu = 0;
	}
	return p_head;

}


/*---------------------------------------------------------------------------*
 |"build_outputs_list"
 |
 |	Build an encoder selection menu given the available encodings. Note we 
 |  simply look to see if this is a VIO card and explicitly build the list.
 |  In the future when XV encodings include an output indicator we'll do it
 |  correctly.
 *----------------------------------------------------------------------------*/
Menu_Linked_E* 
	build_outputs_list(title, cb_head, cb_rtn, n_encodings, p_encodings)
	char*			title;			/* -> title for menu or null. */
	void		    (*cb_head)();   /* -> callback routine for head. */
	void		    (*cb_rtn)();    /* -> callback routine to use. */
    int				n_encodings;	/* =  # of input signals. */
    XvEncodingInfo*	p_encodings;	/* -> input signals. */
{
	int				saw_zero;	/* = 1 if saw digitizer zero in list. */
	int				saw_one;	/* = 1 if saw digitizer zero in list. */
	int			  	i_encode;	/* # of encoding now examining. */
	Menu_Linked_E*	p_digit;	/* Digitizer menu item now accessing. */
    Menu_Linked_E*  p_head;		/* Head of linked list of menu items. */
	Menu_Linked_E*	p_timing;	/* VIdeo timing menu item now accessing. */


	/*  LOOK AT ENCODINGS LIST AND DETERMINE IF THERE ARE TWO DMSEs. */

	saw_zero = 0;
	saw_one = 0;
	for ( i_encode = n_encodings-1; i_encode >= 0; --i_encode )
	{
		switch((p_encodings[i_encode].encoding_id & XV_PLX_DMSD) >> 4)
		{
		  case 0: saw_zero = 1; break;
		  case 1: saw_one = 1;  break;
		}
	}
	if ( !saw_zero || !saw_one ) return 0;

	/*  THERE ARE TWO DMSDs, SO THERE IS A DMSE. BUILD THE LIST.  */

	p_head = (Menu_Linked_E*)malloc(sizeof(Menu_Linked_E));
	p_head->p_next = 0;
	p_head->mnemonic = 0;
	p_head->misc = 0;
	p_head->name = title;
	p_head->cb_data = 0;
	p_head->cb_func = cb_head;
	p_head->sub_menu = 0;
	p_head->sub_menu_title = 0;
	p_head->widget = 0;

	
	p_digit = (Menu_Linked_E*)malloc(sizeof(Menu_Linked_E));
	p_digit->name = "Encoder 1";
	p_digit->p_next = p_head->sub_menu;
	p_head->sub_menu = p_digit;
	p_digit->mnemonic = 0;
	p_digit->misc = 0;
	p_digit->cb_data = 0;
	p_digit->cb_func = 0;
	p_digit->sub_menu = 0;
	p_digit->sub_menu_title = 0;
	p_digit->widget = 0;

	p_timing = (Menu_Linked_E*)malloc(sizeof(Menu_Linked_E));
	p_timing->p_next = p_digit->sub_menu;
	p_digit->sub_menu = p_timing;
	p_timing->mnemonic = 0;
	p_timing->misc = 0;
	p_timing->name = "PAL";
	p_timing->cb_data = XV_COMPOSITE | XV_PAL;
	p_timing->cb_func = cb_rtn;
	p_timing->sub_menu = 0;
	p_timing->sub_menu_title = 0;
	p_timing->widget = 0;

	p_timing = (Menu_Linked_E*)malloc(sizeof(Menu_Linked_E));
	p_timing->p_next = p_digit->sub_menu;
	p_digit->sub_menu = p_timing;
	p_timing->mnemonic = 0;
	p_timing->misc = 0;
	p_timing->name = "NTSC";
	p_timing->cb_data = XV_COMPOSITE | XV_NTSC;
	p_timing->cb_func = cb_rtn;
	p_timing->sub_menu = 0;
	p_timing->sub_menu_title = 0;
	p_timing->widget = 0;


	/*  IF THE HEAD OF THE LIST HAS A CALL-BACK FUNCTION WE ARE A SUB-MENU.
	 *
	 *  So turn the p_head entry in to the first element of the sub-menu
	 *  rather than a menu item pointing to a sub-menu.
	 */
	if ( cb_head )
	{
		p_head->p_next = p_head->sub_menu;
		p_head->sub_menu = 0;
	}

	return p_head;

}


/*---------------------------------------------------------------------------*
 |"build_signal_list"
 |
 |	Build an encoding selection menu given the available signals. Note we 
 |  process the signals in reverse order since we build the menu linked
 |  list by adding entries on the head, not the tail of the list.
 *----------------------------------------------------------------------------*/
Menu_Linked_E* build_signal_list(title, cb_rtn, n_signals, p_signals)
	char*			title;			/* -> title for menu or null. */
	void		    (*cb_rtn)();    /* -> callback routine to use. */
	int				n_signals;		/* =  number of signals. */
	XvplxSignalRec*	p_signals;		/* -> input signals. */
{
	int				dmsd;			/* Digitizer for current encoding. */
	int				format;			/* Video format of input. */
	int				i_digit;		/* # of digitizer now accessing. */
	int			  	i_signal;		/* # of encoding now examining. */
	int				instance;		/* Video format instance of input. */
	CB_Encoding*	p_cb_encoding;	/* Callback encoding record. */
	Menu_Linked_E*	p_digit;		/* Digitizer menu item now accessing. */
	Menu_Linked_E*	p_format;		/* Format menu item now accessing. */
    Menu_Linked_E*	p_head;			/* Head of linked list of menu items. */
	Menu_Linked_E*	p_instance;		/* Format instance menu item now accessing*/
	Menu_Linked_E*	p_timing;		/* Video timing menu item now accessing. */
	int				timing;			/* Video timing of input. */

	p_head = (Menu_Linked_E*)malloc(sizeof(Menu_Linked_E));
	p_head->p_next = 0;
	p_head->mnemonic = 0;
	p_head->misc = 0;
	p_head->name = title;
	p_head->cb_data = 0;
	p_head->cb_func = 0;
	p_head->sub_menu = 0;
	p_head->sub_menu_title = 0;
	p_head->widget = 0;

	for ( i_signal = n_signals-1; i_signal >= 0; --i_signal )
	{

		/* LOOK UP DIGITIZER */

		dmsd = (p_signals[i_signal].encoding_id & XV_PLX_DMSD) >> 4;
		for ( p_digit = p_head->sub_menu; p_digit; p_digit = p_digit->p_next )
		{
			if ( p_digit->misc == dmsd ) break;
		}
		if ( !p_digit )
		{
			p_digit = (Menu_Linked_E*)malloc(sizeof(Menu_Linked_E));
			p_digit->p_next = p_head->sub_menu;
			p_head->sub_menu = p_digit;
			p_digit->mnemonic = 0;
			p_digit->misc = dmsd;
			p_digit->cb_data = 0;
			p_digit->cb_func = 0;
			p_digit->sub_menu = 0;
			p_digit->sub_menu_title = 0;
			p_digit->widget = 0;
		}

	
		/* LOOK UP FORMAT */

		format = p_signals[i_signal].encoding_id & XV_SOURCE_TYPE;
		for ( p_format=p_digit->sub_menu; p_format; p_format=p_format->p_next )
		{
			if ( p_format->misc == format ) break;
		}
		if ( !p_format )
		{
			p_format = (Menu_Linked_E*)malloc(sizeof(Menu_Linked_E));
			p_format->p_next = p_digit->sub_menu;
			p_digit->sub_menu = p_format;
			p_format->mnemonic = 0;
			p_format->misc = format;
			p_format->cb_data = 0;
			p_format->cb_func = 0;
			p_format->sub_menu = 0;
			p_format->sub_menu_title = 0;
			p_format->widget = 0;
			switch ( format )
			{
			  case XV_COMPOSITE:	p_format->name = "COMPOSITE";	break;
			  case XV_YC:			p_format->name = "YC";			break;
			}
		}


		/* LOOK UP INSTANCE NUMBER. */

		instance = p_signals[i_signal].encoding_id & XV_PLX_INPUT;
		for ( p_instance=p_format->sub_menu; p_instance; 
				p_instance=p_instance->p_next )
		{
			if ( p_instance->misc == instance ) break;
		}
		if ( !p_instance )
		{
			p_instance = (Menu_Linked_E*)malloc(sizeof(Menu_Linked_E));
			p_instance->p_next = p_format->sub_menu;
			p_format->sub_menu = p_instance;
			p_instance->mnemonic = 0;
			p_instance->misc = instance;
			p_instance->cb_data = 0;
			p_instance->cb_func = 0;
			p_instance->sub_menu = 0;
			p_instance->sub_menu_title = 0;
			p_instance->widget = 0;
			p_instance->name = instance_labels[instance];
		}


		/* SET UP TIMING (THIS IS LEAF OF TREE.) */

		p_timing = (Menu_Linked_E*)malloc(sizeof(Menu_Linked_E)+sizeof(CB_Encoding));
		p_cb_encoding = (CB_Encoding*)&p_timing[1];
		p_cb_encoding->channel = p_signals[i_signal].channel;
		p_cb_encoding->format = p_signals[i_signal].format;
		p_cb_encoding->instance = p_signals[i_signal].instance;
		p_cb_encoding->standard = p_signals[i_signal].standard;
		p_timing->p_next = p_instance->sub_menu;
		p_instance->sub_menu = p_timing;
		p_timing->mnemonic = 0;
		p_timing->misc = 0;
		p_timing->cb_data = p_cb_encoding;
		p_timing->cb_func = cb_rtn;
		p_timing->sub_menu = 0;
		p_timing->sub_menu_title = 0;
		p_timing->widget = 0;
		switch (p_signals[i_signal].encoding_id & XV_SOURCE_TIMING)
		{
		  case XV_NTSC:  p_timing->name = "NTSC";  break;
		  case XV_PAL:   p_timing->name = "PAL";   break;
		  case XV_SECAM: p_timing->name = "SECAM"; break;
		}
	}

	/* FILL IN DIGITIZER LABELS. */

	for(p_digit=p_head->sub_menu, i_digit=0; p_digit; 
		p_digit = p_digit->p_next, i_digit++)
	{
		p_digit->name = decoder_labels[i_digit];
	}

	return p_head;
}
