/*
 * sun_raster.c:	Support for Sun Raster file format images.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     26 May 93 - Andy Goodrich
 *       - Initial coding
 *
 * $Log:   /plxhp/dave/pvcsrc/O_XV/archives/sun_raster.c_v  $
 * 
 *    Rev 1.1   09 Sep 1996 09:53:32   dave
 * Initial revision
 * Revision 1.1  1995/06/21 17:17:03  dist
 * Initial revision
 *
 *
 *  Copyright(c) 1993, Andrew C. Goodrich
 */
#include <sys/types.h>			/* Standard types for this C environment. */
#include <stdio.h>				/* Stream input/output declarations. */
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <Xvlib.h>
#include "Xvddx.h"
#include "O_XV.h"
#include "rasterfile.h"

/*  FORWARD REFERENCES: */

XImage*	read_sun_raster();
int		write_sun_raster();


/*----------------------------------------------------------------------------*
 |"print_raster_header"
 |
 |  Read a sun raster file and convert it into an XImage.
 |
 *----------------------------------------------------------------------------*/
void print_raster_header( fd, p_hdr )
	FILE*				fd;		/* -> file descriptor to use. */
	struct rasterfile*	p_hdr;  /* -> header for Sun raster format file. */
{
	
	fprintf(fd, "ras_magic:     0x%08x (%d)\n", 
			p_hdr->ras_magic, p_hdr->ras_magic);
	fprintf(fd, "ras_width:     0x%08x (%d)\n", 
			p_hdr->ras_width, p_hdr->ras_width);
	fprintf(fd, "ras_height:    0x%08x (%d)\n", 
			p_hdr->ras_height, p_hdr->ras_height);
	fprintf(fd, "ras_depth:     0x%08x (%d)\n", 
			p_hdr->ras_depth, p_hdr->ras_depth);
	fprintf(fd, "ras_length:    0x%08x (%d)\n", 
			p_hdr->ras_length, p_hdr->ras_length);
	fprintf(fd, "ras_type:      0x%08x (%d)\n", 
			p_hdr->ras_type, p_hdr->ras_type);
	fprintf(fd, "ras_maptype:   0x%08x (%d)\n", 
		p_hdr->ras_maptype, p_hdr->ras_maptype);
	fprintf(fd, "ras_maplength: 0x%08x (%d)\n", 
		p_hdr->ras_maplength, p_hdr->ras_maplength);
}


/*----------------------------------------------------------------------------*
 |"read_sun_raster"
 |
 |  Read a sun raster file and convert it into an XImage.
 |
 *----------------------------------------------------------------------------*/
XImage* read_sun_raster( fd, dump_hdr, p_display, p_width, p_height )
	FILE*		fd;			/* -> file descriptor for file reading. */
	int			dump_hdr;	/* =  1 if should dump contents of header. */
	Display*	p_display;	/* -> display to create XImage for. */
	int*		p_width;	/* -> where to store width of image. */
	int*		p_height;	/* -> where to store height of image. */
{
	struct rasterfile	header;		 /* Header for Sun raster format file. */
	int					i_col;		 /* Column index of p_im_data8. */
	int				    i_image;	 /* Entry in p_im_dataN accessing. */
	int					i_row;		 /* Row index of p_im_data8. */
	int					n_read;		 /* Number of bytes read. */
	int					n_im_data32; /* # of bytes in p_im_data32. */
	int					pad_width;   /* Padded width of p_im_data8. */
	u_char*				p_cmap;		 /* Lookup table (colormap) data. */
	u_char*				p_cmap_blu;	 /* Lookup table blue data. */
	u_char*				p_cmap_grn;	 /* Lookup table green data. */
	u_char*				p_cmap_red;	 /* Lookup table red data. */
	XImage*				p_image;	 /* Image */
	u_char*				p_im_data8;	 /* 8-bit image data. */
	u_int*				p_im_data32; /* 32-bit image data. */


	/*  READ IN THE RASTER HEADER. */

	if ( fread(&header, sizeof(header), 1, fd) != 1 )
	{
		fprintf(stderr, "Could not read in header\n");
		return 0;
	}
	if ( dump_hdr ) print_raster_header(stdout, &header);
	if ( header.ras_magic != RAS_MAGIC )
	{
		fprintf(stderr, "Bad magic number %08x should be %08x\n", 
			header.ras_magic, RAS_MAGIC);
		return 0;
	}


	/*  READ IN COLOR MAP IF THERE IS ONE. */

	if ( header.ras_maplength != 0 )
	{
		if ( !(p_cmap = (u_char*)malloc(header.ras_maplength)) )
		{
			fprintf(stderr, "Unable to allocate %d byte buffer for cmap\n",
				header.ras_maplength);
			return 0;
		}
		p_cmap_red = p_cmap;
		p_cmap_grn = &p_cmap_red[header.ras_maplength/3];
		p_cmap_blu = &p_cmap_grn[header.ras_maplength/3];
		n_read = fread(p_cmap, 1, header.ras_maplength, fd); 
		if ( n_read != header.ras_maplength )
		{
			fprintf(stderr, "Short cmap length expected %d bytes got %d\n",
				header.ras_maplength, n_read);
			return 0;
		}
	}


	/*  ALLOCATE BUFFER FOR RASTER DATA AND READ IT IN. */

	if ( !(p_im_data8 = (u_int*)malloc(header.ras_length)) )
	{
		fprintf(stderr, "Unable to allocate %d byte buffer for image\n",
			header.ras_length);
		return 0;
	}
	n_read = fread(p_im_data8, 1, header.ras_length, fd); 
	if ( n_read != header.ras_length )
	{
		fprintf(stderr, "Short data length expected %d bytes got %d\n",
			header.ras_length, n_read);
		return 0;
	}


	switch ( header.ras_depth )
	{
	  
	  /*  IF THE RASTER DATA IS 8-BIT PSEUDO COLOR CONVERT IT TO TRUE COLOR. */

	  case 8:
		if ( header.ras_maplength == 0 ) 
		{
			fprintf(stderr, "8-bit image with no color map found\n");
			return 0;
		}
		n_im_data32 = header.ras_length * sizeof(int);
		if ( !(p_im_data32 = (u_int*)malloc(n_im_data32)) )
		{
			fprintf(stderr, "Unable to allocate %d byte buffer for image\n",
				n_im_data32);
			return 0;
		}
		pad_width = (header.ras_width+1) & ~1;
		for ( i_row = 0; i_row < header.ras_height; i_row++ )
		{
			for ( i_col = 0; i_col < header.ras_width; i_col++ )
			{
				p_im_data32[i_row*header.ras_width+i_col] = 
					p_cmap_red[p_im_data8[i_row*pad_width+i_col]]      |
					(p_cmap_grn[p_im_data8[i_row*pad_width+i_col]] << 8) |
					(p_cmap_blu[p_im_data8[i_row*pad_width+i_col]] << 16);
			}
		}
		free((char*)p_cmap);
		free(p_im_data8);
		break;


	  /*  PACKED 24-BIT DATA, EXPAND IT TO XBGR FORMAT. */

	  case 24:
		n_im_data32 = header.ras_width * header.ras_height * sizeof(int);
		if ( !(p_im_data32 = (u_int*)malloc(n_im_data32)) )
		{
			fprintf(stderr, "Unable to allocate %d byte buffer for image\n",
				n_im_data32);
			return 0;
		}
		for ( i_image = 0; i_image < header.ras_length; i_image+=3 )
		{
			p_im_data32[i_image/3] = 
				(p_im_data8[i_image]   << 16 )  | 
				(p_im_data8[i_image+1] << 8) |
				(p_im_data8[i_image+2] << 0);
		}
	    break;


	  /*  TRUE COLOR RASTER DATA, JUST USE IT. */

	  case 32:
		p_im_data32 = (int*)p_im_data8;
		break;


	  /*  DON'T KNOW WHAT TO DO WITH THIS ONE! */

	  default:
		fprintf(stderr, "Unknown depth %d\n", header.ras_depth);
		return 0;
	}		

	/*  BUILD X IMAGE AND RETURN IT TO OUR CALLER. */

	*p_width = header.ras_width;
	*p_height = header.ras_height;

	return XCreateImage(p_display, 
		XDefaultVisual(p_display, XDefaultScreen(p_display)), 
		32, ZPixmap, 0, p_im_data32, *p_width, *p_height, 32, 0);
}


/*----------------------------------------------------------------------------*
 |"write_sun_raster"
 |
 |  Read a sun raster file and convert it into an XImage.
 |
 *----------------------------------------------------------------------------*/
int write_sun_raster( fd, p_image )
	FILE*		fd;			/* -> file descriptor for file writing. */
	XImage*		p_image;	/* -> image to be written in Sun Raster format. */
{
	struct rasterfile	header;		 /* Header for Sun raster format file. */


	/*  CHECK FOR OUR CURRENT RESTRICTIONS: */

	if ( p_image->depth != 24 && p_image->depth != 32 )
	{
		printf("*** write_sun_raster: Only able to handle depth 24 pixmaps\n");
		return 0;
	}


	/*  CREATE HEADER FOR SUN RASTER FILE BASED ON XImage INFORMATION. */

	header.ras_magic     = RAS_MAGIC;
	header.ras_width     = p_image->width;
	header.ras_height    = p_image->height;
	header.ras_depth     = p_image->depth;
	header.ras_length    = p_image->width * p_image->height * 4; /* #### */
	header.ras_type      = RT_STANDARD;
	header.ras_maptype   = RMT_NONE;
	header.ras_maplength = 0;
	if ( fwrite(&header, sizeof(header), 1, fd) != 1 )
	{
		fprintf(stderr, "*** write_sun_raster: could not write out header\n");
		return 0;
	}

	/*  WRITE OUT THE DATA */

	if ( fwrite(p_image->data, 1, header.ras_length, fd) != header.ras_length )
	{
		fprintf(stderr, "*** write_sun_raster unable to write all the data\n");
		return 0;
	}
	return 1;
}
