/*
 *  xvoutput.c:	Output test program for Xv video extension access object.
 *              This program creates an video input window on port 0 and
 *              then starts a video output stream on port 1 from that 
 *              video input window.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     18 June 94 - Andy Goodrich
 *       - Initial coding
 *
 * $Log:   /plxhp/dave/pvcsrc/O_XV/archives/xvoutput.c_v  $
 * 
 *    Rev 1.1   09 Sep 1996 09:55:44   dave
 * Initial revision
 * Revision 1.1  1995/05/16 10:10:32  dist
 * Initial revision
 *
 *
 *  Copyright(c) 1994, Parallax Graphics, Inc.
 */
#include <sys/types.h>			/* Standard types for this C environment. */
#include <stdio.h>				/* Stream input/output declarations. */
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <Xvlib.h>
#include "Xvddx.h"
#include "O_XV.h"

static int  colors[] = {	/* Bar colors. */
	0x00a5a5a5,				/* ... white. */
	0x0000a5a5,				/* ... yellow. */
	0x00a5a500,				/* ... cyan. */
	0x0000a500,				/* ... green. */
	0x00a500a5,				/* ... magenta. */
	0x000000a5,				/* ... red. */
	0x00a50000,				/* ... blue. */
};

/*---------------------------------------------------------------------------*
 |"bars_image"
 |
 |  Create and return a 32-bit image which contains NTSC Color Bars.
 |
 |    -> image on success
 |    =  0 if error detected
 *---------------------------------------------------------------------------*/
XImage* bars_image(type, display, visual)
	int			type;		/* =  type of video image. */
	Display*	display;	/* -> display to put image on. */
	Visual*		visual;		/* =  visual type for basing color table et al. */
{
	int			bar_h;		/* Height of bar. */
	int			bar_w;		/* Width of bar. */
	int			i_bar;		/* Color bar now drawing. */
    int     	i_col;      /* Column now processing. */
	int			image_w;	/* Width of image. */
    uint*   	im_data;    /* Data for image. */
    uint*   	im_pixel;   /* Pixel within im_data accessing. */
    int     	i_pixel;    /* Number of pixels in im_data left to process. */
    int     	i_row;      /* Row now processing. */
 
	switch( type )
	{
	  case XV_NTSC:
		bar_w = 80;
		bar_h = 480;
		image_w = 640;
		break;
	  case XV_PAL:
		bar_w = 95;
		bar_h = 576;
		image_w = 768;
		break;
	}

    /*
     *  ALLOCATE STORAGE FOR THE DATA AND CREATE THE GRID
     *  (1) Clear the background.
     *  (2) Generate the horizontal lines (including one at height-1)
     *  (3) Generate the vertical lines (including one at width-1)
     */
    if ( (im_data = (u_int)malloc(image_w*bar_h*4)) == 0 ) return 0;
     
    for ( i_pixel = image_w*bar_h, im_pixel = im_data; i_pixel; i_pixel-- )
    {
        *im_pixel++ = 0x000000;
    }
       
	for ( i_bar = 0; i_bar < 7; i_bar++ )
	{
		
		for ( i_col = i_bar*bar_w; i_col < (i_bar+1)*bar_w; i_col++ )
		{  
			for ( i_row = 0; i_row < bar_h; i_row++ )
			{
				im_data[i_row*image_w+i_col] = colors[i_bar];
			}
		}
    }
 
    /*
     *  CREATE THE XIMAGE AND RETURN IT TO OUR CALLER
     */
    return XCreateImage(display, visual, 32, ZPixmap, 
		0, im_data, image_w, bar_h, 32, 0);
}


/*----------------------------------------------------------------------------*
 |"main"
 |
 |  Entry point for system for this application.
 |
 *----------------------------------------------------------------------------*/
main( argc, argv )
	int		argc;		/* = # of command arguments parsed by shell. */
	char	*argv[];	/* = vector of pointers to command arguments parsed. */
{
	XImage*	 color_bars;		  /* Image containing color bars. */
	char	 command[256];		  /* Command from user to execute. */
	int		 command_mode;		  /* = 1 if should read commands. */
	Display* display;			  /* Display accessing. */
	int		 hue;				  /* Hue value to set. */
	int		 i_argv;			  /* # of element in 'argv' accessing. */
	int		 o_source_format;	  /* Format of video sink. */
	int		 o_source_number;     /* Instance of video sink format. */
	int		 o_source_type;	      /* Type of video sink. */
	int	 	 parse_port;		  /* Port now processing. */	
	O_XV	 *p_xv0;			  /* Xv access object for port 0. */	
	O_XV	 *p_xv1;			  /* Xv access object for port 1. */	
	int		 saturation;		  /* Saturation value to set. */
	int		 v_source_format;	  /* Format of video source. */
	int		 v_source_number;     /* Instance of video source format. */
	int		 v_source_type;	      /* Type of video source. */
	int		 v_start_off;	  	  /* = 1 if should start with input off. */
	int		 w_source_format;	  /* Format of video source. */
	int		 w_source_number;     /* Instance of video source format. */
	int		 w_source_type;	      /* Type of video source. */
	int		 w_input;			  /* = 1 if w input window should be created. */
	
	if ( !(display = XOpenDisplay(0)) ) 
	{
		fprintf(stderr, "\nCould not open display\n");
		return 0;
	}
	XSynchronize(display, True);

	/*
	 *  PROCESS PARAMETER LIST:
	 */
	v_start_off = 0;
	command_mode = 1;
	w_input = 0;
	parse_port = 'V';

	o_source_format = -1;
	o_source_number = 0;
	o_source_type = -1;

	v_source_format = XV_COMPOSITE;
	v_source_number = 0;
	v_source_type = XV_NTSC;

	for ( i_argv = 1; i_argv < argc; i_argv++ )
	{
		if      ( !strcmp(argv[i_argv], "no_cmd") )
		{
			command_mode = 0;
		}
		else if ( !strcmp(argv[i_argv], "help") )
		{
			fprintf(stderr, 
			"Usage: xvoutput [no_cmd] [o: [source]] [v: [source]] [w: [source]]\n");
			fprintf(stderr, "  where source is [ntsc|pal] [compsite|yc] [#]\n");
			fprintf(stderr, "e.g., xvoutput v: pal composite 0\n");
			return 0;
		}
		else if ( !strcmp(argv[i_argv], "o:") ||
		          !strcmp(argv[i_argv], "out:") )
		{
			parse_port = 'O';
		}
		else if ( !strcmp(argv[i_argv], "v:") )
		{
			parse_port = 'V';
		}
		else if ( !strcmp(argv[i_argv], "w:") )
		{
			parse_port = 'W';
			if ( !w_input )
			{
				w_source_format = XV_COMPOSITE;
				w_source_number = 0;
				w_source_type = XV_NTSC;
				w_input = 1;
			}
		}

		else
		{
			switch( parse_port )
			{
			  case 'O':
				if      ( !strcmp(argv[i_argv], "composite") ) 
					o_source_format = XV_COMPOSITE; 
				else if ( !strcmp(argv[i_argv], "ntsc") ) 
					o_source_type = XV_NTSC;
				else if ( !strcmp(argv[i_argv], "off") )
					v_start_off = 1;
				else if ( !strcmp(argv[i_argv], "pal") ) 
					o_source_type = XV_PAL; 
				else if ( !strcmp(argv[i_argv], "yc") ) 
					o_source_format = XV_YC; 
				else if ( sscanf(argv[i_argv], "%d", &o_source_number)!=1 )
					fprintf(stderr, "%s is an unrecognized parameter\n", 
					argv[i_argv]);
				break;
			  case 'V':
				if      ( !strcmp(argv[i_argv], "composite") ) 
					v_source_format = XV_COMPOSITE; 
				else if ( !strcmp(argv[i_argv], "ntsc") ) 
					v_source_type = XV_NTSC;
				else if ( !strcmp(argv[i_argv], "pal") ) 
					v_source_type = XV_PAL; 
				else if ( !strcmp(argv[i_argv], "yc") ) 
					v_source_format = XV_YC; 
				else if ( sscanf(argv[i_argv], "%d", &v_source_number)!=1 )
					fprintf(stderr, "%s is an unrecognized parameter\n", 
					argv[i_argv]);
				break;
			  case 'W':
				if      ( !strcmp(argv[i_argv], "composite") ) 
					w_source_format = XV_COMPOSITE; 
				else if ( !strcmp(argv[i_argv], "ntsc") ) 
					w_source_type = XV_NTSC;
				else if ( !strcmp(argv[i_argv], "pal") ) 
					w_source_type = XV_PAL; 
				else if ( !strcmp(argv[i_argv], "yc") ) 
					w_source_format = XV_YC; 
				else if ( sscanf(argv[i_argv], "%d", &w_source_number)!=1 )
					fprintf(stderr, "%s is an unrecognized parameter\n", 
					argv[i_argv]);
				break;
			}
		}
	}
    if ( o_source_format == -1 ) o_source_format = v_source_format;
    if ( o_source_type == -1 ) o_source_type = v_source_type;


	/*
	 *  CREATE OBJECT FOR PORT 0 WHICH WILL SERVE AS OUR INPUT IMAGE
	 */
	if ( !(p_xv0 = O_XV_O_XV( 0, display, 0, 0 )) )
	{
		fprintf(stderr, "\nCould not open display\n");
		return 0;
	}
	color_bars = bars_image(v_source_type, p_xv0->display, 
		XDefaultVisual(p_xv0->display, p_xv0->screen));

	O_XV_Source(p_xv0, 0, v_source_type, v_source_format, v_source_number);
	O_XV_Window( p_xv0, -1, -1 );
	O_XV_Video_On(p_xv0);

	/*  
     *  CREATE OBJECT FOR PORT 1 WHICH WILL BE THE OUTPUT IMAGE
	 */
	if ( !(p_xv1 = O_XV_O_XV( 0, display, 0, 1 )) )
	{
		fprintf(stderr, "\nCould not open display\n");
		return 0;
	}
	if ( w_input )
	{
		O_XV_Source(p_xv1, 1, w_source_type, w_source_format, w_source_number);
		O_XV_Window( p_xv1, -1, -1 );
		O_XV_Video_On(p_xv1);
	}

	/*
	 *  START UP VIDEO ON THE OUTPUT CHANNEL (PORT 1) AND PROCESS COMMANDS
	 */
	printf("About to perform get video\n");
	if ( !w_input )
	{
		O_XV_Source(p_xv1, 1, o_source_type, o_source_format, o_source_number);
		XvGetVideo(display, p_xv1->port_id, p_xv0->w_main, p_xv0->gc, 
			0, 0, p_xv0->sig_width, p_xv0->sig_height, 
			0, 0, p_xv0->sig_width, p_xv0->sig_height);
	}

	for(;command_mode;)
	{
		printf("cmd:");
		if ( scanf("%s", command) != 1 ||
		     !strcmp(command, "exit")  ||
		     !strcmp(command, "quit") )
		{
			return 0;
		}
		else if ( !strcmp(command, "help") ||
				  !strcmp(command, "?") )
		{
			fprintf(stderr, "bars, exit, help, hue N, sat N, ");
			fprintf(stderr, "v_off, v_on, ");
			if ( w_input ) fprintf(stderr, "w_bars, w_off, w_on, ");
			fprintf(stderr, "out_off, out_on\n");
		}
		else if ( !strcmp(command, "hue") )
		{
			if ( scanf("%d", &hue) != 1 )
			{
				fprintf(stderr, "*** You must give a hue value\n");
			}
			else
			{
				printf("#### Setting hue to %d\n", hue);
				O_XV_Set_Attribute(p_xv0, p_xv0->a_hue, hue);
			}
		}
		else if ( !strcmp(command, "sat") )
		{
			if ( scanf("%d", &saturation) != 1 )
			{
				fprintf(stderr, "*** You must give a saturation value\n");
			}
			else
			{
				O_XV_Set_Attribute(p_xv0, p_xv0->a_saturation, saturation);
			}
		}
		else if ( !strcmp(command, "bars") ||
				  !strcmp(command, "v_bars") )
		{
			O_XV_Video_Off(p_xv0);
			XPutImage(p_xv0->display, p_xv0->w_main, p_xv0->gc, color_bars, 
				0, 0, 0, 0, p_xv0->sig_width, p_xv0->sig_height);
		}
		else if ( !strcmp(command, "v_off") )
		{
			O_XV_Video_Off(p_xv0);
		}
		else if ( !strcmp(command, "v_on") )
		{
			O_XV_Video_On(p_xv0);
		}
		else if ( !strcmp(command, "w_bars") )
		{
			O_XV_Video_Off(p_xv1);
			XPutImage(p_xv1->display, p_xv1->w_main, p_xv1->gc, color_bars,
				0, 0, 0, 0, p_xv0->sig_width, p_xv0->sig_height);
		}
		else if ( w_input && !strcmp(command, "w_off") )
		{
			O_XV_Video_Off(p_xv1);
		}
		else if ( w_input && !strcmp(command, "w_on") )
		{
			O_XV_Source( p_xv1, 1, XV_NTSC, XV_COMPOSITE, 0 );
			O_XV_Video_On(p_xv1);
		}
		else if ( !strcmp(command, "out_off") )
		{
			XvStopVideo(display, p_xv1->port_id, p_xv0->w_main);
		}
		else if ( !strcmp(command, "out_on") )
		{
			XvGetVideo(display, p_xv1->port_id, p_xv0->w_main, p_xv0->gc, 
				0, 0, p_xv0->sig_width, p_xv0->sig_height, 
				0, 0, p_xv0->sig_width, p_xv0->sig_height);
		}
		else
		{
			fprintf(stderr, 
				"%s is unavailable, enter replacement or cancel\n", command);
		}
	}
	printf("hit return to continue:");
	fgetc(stdin);
}

