/*
 *  pv700_main.c: PowerVideo 700 platform specific routines.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     15 Nov 94 - Andy Goodrich
 *       - Initial coding
 *
 *  Copyright(c) 1994, Parallax Graphics, Inc.
 *
 * $Log:   /plxhp/dave/pvcsrc/hwlib/archives/pv700_main.c_v  $
 * 
 *    Rev 1.2   09 Sep 1996 10:12:36   dave
 * Andy Goodrich: Sun (hedge), HP, and IBM combined hardware library.
 * Revision 1.2  1995/12/11 12:36:50  dist
 * Andy Goodrich: Sun (hedge), HP, and IBM combined hardware library.
 *
 * Revision 1.1  95/12/08  12:19:19  12:19:19  dist ()
 * Initial revision
 * 
 * Revision 1.5  95/06/02  11:38:28  11:38:28  dist ()
 * Andy Goodrich: Improved tracing of routine calls.
 * 
 * Revision 1.4  95/05/25  17:11:35  17:11:35  dist ()
 * Andy Goodrich - HP 9.1 code freeze version.
 * 
 * Revision 1.2  95/04/17  12:28:57  12:28:57  dist ()
 * Andy Goodrich - added RCS header.
 * 
 */
#include <sys/types.h>			/* Standard types for this C environment. */
#include <stdio.h>				/* Stream input/output declarations. */
#include <fcntl.h>				/* Pick up O_RDWR and friends. */
#include "objects_internal.h"	/* PowerVideo 700 object defintions. */
#include "plxv_xpoint.h"		/* Cross point switch definitions. */
#include <sys/iomap.h> /* */
#include <sys/framebuf.h>

/*  XV_DIAG bit definitions:
 *
 *    0x01 - normal one-shot diagnostics
 *    0x40 - method call tracing
 *    0x80 - hardware library initialization tracing
 */
/* #define XV_DIAG 0x41   */
#include "xvdebug.h"


typedef union _Mode_Reg_1 {
    int word;           /* 32-bit word access. */
    struct {            /* Bit fields within register: */
        int unused:25;  /* ... there is nothing in top 26 bits. */
        int DENB:1;     /* ... delay line serial load enable. */
        int DLSD:1;     /* ... delay line serial data input. */
        int DLCK:1;     /* ... delay line serial clock. */
        int ENVCO:2;    /* ... high res. clock oscillator. */
        int BYPASS:1;   /* ... genlock: = 0 encode RGB, 1 CVBS genlock. */
        int VOBLK:1;    /* ... video output blank: 0 use low res. encoder */
    } bits;
} Mode_Reg_1;

static O_Draw_Gc erase_gc = {	/* Drawing context for screen erase. */
	0, 				/* Color */
	0,				/* Transparency flag. */
	PLXV_ROP_COPY,	/* Raster operation. */
	0,				/* Stencil mask. */
	0,				/* Stencil value. */
	0,				/* Window id mask. */
	0,				/* Window id value. */
	PLXV_WMASK_ALL  /* Write mask. */
};


/*  FORWARD FUNCTION REFERENCES: */

void        m_set_delay();
O_Input*    O_Input_O_Input();
O_Output*   O_Output_O_Output();


/*---------------------------------------------------------------------------*
 |"calc_serration"
 |
 |  Calculate serration values given display geometry.
 |
 *---------------------------------------------------------------------------*/
static void calc_serration(front_porch, sync, back_porch, visible, p_hs, p_hn)
	int		front_porch;	/* =  front porch of horiziontal signal. */
	int		sync;			/* =  horizontal sync pulse width. */
	int		back_porch;		/* =  front porch of horiziontal signal. */
	int		visible;		/* =  horizontal visible pixel width. */
	int*	p_hs;			/* -> where to store hs value. */
	int*	p_hn;			/* -> where to store hn value. */
{
	int		total;			/* Total line time. */

	total = front_porch + sync + back_porch + visible;
	*p_hs = visible + back_porch + front_porch;
	*p_hn = sync - back_porch - front_porch;

	/*
	 *  SUBTRACT ONE FROM HS AND HN VALUES. IF THE HN VALUE
	 *  IS NEGATIVE ADJUST IT TO 0 AND REDUCE HS BY THE SAME
	 *  AMOUNT.
	 */
	*p_hn -= 1;
	*p_hs -= 1;
	if ( *p_hn < 0 )
	{
		*p_hs += *p_hn;
		*p_hn = 0;
	}
	*p_hn |= 0x8000;
}


/*---------------------------------------------------------------------------*
 |"m_ccube_present"
 |
 |  Return whether a ccube chip is present on this board or not.
 |
 |    = 1 if ccube present, 0 if not.
 *---------------------------------------------------------------------------*/
static int m_ccube_present( p_me )
	XV_struct*	p_me;	/* -> hardware context. */
{
    int interrupts;

	DEBUGF(0x40, (fd, "%s: m_ccube_present: entry\n", p_me->p_mmap->name));
    PV_RREG(p_me,interrupts,IntStatC.l);
    PV_RREG(p_me,interrupts,IntStatC.l);
    p_me->main.c3_rev =  (interrupts & (1<<1)) ? 0 : 1;
	DEBUGF(0x40, (fd, "%s: m_ccube_present: exit ccube %s present\n", 
		p_me->p_mmap->name, p_me->main.c3_rev ? "" : "not")
	);
    return p_me->main.c3_rev;
}


/*---------------------------------------------------------------------------*
 |"m_dump"
 |
 |  Dump the contents of a Parallax access object. 
 |
 *---------------------------------------------------------------------------*/
static void m_dump( p_me, fd )
	XV_struct*	p_me;	/* -> hardware context. */
	FILE*		fd;		/* -> descriptor for file to dump to. */
{
	int		i_unit;		/* Unit # of object pointer now dumping. */
	int		n_units;	/* # of objects to dump. */

	fprintf(fd, "\n\nDump of XV_struct for device %s at 0x%08x:\n", 
		p_me->p_mmap->name, p_me);
	fprintf(fd, "   atv          = 0x%08x \n", p_me->atv);
    fprintf(fd, "   c3base       = 0x%08x \n", p_me->c3base);
    fprintf(fd, "   c3_rreg      = 0x%08x \n", p_me->c3_rreg);
    fprintf(fd, "   c3_wreg      = 0x%08x \n", p_me->c3_wreg);
    fprintf(fd, "   eprombase    = 0x%08x \n", p_me->eprombase);
	fprintf(fd, "   fd_ccube     = 0x%08x \n", p_me->fd_ccube);
    fprintf(fd, "   maps         = 0x%08x \n", p_me->maps);
    fprintf(fd, "   m1base       = 0x%08x \n", p_me->m1base);
    fprintf(fd, "   m8base       = 0x%08x \n", p_me->m8base);
    fprintf(fd, "   m24base      = 0x%08x \n", p_me->m24base);
	n_units = sizeof(p_me->p_inputs) / sizeof(p_me->p_inputs[0]);
	for( i_unit = 0; i_unit < n_units; i_unit++ )
	{
		fprintf(fd, "   p_inputs[%d]  = 0x%08x \n", i_unit, 
			p_me->p_inputs[i_unit]);
	}
	for( i_unit = 0; i_unit < n_units; i_unit++ )
	{
		fprintf(fd, "   p_outputs[%d] = 0x%08x \n", i_unit, 
			p_me->p_outputs[i_unit]);
	}
	fprintf(fd, "   p_i2c        = 0x%08x \n", p_me->p_i2c);
	fprintf(fd, "   p_rsrc       = 0x%08x \n", p_me->p_rsrc);
    fprintf(fd, "   regbase      = 0x%08x \n", p_me->regbase);
	fprintf(fd, "   xvm          = 0x%08x \n", p_me->xvm);
    fprintf(fd, "   xv_rreg      = 0x%08x \n", p_me->xv_rreg);
    fprintf(fd, "   xv_shdw      = 0x%08x \n", p_me->xv_shdw);
    fprintf(fd, "   xv_wreg      = 0x%08x \n", p_me->xv_wreg);
}

/*---------------------------------------------------------------------------*
 |"m_get_zoom"
 |
 |  Return current zoom factor.
 |
 |    = current zoom factor.
 *---------------------------------------------------------------------------*/
static int m_get_zoom( p_me )
	XV_struct*	p_me;	/* -> hardware context. */
{
	DEBUGF(0x40, (fd, "%s: m_get_zoom: returning %d\n", 
		p_me->p_mmap->name, p_me->main.zoom)
	);
	return p_me->main.zoom;
}


/*---------------------------------------------------------------------------*
 |"m_init_context"
 |
 |  Initialize context from current resource object values.
 *---------------------------------------------------------------------------*/
static void m_init_context( p_me )
	XV_struct*	p_me;	/* -> hardware context. */
{
    XV_addr_maps*   ap;
	int				i_row;
    int 			Xres;
	int				Yres;

	DEBUGF(0x40, (fd, "%s: m_init_context: entry\n", p_me->p_mmap->name));
	O_Rsrc_store_to_context(p_me->p_rsrc, p_me);
	ap = p_me->maps;

    switch ( Xres = p_me->main.linebytes )
	{
		case 1152:	p_me->addr_wrap = 0x100000;	break;
		case 1280:	p_me->addr_wrap = 0x140000;	break;
	}
    Yres = p_me->main.ysize;
 
    for ( i_row=0; i_row<Yres; i_row++) 
		ap->reg[i_row] =  (Xres *  i_row);
    for ( i_row=0; i_row<Yres; i_row++) 
		ap->m24[i_row] = (int *)( (int) p_me->m24base + ((Xres*4) * i_row));
    for ( i_row=0; i_row<Yres; i_row++) 
		ap->m1[i_row] = (int *)( (int) p_me->m1base + ((Xres/8) * i_row));

	DEBUGF(0x40, (fd, "%s: m_init_context: exit\n", p_me->p_mmap->name));
}


/*---------------------------------------------------------------------------*
 |"m_init_disp"
 |
 |  Initialize display hardware from current context values. 
 *---------------------------------------------------------------------------*/
static void m_init_disp( p_me )
	XV_struct*	p_me;	/* -> hardware context. */
{
    int   h_sync_n;	/* Serration value adjusted to short. */
    int   h_sync_s;	/* Serration value adjusted to short. */

	DEBUGF(0x40, (fd, "%s: m_init_disp: entry\n", p_me->p_mmap->name));


	/* INITIALIZE PARAMETERS WHICH ARE ZOOM-INDEPENDENT. */

    PV_WREG(p_me, StartUp.l,    p_me->main.StartUp);
	PV_WREG(p_me, VFP.l,        p_me->main.v.fp); 
	PV_WREG(p_me, VBP.l,        p_me->main.v.bp);
	PV_WREG(p_me, VDisp.l,      p_me->main.v.disp);
	PV_WREG(p_me, VSync.l,      p_me->main.v.sy);


	/* INITIALIZE PARAMETERS WHICH CHANGE BASED ON ZOOM. */

	switch ( p_me->main.zoom )
	{
      default: 
		PV_WREG(p_me, DispFlgs.l,   p_me->main.DispFlgs);
		PV_WREG(p_me, HFP.l,        p_me->main.h.fp);
		PV_WREG(p_me, HSync.l,      p_me->main.h.sy);
		PV_WREG(p_me, HBP.l,        p_me->main.h.bp);
		PV_WREG(p_me, HDisp.l,      p_me->main.h.disp);  
		calc_serration(p_me->main.h.fp, p_me->main.h.sy, 
			p_me->main.h.bp, p_me->main.h.disp, &h_sync_s, &h_sync_n);
		break;

	  case 2:
		PV_WREG(p_me, DispFlgs.l,   p_me->zoom[0].DispFlgs); 
		PV_WREG(p_me, HFP.l,        p_me->zoom[0].h.fp);
		PV_WREG(p_me, HSync.l,      p_me->zoom[0].h.sy);
		PV_WREG(p_me, HBP.l,        p_me->zoom[0].h.bp);
		PV_WREG(p_me, HDisp.l,      p_me->zoom[0].h.disp);  
		calc_serration(p_me->zoom[0].h.fp, p_me->zoom[0].h.sy, 
			p_me->zoom[0].h.bp, p_me->zoom[0].h.disp, &h_sync_s, &h_sync_n);
		break;
	}
	PV_WREG(p_me, HSyncS.l,     h_sync_s);
	PV_WREG(p_me, HNSyncS.l,    h_sync_n);
	PV_WREG(p_me, DispKludge.l, p_me->main.DispKludge);

	DEBUGF(0x40, (fd, "%s: m_init_disp: exit\n", p_me->p_mmap->name));
}


/*---------------------------------------------------------------------------*
 |"m_init_hw"
 |
 |  Initialize hardware from current context values. 
 *---------------------------------------------------------------------------*/
static void m_init_hw( p_me )
	XV_struct*	p_me;	/* -> hardware context. */
{
	static XV_video_format  loop_format = { /* Default video format. */
		640,482,43,7,21
	};

	DEBUGF(0x40, (fd, "%s: m_init_hw: entry\n", p_me->p_mmap->name));


	/*  TELL DD SCREEN HEIGHT. */

	m_vertical(p_me, p_me->main.ysize);


	/*  INITIALIZE THE STANDARD COMPONENTS AND ERASE THE SCREEN TO BLACK. */

    plxv_init_bus(p_me);
	PV_WREG(p_me, DispKludge.l, 0x5000);
	O_Lut_init_hw(p_me->p_lut);
    m_init_disp(p_me);
    O_Draw_init_hw(p_me->p_draw);
#ifdef REMOVE
	plxv_init_timer(p_me);
#endif
	plxv_init_video(p_me);
    O_PV700_init_pll(p_me, 0);
	O_Draw_set_gc(p_me->p_draw, &erase_gc);
	O_Draw_fill_box(p_me->p_draw, 0, 0, p_me->main.xsize, p_me->main.ysize);


	/* INITIALIZE INPUT/OUTPUT PORTS: */

	DEBUGF(0x40, (fd, "%s: m_init_hw: before i2c devices\n", p_me->p_mmap->name));
	if ( p_me->p_outputs[PORT_A] ) O_Output_init_hw(p_me->p_outputs[PORT_A]);
	if ( p_me->p_inputs[PORT_B] ) O_Input_init_hw(p_me->p_inputs[PORT_B]);
	if ( p_me->p_inputs[PORT_D] ) O_Input_init_hw(p_me->p_inputs[PORT_D]);
	p_me->plxv_io_format[4] = &loop_format;
	p_me->format_v_ptr = &(p_me->plxv_io_format[4]);
	p_me->format_w_ptr = &(p_me->plxv_io_format[4]);


	/*  INITIALIZE CCUBE IF WE HAVE ONE. */

	if ( m_ccube_present(p_me) )
	{
		C3Init(p_me);
	}

	DEBUGF(0x40, (fd, "%s: m_init_hw: exit\n", p_me->p_mmap->name));
}


/*---------------------------------------------------------------------------*
 |"m_init_pll"
 |
 |  Set the values associated with a host frame buffer frequency.
 |  A frequency of zero defaults frequency to nvram setting.
 |  Note we set the mode register to disable the second video input and
 |  the video output. These will be enabled by their access objects when 
 |  they are selected.
 |
 |    = 1 on success, 0 on failure
 *---------------------------------------------------------------------------*/
static int m_init_pll( p_me, frequency )
	XV_struct*	p_me;		/* -> hardware library context. */
	int			frequency;	/* =  frequency to set up PLL for. */
{
	int		delay;		/* Delay value to use base on zoom factor. */
	int		skip;		/* Skip value to use based on zoom factor. */
	int		zoom_bits;	/* Zoom specification. */
	
	DEBUGF(0x40, (fd, "%s: m_init_pll: entry\n", p_me->p_mmap->name));


	 /*  SET MODE REGISTER TWO WITH DMSD #2 AND DMSE DISABLED */
	 
	PV_WREG(p_me, Bt.grn.reg2, 0x5e);

	switch ( p_me->main.zoom )
	{
	  default:
		delay = p_me->main.delay;
		skip = p_me->main.skip >> 1;
		zoom_bits = 0x00000003;
		break;
	  case 2:
		delay = p_me->zoom[0].delay;
		skip = p_me->zoom[0].skip >> 1;
		zoom_bits = 0x00000002;
		break;
	}


	/*  PROGRAM FOR THE PROPER FREQUENCY: */
	 
	if ( !frequency )
	{
		frequency = p_me->main.pixel_clock;
	}

	switch(frequency)
	{
	  case 135:
		PV_WREG(p_me, Bt.blu.reg0, 0xaa); 		  /* Load pll window counter. */
		PV_WREG(p_me, Bt.grn.reg1, 0x01);
		m_set_delay(p_me, delay);
		PV_WREG(p_me, MiscMode.b.grn, zoom_bits);
		PV_WREG(p_me, MiscMode.b.red, 0x00000000); 			   /* Select vco. */
		PV_WREG(p_me, MiscMode.b.blu, skip);
		break;
	
	  case 108:
		PV_WREG(p_me, Bt.blu.reg0, 0xaa);
		PV_WREG(p_me, Bt.grn.reg1, 0x05);
		m_set_delay(p_me, delay);
		PV_WREG(p_me, MiscMode.b.red, 0x00000000); 			   /* Select vco. */
		PV_WREG(p_me, MiscMode.b.grn, zoom_bits);
		PV_WREG(p_me, MiscMode.b.blu, skip);
		break;
	
	  case 84:
		PV_WREG(p_me, Bt.blu.reg0, 0xaa);
		PV_WREG(p_me, Bt.grn.reg1, 0x09);
		m_set_delay(p_me, delay);
		PV_WREG(p_me, MiscMode.b.red, 0x00000000); 			   /* Select vco. */
		PV_WREG(p_me, MiscMode.b.grn, 0x00000004 | zoom_bits);
		PV_WREG(p_me, MiscMode.b.blu, skip);
		break;
	
	  case 64:
		PV_WREG(p_me, Bt.blu.reg0, 0xaa);
		PV_WREG(p_me, Bt.grn.reg1, 0x0d);
		m_set_delay(p_me, delay);
		PV_WREG(p_me, MiscMode.b.red, 0x00000000); 			   /* Select vco. */
		PV_WREG(p_me, MiscMode.b.grn, 0x00000004 | zoom_bits);
		PV_WREG(p_me, MiscMode.b.blu, skip);
		break;
	
	  default:
		PV_WREG(p_me, Bt.blu.reg0, 0xaa);
		PV_WREG(p_me, Bt.grn.reg1, 0x01);
		m_set_delay(p_me, frequency);
		PV_WREG(p_me, MiscMode.b.red, 0x00000004);
		PV_WREG(p_me, MiscMode.b.grn, zoom_bits);
		PV_WREG(p_me, MiscMode.b.blu, skip);
		break;
	}
	m_vertical(p_me, 1);

	DEBUGF(0x40, (fd, "%s: m_init_pll: exit\n", p_me->p_mmap->name));
	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_set_delay"
 |
 |  Store the delay value. This is done through a serial interface in mode
 |  register 2. For each bit (from the most significant to the least 
 |  significant) of the 8-bit delay value we perform the following steps:
 |    (1) Pick up original bits for register 2 from the shadow area.
 |    (2) Set the serial clock bit to 0, and the serial data bit to the value
 |        of the bit in the 'delay' parameter to be set.
 |    (3) Set the serial clock bit to 1, with the serial data bit still set
 |        to the bit in the 'delay' parameter to be set.
 |    (4) Set the serial clock bit to 0.
 |
 |	
 *---------------------------------------------------------------------------*/
static void m_set_delay( p_me, delay )
    XV_struct* p_me;
	int		delay;		/* =  delay value to be loaded. */
{
	int			i_bit;	/* # of bit (from MSB to LSB) clocking into register. */
	Mode_Reg_1	reg1;	/* Value for mode register 1 from the shadow area. */

	DEBUGF(0x40, (fd, "%s: m_set_delay: entry\n", p_me->p_mmap->name));

	reg1.word = p_me->xv_shdw->b.Bt.grn.reg1;


	/*  ENABLE USE OF DELAY LINE SERIAL INTERFACE.  */

	reg1.bits.DLSD = 0;
	reg1.bits.DLCK = 0;
	PV_WREG(p_me, Bt.grn.reg1, reg1.word);
	reg1.bits.DENB = 1;
	PV_WREG(p_me, Bt.grn.reg1, reg1.word);

	for ( i_bit = 0; i_bit < 8; i_bit++ )
	{
		reg1.bits.DLSD = (delay & 0x80) ? 1 : 0;
		reg1.bits.DLCK = 0;
		PV_WREG(p_me, Bt.grn.reg1, reg1.word);

		reg1.bits.DLCK = 1;
		PV_WREG(p_me, Bt.grn.reg1, reg1.word);

		reg1.bits.DLCK = 0;
		PV_WREG(p_me, Bt.grn.reg1, reg1.word);

		delay <<= 1;
	}

	/*
	 *  DISABLE USE OF DELAY LINE SERIAL INTERFACE.
	 */
	reg1.bits.DLSD = 0;
	reg1.bits.DLCK = 0;
	PV_WREG(p_me, Bt.grn.reg1, reg1.word);
	reg1.bits.DENB = 0;
	PV_WREG(p_me, Bt.grn.reg1, reg1.word);

	DEBUGF(0x40, (fd, "%s: m_set_delay: exit\n", p_me->p_mmap->name));
}


/*---------------------------------------------------------------------------*
 |"m_set_zoom"
 |
 |  Set zoom factor.
 *---------------------------------------------------------------------------*/
static int m_set_zoom( p_me, factor )
	XV_struct*	p_me;	/* -> hardware context. */
	int			factor;	/* =  zoom factor. */
{
	DEBUGF(0x40, (fd, "%s: m_set_zoom: entry setting zoom to %d\n", 
		p_me->p_mmap->name, factor));

	p_me->main.zoom = factor;
	m_vertical(p_me, 0);
	switch(factor)
	{
	  case 1:
		PV_WREG(p_me, MiscMode.b.grn, 0x00000003);
		m_init_pll(p_me, 0);
		m_init_disp(p_me);
		O_Draw_set_gc(p_me->p_draw, &erase_gc);
		O_Draw_fill_box(p_me->p_draw, 0, 0, p_me->main.xsize, p_me->main.ysize);
		break;

	  case 2:
		PV_WREG(p_me, MiscMode.b.grn, 0x00000002);
		PV_WREG(p_me, DispKludge.l, 0x5000);
		m_init_disp(p_me);
		m_init_pll(p_me, 0);
		break;

	  case 4:
		PV_WREG(p_me, MiscMode.b.grn, 0x00000001);
		switch( p_me->main.board_code )
		{
		  default:
			break;
		}
		m_set_delay(p_me, 0x14);
		break;
	}
	m_vertical(p_me, 1);
	DEBUGF(0x40, (fd, "%s: m_set_zoom: exit\n", p_me->p_mmap->name));
	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_terminate"
 |
 |  Terminate use of this hardware context, freeing its storage.
 *---------------------------------------------------------------------------*/
static void m_terminate( p_me )
	XV_struct*	p_me;	/* -> hardware context. */
{
	DEBUGF(0x40, (fd, "%s: m_terminate: entry\n", p_me->p_mmap->name));

	if ( p_me->p_draw ) O_Draw_terminate(p_me->p_draw);
	if ( p_me->p_i2c ) O_I2C_terminate(p_me->p_i2c);
	if ( p_me->p_inputs[PORT_B] )  O_Input_terminate(p_me->p_inputs[PORT_B]);
	if ( p_me->p_inputs[PORT_D] )  O_Input_terminate(p_me->p_inputs[PORT_D]);
	if ( p_me->p_lut ) O_Lut_terminate(p_me->p_lut);
	if ( p_me->p_mmap ) O_Mmap_terminate(p_me->p_mmap);
	if ( p_me->p_outputs[PORT_A] ) O_Output_terminate(p_me->p_outputs[PORT_A]);
	if ( p_me->p_rsrc ) O_Rsrc_terminate(p_me->p_rsrc);

	free((char*)p_me->maps);
	free((char*)p_me->xv_shdw);
	free((char*)p_me);
	DEBUGF(0x40, (fd, "%s: m_terminate: exit\n", p_me->p_mmap->name));
}


/*---------------------------------------------------------------------------*
 |"m_vertical"
 |
 |  Turn on or off vertical interrupt processing in the device driver.
 | 
 |  	= return code from ioctl (0 is success, ~0 is failure)
 *---------------------------------------------------------------------------*/
static int m_vertical( p_me, on_off )
	XV_struct*	p_me;	/* -> hardware context. */
	int			on_off;	/* =  1 if enable, 0 if disable, resolution otherwise */
{
    int             ioctl_data;
	int				rc;

	DEBUGF(0x40, (fd, "%s: m_vertical: entry, request for %d\n", 
		p_me->p_mmap->name, on_off));

    ioctl_data = on_off;
    rc = ioctl(p_me->fd_ccube, GCTEST, &ioctl_data);
    if ( rc )
	{
        ERRORF((fd, "**** ioctl error sending %d to driver\n", on_off));
	}
	DEBUGF(0x40, (fd, "%s: m_vertical: exit, rc=%d\n", p_me->p_mmap->name, rc));
	return rc;
}


static TV_PV main_methods_tv = {
	m_ccube_present,
	m_dump,
	m_get_zoom,
	m_init_context,
	m_init_disp,
	m_init_hw,
	m_init_pll,
	m_set_delay,
	m_set_zoom,
	m_terminate,
	m_vertical
};


/*---------------------------------------------------------------------------*
 |"O_HWL_O_HWL"
 |
 |  Open a Parallax PowerVideo 700 device to a specified initialization level:
 |     "minimum"    - create only its I2C and resource objects.
 |     "no_hw_init" - create all hardware access objects, but don't 
 |                    initialize the hardware.
 |     "full"       - create a fully initialized access object.
 *---------------------------------------------------------------------------*/
XV_struct* O_HWL_O_HWL(level, device, rsrc_file)
	char* level;		/* -> object level: "minimum", "no_hw_init", "full". */
    char* device;		/* -> name of device to open. */
	char* rsrc_file;	/* -> resource name or 0. */
{
    XV_addr_maps*       ap;			/* Address maps structure. */
	FILE*				fd_rsrc;	/* File to load resource values from. */
	int					i_shadow;	/* # of word in shadow registers zeroing. */
	int					n_shadow;	/* # of words in shadow registers. */
	O_Mmap*				p_mmap;		/* Memory map of PV700 device. */
	int*				p_shadow;	/* Word in shadow registers zeroing. */
    XV_struct*         	p_xv;		/* Context we are filling in. */
    XV_shdw_regs*      	sp;			/* Shadow register area. */
    XV_mem_map*        	xvm;		/* Map for card. */

	TRACEF(0x80, (fd, "O_HWL_O_HWL(%s, %s, %s): Entry, stack @ %08x\n", 
		level, device, rsrc_file, &ap));

	/*  OPEN DEVICE AND ACQUIRE AN ADDRESS MAP FOR IT. */

	if ( !(p_mmap = O_Mmap_O_Mmap(device)) )
	{
		fprintf(stderr, "*** Unable to memory map %s\n", device);
	}


	/*  ALLOCATE STRUCTURES WE NEED, INCLUDING OUR CONTEXT: */
 
    if ((p_xv = (XV_struct *)malloc(sizeof(XV_struct))) == NULL)
    {   /* malloc failed */
        printf("malloc failed on plxv_init_struct:main struct\n");
        exit(-1);
    }
    if ((sp = (XV_shdw_regs *)malloc(sizeof(XV_shdw_regs))) == NULL)
    {   /* malloc failed */
        printf("malloc failed on plxv_init_struct:xv_shdw\n");
        exit(-1);
    }
    if ((ap = (XV_addr_maps *)malloc(sizeof(XV_addr_maps))) == NULL)
    {   /* malloc failed */
        printf("malloc failed on plxv_init_struct:addr_maps\n");
        exit(-1);
    }


	/* FILL IN CONTEXT WITH MEMORY MAP ADDRESSES: */

	TRACEF(0x80, (fd, "O_HWL_O_HWL(%s, %s, %s): setting up p_xv\n",
		level, device, rsrc_file));
	xvm = (XV_mem_map*)O_Mmap_virtual(p_mmap, 0);
	p_xv->p_mmap = p_mmap;
	p_xv->atv = (void*)&main_methods_tv;
    p_xv->c3base = (int*)&xvm->Ccube;
    p_xv->c3_rreg = (C3_RREGS *)p_xv->c3base;
    p_xv->c3_wreg = (C3_WREGS *)p_xv->c3base;
    p_xv->eprombase = (int*)&xvm->Prom0;
	p_xv->fd_ccube = p_mmap->fd_ccube;
    p_xv->maps = ap;
    p_xv->m1base = (int*)&xvm->M1;
    p_xv->m8base = (int*)&xvm->M8;
    p_xv->m24base = (int*)&xvm->M24;
    p_xv->regbase = (int*)&xvm->RW_regs;
	p_xv->xvm = xvm;
    p_xv->xv_rreg = (XV_read_regs*)p_xv->regbase;
    p_xv->xv_shdw = (XV_shdw_regs*)sp;
    p_xv->xv_wreg = (XV_write_regs*)p_xv->regbase;


    /* ZERO REGISTER FILE SHADOW STRUCTURE. */

    n_shadow = sizeof(XV_shdw_regs) >> 2;
    for (p_shadow = (int*)sp, i_shadow = 0; i_shadow < n_shadow; i_shadow++ ) 
	{
		*p_shadow++ = 0;
	}


	/* ALLOCATE MINIMUM SET OF OBJECTS AND INITIALIZE CONTEXT FROM RESOURCES. */

	TRACEF(0x80, (fd, "O_HWL_O_HWL(%s, %s, %s): creating i2c & rsrc objects\n",
		level, device, rsrc_file));
    p_xv->p_i2c = O_I2C_O_I2C(p_xv);
    p_xv->p_draw = O_Draw_O_Draw(0, p_xv);
    p_xv->p_rsrc = O_Rsrc_O_Rsrc();
	if ( rsrc_file )
	{	
		if ( fd_rsrc = fopen(rsrc_file, "r") )
		{
			O_Rsrc_load_from_file(p_xv->p_rsrc, fd_rsrc);
			fclose(fd_rsrc);
		}
	}


	/*  IF A MINIMUMAL OBJECT WAS REQUESTED WE ARE DONE. */

    if ( !strcmp(level, "minimum") ) 
	{
		m_init_context(p_xv);
		TRACEF(0x80, (fd, 
		   "O_HWL_O_HWL(%s, %s, %s): Exit, with a minimal hardware object\n",
			level, device, rsrc_file)
		);
		return p_xv;
	}


	/* INITIALIZE OBJECTS FOR INPUT OUTPUT PORTS AND LOOKUP TABLE: */

	TRACEF(0x80, (fd, "O_HWL_O_HWL(%s, %s, %s): creating i/o objects\n",
		level, device, rsrc_file));
	p_xv->p_outputs[PORT_A] = O_Output_O_Output(0, 0, p_xv); 
	p_xv->p_inputs[PORT_B]  = O_Input_O_Input( 0, 1, p_xv );
	p_xv->p_inputs[PORT_D] = O_Input_O_Input( 0, 0, p_xv ); 

	TRACEF(0x80, (fd, "O_HWL_O_HWL(%s, %s, %s): creating lookup table object\n",
		level, device, rsrc_file));
	p_xv->p_lut = O_Lut_Bt495_O_Lut_Bt495(0, p_xv);


	TRACEF(0x80, (fd, "O_HWL_O_HWL(%s, %s, %s): initializing context\n",
		level, device, rsrc_file));
	m_init_context(p_xv);     /* Necessary after creating inputs. */


	/*  IF AN OBJECT WITHOUT HARDWARE INITIALIZATION IS WANTED WE ARE DONE. */

	if ( !strcmp(level, "no_hw_init") ) 
	{
		TRACEF(0x80, (fd, 
		   "O_HWL_O_HWL(%s, %s, %s): Exit, w/a hardware object w/no hwinit\n",
			level, device, rsrc_file)
		);
		return p_xv;
	}

		
	/*  INITIALIZE ALL THE HARDWARE... */

	TRACEF(0x80, (fd, "O_HWL_O_HWL(%s, %s, %s): initializing hardware\n",
		level, device, rsrc_file));
	m_init_hw( p_xv );

	TRACEF(0x80, (fd, "O_HWL_O_HWL(%s, %s, %s): Exit, stack @ %08x\n", 
		level, device, rsrc_file, &ap));
	return p_xv;
}


/*---------------------------------------------------------------------------*
 |"O_HWL_device"
 |
 |  Return name of Parallax device to use. This will either be the value of
 |  the environment variable PLX_DEVICE or a default of "/dev/plxv0".
 *---------------------------------------------------------------------------*/
char* O_HWL_device()
{
	char*	device;

	device = getenv("PLX_DEVICE");

	return device ? device : "/dev/plxv0";
}


/*---------------------------------------------------------------------------*
 |"O_HWL_rsrc"
 |
 |  Return name of resource file to use. This will either be the value of the
 |  environment variable PLX_RSRC or a default of "/opt/parallax/etc/plxv0.
 *---------------------------------------------------------------------------*/
char* O_HWL_rsrc()
{
	char*	rsrc;

	rsrc = getenv("PLX_RSRC");

	return rsrc ? rsrc : "/opt/parallax/etc/plxv0";
}

/*---------------------------------------------------------------------------*
 |"O_Input_O_Input"
 |
 |  Return an input device given a hardware context and unit number for the
 |  device.
 |
 |    -> object on success, 0 if error detected or device not present.
 *---------------------------------------------------------------------------*/
O_Input* O_Input_O_Input(p_me, unit, p_xv)
    O_Input*    p_me;   /* -> object storage or 0. */
    int         unit;   /* =  unit number of device. */
    XV_struct*  p_xv;   /* -> hardware library context to use. */
{
    return (O_Input*)O_Input_Dmsd_O_Input_Dmsd(p_me, unit, p_xv);
}


/*---------------------------------------------------------------------------*
 |"O_Output_O_Output"
 |
 |  Return an output device given a hardware context and unit number for the
 |  device.
 |
 |    -> object on success, 0 if error detected or device not present.
 *---------------------------------------------------------------------------*/
O_Output* O_Output_O_Output(p_me, unit, p_xv)
    O_Output*   p_me;   /* -> object storage or 0. */
    int         unit;   /* =  unit number of device. */
    XV_struct*  p_xv;   /* -> hardware library context to use. */
{
    return (O_Output*)O_Output_Dmse_O_Output_Dmse(p_me, unit, p_xv);
}

