/*
 *  pvpci_main.c: PowerVideo PCI platform specific routines.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     15 Nov 94 - Andy Goodrich
 *       - Initial coding
 *
 *  Copyright(c) 1994, Parallax Graphics, Inc.
 *
 * $Log:   /plxhp/dave/pvcsrc/hwlib/archives/pvpci_main.c_v  $
 * 
 *    Rev 1.10   09 Sep 1996 10:13:42   dave
 * shut off output on terminate
 * Revision 1.10  1996/04/20 00:12:11  dist
 * shut off output on terminate
 *
 * Revision 1.9  1995/12/28  12:55:30  dist
 * Andy Goodrich: Set Sun platform default value for O_HWL_device().
 *
 * Revision 1.8  1995/12/22  18:56:45  dist
 * Andy Goodrich: Added visibility transfer vector routine.
 *
 * Revision 1.7  1995/12/18  11:15:17  dist
 * Andy Goodrich: Fixes to vert() non-driver overlay synchronization.
 *
 * Revision 1.6  95/12/11  14:38:35  14:38:35  dist ()
 * Andy Goodrich: Converted over to Lookup table objects like HP.
 * 
 * Revision 1.5  95/12/09  00:31:32  00:31:32  dist ()
 * Andy Goodrich: Added diagnostic trace information to vert().
 * 
 * Revision 1.4  1995/12/08  23:59:52  dist
 * Andy Goodrich: Added object parameter to front of Input and Output object
 *                creation invocations.
 *
 * Revision 1.3  95/12/08  14:52:33  14:52:33  dist ()
 * Andy Goodrich: Added forward references to keep AIX C compiler happy.
 * 
 * Revision 1.2  95/12/08  14:49:03  14:49:03  dist ()
 * Andy Goodrich: Added O_Input_O_Input and O_Output_O_Output.
 * 
 * Revision 1.1  95/12/08  10:28:43  10:28:43  dist ()
 * Initial revision
 * 
 * Revision 1.4  95/09/13  10:04:51  10:04:51  dist ()
 * Andy Goodrich: Added device name to memory map creation call
 *                to allow for multiple cards in a system.
 * 
 * Revision 1.3  95/08/16  13:32:55  13:32:55  dist ()
 * Andy Goodrich: Added check on return value of O_Mmap_O_Mmap for null.
 *                If we get a null we return a null rather than dying a
 *                horrible death.
 * 
 * Revision 1.2  95/08/16  13:27:49  13:27:49  dist ()
 * Andy Goodrich: Changed code to use mmap object.
 * 
 * Revision 1.1  95/06/02  11:51:13  11:51:13  dist ()
 * Initial revision
 * 
 * Revision 1.2  95/04/17  12:28:57  12:28:57  dist ()
 * Andy Goodrich - added RCS header
 */
#include <sys/types.h>			/* Standard types for this C environment. */
#include <stdio.h>				/* Stream input/output declarations. */
#include <fcntl.h>				/* Pick up O_RDWR and friends. */
#include "objects_internal.h"	/* PowerVideo 700 object defintions. */
#include "plxv_xpoint.h"		/* Cross point switch definitions. */
#if defined(AIX) || defined(sun)
#include "pvioctl.h"			/* PowerVideo 6000 ioctl codes */
#endif

/*  XV_DIAG bit definitions:
 *
 *    0x01 - normal one-shot diagnostics
 *    0x80 - hardware library initialization tracing
 */
/* #define XV_DIAG 0x81 */
#include "xvdebug.h"

typedef union _YBus4reg1 {
    int word;           /* 32-bit word access. */
    struct {            /* Bit fields within register: */
        int unused:28;  /* ... there is nothing in top 28 bits. */
		int VIDCLK:2;	/* ... dvid clock source select. */
		int unused1:1;  /* ... unused. */
        int VOBLK:1;    /* ... video output blank: 0 use low res. encoder */
    } bits;
} YBus4reg1;

typedef union _YBus5reg1 {
    int word;           /* 32-bit word access. */
    struct {            /* Bit fields within register: */
        int unused:25;  /* ... there is nothing in top 25 bits. */
		int HPOL:1;		/* ... 0 if Hsyn negative true, 1 if positive true. */
		int VPOL:1;		/* ... 0 if Vsyn negative true, 1 if positive true. */
        int DENB:1;     /* ... delay line serial load enable. */
        int DLCK:1;     /* ... delay line serial clock. */
        int DLSD:1;     /* ... delay line serial data input. */
		int HMUX:1;     /* ... hsync: 0 = use sog, 1 use TTL separate. */
		int VMUX:1;     /* ... vsync: 0 = use sog, 1 use TTL separate. */
    } bits;
} YBus5reg1;

static O_Draw_Gc erase_gc = {   /* Drawing context for screen erase. */
    0,              /* Color */
    0,              /* 1-bit: background color. */
    0,              /* 1-bit: foreground color. */
    0,              /* Transparency flag. */
    0,              /* Readback red. */
    PLXV_ROP_COPY,  /* Raster operation. */
    0,              /* Stencil mask. */
    0,              /* Stencil value. */
    0,              /* Window id mask. */
    0,              /* Window id value. */
    PLXV_WMASK_ALL  /* Write mask. */
};

static int        m_init_pll();
static void       m_set_delay();
static XV_struct* pvpci_minimum();
static XV_struct* pvpci_open();
static XV_struct* pvpci_no_hw_init();
O_Input*          O_Input_O_Input();
O_Output*         O_Output_O_Output();



/*---------------------------------------------------------------------------*
 |"calc_serration"
 |
 |  Calculate serration values given display geometry.
 |
 *---------------------------------------------------------------------------*/
static void calc_serration(front_porch, sync, back_porch, visible, p_hs, p_hn)
    int     front_porch;
    int     sync;
    int     back_porch;
    int     visible;
    int*    p_hs;           /* -> where to store hs value. */
    int*    p_hn;           /* -> where to store hn value. */
{
    int     total;          /* Total line time. */
 
    total = front_porch + sync + back_porch + visible;
    *p_hs = visible + back_porch + front_porch;
    *p_hn = sync - (back_porch + front_porch);
 
    /*
     *  SUBTRACT ONE FROM HS AND HN VALUES. IF THE HN VALUE
     *  IS NEGATIVE ADJUST IT TO 0 AND REDUCE HS BY THE SAME
     *  AMOUNT.
     */
#ifdef NOTDEF /* #### */
    *p_hn -= 1;
    *p_hs -= 1;
#endif
    if ( *p_hn < 0 )
    {
        *p_hs += *p_hn;
        *p_hn = 0;
    }
    *p_hn |= 0x8000;
}
 

/*---------------------------------------------------------------------------*
 |"i_init_bus"
 |
 |  Internal routine to initialize bus, including interrupts.
 |
 |    = 1 if success, 0 if error detected.
 *---------------------------------------------------------------------------*/
static int i_init_bus( p_me )
    XV_struct*  p_me;   /* -> hardware context. */
{
    int i_write;    /* # of times we have written register. */

    PV_WREG(p_me,StartUp.l, 0x8000618);
    for(i_write=0;i_write<10000;i_write++);
        PV_WREG(p_me,StartUp.l,0x8000608);
    for(i_write=0;i_write<10000;i_write++);
        PV_WREG(p_me,DispFlgs.l,0xa28);

    PV_WREG(p_me,Attr.l,p_me->main.Attr);
    PV_WREG(p_me,BIntMask.l,0);
    PV_WREG(p_me,IntMask.l,0x8000);
    PV_WREG(p_me,RasMode.l,p_me->main.RasMode);

    return 1;
}


/*---------------------------------------------------------------------------*
 |"m_ccube_present"
 |
 |  Return whether a ccube chip is present on this board or not.
 |
 |    = 1 if ccube present, 0 if not.
 *---------------------------------------------------------------------------*/
static int m_ccube_present( p_me )
	XV_struct*	p_me;	/* -> hardware context. */
{
    int interrupts;

    PV_RREG(p_me,interrupts,IntStatC.l);
    PV_RREG(p_me,interrupts,IntStatC.l);
    p_me->main.c3_rev =  (interrupts & (1<<1)) ? 0 : 1;
    return p_me->main.c3_rev;
}


/*---------------------------------------------------------------------------*
 |"m_dump"
 |
 |  Dump the contents of a Parallax access object.
 |
 *---------------------------------------------------------------------------*/
static void m_dump( p_me, fd )
    XV_struct*  p_me;   /* -> hardware context. */
    FILE*       fd;     /* -> descriptor for file to dump to. */
{
    int     i_unit;     /* Unit # of object pointer now dumping. */
    int     n_units;    /* # of objects to dump. */

    fprintf(fd, "\n\nDump of XV_struct for device %s at 0x%08x:\n",
        p_me->p_mmap->name, p_me);
    fprintf(fd, "   atv          = 0x%08x \n", p_me->atv);
    fprintf(fd, "   c3base       = 0x%08x \n", p_me->c3base);
    fprintf(fd, "   c3_rreg      = 0x%08x \n", p_me->c3_rreg);
    fprintf(fd, "   c3_wreg      = 0x%08x \n", p_me->c3_wreg);
    fprintf(fd, "   eprombase    = 0x%08x \n", p_me->eprombase);
    fprintf(fd, "   fd_ccube     = 0x%08x \n", p_me->fd_ccube);
    fprintf(fd, "   maps         = 0x%08x \n", p_me->maps);
    fprintf(fd, "   m1base       = 0x%08x \n", p_me->m1base);
    fprintf(fd, "   m8base       = 0x%08x \n", p_me->m8base);
    fprintf(fd, "   m24base      = 0x%08x \n", p_me->m24base);
    n_units = sizeof(p_me->p_inputs) / sizeof(p_me->p_inputs[0]);
    for( i_unit = 0; i_unit < n_units; i_unit++ )
    {
        fprintf(fd, "   p_inputs[%d]  = 0x%08x \n", i_unit,
            p_me->p_inputs[i_unit]);
    }
    for( i_unit = 0; i_unit < n_units; i_unit++ )
    {
        fprintf(fd, "   p_outputs[%d] = 0x%08x \n", i_unit,
            p_me->p_outputs[i_unit]);
    }
    fprintf(fd, "   p_i2c        = 0x%08x \n", p_me->p_i2c);
    fprintf(fd, "   p_rsrc       = 0x%08x \n", p_me->p_rsrc);
    fprintf(fd, "   regbase      = 0x%08x \n", p_me->regbase);
    fprintf(fd, "   xvm          = 0x%08x \n", p_me->xvm);
    fprintf(fd, "   xv_rreg      = 0x%08x \n", p_me->xv_rreg);
    fprintf(fd, "   xv_shdw      = 0x%08x \n", p_me->xv_shdw);
    fprintf(fd, "   xv_wreg      = 0x%08x \n", p_me->xv_wreg);
}


/*---------------------------------------------------------------------------*
 |"m_get_zoom"
 |
 |  Return current zoom factor.
 |
 |    = current zoom factor.
 *---------------------------------------------------------------------------*/
static int m_get_zoom( p_me )
	XV_struct*	p_me;	/* -> hardware context. */
{
	return p_me->main.zoom;
}


/*---------------------------------------------------------------------------*
 |"m_init_context"
 |
 |  Initialize context from current resource object values.
 *---------------------------------------------------------------------------*/
static void m_init_context( p_me )
	XV_struct*	p_me;	/* -> hardware context. */
{
    XV_addr_maps*   ap;
	int				i_row;
    int 			Xres;
	int				Yres;

	O_Rsrc_store_to_context(p_me->p_rsrc, p_me);
	ap = p_me->maps;

    switch ( Xres = p_me->main.linebytes )
    {
        case 1152:  p_me->addr_wrap = 0x100000; break;
        case 1280:  p_me->addr_wrap = 0x140000; break;
    }
    Yres = p_me->main.ysize;

    for ( i_row=0; i_row<Yres; i_row++) 
		ap->reg[i_row] =  (Xres *  i_row);
    for ( i_row=0; i_row<Yres; i_row++) 
		ap->m24[i_row] = (int *)( (int) p_me->m24base + ((Xres*4) * i_row));
    for ( i_row=0; i_row<Yres; i_row++) 
		ap->m1[i_row] = (int *)( (int) p_me->m1base + ((Xres/8) * i_row));

}


/*---------------------------------------------------------------------------*
 |"m_init_disp"
 |
 |  Initialize display hardware from current context values.
 *---------------------------------------------------------------------------*/
static void m_init_disp( p_me )
    XV_struct*  p_me;   /* -> hardware context. */
{
    int   h_sync_n;		 /* Serration value adjusted to short. */
    int   h_sync_s;		 /* Serration value adjusted to short. */
	YBus4reg1	ybus4r1; /* Temp to build YBUS 4 initial value. */
	YBus5reg1	ybus5r1; /* Temp to build YBUS 5 initial value. */
 
    /* INITIALIZE PARAMETERS WHICH ARE ZOOM-INDEPENDENT. */

    PV_WREG(p_me, StartUp.l,    p_me->main.StartUp);
    PV_WREG(p_me, VFP.l,        p_me->main.v.fp);
    PV_WREG(p_me, VBP.l,        p_me->main.v.bp);
    PV_WREG(p_me, VDisp.l,      p_me->main.v.disp);
    PV_WREG(p_me, VSync.l,      p_me->main.v.sy);
 
 
    /* INITIALIZE PARAMETERS WHICH CHANGE BASED ON ZOOM. */
 
    switch ( p_me->main.zoom )
    {
      default:
        PV_WREG(p_me, DispFlgs.l,   p_me->main.DispFlgs);
        PV_WREG(p_me, HFP.l,        p_me->main.h.fp);
        PV_WREG(p_me, HSync.l,      p_me->main.h.sy);
        PV_WREG(p_me, HBP.l,        p_me->main.h.bp);
        PV_WREG(p_me, HDisp.l,      p_me->main.h.disp);
#if NOTDEF
		h_sync_s = p_me->main.h.hsync_ser;
		if(h_sync_s & 0x80) h_sync_s += 0x80;
		h_sync_n = p_me->main.h.hsync_nser;
#endif
		calc_serration(p_me->main.h.fp, p_me->main.h.sy,
			p_me->main.h.bp, p_me->main.h.disp, &h_sync_s, &h_sync_n);
        break;
 
      case 2:
        PV_WREG(p_me, DispFlgs.l,   p_me->zoom[0].DispFlgs);
        PV_WREG(p_me, HFP.l,        p_me->zoom[0].h.fp);
        PV_WREG(p_me, HSync.l,      p_me->zoom[0].h.sy);
        PV_WREG(p_me, HBP.l,        p_me->zoom[0].h.bp);
        PV_WREG(p_me, HDisp.l,      p_me->zoom[0].h.disp);
		calc_serration(p_me->zoom[0].h.fp, p_me->zoom[0].h.sy,
			p_me->zoom[0].h.bp, p_me->zoom[0].h.disp, &h_sync_s, &h_sync_n);
        break;
    }
	PV_WREG(p_me, HSyncS.l,     h_sync_s);
	PV_WREG(p_me, HNSyncS.l,    h_sync_n);
	PV_WREG(p_me, DispKludge.l, p_me->main.DispKludge);


	/*  SET UP YBUS REGISTER INITIAL VALUES, AND SET UP THE PLL. */

	ybus4r1.word = 0;
	ybus4r1.bits.VIDCLK = 0;
	ybus4r1.bits.VOBLK = 0;
	PV_WREG(p_me, Bt.grn.reg1, ybus4r1.word);

	ybus5r1.bits.DENB = 0;
	ybus5r1.bits.DLCK = 0;
	ybus5r1.bits.DLSD = 0;
	ybus5r1.bits.HMUX = 1;
	ybus5r1.bits.VMUX = 1;
	PV_WREG(p_me, Bt.blu.reg1, ybus5r1.word);
}


/*---------------------------------------------------------------------------*
 |"m_init_hw"
 |
 |  Initialize hardware from current context values. 
 *---------------------------------------------------------------------------*/
static void m_init_hw( p_me, screen_erase )
	XV_struct*	p_me;			/* -> hardware context. */
	int         screen_erase;	/* = 1 if should erase screen. */
{
    int             		ioctl_data;		/* Value to pass in ioctl. */
	static XV_video_format  loop_format = { /* Default video format. */
		640,482,43,7,21
	};
#if 0
	YBus4reg1				ybus4r1; /* Temp to build YBUS 4 initial value. */
	YBus5reg1				ybus5r1; /* Temp to build YBUS 5 initial value. */
#endif


	/*  INITIALIZE THE STANDARD COMPONENTS AND ERASE THE SCREEN TO BLACK. */

	TRACEF(0x80, (fd, "m_init_hw: entry\n"));
	m_vertical(p_me, 0);	
	ioctl_data = p_me->main.ysize;
#if defined( _AIX) || defined(sun)
	if ( ioctl(p_me->fd_ccube, PV_RES_SET, &ioctl_data) )
		ERRORF((fd, "**** ioctl error on resolution set\n"));
#endif
    i_init_bus(p_me);
	O_Lut_init_hw(p_me->p_lut);
    m_init_disp(p_me); 
    O_Draw_init_hw(p_me->p_draw);
	plxv_init_video(p_me);
	m_init_pll(p_me, 0);
    O_Draw_set_gc(p_me->p_draw, &erase_gc);
	if ( screen_erase )
	{
		O_Draw_fill_box(p_me->p_draw, 0, 0, p_me->main.xsize, p_me->main.ysize);
	}


	/* INITIALIZE INPUT/OUTPUT PORTS: */

    if ( p_me->p_outputs[PORT_A] ) O_Output_init_hw(p_me->p_outputs[PORT_A]);
    if ( p_me->p_inputs[PORT_B] ) O_Input_init_hw(p_me->p_inputs[PORT_B]);
    if ( p_me->p_inputs[PORT_D] ) O_Input_init_hw(p_me->p_inputs[PORT_D]);
	p_me->plxv_io_format[5] = &loop_format;
	p_me ->format_v_ptr = &(p_me->plxv_io_format[5]);
	p_me ->format_w_ptr = &(p_me->plxv_io_format[5]);


	/*  INITIALIZE CCUBE IF WE HAVE ONE. */

	if ( m_ccube_present(p_me) )
	{
		C3Init(p_me);
	}
	TRACEF(0x80, (fd, "m_init_hw: exit\n"));
}


/*---------------------------------------------------------------------------*
 |"m_init_pll"
 |
 |  Set the values associated with a host frame buffer frequency.
 |  A frequency of zero defaults frequency to nvram setting.
 |  Note we set the mode register to disable the second video input and
 |  the video output. These will be enabled by their access objects when
 |  they are selected.
 |
 *---------------------------------------------------------------------------*/
static int m_init_pll( p_me, frequency )
	XV_struct*	p_me;		/* -> hardware library context. */
	int			frequency;	/* =  frequency to set up PLL for. */
{
    int     delay;      	/* Delay value to use base on zoom factor. */
	int		ioctl_data;		/* Data to pass to driver. */
    int     skip;       	/* Skip value to use based on zoom factor. */
	int		vco_select;		/* Clock to load into MiscMode.b.red. */
    int     zoom_bits;  	/* Zoom factor to load into MiscMode.b.grn. */
    int     zoom_enable;	/* Zoom enable to load into MiscMode.b.red. */
    
     /*  SET MODE REGISTER TWO WITH DMSD #2 AND DMSE DISABLED */
     
	m_vertical(p_me, 0);	
	ioctl_data = p_me->main.ysize;
#ifdef _AIX
	if ( ioctl(p_me->fd_ccube, PV_RES_SET, &ioctl_data) )
		ERRORF((fd, "**** ioctl error on resolution set\n"));
#endif
    PV_WREG(p_me, Bt.grn.reg2, 0x5e);
 
    switch ( p_me->main.zoom )
    {
      default:
        delay = p_me->main.delay;
        skip = p_me->main.skip >> 1;
        zoom_bits = 0x00000003;
		zoom_enable = 0;
        break;
      case 2:
        delay = p_me->zoom[0].delay;
        skip = p_me->zoom[0].skip >> 1;
        zoom_bits = 0x00000002;
		zoom_enable = 4;
        break;
    }
 

	if ( !frequency )
	{
		switch(p_me->main.pixel_clock)
		{
		  default:
		  case 135: vco_select = 0; break;
		  case 108: vco_select = 1; break;
		  case  84: vco_select = 2;  break;
		  case  64: vco_select = 3;  break;
		}
	}


	/*  PROGRAM FOR THE PROPER FREQUENCY: */

	m_set_delay(p_me, delay);
	PV_WREG(p_me, MiscMode.b.grn, zoom_bits);
	PV_WREG(p_me, MiscMode.b.red, vco_select | zoom_enable);
	PV_WREG(p_me, MiscMode.b.blu, skip);
	m_vertical(p_me, 1);	

	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_set_delay"
 |
 |  Store the delay value. This is done through a serial interface in mode
 |  register 2. For each bit (from the most significant to the least 
 |  significant) of the 8-bit delay value we perform the following steps:
 |    (1) Pick up original bits for register 2 from the shadow area.
 |    (2) Set the serial clock bit to 0, and the serial data bit to the value
 |        of the bit in the 'delay' parameter to be set.
 |    (3) Set the serial clock bit to 1, with the serial data bit still set
 |        to the bit in the 'delay' parameter to be set.
 |    (4) Set the serial clock bit to 0.
 |
 |
 |	use ybus5 (Bt.blu.reg1) to load delay line
 |	
 *---------------------------------------------------------------------------*/
static void m_set_delay( p_me, delay )
    XV_struct* p_me;
	int		delay;		/* =  delay value to be loaded. */
{
	int			i_bit;	/* # of bit (from MSB to LSB) clocking into register. */
	YBus5reg1	reg1;	/* Value for mode register 1 from the shadow area. */

	reg1.word = p_me->xv_shdw->b.Bt.blu.reg1;

	/*
	 *  ENABLE USE OF DELAY LINE SERIAL INTERFACE.
	 */
	reg1.bits.DLSD = 0;
	reg1.bits.DLCK = 0;
	PV_WREG(p_me, Bt.blu.reg1, reg1.word);
	reg1.bits.DENB = 1;
	PV_WREG(p_me, Bt.blu.reg1, reg1.word);

	for ( i_bit = 0; i_bit < 8; i_bit++ )
	{
		reg1.bits.DLSD = (delay & 0x80) ? 1 : 0;
		reg1.bits.DLCK = 0;
		PV_WREG(p_me, Bt.blu.reg1, reg1.word);

		reg1.bits.DLCK = 1;
		PV_WREG(p_me, Bt.blu.reg1, reg1.word);

		reg1.bits.DLCK = 0;
		PV_WREG(p_me, Bt.blu.reg1, reg1.word);

		delay <<= 1;
	}

	/*
	 *  DISABLE USE OF DELAY LINE SERIAL INTERFACE.
	 */
	reg1.bits.DLSD = 0;
	reg1.bits.DLCK = 0;
	PV_WREG(p_me, Bt.blu.reg1, reg1.word);
	reg1.bits.DENB = 0;
	PV_WREG(p_me, Bt.blu.reg1, reg1.word);
}


/*---------------------------------------------------------------------------*
 |"m_set_zoom"
 |
 |  Set zoom factor.
 *---------------------------------------------------------------------------*/
static int m_set_zoom( p_me, factor )
	XV_struct*	p_me;	/* -> hardware context. */
	int			factor;	/* =  zoom factor. */
{
	p_me->main.zoom = factor;
	m_vertical(p_me, 0);
	switch(factor)
	{
	  case 1:
		m_init_disp(p_me);
		m_init_pll(p_me, 0);
		O_Draw_set_gc(p_me->p_draw, &erase_gc);
		O_Draw_fill_box(p_me->p_draw, 0, 0, p_me->main.xsize, p_me->main.ysize);
#ifdef REMOVE
		plxv_tag(p_me, 0, 0);
		plxv_stencil(p_me, 0, 0);
		plxv_wmask(p_me, PLXV_WMASK_ALL);
		plxv_color_24(p_me, 0);
		plxv_box(p_me, 0, 0, p_me->main.xsize, p_me->main.ysize);
#endif
		break;

	  case 2:
        PV_WREG(p_me, DispKludge.l, 0x5000);
        m_init_disp(p_me);
        m_init_pll(p_me, 0);
		break;

	  case 4:
        PV_WREG(p_me, DispKludge.l, 0x5000);
		m_init_disp(p_me);
		m_init_pll(p_me, 0);
		break;
	}
	m_vertical(p_me, 1);
	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_terminate"
 |
 |  Terminate use of this hardware context, freeing its storage.
 *---------------------------------------------------------------------------*/
static void m_terminate( p_me )
    XV_struct*  p_me;   /* -> hardware context. */
{
    if ( p_me->p_draw )
	{
		O_Draw_set_gc(p_me->p_draw, &erase_gc);
		O_Draw_fill_box(p_me->p_draw, 0, 0, p_me->main.xsize, p_me->main.ysize);
		O_Draw_terminate(p_me->p_draw);
	}

    if ( p_me->p_i2c ) O_I2C_terminate(p_me->p_i2c);
	if ( p_me->p_lut ) O_Lut_terminate(p_me->p_lut);
    if ( p_me->p_mmap ) O_Mmap_terminate(p_me->p_mmap);
    if ( p_me->p_rsrc ) O_Rsrc_terminate(p_me->p_rsrc);
    if ( p_me->p_outputs[PORT_A] ) O_Output_terminate(p_me->p_outputs[PORT_A]);
    if ( p_me->p_inputs[PORT_B] )  O_Input_terminate(p_me->p_inputs[PORT_B]);
    if ( p_me->p_inputs[PORT_D] )  O_Input_terminate(p_me->p_inputs[PORT_D]);
    free((char*)p_me);
}


/*---------------------------------------------------------------------------*
 |"m_vertical"
 |
 |  Turn on or off vertical interrupt processing in the device driver.
 *---------------------------------------------------------------------------*/
static int m_vertical( p_me, on_off )
	XV_struct*	p_me;	/* -> hardware context. */
	int			on_off;	/* =  1 if enabled, 0 if not. */
{
    int             ioctl_data;

    ioctl_data = on_off ? 1 : 0;
#if 0 /* def sun */
	vert( p_me );
#else
    if ( ioctl(p_me->fd_ccube, PV_DISP_ONOFF, &ioctl_data) )
        fprintf(stderr, "**** ioctl error on off\n");
#endif
}


/*---------------------------------------------------------------------------*
 |"m_visibility"
 |
 |  Turn on or off visibility.
 *---------------------------------------------------------------------------*/
static void m_visibility( p_me, on_off )
	XV_struct*	p_me;	/* -> hardware context. */
	int			on_off;	/* =  1 if visible, 0 if blanked. */
{
	int	vis;	/* Binary version of on_off. */

	vis = (on_off) ? 1 : 0;
	PV_WREG(p_me, DispKludge.b.disp_blank, vis);
}


static TV_PV main_methods_tv = {
	m_ccube_present,
	m_dump,
	m_get_zoom,
	m_init_context,
	m_init_disp,
	m_init_hw,
	m_init_pll,
	m_set_delay,
	m_set_zoom,
	m_terminate,
	m_vertical,
	m_visibility
};

/*---------------------------------------------------------------------------*
 |"O_HWL_O_HWL"
 |
 |  Open a Parallax PowerVideo PCI device to a specified initialization level:
 |     "minimum"    - create only its I2C and resource objects.
 |     "no_hw_init" - create all hardware access objects, but don't
 |                    initialize the hardware.
 |     "full"       - create a fully initialized access object.
 *---------------------------------------------------------------------------*/
XV_struct* O_HWL_O_HWL(level, device, rsrc_file)
    char* level;        /* -> object level:	"minimum", "no_hw_init", "full". */
    char* device;       /* -> name of device to open. */
    char* rsrc_file;    /* -> resource name or 0. */
{
	if ( !strcmp(level, "minimum" ) )
	{
		return pvpci_minimum(device, rsrc_file);
	}
	else if ( !strcmp(level, "no_hw_init" ) )
	{
		return pvpci_no_hw_init(device, rsrc_file);
	}
	else if ( !strcmp(level, "no_erase" ) )
	{
		return pvpci_open(device, rsrc_file, 0);
	}
	else if ( !strcmp(level, "full" ) )
	{
		return pvpci_open(device, rsrc_file, 1);
	}
	else
	{
		printf("O_HWL_O_HWL: unknown level requested %s\n");
		return 0;
	}
}

/*---------------------------------------------------------------------------*
 |"pvpci_minimum"
 |
 |  Open a Parallax PowerVideo pci device and return a minimal context object.
 *---------------------------------------------------------------------------*/
static XV_struct* pvpci_minimum(device, rsrc_file)
    char* device;
	char* rsrc_file;
{
    XV_addr_maps*   ap;	        /* Address maps structure. */
	FILE*			fd;	        /* File descriptor for 'nvram' values. */
	int				i_shadow;   /* # of word in shadow registers zeroing. */
	int				n_shadow;	/* # of words in shadow registers. */
	O_Mmap*			p_mmap;		/* Memory map object. */
	int*			p_shadow;	/* Word in shadow registers zeroing. */
    XV_struct*      p_xv;		/* Context we are filling in. */
    XV_shdw_regs*   sp;			/* Shadow register area. */
    XV_mem_map*     xvm;		/* Map for card. */
	int i;

	TRACEF(0x80, (fd, "pvpci_minimum: entry\n"));
	if ( !(p_mmap = (O_Mmap*)O_Mmap_O_Mmap(device)) ) return 0;


	/*  ALLOCATE STRUCTURES WE NEED, INCLUDING OUR CONTEXT: */
 
    if ((p_xv = (XV_struct *)malloc(sizeof(XV_struct))) == NULL)
    {   /* malloc failed */
        printf("malloc failed on plxv_init_struct:main struct\n");
        exit(-1);
    }
    if ((sp = (XV_shdw_regs *)malloc(sizeof(XV_shdw_regs))) == NULL)
    {   /* malloc failed */
        printf("malloc failed on plxv_init_struct:xv_shdw\n");
        exit(-1);
    }
    if ((ap = (XV_addr_maps *)malloc(sizeof(XV_addr_maps))) == NULL)
    {   /* malloc failed */
        printf("malloc failed on plxv_init_struct:addr_maps\n");
        exit(-1);
    }


	/* FILL IN CONTEXT WITH MEMORY MAP ADDRESSES: */

#ifdef sun
    xvm = (XV_mem_map*)((u_int)O_Mmap_virtual(p_mmap, 0x10000) - 0x10000);
    p_xv->p_mmap = p_mmap;
    p_xv->atv = (void*)&main_methods_tv;
    p_xv->c3base = (int*)O_Mmap_virtual(p_mmap, 0x50000);
    p_xv->c3_rreg = (C3_RREGS *)p_xv->c3base;
    p_xv->c3_wreg = (C3_WREGS *)p_xv->c3base;
    p_xv->eprombase = (int*)&xvm->Prom0;
    p_xv->fd_ccube = p_mmap->fd_ccube;
    p_xv->maps = ap;
    p_xv->m1base = (int*)O_Mmap_virtual(p_mmap, 0x10000);
    p_xv->m8base = (char*)O_Mmap_virtual(p_mmap, 0x80000);
    p_xv->m24base = (int*)O_Mmap_virtual(p_mmap, 0x200000);
    p_xv->regbase = (int*)O_Mmap_virtual(p_mmap, 0x40000);
    p_xv->xvm = xvm;
    p_xv->xv_rreg = (XV_read_regs*)p_xv->regbase;
    p_xv->xv_shdw = (XV_shdw_regs*)sp;
    p_xv->xv_wreg = (XV_write_regs*)p_xv->regbase;
#else
	xvm = (XV_mem_map*)p_mmap->maps[MM_Eprom].virtual;
	p_xv->atv = (void*)&main_methods_tv;
    p_xv->c3base = (int*)&xvm->Ccube;
    p_xv->c3_rreg = (C3_RREGS *)&xvm->Ccube;
    p_xv->c3_wreg = (C3_WREGS *)&xvm->Ccube;
    p_xv->eprombase = (int*)&xvm->Prom0;
	p_xv->fd_ccube = p_mmap->fd_ccube;
	p_xv->p_mmap = p_mmap;
    p_xv->maps = ap;
    p_xv->m1base = (int*)&xvm->M1;
    p_xv->m8base = (char*)&xvm->M8;
    p_xv->m24base = (int*)&xvm->M24;
    p_xv->regbase = (int*)&xvm->RW_regs;
	p_xv->xvm = xvm;
    p_xv->xv_shdw = (XV_shdw_regs*)sp;
    p_xv->xv_wreg = (XV_write_regs*)&xvm->RW_regs;
    p_xv->xv_rreg = (XV_read_regs*)&xvm->RW_regs;
#endif


    /* ZERO REGISTER FILE SHADOW STRUCTURE. */

    n_shadow = sizeof(XV_shdw_regs) >> 2;
    for (p_shadow = (int*)sp, i_shadow = 0; i_shadow < n_shadow; i_shadow++ ) 
	{
		*p_shadow++ = 0;
	}


	/* ALLOCATE MINIMUM SET OF DEVICE OBJECTS: */

	TRACEF(0x80, (fd, "pvpci_minimum: allocating objects for %s\n", 
		p_xv->p_mmap->name));
    p_xv->p_i2c = (O_I2C*)O_I2C_O_I2C(p_xv);
	p_xv->p_draw = O_Draw_O_Draw(0, p_xv);
    p_xv->p_rsrc = (O_Rsrc*)O_Rsrc_O_Rsrc();

	if ( fd = fopen(rsrc_file, "r") )
	{	
		O_Rsrc_load_from_file(p_xv->p_rsrc, fd);
		fclose(fd);
	}

	/*  INITIALIZE REMAINDER OF CONTEXT FROM RESOURCE VALUES. */

	TRACEF(0x80, (fd, "pvpci_minimum: initializing context for %s\n", 
		p_xv->p_mmap->name));
	m_init_context(p_xv);

	/* #### ???? i = *p_xv->eprombase; */
	TRACEF(0x80, (fd, "pvpci_minimum: exit\n"));
    return p_xv;
}


/*---------------------------------------------------------------------------*
 |"pvpci_open"
 |
 |  Open a Parallax PowerVideo pci device, initialize it,  and return its 
 |  context object.
 |
 |	  -> context object on success, 0 on failure.
 *---------------------------------------------------------------------------*/
static XV_struct* pvpci_open(device, rsrc_file, screen_erase)
    char* device;
	char* rsrc_file;
	int	  screen_erase;
{
    XV_struct*         	p_me;

	TRACEF(0x80, (fd, "pvpci_open: entry\n"));
	if ( !(p_me = pvpci_no_hw_init(device, rsrc_file)) )
	{
		ERRORF((fd, "Unable to open device %s\n", device ? device : "0"));
		return 0;
	}
	TRACEF(0x80, (fd, "pvpci_open: device %s calling init_hw\n", 
		p_me->p_mmap->name));
	m_init_hw( p_me, screen_erase );
	if(p_me->p_outputs[0]) p_me->opt.board_code = EISA_BUS_1_135;

	TRACEF(0x80, (fd, "pvpci_open: exit\n"));
	return p_me;
}

/*---------------------------------------------------------------------------*
 |"pvpci_no_hw_init"
 |
 |  Open a Parallax PowerVideo pci device, but don't initialize it,  and 
 |  return its context object.
 |
 |	  -> context object on success, 0 on failure.
 *---------------------------------------------------------------------------*/
static XV_struct* pvpci_no_hw_init(device, rsrc_file)
    char* device;
	char* rsrc_file;
{
    XV_struct*         	p_me;


	/*  ALLOCATE MINIMAL POWERVIDEO PCI CONTEXT OBJECT: */

	TRACEF(0x80, (fd, "pvpci_no_hw_init: entry\n"));
	if ( !(p_me = pvpci_minimum(device, rsrc_file)) )
	{	
		fprintf(stderr, "*** Unable to open device %s\n", device ? device :"0");
		return 0;
	}
	TRACEF(0x80, (fd, "pvpci_no_hw_init: after pvpci_minimum\n"));


	/* INITIALIZE OBJECTS FOR PORTS A AND B AND LUT OBJECT: */

	p_me->p_outputs[PORT_A] = (O_Output*)O_Output_O_Output(0, 0, p_me);
	p_me->p_outputs[PORT_B] = 0;
	p_me->p_outputs[PORT_C] = 0;
	p_me->p_outputs[PORT_D] = 0;

	p_me->p_inputs[PORT_A] = 0;
	p_me->p_inputs[PORT_B] = (O_Input*)O_Input_O_Input( 0, 1, p_me );
	p_me->p_inputs[PORT_C] = 0;
	p_me->p_inputs[PORT_D] = (O_Input*)O_Input_O_Input( 0, 0, p_me ); 

	TRACEF(0x80, (fd, "pvpci_no_hw_init: before lut creation\n"));
	p_me->p_lut = (O_Lut*)O_Lut_Bt495_O_Lut_Bt495(0, p_me);


	TRACEF(0x80, (fd, "pvpci_no_hw_init: exit\n"));
	return p_me;
}


/*---------------------------------------------------------------------------*
 |"O_HWL_device"
 |
 |  Return name of Parallax device to use. This will either be the value of
 |  the environment variable PLX_DEVICE or a platform-specific default (e.g.,
 |  "/dev/pv0").
 *---------------------------------------------------------------------------*/
char* O_HWL_device()
{
    char*   device;

    device = (char*)getenv("PLX_DEVICE");

#ifdef _AIX
    return device ? device : "/dev/pv0";
#else /* !_AIX */
    return device ? device : "/dev/plxv0";
#endif /* _AIX */
}


/*---------------------------------------------------------------------------*
 |"O_HWL_rsrc"
 |
 |  Return name of resource file to use. This will either be the value of the
 |  environment variable PLX_RSRC or a platform-specific default (e.g.,
 |  "/opt/parallax/etc/plxv0").
 *---------------------------------------------------------------------------*/
char* O_HWL_rsrc()
{
    char*   rsrc;

    rsrc = (char*)getenv("PLX_RSRC");

#ifdef _AIX
    return rsrc ? rsrc : "/usr/lpp/parallax/etc/pv0";
#else /* !_AIX */
    return rsrc ? rsrc : "/opt/parallax/etc/plxv0";
#endif /* _AIX */
}


/*---------------------------------------------------------------------------*
 |"O_Input_O_Input"
 |
 |  Return an input device given a hardware context and unit number for the
 |  device.
 |
 |    -> object on success, 0 if error detected or device not present.
 *---------------------------------------------------------------------------*/
O_Input* O_Input_O_Input(p_me, unit, p_xv)
    O_Input*    p_me;   /* -> object storage or 0. */
    int         unit;   /* =  unit number of device. */
    XV_struct*  p_xv;   /* -> hardware library context to use. */
{
    return (O_Input*)O_Input_7110_O_Input_7110(p_me, unit, p_xv);
}


/*---------------------------------------------------------------------------*
 |"O_Output_O_Output"
 |
 |  Return an output device given a hardware context and unit number for the
 |  device.
 |
 |    -> object on success, 0 if error detected or device not present.
 *---------------------------------------------------------------------------*/
O_Output* O_Output_O_Output(p_me, unit, p_xv)
    O_Output*   p_me;   /* -> object storage or 0. */
    int         unit;   /* =  unit number of device. */
    XV_struct*  p_xv;   /* -> hardware library context to use. */
{
    return (O_Output*)O_Output_Dmse_O_Output_Dmse(p_me, unit, p_xv);
}


int icd_init( p_me ,data)
	XV_struct*	p_me;	/* -> hardware context. */
	int data;
{
	int i;
	icd_write(p_me, data);
	icd_write(p_me, 0x67f7a6);
	icd_write(p_me, 0xa40000);
	icd_write(p_me, 0xc18000);
	for (i=0; i<1000000;i++);
	icd_write(p_me, 0xc38000);
	for (i=0; i<1000000;i++);
	icd_write(p_me, 0xc18000);
}

#define mm(x) PV_WREG(p_me, MiscMode.l,(x)) /* */

icd_write(p_me, val) 
    XV_struct* p_me;
{
    int i,n=val;
    /* start sequence */
    /* printf("icd write = %x\n",val); */
    mm(5);
    for (i=0; i<7; i++) { mm(7); mm(6);}
    mm(4); mm(5); mm(4); mm(5);
    for (i=0; i<24; i++) {
        /* bit sequence */
        if (n&1) {
            mm(5); mm(4); mm(6); mm(7);
        } else {
            mm(7); mm(6); mm(4); mm(5);
        }
        n = n>>1;
    }
    mm(7); mm(6); mm(7); mm(4);
}

static char nv_address[3] = {0x50,0x51,0x52};
static nv_read(p_i2c, unit, ptr, nv_addr, count)
    O_I2C* p_i2c;
	int unit,nv_addr,count;
	char *ptr;
{
    char *j;
    int i,k,paddr,data;

    paddr = nv_address[unit];
    j = ptr;
    for (i=0; i<count; i++)
    {
        k = O_I2C_read_data(p_i2c, paddr,nv_addr++,&data);
        *j++ = data;
        if (k) 
        {
           printf("nv_read: k= %d\n",k);
           return(k);
        }
    }
    return(0);
}


#include <sys/time.h>               /* General timer parameters. */

#define READ(offset) *((int *) (reg+offset))
#define READB(offset) (READ((0x4a0+offset))&0xff)
#define WRITE(offset,data) *((int *) (reg+offset))=data;
#define LINES(data) WRITE(0x9c,vlines+(data))

#define INT_BIT 1

vert( p_me )
	XV_struct* p_me;	/* Hardware library context. */
{
	struct timeval   to;            /* Time stamp for performing wait. */
	int	loop_cnt;
	int force_loop;
	char *reg;
	int vert_count=0;
	int plx_v;
	int done=0;
	int vlines = p_me->main.ysize-1;
	int wait;						/* Amount of time to wait. */

	TRACEF(0x80, (fd, "vert: entry  v.fp: %d v.bp: %d v.sy:%d v.disp: %d ",
		p_me->main.v.fp, p_me->main.v.bp, p_me->main.v.sy, p_me->main.v.disp));
	reg = (char*)O_Mmap_virtual(p_me->p_mmap,0x40000);
	TRACEF(0x80, (fd, "plx_v = %x\n", READB(0x0)));

	/* PV_WREG(p_me, Bt.blu.reg2, 0); /* Reset the EPLD. */ 

	force_loop = 1;

	switch(p_me->main.pixel_clock)
	{
	  default:
	  case 135: wait = 1000000 / 72; break;
	  case 108: wait = 1000000 / 66; break;
	}
	
	done=0;
	for ( loop_cnt=200; (done < 7) && force_loop && loop_cnt > 0; --loop_cnt )
	{
		to.tv_sec = 0;
		to.tv_usec = wait;
		select(0,0,0,0,&to);
		{
			/* we have seen a vertical interrupt */
			plx_v = READB(0x0);
			if (0==(plx_v&0x80)) {
				LINES(-64);
				vert_count=0;
				done = 0;
			} else if (0==(plx_v&0x40)) {
				LINES(-8);
				vert_count=0;
				done = 0;
			} else if (((plx_v)&0xf)>=2) {
				if (vert_count&1)	{LINES(-3);}
				else {LINES(0);}
				vert_count++;
			} else if (((plx_v)&0xf)>0) {
				if (vert_count&1)	{LINES(-1);}
				else {LINES(0);}
				vert_count++;
			} else {
				LINES(0);
				done++;
				vert_count=0;
			}
		    /* printf("%x\n", IntStat); */
		}
	}
	LINES(0);
	printf("read %x\n",READB(0x0)&0xff);
	TRACEF(0x80, (fd, "vert: exit lines: %x\n", plx_v));
}
