/*
 *  bt473_lut.c: Support for Brooktree 473 Lookup Table Object.
 *
 *   The Bt473 is a color space converter, as well as a set of color lookup 
 *   tables individually for red, green, and blue. The color space converter 
 *   performs normalized operations, the color tables are used to set offset 
 *   and gain.
 *
 * (c) Copyright 1995 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 * $Log:   /plxhp/dave/pvcsrc/hwlib/archives/bt473_lut.c_v  $
 * 
 *    Rev 1.1   09 Sep 1996 10:08:50   dave
 * Initial revision
 * Revision 1.1  1995/12/11 20:42:56  dist
 * Initial revision
 *
 */
#include <stdio.h>			/* Pickup stdout and friends. */
#include <sys/types.h>		/* Pick up u_int and friends. */
#include "objects.h"		/* Pick up object definitions. */
#include "video_stream.h"	/* Pick up Parallax hardware registers defs. */
#include "hw_context.h"		/* Pick up Parallax hardware library context. */
#include "xvdebug.h"		/* Pick up XV debugging macros. */

/*  INTERNAL OBJECT DEFINITION FOR BROOKTREE 473 OBJECT: */

typedef struct _O_Lut_Bt473 {
	TV_Lut*		atv;		/* Methods: first entry in all derived objects. */
	XV_struct*	p_xv;		/* Parallax hardware library context structure. */
} O_Lut_Bt473;


/* FORWARD FUNCTION REFERENCES: */

static void i_Lut_Bt473_set_option_rsaddr(/* p_me, device_code,rsaddr */);
static int  m_Lut_Bt473_control(/* p_me, op, arg */);
static void m_Lut_Bt473_dump(/* p_me, fd */);
static int  m_Lut_Bt473_get_entries(/* p_me, start, count, p_value */);
static int  m_Lut_Bt473_init_hw(/* p_me */);
static int  m_Lut_Bt473_set_entries(/* p_me, start, count, p_value */);
static void m_Lut_Bt473_terminate(/* p_me */);


static void i_Lut_Bt473_set_option_rsaddr(p_me, device_code, rsaddr)
	O_Lut_Bt473* p_me;			/* -> context for this object. */
	int			 device_code;	/* =  device code for option. */
	int			 rsaddr;		/* = */
{
    int i,j,k;
 
    i = (device_code & 3) << 3;
    j = rsaddr & 7;
    k = i | j;
    PV_WREG(p_me->p_xv, Ext.reg0,k);
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt473_control"
 |
 |  Perform a device-dependent control operation.
 |
 |    = 1 if success, 0 if error detected.
 *---------------------------------------------------------------------------*/
static int m_Lut_Bt473_control( p_me, op, arg )
	O_Lut_Bt473* p_me;	/* -> context for this object. */
	char*		 op;	/* -> string specifying operation to perform. */
	int			 arg;	/* =  optional argument for operation. */
{
	{
		ERRORF((fd, "O_Lut_Bt473_control: unknown option: %s\n", op));
		return 0;
	}
	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt473_dump"
 |
 |  Dump the contents of this object to a file descriptor.
 *---------------------------------------------------------------------------*/
static void m_Lut_Bt473_dump( p_me, fd )
	O_Lut_Bt473* p_me;	/* -> context for this object. */
	FILE*		 fd;	/* -> file descriptor to write object contents to. */
{
	fprintf(fd, "\nBrooktree 473 object @ 0x%08x: \n");
	fprintf(fd, "   Hardware library context at 0x%08x\n", p_me->p_xv);
	if ( p_me->atv->control != m_Lut_Bt473_control ) 
	{
		fprintf(fd, 
			"  *** control method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->control, m_Lut_Bt473_control);
	}
	else if ( p_me->atv->dump != m_Lut_Bt473_dump ) 
	{
		fprintf(fd, 
			"  *** dump method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->dump, m_Lut_Bt473_dump);
	}
	if ( p_me->atv->get_entries != m_Lut_Bt473_get_entries ) 
	{
		fprintf(fd, 
			"  *** get_entries method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->get_entries, m_Lut_Bt473_get_entries);
	}
	if ( p_me->atv->init_hw != m_Lut_Bt473_init_hw ) 
	{
		fprintf(fd, 
			"  *** init_hw method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->init_hw, m_Lut_Bt473_init_hw);
	}
	if ( p_me->atv->set_entries != m_Lut_Bt473_set_entries ) 
	{
		fprintf(fd, 
			"  *** set_entries method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->set_entries, m_Lut_Bt473_set_entries);
	}
	if ( p_me->atv->terminate != m_Lut_Bt473_terminate ) 
	{
		fprintf(fd, 
			"  *** terminate method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->terminate, m_Lut_Bt473_terminate);
	}
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt473_get_entries"
 |
 |  Return the values of a range of lookup table entries. There are three 
 |  8-in 8-out lookup tables, one for each color component. The values 
 |  pointed to by p_values are integers of the form XBGR.
 |
 |    = 1 on success, 0 if error detected.
 *---------------------------------------------------------------------------*/
static int m_Lut_Bt473_get_entries( p_me, type, start, count, p_values )
	O_Lut_Bt473*	p_me;		/* -> context for this object. */
	int				type;		/* =  type of entries (see objects.h). */
	int				start;		/* =  first location in table to return. */
	int				count;		/* =  # of entries to return. */
	int*			p_values;	/* -> where to place entries being returned. */
{
	int			blue;		/* Blue component of value. */
	int			green;		/* Green component of value. */
	int			i_entry;	/* # of entry in lut now getting. */
	int*		p_entry;	/* Data for entry in lut now getting. */
	XV_struct*	p_xv;		/* Fast access to hardware context: p_me->p_xv. */
	int			red;		/* Red component of value. */

	p_xv = p_me->p_xv;
	switch ( type & O_Lut_Entry_Type_Mask )
	{
	  case O_Lut_Entry_Lut:
		p_entry = p_values;
		i_Lut_Bt473_set_option_rsaddr(p_xv, 3, 3);
		PV_WREG(p_xv, Ext.reg1, start);
		i_Lut_Bt473_set_option_rsaddr(p_xv, 3, 1);
		for ( i_entry = start; i_entry < start+count; i_entry++ )
		{
			PV_RREG(p_xv, red, Ext.reg1);
			PV_RREG(p_xv, green, Ext.reg1);
			PV_RREG(p_xv, blue, Ext.reg1);
			*p_entry++ = (blue << 16) | (green << 8) | red;
		}
		break;
	  case O_Lut_Entry_Overlay:
		p_entry = p_values;
		i_Lut_Bt473_set_option_rsaddr(p_xv, 3, 4 | 3);
		PV_WREG(p_xv, Ext.reg1, start);
		i_Lut_Bt473_set_option_rsaddr(p_xv, 3, 4 | 1);
		for ( i_entry = start; i_entry < start+count; i_entry++ )
		{
			PV_RREG(p_xv, red, Ext.reg1);
			PV_RREG(p_xv, green, Ext.reg1);
			PV_RREG(p_xv, blue, Ext.reg1);
			*p_entry++ = (blue << 16) | (green << 8) | red;
		}
		break;
	  default:
		return 0;
	}
	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt473_init_hw"
 |
 |  Initialize hardware to its default state.
 |
 |    = 1 on success, 0 if error detected.
 *---------------------------------------------------------------------------*/
static int m_Lut_Bt473_init_hw( p_me )
	O_Lut_Bt473*	p_me;		/* -> context for this object. */
{
	XV_struct*	p_xv;		/* Fast access to hardware context: p_me->p_xv. */

	p_xv = p_me->p_xv;
	i_Lut_Bt473_set_option_rsaddr(p_xv, 3, 6);	/* Command register. */
	PV_WREG(p_xv, Ext.reg1, 0x30);
	i_Lut_Bt473_set_option_rsaddr(p_xv, 3, 2);	/* Read mask register. */
	PV_WREG(p_xv, Ext.reg1, 0xff);

	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt473_set_entries"
 |
 |  Set the values of a range of lookup table entries. There are three 8-in
 |  8-out lookup tables, one for each color component. The values pointed to
 |  by p_values are integers of the form XBGR.
 |
 |    = 1 on success, 0 if error detected.
 *---------------------------------------------------------------------------*/
static int m_Lut_Bt473_set_entries( p_me, type, start, count, p_values )
	O_Lut_Bt473*	p_me;		/* -> context for this object. */
	int				type;		/* =  type of entries (see objects.h). */
	int				start;		/* =  first location in table to set. */
	int				count;		/* =  # of entries to set. */
	int*			p_values;	/* -> where to get values for entries. */
{
	int			i_entry;	/* # of entry in lut now setting. */
	int*		p_entry;	/* Data for entry in lut now setting. */
	XV_struct*	p_xv;		/* Fast access to hardware context: p_me->p_xv. */

	p_xv = p_me->p_xv;
	switch ( type & O_Lut_Entry_Type_Mask )
	{
	  case O_Lut_Entry_Cursor:
     case BT473:
		for ( i_entry = start; i_entry < start+count; i_entry++, p_entry++ )
		{
			i_Lut_Bt473_set_option_rsaddr(p_xv, 3, 4);
			if ( i_entry )	/* Foreground color at addr 2 in overlay pallette */
			{
				PV_WREG(p_xv, Ext.reg1, 2);  
				i_Lut_Bt473_set_option_rsaddr(p_xv, 3, 5);
			}
			else			/* Background color at addr 1 in overlay palette. */
			{
				PV_WREG(p_xv, Ext.reg1, 1);  
				i_Lut_Bt473_set_option_rsaddr(p_xv, 3, 5);
			}
			PV_WREG(p_xv, Ext.reg1, *p_entry & 0xff);
			PV_WREG(p_xv, Ext.reg1, (*p_entry>>8) & 0xff);
			PV_WREG(p_xv, Ext.reg1, (*p_entry>>16) & 0xff);
		}
		break;
	  case O_Lut_Entry_Lut:
		p_entry = p_values;
		i_Lut_Bt473_set_option_rsaddr(p_xv, 3, 0);
		PV_WREG(p_xv, Ext.reg1, start);
		i_Lut_Bt473_set_option_rsaddr(p_xv, 3, 1);
		for ( i_entry = start; i_entry < start+count; i_entry++, p_entry++ )
		{
			PV_WREG(p_xv, Ext.reg1, *p_entry & 0xff);
			PV_WREG(p_xv, Ext.reg1, (*p_entry>>8) & 0xff);
			PV_WREG(p_xv, Ext.reg1, (*p_entry>>16) & 0xff);
		}
		break;
	  case O_Lut_Entry_Overlay:
		p_entry = p_values;
		i_Lut_Bt473_set_option_rsaddr(p_xv, 3, 4 | 0);
		PV_WREG(p_xv, Ext.reg1, start);
		i_Lut_Bt473_set_option_rsaddr(p_xv, 3, 4 | 1);
		for ( i_entry = start; i_entry < start+count; i_entry++, p_entry++ )
		{
			PV_WREG(p_xv, Ext.reg1, *p_entry & 0xff);
			PV_WREG(p_xv, Ext.reg1, (*p_entry>>8) & 0xff);
			PV_WREG(p_xv, Ext.reg1, (*p_entry>>16) & 0xff);
		}
		break;
	  default:
		return 0;
	}
	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt473_terminate"
 |
 |  Terminate use of this object.
 *---------------------------------------------------------------------------*/
static void m_Lut_Bt473_terminate( p_me )
	O_Lut_Bt473*	p_me;		/* -> context for this object. */
{
	free((char*)p_me);
}


static TV_Lut methods_Lut_Bt473 = {
	m_Lut_Bt473_control,
	m_Lut_Bt473_dump,
	m_Lut_Bt473_get_entries,
	m_Lut_Bt473_init_hw,
	m_Lut_Bt473_set_entries,
	m_Lut_Bt473_terminate
};


/*---------------------------------------------------------------------------*
 |"O_Lut_Bt473_O_Lut_Bt473"
 |
 |  Create an instance of a Brooktree 473 lookup table object.
 |
 |    -> object's context on success, or 0 if error occurred.
 *---------------------------------------------------------------------------*/
O_Lut* O_Lut_Bt473_O_Lut_Bt473( p_me, p_xv )
	O_Lut_Bt473* p_me;		/* -> context to fill in, or 0 for dynamic alloc. */
	XV_struct*	 p_xv;		/* -> context for hardware library. */
{
	if ( !p_me )
	{
		if ( !(p_me = (O_Lut_Bt473*)malloc(sizeof(O_Lut_Bt473))) )
		{
			ERRORF((fd, 
				"O_Lut_Bt473_O_Lut_Bt473: Unable to allocate storage\n"));
		}
	}
	p_me->atv = &methods_Lut_Bt473;
	p_me->p_xv = p_xv;
	
	return (O_Lut*)p_me;
}
