/*
 *  dmsd_input.c: DMSD input object implementation.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     25 Sep 94 - Andy Goodrich
 *       - Initial coding
 *
 *  Copyright(c) 1994, Parallax Graphics, Inc.
 * $Log:   /export/home/dist1/source/hwlib/archives/dmsd_input.c_v  $
 * 
 *    Rev 1.3   31 Dec 1996 11:53:30   dave
 *  
 * 
 *    Rev 1.2   09 Sep 1996 10:10:40   dave
 * Andy Goodrich: Added additional parameter to O_I2C_acknowledge call for 
 *                better diagnostics.
 * Revision 1.2  1995/12/14 16:15:31  dist
 * Andy Goodrich: Added additional parameter to O_I2C_acknowledge call for
 *                better diagnostics.
 *
 * Revision 1.1  1995/12/11  12:35:53  dist
 * Initial revision
 *
 * Revision 1.7  95/09/13  09:54:45  09:54:45  dist ()
 * Andy Goodrich: Added set_register method and changed trace code to
 *                include device name in messages.
 * 
 * Revision 1.6  95/06/02  11:38:00  11:38:00  dist ()
 * Andy Goodrich: Improved tracing of routine calls.
 * 
 * Revision 1.5  95/05/25  17:11:22  17:11:22  dist ()
 * Andy Goodrich - Combined HP/AIX version for HP 9.1 code freeze.
 * 
 * Revision 1.3  95/04/17  12:52:39  12:52:39  dist ()
 * Remove debugging statements for 9.07 release.
 * 
 * Revision 1.2  95/04/17  12:28:52  12:28:52  dist ()
 * Andy Goodrich - added RCS header.
 * 
 */
#include <sys/types.h>		/* Standard types for this C environment. */
#include <stdio.h>			/* Stream input/output declarations. */
#include "objects.h"		/* Hardware library object definitions. */
#include "hw_context.h"		/* Hardware library context definition. */
#include "plxv_xpoint.h"	/* Cross point switch definitions. */

/*  XV_DIAG BIT MEANINGS:
 *
 *    0x01 - normal one-shot debugging
 *    0x20 - I2C bus debugging
 *    0x40 - method call tracing
 *    0x80 - initialization debugging
 */
/* #define XV_DIAG 1 */
#include "../include/xvdebug.h"

/*
 *  INTERNAL INPUT OBJECT DEFINITION:
 */
typedef struct _I_Input_Dmsd {
    TV_Input*   atv;            /* Methods: must be the first entry!! */
 
    int         dcsc_address;       /* Address of dcsc for this input. */
    int         dmsd_address;       /* Address of dmsd for this input. */
    int         inverse_video;      /* = 1 if we have inverse video. */
    int         port;               /* Unit number of this input. */
    XV_struct*  p_xv;               /* Hardware information. */
    char*       values_now;         /* Current dmsd values in use. */
    char        values_ntsc[0x19];  /* Values for ntsc video. */
    char        values_pal[0x19];   /* Values for pal video. */
    char        values_secam[0x19]; /* Values for secam video. */
} I_Input_Dmsd;


/*
 *  DMSD VALUES:
 */
static char dcsc_addresses[2] = {0x70,0x71};
static char dmsd_addresses[2] = {0x45,0x47};


static XV_video_format input_format[2] = {
	{640,482,43,10,20},
	{768,574,51,7,23}
};

static int clip(val)
int val;
{
    if (val < 0) return(0);
    if (val > 0xff) return(0xff);
    return(val);
}


/*---------------------------------------------------------------------------*
 |"m_Input_init_hw_pv700"
 |
 |  Initialize hardware settings for this channel.
 |
 *---------------------------------------------------------------------------*/
static void m_Input_init_hw_pv700( p_me )
	I_Input_Dmsd*	p_me;		/* -> object context. */
{
	DEBUGF(0x40,(fd, "%s: m_Input_init_hw_pv700: entry\n", 
		p_me->p_xv->p_mmap->name));

	p_me->inverse_video = 0;
	p_me->values_now = p_me->values_ntsc;

	switch( p_me->port )
	{
	  case PORT_D:
		O_Rsrc_get_long(p_me->p_xv->p_rsrc, "d.dmsd.ntsc", p_me->values_ntsc);
		O_Rsrc_get_long(p_me->p_xv->p_rsrc, "d.dmsd.pal", p_me->values_pal);
		O_Rsrc_get_long(p_me->p_xv->p_rsrc, "d.dmsd.secam", p_me->values_secam);
		plxv_xpoint_a(p_me->p_xv, (CE_HIGH | HSYNC_HIGH | VSYNC_HIGH
			| ODD_HIGH  | ODD_INMODE_A | SHUTTLE_EN));
		p_me->p_xv->plxv_io_format[0] = &input_format[0];
		break;
	  case PORT_B:
		O_Rsrc_get_long(p_me->p_xv->p_rsrc, "b.dmsd.ntsc", p_me->values_ntsc);
		O_Rsrc_get_long(p_me->p_xv->p_rsrc, "b.dmsd.pal", p_me->values_pal);
		O_Rsrc_get_long(p_me->p_xv->p_rsrc, "b.dmsd.secam", p_me->values_secam);
		plxv_xpoint_b(p_me->p_xv, (CE_HIGH | HSYNC_HIGH | VSYNC_HIGH
	        | ODD_HIGH  | ODD_INMODE_A | SHUTTLE_EN));
		p_me->p_xv->plxv_io_format[1] = &input_format[0];
		break;
	}
	O_Input_set_type(p_me, "NTSC", "COMPOSITE", 0);
	O_Input_set_levels(p_me, 0x7f, 0x7f, 0x7f, 0x7f);

	DEBUGF(0x40,(fd, "%s: m_Input_init_hw_pv700: exit\n", 
		p_me->p_xv->p_mmap->name));
}


/*---------------------------------------------------------------------------*
 |"m_Input_off_pv700"
 |
 |  Disable an input.
 |
 *---------------------------------------------------------------------------*/
static void m_Input_off_pv700( p_me )
	I_Input_Dmsd*	p_me;		/* -> object context. */
{
    int     bits;       /* Shadow value for register. */

	DEBUGF(0x40,(fd, "%s: m_Input_off_pv700: entry\n", 
		p_me->p_xv->p_mmap->name));

    if ( p_me->port == PORT_B ) 
    {   
		bits = p_me->p_xv->xv_shdw->b.Bt.grn.reg1;
		bits = bits | 0x1;
		PV_WREG(p_me->p_xv,Bt.grn.reg1, bits);
		bits = p_me->p_xv->xv_shdw->b.Bt.grn.reg2;
		bits = (bits & 0xf9) | 0x2;
		PV_WREG(p_me->p_xv,Bt.grn.reg2, bits);
	}
	DEBUGF(0x40,(fd, "%s: m_Input_off_pv700: exit\n",p_me->p_xv->p_mmap->name));
}


/*---------------------------------------------------------------------------*
 |"m_Input_on_pv700"
 |
 |  Enable an input.
 |
 *---------------------------------------------------------------------------*/
static void m_Input_on_pv700( p_me )
	I_Input_Dmsd*	p_me;		/* -> object context. */
{
    int     bits;       /* Shadow value for register. */
 
	DEBUGF(0x40,(fd, "%s: m_Input_on_pv700: entry\n",p_me->p_xv->p_mmap->name));

    if ( p_me->port == PORT_B ) 
    {   
        bits = p_me->p_xv->xv_shdw->b.Bt.grn.reg1;
        bits = bits & 0xfe;
        PV_WREG(p_me->p_xv,Bt.grn.reg1, bits);
        bits = p_me->p_xv->xv_shdw->b.Bt.grn.reg2;
		bits = (bits & 0xf9) | 0x4;
        PV_WREG(p_me->p_xv,Bt.grn.reg2, bits);
		plxv_xpoint_b(p_me->p_xv, CE_HIGH | HSYNC_HIGH | VSYNC_HIGH
			| ODD_HIGH  | ODD_INMODE_A | SHUTTLE_EN);
    } 
	else
	{
		plxv_xpoint_a(p_me->p_xv, CE_HIGH | HSYNC_HIGH | VSYNC_HIGH
			| ODD_HIGH  | ODD_INMODE_A | SHUTTLE_EN);
	}

	DEBUGF(0x40,(fd, "%s: m_Input_on_pv700: exit\n", p_me->p_xv->p_mmap->name));
}


/*---------------------------------------------------------------------------*
 |"m_Input_set_levels_pv700"
 |
 |  Set color characteristics of video input.
 |
 *---------------------------------------------------------------------------*/
static void m_Input_set_levels_pv700( p_me, brightness,contrast,hue,saturation )
	I_Input_Dmsd*	p_me;		/* -> object context. */
	int				brightness;	/* =  brightness level to set. */
	int				contrast;	/* =  contrast level to set. */
	int				hue;		/* =  hue level to set. */
	int				saturation;	/* =  satruation level to set. */
{
    float   a,b,c;			/* Coefficients for calculating clt entries. */
	int		i_addr_retry;	/* # of retries performed on address set so far. */
	int		i_data_retry;	/* # of retries performed on data set so far. */
    int 	i_entry;		/*	Entry in clt now filling in. */
	int		temp;			/* Temporary for register values. */
 
	DEBUGF(0x40,(fd, "%s: m_Input_set_levels_pv700: entry\n",
		p_me->p_xv->p_mmap->name));

	/* SET SATURATION (DMSD VALUE 17): */

	p_me->values_now[17] = saturation;
	for ( i_data_retry = 0; i_data_retry < 3; i_data_retry++ )
	{
		if ( !O_I2C_write(p_me->p_xv->p_i2c, p_me->dmsd_address, 17, 
			p_me->values_now[17]) ) break;
		ERRORF((fd,"%s:m_Input_set_levels_pv700: error 7191 %x loc 17 try %d\n",
			p_me->p_xv->p_mmap->name, p_me->dmsd_address, i_data_retry+1));
	}


	/* SET HUE (DMSD VALUE 7): */

	p_me->values_now[7] = hue - 0x80;

	for ( i_data_retry = 0; i_data_retry < 3; i_data_retry++ )
	{
		if ( !O_I2C_write(p_me->p_xv->p_i2c, p_me->dmsd_address, 7, 
			p_me->values_now[7]) ) break;
		ERRORF((fd,"%s: m_Input_set_levels_pv700: error 7191 %x loc 7 try %d\n",
			p_me->p_xv->p_mmap->name, p_me->dmsd_address, i_data_retry+1));
	}


    /* CALCULATE COEFFICIENTS FOR CALCULATING BRIGHTNESS/CONTRAST TABLE. */

    a = (float)(brightness - 0x80) * 0.6;
    b = (float) (contrast - 0x80) /128.0 + 1.00;
    if ( p_me->inverse_video )
    {
        a = 256.0 - a;
        b = -b;
    }   
 
    
	/*  PLACE DMSE INTO BYPASS MODE (BIT 3 OF MODE REGISTER 2 ) TO HIDE THE 
     *  LOADING OF THE TABLE. THEN PREPARE THE DCSC (7192) TO RECEIVE THE TABLE.
	 */
	temp = p_me->p_xv->xv_shdw->b.Bt.grn.reg2;
	temp &= ~0x08;
	PV_WREG(p_me->p_xv,Bt.grn.reg2, temp); /* the bit is in mode register 2. */


	O_I2C_write(p_me->p_xv->p_i2c,p_me->dcsc_address,0,0x3a); /* vlut write. */
	for ( i_addr_retry = 0; i_addr_retry < 10; i_addr_retry++ )
	{
		O_I2C_set_address(p_me->p_xv->p_i2c, p_me->dcsc_address, 4);
		O_I2C_write_data(p_me->p_xv->p_i2c, 0);
		if ( !O_I2C_acknowledge(p_me->p_xv->p_i2c, p_me->dcsc_address, 3) )
			break;
		DEBUGF(0x20, (fd, "%s: DCSC(%x) address set failed attempt %d\n", 
			p_me->p_xv->p_mmap->name, p_me->dcsc_address, i_addr_retry+1));
	}


	/*  LOOP CALCULATING THE VALUES AND LOADING THEM INTO THE DCSC. */
    
    for ( i_entry=0; i_entry < 256; i_entry )
    {
        c = a + (b * (float)i_entry) + 0.5;
        temp = clip((int)c);
		for ( i_data_retry = 0; i_data_retry < 10; i_data_retry++ )
		{
			O_I2C_write_data(p_me->p_xv->p_i2c, temp);
        	if ( !O_I2C_acknowledge(p_me->p_xv->p_i2c, p_me->dcsc_address, 3) ) 
			{
				i_entry++;
				break;
			}
			DEBUGF(0x20, (fd, "%s DCSC(%x) data failed on entry %d retry %d\n", 
				p_me->p_xv->p_mmap->name, p_me->dcsc_address, i_entry, 
				i_data_retry)
			);
			for ( i_addr_retry = 0; i_addr_retry < 10; i_addr_retry++ )
			{
				O_I2C_set_address(p_me->p_xv->p_i2c, p_me->dcsc_address, 4);
				O_I2C_write_data(p_me->p_xv->p_i2c, i_entry);
				if (O_I2C_acknowledge(p_me->p_xv->p_i2c, p_me->dcsc_address, 3))
					break;
				DEBUGF(0x20,
					(fd,"%s: DCSC(%x) address set failed attempt %d entry %d\n",
					p_me->p_xv->p_mmap->name, p_me->dcsc_address, 
					i_addr_retry+1, i_entry)
				);
			}
		}
	}
	O_I2C_write(p_me->p_xv->p_i2c,p_me->dcsc_address,0,0xfa); /* vlut read. */


	/*  SET DMSD INTO NORMAL MODE BY TURNING OFF BIT 3 OF MODE REGISTER 2. */

	temp = p_me->p_xv->xv_shdw->b.Bt.grn.reg2;
	temp |= 0x08;
	PV_WREG(p_me->p_xv, Bt.grn.reg2, temp); 

	DEBUGF(0x40,(fd, "%s: m_Input_set_levels_pv700: exit\n",
		p_me->p_xv->p_mmap->name));
}


/*---------------------------------------------------------------------------*
 |"m_Input_set_register_pv700"
 |
 |  Set the value of a 7110 register.
 |
 |    = 1 if success, 0 if error detected
 *---------------------------------------------------------------------------*/
int m_Input_set_register_pv700( p_me, reg, value)
	I_Input_Dmsd*	p_me;		/* -> object context. */
	int				reg;		/* =  register number to set value of. */
	int				value;		/* =  value to set register to. */
{
	DEBUGF(0x40,(fd, "%s: m_Input_set_register_pv700: entry\n",
		p_me->p_xv->p_mmap->name));
	if( O_I2C_write(p_me->p_xv->p_i2c, p_me->dmsd_address, reg, value) )
	{
		DEBUGF(0x40,(fd, "%s: m_Input_set_register_pv700: i2c error exit\n",
			p_me->p_xv->p_mmap->name));
		return 0;
	}
	DEBUGF(0x40,(fd, "%s: m_Input_set_register_pv700: exit\n",
		p_me->p_xv->p_mmap->name));
	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Input_set_type_pv700"
 |
 |  Set type of video input.
 |
 |    Notes:
 |      (1) For PORT_B we need to disable output and enable input since the
 |          port is shared.
 |
 |    = 1 if success, 0 if error detected
 *---------------------------------------------------------------------------*/
static int m_Input_set_type_pv700( p_me, timing, format, instance )
	I_Input_Dmsd*	p_me;		/* -> object context. */
	char*			timing;		/* =  timing to use (NTSC, PAL, SECAM).
	char*			format;		/* =  input format to use (COMPOSITE, YC).
	int				instance;	/* =  instance of format. */
{
	int		bits;		/* Temporary to manipulate register bits in. */
	int 	i_value;    /* # of value now loading. */

	DEBUGF(0x40,(fd, "%s: m_Input_set_type_pv700: entry\n",
		p_me->p_xv->p_mmap->name));

	/*  PROCESS TIMING INFORMATION: */

	if ( !strcmp(timing, "NTSC") )
	{
		p_me->p_xv->plxv_io_format[p_me->port] = &(input_format[0]);
		p_me->values_now = p_me->values_ntsc;
	}
	else if ( !strcmp(timing, "PAL") )
	{
		p_me->p_xv->plxv_io_format[p_me->port] = &(input_format[1]);
		p_me->values_now = p_me->values_pal;
	}
	else if ( !strcmp(timing, "SECAM") )
	{
		p_me->p_xv->plxv_io_format[p_me->port] = &(input_format[1]);
		p_me->values_now = p_me->values_secam;
	}
	else
	{
		fprintf(stderr, "Unrecognized timing type %s\n", timing);
		return 0;
	}
	for ( i_value = 0; i_value < sizeof(p_me->values_ntsc); i_value++ )
	{
		O_I2C_write(p_me->p_xv->p_i2c, p_me->dmsd_address, 
			i_value, p_me->values_now[i_value]);
	}


	/*  PROCESS CONNECTION INFORMATION: */

	if ( !strcmp(format, "COMPOSITE") )
	{
		if ( p_me->port == PORT_B ) /* Enable input port, disable output port.*/
		{
			bits = p_me->p_xv->xv_shdw->b.Bt.grn.reg2;
			bits = (bits & 0xf9) | 0x4;
			PV_WREG(p_me->p_xv, Bt.grn.reg2, bits);
			plxv_xpoint_b(p_me->p_xv, CE_HIGH | HSYNC_HIGH | VSYNC_HIGH
				| ODD_HIGH  | ODD_INMODE_A | SHUTTLE_EN);
		}
		O_I2C_write(p_me->p_xv->p_i2c, p_me->dmsd_address, 
			0xe, p_me->values_now[0x0e]|instance);
		O_I2C_write(p_me->p_xv->p_i2c,p_me->dmsd_address,	/* Set bpf, etc. */
			0x6,p_me->values_now[6]&~0x80);
	}
	else if ( !strcmp(format, "YC") )
	{
		if ( p_me->port == PORT_D )
		{
			ERRORF((fd, "*** There is no YC on port %d\n", PORT_D));
			return 0;
		}
		else
		{
			bits = p_me->p_xv->xv_shdw->b.Bt.grn.reg2;
			bits = (bits & 0xf9) | 0x4;
			PV_WREG(p_me->p_xv, Bt.grn.reg2, bits);
			plxv_xpoint_b(p_me->p_xv, CE_HIGH | HSYNC_HIGH | VSYNC_HIGH
				| ODD_HIGH  | ODD_INMODE_A | SHUTTLE_EN);
		}
		PV_WREG(p_me->p_xv, MiscMode.b.red, instance ? 4 : 0);
		O_I2C_write(p_me->p_xv->p_i2c, p_me->dmsd_address, 
			0xe, p_me->values_now[0xe] | 2);
		O_I2C_write(p_me->p_xv->p_i2c, p_me->dmsd_address,
			0x6, p_me->values_now[6]|0x80);
	}
	else
	{
		ERRORF((fd, "Unrecognized format type %s\n", format));
		return 0;
	}

	DEBUGF(0x40,(fd, "%s: m_Input_set_type_pv700: exit\n",
		p_me->p_xv->p_mmap->name));
	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Input_status_pv700"
 |
 |  Return status of input signal.
 |
 |    = status value from DMSD
 *---------------------------------------------------------------------------*/
static int m_Input_status_pv700( p_me )
    I_Input_Dmsd*  p_me;       /* -> object context. */
{
    int     status;       /* Bits from dmsd register giving status. */

	DEBUGF(0x40,(fd, "%s: m_Input_status_pv700: entry\n",
		p_me->p_xv->p_mmap->name));
	if ( O_I2C_read_no_sub(p_me->p_xv->p_i2c, p_me->dmsd_address, &status) )
	{
		ERRORF((fd, "%s: m_Input_status_pv700: i2c read error 7191 %x\n",
			p_me->p_xv->p_mmap->name, p_me->dmsd_address));
		return(-1);
	}
	DEBUGF(0x40,(fd, "%s: m_Input_status_pv700: entry\n",
		p_me->p_xv->p_mmap->name));
	return status;
}

/*---------------------------------------------------------------------------*
 |"m_Input_terminate_pv700"
 |
 |  Destructor for this object: free its dynamically allocated storage.
 |
 *---------------------------------------------------------------------------*/
static void m_Input_terminate_pv700( p_me )
    I_Input_Dmsd*  p_me;       /* -> object context. */
{
	DEBUGF(0x40,(fd, "%s: m_Input_terminate_pv700: entry\n",
		p_me->p_xv->p_mmap->name));
	free((char*)p_me);
	DEBUGF(0x40,(fd, "m_Input_terminate_pv700: exit\n"));
}

/*---------------------------------------------------------------------------*
 |"m_Input_parameter_string"
 |
 |  set video parameter based on the string argument
 |
 *---------------------------------------------------------------------------*/
static int m_Input_parameter_string(p_me, s)
    I_Input_Dmsd*  p_me;       /* -> object context. */
    char *s;
{
  int temp, i;
  while(*s)
  {
	for(i = 0; s[i] && (s[i] != ' ') && (s[i] != '\t'); ++i);
	if(s[i])
	{
		s[i++] = 0;
		while(s[i] && ((s[i] == ' ') || (s[i] == '\t'))) ++i;
	}
    if(!strcmp(s, "INVERTED"))
    {
		p_me->inverse_video = 1;
		m_Input_set_levels_pv700(p_me, 0x80, 0x80, 0x80 + p_me->values_now[7],
						   p_me->values_now[17]); 
	}
    else if(!strcmp(s, "NONINVERTED"))
	{
		p_me->inverse_video = 0;
		m_Input_set_levels_pv700(p_me, 0x80, 0x80, 0x80 + p_me->values_now[7],
						   p_me->values_now[17]); 
	}
    else if(!strcmp(s, "BW"))
	{
	
		O_I2C_write(p_me->p_xv->p_i2c, p_me->dmsd_address, 8, 0);
		temp = p_me->values_now[0x6] | 0x80;
		p_me->values_now[0x6] = temp;
		O_I2C_write(p_me->p_xv->p_i2c, p_me->dmsd_address, 6, temp);
	}
    else if(!strcmp(s, "COLOR"))
	{
		temp = p_me->values_now[0x6] & ~0x80;
		p_me->values_now[0x6] = temp;
		O_I2C_write(p_me->p_xv->p_i2c, p_me->dmsd_address, 6, temp);
		O_I2C_write(p_me->p_xv->p_i2c, p_me->dmsd_address, 8,
					p_me->values_now[8]);
	}
    else
		return 0;
	s = &s[i];
  }
  return 1;
}

/*
 *  I_Input_Dmsd Object Methods:
 */
static TV_Input O_Input_methods_pv700 = {
	m_Input_init_hw_pv700,
	m_Input_off_pv700,
	m_Input_on_pv700,
	m_Input_set_levels_pv700,
	m_Input_set_register_pv700,
	m_Input_set_type_pv700,
	m_Input_status_pv700,
	m_Input_terminate_pv700,
    m_Input_parameter_string
};

/*---------------------------------------------------------------------------*
 |"O_Input_O_Input"
 |
 |  Create a DMSD-based input object and return a pointer to it. Note we do 
 |  not initialize the hardware, this is done by the init_hw method.
 |
 |    -> object on success, 0 on failure
 *---------------------------------------------------------------------------*/
O_Input* O_Input_Dmsd_O_Input_Dmsd( p_me, unit, p_xv )
	I_Input_Dmsd*	p_me;	/* -> object initializing, or 0. */
	int				unit;	/* =  # of input creating object for. */
	XV_struct*		p_xv;	/* -> hardware definition. */
{
	int				i_poll;	/* # of times around device existence poll loop. */
	int				status;	/* Status register value. */

	DEBUGF(0x80, (fd, "%s: O_Input_Dmsd_O_Input_Dmsd: entry dmsd %d(%x)\n", 
				p_xv->p_mmap->name, unit, dmsd_addresses[unit])); 


	/*  READ THE STATUS REGISTER TO VERIFY THE DMSD IS THERE. */

	for ( i_poll = 0; i_poll < 5; i_poll++ )
	{
	    if ( !O_I2C_read_no_sub(p_xv->p_i2c, dmsd_addresses[unit], &status) )
			break;
	}
	if ( i_poll >= 5 )
	{
		DEBUGF(0xa0, (fd, 
			"%s: O_Input_Dmsd_O_Input_Dmsd: could not set up dmsd %d(%x)!!!\n",
				p_xv->p_mmap->name, unit, dmsd_addresses[unit])
		); 
		return 0;
	}


	/*  DMSD IS THERE, ALLOCATE AND INITIALIZE AN INPUT OBJECT. */

	if ( !p_me )
	{
		if ( !(p_me = (I_Input_Dmsd*)malloc(sizeof(I_Input_Dmsd))) )
		{
			fprintf(stderr, 
				"*** Unable to allocate input object for unit %d\n", unit);
			return 0;
		}
	}

	p_me->atv = &O_Input_methods_pv700;
	p_me->dcsc_address = dcsc_addresses[unit];
	p_me->dmsd_address = dmsd_addresses[unit];
	p_me->port = unit ? PORT_B : PORT_D;
	p_me->p_xv = p_xv;

	DEBUGF(0x80, (fd, "%s: O_Input_Dmsd_O_Input_Dmsd: exit dmsd %d(%x)\n", 
				p_xv->p_mmap->name, unit, dmsd_addresses[unit])); 

	return (O_Input*)p_me;
}
