/*
 *  generic_draw.c: Generic Support for Drawing Operations.
 *
 *   This code supports the Video Stream I architecture.
 *
 * (c) Copyright 1995 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 * $Log:   /plxhp/dave/pvcsrc/hwlib/archives/generic_draw.c_v  $
 * 
 *    Rev 1.3   09 Sep 1996 10:11:22   dave
 * fixed various interleave things
 * Revision 1.3  1996/07/22 20:10:20  dist
 * fixed various interleave things
 *
 * Revision 1.1  1995/12/08  10:27:34  dist
 * Initial revision
 *
 */
#include <stdio.h>				/* Pickup stdout and friends. */
#include <sys/types.h>			/* Pick up u_int and friends. */
#include "objects_internal.h"	/* Pick up object definitions. */

/*  XV_DIAG MEANINGS: 
 *
 *    0x80 - object creation and termination.
 *    0x40 - method tracing
 *       1 - normal one-shot debugging.
 */
#define XV_DIAG 1
#include "xvdebug.h"		/* Pick up XV debugging macros. */

/*  INTERNAL OBJECT DEFINITION FOR DRAW OBJECT: */

typedef struct _I_Draw {
	TV_Draw*	atv;		/* Methods: first entry in all derived objects. */
	int			force_gc;	/* = 1 if should force loading of gc registers. */
	O_Draw_Gc	gc;			/* Graphics context for drawing operations. */
	XV_struct*	p_xv;		/* Parallax hardware library context structure. */
} I_Draw;


/* FORWARD FUNCTION REFERENCES: */

static int  m_Draw_copy_area(/* p_me, src_x, src_y, dst_x, dst_y, w, h */);
static void m_Draw_dump(/* p_me, fd */);
static int  m_Draw_fill_box(/* p_me, x, y, w, h */);
static void m_Draw_get_gc(/* p_me, p_gc */);
static void m_Draw_init_hw(/* p_me, start, count, p_value */);
static int  m_Draw_set_gc(/* p_me, p_gc */);
static void m_Draw_terminate(/* p_me */);
static int  m_Draw_vector(/* p_me, start_x, start_y, len_x, len_y */);
static void m_Draw_wait_for_done( /* p_me */);
static int  m_Draw_vector_uncook(/* p_me, x0,y0,num_pix,i1,i2,e,left_flag,
				 up_flag,x_major_flag */);


/*  VARIOUS USEFUL MACROS: */

#undef  PLXV_ADDR_REG
#define PLXV_ADDR_REG(x,y) /* Calculate starting address for accelerator ops */\
	( (int)p_xv->maps->reg[(y)]  + (x))

#define Vector_Done( p_xv ) /* Wait for vector hardware completion, */ \
{ \
    int loop=1; \
    while (loop) { PV_RREG(p_xv, loop, VBusy.l); } \
}

    /* routine to support the offsetting map hardware in all access modes */

static void plxv_map_ctl(p_me, start,end,offset)
XV_struct *p_me;
int offset,start,end;
{
    O_Draw_wait_for_done(p_me->p_draw);
    PV_WREG(p_me, MStart.l,start);
    PV_WREG(p_me, MEnd.l,end);
    PV_WREG(p_me, MOffset.l,offset);
}

static void plxv_ctl_1(p_me, offset,length)
XV_struct *p_me;
int offset,length;
{
    plxv_map_ctl(p_me, 0,length - 1,offset);
}


/*---------------------------------------------------------------------------*
 |"i_Draw_set_rwm"
 |
 |  Internal function to set read-write mode bit based on current graphics 
 |  context. This routine should be called any time the rop, stencil_mask,
 |  or write masks are changed.
 *---------------------------------------------------------------------------*/
static void i_Draw_set_rwm( p_me )
	I_Draw* p_me;	/* -> context for this object. */
{
	int			rwm_value;	/* New read-write mode value to set. */

	rwm_value = ( (p_me->gc.rop == PLXV_ROP_COPY) &&
				  (p_me->gc.stencil_mask == 0) &&
				  ((p_me->gc.write_mask & 0xff000000) == 0) ) ? 0 : 1;
	PV_WREG(p_me->p_xv, RopCtl.b.rmw, rwm_value);
}


/*---------------------------------------------------------------------------*
 |"m_Draw_copy_area"
 |
 |  Copy a rectangular area using the bitblt hardware, based on the current
 |  graphics context. The rectangle is specified by a corner and width and 
 |  direction vectors. The vectors' sign determine the copy direction:
 |               +     -             +    -
 |     width:  right left  height: down  up
 |
 |    = 1 on success, 0 on error.
 *---------------------------------------------------------------------------*/
static int m_Draw_copy_area( p_me, src_x, src_y, dst_x, dst_y, w, h )
	I_Draw* p_me;	/* -> context for this object. */
	int		src_x;	/* =  x position of source for copy. */
	int		src_y;	/* =  y position of source for copy. */
	int		dst_x;	/* =  x position of destination for copy. */
	int		dst_y;	/* =  y position of destination for copy. */
	int     w;		/* =  width direction vector for copy. */
	int		h;		/* =  height direction vector for copy. */
{
    int 		src_start;	/* Linear address of source start. */
	int			swap;		/* Temporary for swapping source and dest. */
	XV_struct*	p_xv;		/* Fast access to hardware context (p_me->p_xv). */

	DEBUGF(0x40, (fd, "m_Draw_copy_area: entry\n"));


    if (w == 0 || h == 0) return(0);
	p_xv = p_me->p_xv;
    m_Draw_wait_for_done(p_me);


#if 0	/* dont do this */
	/*  ADJUST SIGNS OF WIDTH AND HEIGHT TO REFLECT THE DIRECTION OF MOVE.
	 *
	 *  If moving in -x and source offset is larger than dest. move right 4. 
	 */
	if ( src_x < dst_x )
	{
		src_x += w-1;
		dst_x += w-1;
		w = -w;
    }
	if ( src_y < dst_y ) 
	{
		src_y += h-1;
		dst_y += h-1;
		h = -h;
	}
#endif

	src_start = PLXV_ADDR_REG(src_x,src_y);
	if ( w < 0 )
	{
		if(p_xv->main.xsize == 1280)
		{
			if (( src_x % 5) > (dst_x % 5 )) src_start +=5;
		}
		else
		{
			if (( src_x & 0x3) > (dst_x & 3 )) src_start +=4;
		}
	}


	/*  GET AROUND A BUG IN GA HARDWARE HAVING TO DO WITH VERTICAL SIZE. 
	 *    We set the write mask to zero and draw a vector of length one in the 
	 *    vertical direction of the box we will render.
	 */
    if ( p_xv->main.ga_rev == GAREV_2 ) 
    {
		int	dy;		/* Vertical direction of draw. */

		PV_WREG(p_xv, DrawMask.l, 0); 
		p_xv->xv_wreg->DstW.l = 0;
		p_xv->xv_wreg->VectDY.l = ((h<0) ? -1 : 1);
		Vector_Done(p_xv);
		PV_WREG(p_xv, DrawMask.l, p_me->gc.write_mask);
	}


	/*  LOAD UP THE BLIT HARDWARE AND WAIT FOR IT TO FINISH. */
    
    PV_WREG(p_xv, SrcStartAdd.l, src_start);
    PV_WREG(p_xv, DstStartAdd.l, PLXV_ADDR_REG(dst_x,dst_y));
    PV_WREG(p_xv, DstW.l, w);
    PV_WREG(p_xv, DstH.l, h);
    m_Draw_wait_for_done(p_me);


	DEBUGF(0x40, (fd, "m_Draw_copy_area: exit\n"));
    return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Draw_dump"
 |
 |  Dump the contents of this object to a file descriptor.
 *---------------------------------------------------------------------------*/
static void m_Draw_dump( p_me, fd )
	I_Draw* p_me;	/* -> context for this object. */
	FILE*	fd;		/* -> file descriptor to write object contents to. */
{
	DEBUGF(0x40, (fd, "m_Draw_dump: entry\n"));

	fprintf(fd, "\nDrawing object @ 0x%08x: \n", p_me);
	fprintf(fd, "   Hardware library context at 0x%08x\n", p_me->p_xv);
	fprintf(fd, "   Graphics context is: \n");
    fprintf(fd, "     color1_bg      = 0x%08x\n", p_me->gc.color1_bg); 
    fprintf(fd, "     color1_fg      = 0x%08x\n", p_me->gc.color1_fg); 
    fprintf(fd, "     f_transparency = %d\n",     p_me->gc.f_transparency);
    fprintf(fd, "     readback       = ");
	switch ( p_me->gc.readback )
	{
	  case 0:	fprintf(fd, "red\n"); 	break;
	  case 1:	fprintf(fd, "green\n"); break;
	  case 2:	fprintf(fd, "blue\n");  break;
	  case 3:	fprintf(fd, "tags\n");  break;
	  default:  fprintf(fd, "Unknown");	break;
	}
    fprintf(fd, "     rop            = 0x%08x\n", p_me->gc.rop);
    fprintf(fd, "     stencil_mask   = 0x%02x\n", p_me->gc.stencil_mask);
    fprintf(fd, "     stencil_value  = 0x%02x\n", p_me->gc.stencil_value);
    fprintf(fd, "     wid_mask       = 0x%02x\n", p_me->gc.wid_mask);
    fprintf(fd, "     wid_value      = 0x%02x\n", p_me->gc.wid_value);
    fprintf(fd, "     write_mask     = 0x%08x\n", p_me->gc.write_mask);

#ifdef XV_DIAG
	if ( (int)p_me->atv->dump != (int)m_Draw_dump ) 
	{
		fprintf(fd, 
			"  *** dump method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->dump, m_Draw_dump);
	}
	if ( (int)p_me->atv->fill_box != (int)m_Draw_fill_box ) 
	{
		fprintf(fd, 
			"  *** fill_box method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->fill_box, m_Draw_fill_box);
	}
	if ( (int)p_me->atv->get_gc != (int)m_Draw_get_gc ) 
	{
		fprintf(fd, 
			"  *** get_gc method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->get_gc, m_Draw_get_gc);
	}
	if ( (int)p_me->atv->init_hw != (int)m_Draw_init_hw ) 
	{
		fprintf(fd, 
			"  *** init_hw method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->init_hw, m_Draw_init_hw);
	}
	if ( (int)p_me->atv->set_gc != (int)m_Draw_set_gc ) 
	{
		fprintf(fd, 
			"  *** set_gc method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->set_gc, m_Draw_set_gc);
	}
	if ( (int)p_me->atv->terminate != (int)m_Draw_terminate ) 
	{
		fprintf(fd, 
			"  *** terminate method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->terminate, m_Draw_terminate);
	}
#endif /* XV_DIAG */
	DEBUGF(0x40, (fd, "m_Draw_dump: exit\n"));
}


/*---------------------------------------------------------------------------*
 |"m_Draw_fill_box"
 |
 |  Fill a rectangular area using the box fill hardware, based on current
 |  graphics context. The box is specified by a corner and width and height 
 |  direction vectors. The vectors' sign determine the drawing direction:
 |               +     -             +    -
 |     width:  right left  height: down  up
 |
 |    = 1 on success, 0 on error.
 *---------------------------------------------------------------------------*/
static int m_Draw_fill_box( p_me, x, y, w, h )
	I_Draw* p_me;	/* -> context for this object. */
	int		x;		/* =  x position of box origin. */
	int		y;		/* =  y position of box origin. */
	int     w;		/* =  width direction vector for fill. */
	int     h;		/* =  height direction vector for fill. */
{
	XV_struct*	p_xv;	/* Fast access to hardware context (p_me->p_xv). */

	DEBUGF(0x40, (fd, "m_Draw_fill_box(%x,%d,%d,%d,%d: entry\n", p_me,x,y,w,h));

	if ( w == 0 || h == 0 ) return 0;

    m_Draw_wait_for_done(p_me);
	p_xv = p_me->p_xv;


#define NARROW_BOX_BUG 
#ifdef NARROW_BOX_BUG
	/*  GET AROUND A BUG WITH CERTAIN BOXES NARROWER THAN THE INTERLEAVE FACTOR.
	 *    If the width of the box is 3 we recursively call ourselves to fill
	 *    boxes of width 1.
     */
	{
		int plus_w; 
		int xinc;

		if(w < 0) { plus_w= -w; xinc= -1; }
		else       { plus_w= w;  xinc=  1; }
		if((plus_w > 1) && (((p_xv->main.linebytes > 1152) && (plus_w < 5)) ||
		   (plus_w < 3)))	/* could do a 2 then a 1, but why bother? */
		{
			int i,j;
			j = x;
			for( i = plus_w; i--; )
			{
				m_Draw_fill_box(p_me, j, y, 1, h);
				j += xinc;
			}
			return 1;
		}
	}
#endif


	/*  GET AROUND A BUG IN GA HARDWARE HAVING TO DO WITH VERTICAL SIZE. 
	 *    We set the write mask to zero and draw a vector of length one in the 
	 *    vertical direction of the box we will render.
	 */
    if ( p_xv->main.ga_rev == GAREV_2 ) 
    {
		int	dy;		/* Vertical direction of draw. */

		PV_WREG(p_xv, DrawMask.l, 0); 
		p_xv->xv_wreg->DstW.l = 0;
		p_xv->xv_wreg->VectDY.l = ((h<0) ? -1 : 1);
		Vector_Done(p_xv);
		PV_WREG(p_xv, DrawMask.l, p_me->gc.write_mask);
	}
    

	/*  DRAW THE BOX. */

    PV_WREG(p_xv, SolidCol.l, p_me->gc.color);
    PV_WREG(p_xv, DstStartAdd.l, PLXV_ADDR_REG(x,y));
    PV_WREG(p_xv, DstW.l, w);
    PV_WREG(p_xv, DstH.l, h);
    m_Draw_wait_for_done(p_me);

	DEBUGF(0x40, (fd, "m_Draw_fill_box: exit\n"));
	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Draw_get_gc"
 |
 |  Get the current drawing context.
 |
 *---------------------------------------------------------------------------*/
static void m_Draw_get_gc( p_me, p_gc )
	I_Draw* p_me;	/* -> context for this object. */
	O_Draw_Gc*	 p_gc;	/* -> where to return gc information. */
{
	DEBUGF(0x40, (fd, "m_Draw_get_gc: entry\n"));
	memcpy(p_gc, &p_me->gc, sizeof(O_Draw_Gc));
	DEBUGF(0x40, (fd, "m_Draw_get_gc: exit\n"));
}


/*---------------------------------------------------------------------------*
 |"m_Draw_init_hw"
 |
 |  Initialize the hardware to its default state.
 |
 *---------------------------------------------------------------------------*/
static void m_Draw_init_hw( p_me )
	I_Draw* p_me;	/* -> context for this object. */
{
	O_Draw_Gc	gc;	/* Area to hold gc defaults. */
    XV_struct*	p_xv;	/* Fast access to h/w context: p_me->p_xv. */
 

	DEBUGF(0x40, (fd, "m_Draw_init_hw: entry\n"));

    m_Draw_wait_for_done(p_me);
	p_xv = p_me->p_xv; 


	/* INITIALIZE STATIC (NON-GC) REGISTERS.  */

    PV_WREG(p_xv, KeyColCtl.l, 0);  /* disable video blue hardware for now */
    PV_WREG(p_xv, MStart.l, 0);	    /* Set up offsetting hardware. */
    PV_WREG(p_xv, MEnd.l, 0x1f);	/* ... */
    PV_WREG(p_xv, MOffset.l, 0);	/* ... */
	PV_WREG(p_xv, WindowX.l, 0);	/* set vector window origin. */
	PV_WREG(p_xv, WindowY.l, 0);	/* ... */

    plxv_ctl_1(p_xv, 0, 32);		/* write all bits */

	/*  INITIALIZE GRAPHICS CONTEXT AND PLACE IT IN THE HARDWARE. */

    gc.color1_bg      	  = 0;
    gc.color1_fg      	  = 0;
    gc.f_transparency     = 0;
    gc.readback           = 0;  /* red */
    gc.rop           	  = PLXV_ROP_COPY;
    gc.stencil_mask  	  = 0;
    gc.stencil_value 	  = 0;
    gc.wid_mask      	  = PLXV_WMASK_PIXEL_ALL;
    gc.wid_value     	  = 0;
    gc.write_mask    	  = 0;
	p_me->force_gc = 1;
	m_Draw_set_gc(p_me, &gc);
	p_me->force_gc = 0;

	DEBUGF(0x40, (fd, "m_Draw_init_hw: exit\n"));
}


/*---------------------------------------------------------------------------*
 |"m_Draw_set_gc"
 |
 |  Set the current drawing context.
 |
 |    = 1 on success, 0 on error.
 *---------------------------------------------------------------------------*/
static int m_Draw_set_gc( p_me, p_gc )
	I_Draw* p_me;	/* -> context for this object. */
	O_Draw_Gc*	 p_gc;	/* -> where to return gc information. */
{
    XV_struct*	p_xv;		/* Fast access to h/w context: p_me->p_xv. */
	int			set_rwm;	/* = 1 need to set read-write mode. */


	DEBUGF(0x40, (fd, "m_Draw_set_gc: entry\n"));

    m_Draw_wait_for_done(p_me);
	p_xv = p_me->p_xv; 
	set_rwm = 0;

	/*  UPDATE ANY REGISTER VALUES WHICH HAVE CHANGED. */

    if ( (p_gc->color != p_me->gc.color) || p_me->force_gc ) 
	{
		PV_WREG(p_xv, SolidCol.l, p_gc->color);
	}
    if ( (p_gc->color1_bg != p_me->gc.color1_bg) || p_me->force_gc ) 
	{
		PV_WREG(p_xv, M1BakCol.l, p_gc->color1_bg);
	}
    if ( (p_gc->color1_fg != p_me->gc.color1_fg) || p_me->force_gc ) 
	{
		PV_WREG(p_xv, M1ForCol.l, p_gc->color1_fg);
	}
    if ( (p_gc->f_transparency != p_me->gc.f_transparency) || p_me->force_gc )
	{
		PV_WREG(p_xv, MMode.b.trans, p_gc->f_transparency);
	}
    if ( (p_gc->readback != p_me->gc.readback) || p_me->force_gc )
	{
#if GA_REV==GAREV_1
    	PV_WREG(p_xv, TagCtl.b.m8rb, p_gc->readback);
#else
    	PV_WREG(p_xv, MMode.b.m8rb, (p_gc->readback & 0x3));
#endif
	}
    if ( (p_gc->rop != p_me->gc.rop) || p_me->force_gc )
	{
		set_rwm = 1;
		PV_WREG4(p_xv, DrawRop.l, p_gc->rop);
	}
    if ( (p_gc->stencil_mask != p_me->gc.stencil_mask) || p_me->force_gc )
	{
		set_rwm = 1;
		PV_WREG(p_xv, TagClip.b.mask, p_gc->stencil_mask);
	}
    if ( (p_gc->stencil_value != p_me->gc.stencil_value) || p_me->force_gc )
	{
		PV_WREG(p_xv, TagClip.b.comp, p_gc->stencil_value);
	}
    if ( (p_gc->wid_mask != p_me->gc.wid_mask) || p_me->force_gc )
	{
		PV_WREG(p_xv, TagCtl.b.ovr_mux, p_gc->wid_mask);
	}
    if ( (p_gc->wid_value != p_me->gc.wid_value) || p_me->force_gc )
	{
		PV_WREG(p_xv, TagCtl.b.ovr_val, p_gc->wid_value);
	}
    if ( (p_gc->write_mask != p_me->gc.write_mask) || p_me->force_gc )
	{
		set_rwm = 1;
		PV_WREG(p_xv, DrawMask.l, p_gc->write_mask);
	}


	/*  COPY NEW VALUES TO OUR CONTEXT AND EXIT, THEN SET THE READ-WRITE
     *  MODE BIT IF THE USUAL SUSPECTS CHANGED. (Order is important!)
     */
	memcpy(&p_me->gc, p_gc, sizeof(O_Draw_Gc));
	if ( set_rwm ) i_Draw_set_rwm( p_me );

	DEBUGF(0x40, (fd, "m_Draw_set_gc: exit\n"));
	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Draw_terminate"
 |
 |  Terminate use of this object.
 *---------------------------------------------------------------------------*/
static void m_Draw_terminate( p_me )
	I_Draw*	p_me;		/* -> context for this object. */
{
	DEBUGF(0xC0, (fd, "m_Draw_terminate: entry\n"));
	free((char*)p_me);
	DEBUGF(0xC0, (fd, "m_Draw_terminate: exit\n"));
}


/*---------------------------------------------------------------------------*
 |"m_Draw_vector"
 |
 |  Draw a line using the vector hardware, based on the current
 |  graphics context. The line is specified by a corner and width and 
 |  direction vectors.
 |
 |    = 1 on success, 0 on error.
 *---------------------------------------------------------------------------*/
static int m_Draw_vector( p_me, x_bgn, y_bgn, x_len, y_len )
	I_Draw* p_me;	/* -> context for this object. */
	int		x_bgn;	/* =  x position of start of vector. */
	int		y_bgn;	/* =  y position of start of vector. */
	int		x_len;	/* =  x position of end of vector. */
	int		y_len;	/* =  y position of end of vector. */
{
    XV_struct*	p_xv;	/* Fast access to h/w context: p_me->p_xv. */

	DEBUGF(0x40, (fd, "m_Draw_vector(%x, %d, %d, %d): entry\n",
		p_me, x_bgn, y_bgn, x_len, y_len));

	p_xv = p_me->p_xv; 
    m_Draw_wait_for_done(p_me);
    PV_WREG(p_xv, SolidCol.l, p_me->gc.color);
    PV_WREG(p_xv, VectX0.l, x_bgn);
    PV_WREG(p_xv, VectY0.l, y_bgn);
#if 1
    Vector_Done(p_xv);
    PV_WREG(p_xv, DstW.l, x_len);
    PV_WREG(p_xv, VectDY.l, y_len);
#if 0
    Vector_Done(p_xv);
#endif
#else
    PV_WREG(p_xv, VectX1.l, x_len);
    PV_WREG(p_xv, VectY1.l, y_len);
#endif
    m_Draw_wait_for_done(p_me);
	DEBUGF(0x40, (fd, "m_Draw_vector exit\n"));
	return 1;
}

/*---------------------------------------------------------------------------*
 |"m_Draw_vector_uncook"
 |
 |  Draw a line using the vector hardware, based on the current
 |  graphics context. The line is specified by a corner, width and 
 |  direction vectors and breshingham parms.
 |
 |    = 1 on success, 0 on error.
 *---------------------------------------------------------------------------*/

static int m_Draw_vector_uncook(p_me,x0,y0,num_pix,i1,i2,e,left_flag,up_flag,
				x_major_flag)
I_Draw* p_me;	/* -> context for this object. */
int x0,y0,num_pix,i1,i2,e,left_flag,up_flag,x_major_flag;
{
    XV_struct*	p_xv;	/* Fast access to h/w context: p_me->p_xv. */
    int i,addr, temp;
    /* all colors are 28 bit (include tag data */
    /* how to vector:
	load rop,stencil, plane mask, etc
	set color
	load start address
	load i1, i2, e
	set number of pixels, flags, and major_axis
	wait for done
    */

/* preceed uncooked vectors with a 'null' cooked vector to prime mode */

	p_xv = p_me->p_xv; 

    O_Draw_wait_for_done(p_me);

    addr = PLXV_ADDR_REG(x0,y0);
    PV_WREG(p_xv, SolidCol.l, p_me->gc.color);


#if GA_REV == GAREV_1
	PV_WREG(p_xv, DrawMask.l, 0);	/* so we don't draw anything */
    p_xv->xv_wreg->DstStartAdd.l = addr;
    p_xv->xv_wreg->DstW.l = 0;
    p_xv->xv_wreg->VecH.l = 1;		/* NOTE must not be horizontal */
    O_Draw_wait_for_done(p_me);
	PV_WREG(p_xv, DrawMask.l, p_me->gc.write_mask);
#endif


/* now do the real vector */

#if 0
    PV_WREG(p_xv, VecI1.l, i1);
    PV_WREG(p_xv, VecI2.l, i2);
    PV_WREG(p_xv, VecAcc.l, e);
#else
    p_xv->xv_wreg->VecI1.l = i1;              /* now do the real vector */
    p_xv->xv_wreg->VecI2.l = i2;
    p_xv->xv_wreg->VecAcc.l = e;
#endif

    PV_WREG(p_xv, DstStartAdd.l, addr);

    i  = ((x_major_flag & 1) << 15) | ((up_flag  & 1) << 14);
    i |= ((left_flag & 1) <<13) | (num_pix & 0xfff);

#if 0
    PV_WREG(p_xv, VecMsc.l, i);
#else
    p_xv->xv_wreg->VecMsc.l = i;
#endif
    m_Draw_wait_for_done(p_me);
    return 1;
}

/*---------------------------------------------------------------------------*
 |"m_Draw_wait_for_done"
 |
 |  We can't use access maps while accelerator busy, so this method waits for 
 |  the accellerator to be done, and the fifo's empty. This method must be 
 |  called before any mode control changes, and after accellerator operations.
 *---------------------------------------------------------------------------*/
static void m_Draw_wait_for_done( p_me )
	I_Draw* p_me;	/* -> context for this object. */
{
    int 		loop;	/* Loop control. */
    XV_struct*	p_xv;	/* Fast access to h/w context: p_me->p_xv. */
 
	p_xv = p_me->p_xv; 

	for ( loop = 1; loop; )
    {
    	loop = (volatile int) p_xv->xv_rreg->DBusy.l;
    }
}


static TV_Draw methods_Draw = {
	m_Draw_copy_area,
	m_Draw_dump,
	m_Draw_fill_box,
	m_Draw_get_gc,
	m_Draw_init_hw,
	m_Draw_set_gc,
	m_Draw_terminate,
	m_Draw_vector,
	m_Draw_wait_for_done,
	m_Draw_vector_uncook
};


/*---------------------------------------------------------------------------*
 |"O_Draw_O_Draw"
 |
 |  Create an instance of a Video Stream I drawing object.
 |
 |    -> object's context on success, or 0 if error occurred.
 *---------------------------------------------------------------------------*/
O_Draw* O_Draw_O_Draw( p_me, p_xv )
	I_Draw* 	p_me;	/* -> context to fill in, or 0 for dynamic alloc. */
	XV_struct*	p_xv;	/* -> context for hardware library. */
{
	DEBUGF(0x80, (fd, "O_Draw_O_Draw(%x,%x): entry\n", p_me, p_xv));

	if ( !p_me )
	{
		if ( !(p_me = (I_Draw*)malloc(sizeof(I_Draw))) )
		{
			ERRORF((fd, "O_Draw_O_Draw: Unable to allocate storage\n"));
		}
	}
	p_me->atv = &methods_Draw;
	p_me->p_xv = p_xv;
	
	DEBUGF(0x80, (fd, "O_Draw_O_Draw: exit with object @ %x\n", p_me));
	return (O_Draw*)p_me;
}

#if 0
void plxv_transp_on(p_me)
XV_struct *p_me;
{

    ((I_Draw *)(p_me->p_draw))->gc.f_transparency = 1;
    PV_WREG(p_me, MMode.b.trans, ((I_Draw *)(p_me->p_draw))->gc.f_transparency);
}

void plxv_transp_off(p_me)
XV_struct *p_me;
{
    ((I_Draw *)(p_me->p_draw))->gc.f_transparency = 0;
    PV_WREG(p_me, MMode.b.trans, ((I_Draw *)(p_me->p_draw))->gc.f_transparency);
}
#endif
