/*
 * (c) Copyright 1994 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 *  10 Nov 1994
 *    - rewrite of original i2clib.c as an access object.
 *    - changed delay marcros into calls to m_I2C_delay which utilizes the
 *      timer on the parallax board.
 *
 */
#include <stdio.h>
#include "video_stream.h"
#include "hw_context.h"
/*  XV_DIAG FLAG MEANINGS:
 *
 *    0x20 - I2C acknowledge errors.
 */
/* #define XV_DIAG 0x20 */
#include "xvdebug.h"


/*
 *  FORWARD REFERENCES:
 */
u_int        i_I2C_get_timer();
static int 	 m_I2C_acknowledge();
static void  m_I2C_delay();
static void  m_I2C_end_of_msg();
static int   m_I2C_ping();
static int   m_I2C_read_data();
static int   m_I2C_read_no_sub();
static int   m_I2C_set_address();
static void  m_I2C_slave_address();
static void  m_I2C_start_of_msg();
static void  m_I2C_sub_address();
static int   m_I2C_write();
static void  m_I2C_write_data();
static int   m_I2C_write_indexed();
static int   m_I2C_write_indexed3();

static char nv_address[3] = {0x50,0x51,0x52};

/*
 *  IMPLEMENTATION VERSION OF O_I2C:
 */
typedef struct _I_I2C { /* Implementation version of I2C object. */
    TV_I2C*     atv;    /* Methods transfer vector. */
    XV_struct*  p_xv;   /* Hardware context for board. */
} I_I2C;

#define mm(x) p_me->xv_wreg->MiscMode.l = x

u_int i_I2C_get_timer(p_me)
    I_I2C* p_me;
{
    int timer_value;

    PV_RREG(p_me->p_xv, timer_value, TmrCnt.l);
    return timer_value;
}


/*---------------------------------------------------------------------------*
 |"m_I2C_acknowledge"
 |
 |  Read back the acknowledgement for the last operation.
 |  The context parameter is used to tell this routine what is trying to be 
 |  acknowledged so that a more informative error message can be generated
 |
 |       context         meaning
 |       -------         -------
 |        1               primary-address
 |        2               sub-address
 |        3               data-write
 |		  4               poll - don't report errors.
 |
 |	  = 0 on success, = value of context on error.
 *---------------------------------------------------------------------------*/
static int m_I2C_acknowledge( p_me, i2c_addr, context ) /* Formerly i2cack. */
    I_I2C* p_me;		/* -> context for this object. */
	int	   i2c_addr;	/* =  address operation performed on. */
	int    context;     /* =  acknowledgment context (see comments above.) */
{
	static char* errors[] = /* Error messages: */
	{
		"i2c error writing slave address",
		"i2c error writing sub-address",
		"i2c_error writing data"
	};
	int			 i_poll;	/* # of time around the polling loop. */
	XV_struct*	 p_xv;		/* Fast access to p_me->p_xv, our board context. */
    int 		 readback;  /* Acknowledgement value. */

	p_xv = p_me->p_xv;

    PV_WREG(p_xv, I2C.b.enbl, 0);      /* Set the data direction to read   */
    m_I2C_delay(p_me, 15);                    
    PV_WREG(p_xv, I2C.b.clk, 1);       /* Low-to-high SCL transition       */

	for ( i_poll = 0; i_poll < 10; i_poll++ )
	{
		m_I2C_delay(p_me, 5);         /* Wait for SCL Low Time=5us         */
		PV_RREG(p_xv, readback, I2CStat.b.data);   
		if ((readback & 1) == 0) break;
/*		m_I2C_delay(p_me, 10);         /* Wait for SCL Low Time=5us         */
	}
	if ( ((readback & 1) == 1) && context != 4 )
	{
		DEBUGF(0x20, (fd, "%s: device is 0x%02x status is 0x%02x\n", 
			errors[context-1], i2c_addr, readback));
	}

    PV_WREG(p_xv,I2C.b.clk,0);          /* High-to-low SCL transition       */
    m_I2C_delay(p_me, 5);
    m_I2C_delay(p_me, 5);               /* Wait for SCL Low Time=5us        */

    return ( (readback & 1) == 1) ? context : 0;
}

 
/*---------------------------------------------------------------------------*
 |"m_I2C_delay"
 |
 |  Delay for a specified number of microseconds. This method uses the 
 |  on-board timer of the Parallax board, so the minimum delay interval
 |  is 10 microseconds. We also wait one additional tick to be sure we get
 |  a full 10 microseconds for the first tick. This means the accuracy of 
 |  the delay could be off by as much as 19 microseconds for a 1 microsecond
 |  delay.
 *---------------------------------------------------------------------------*/
static void m_I2C_delay(p_me, usec)		/* Formerly plxv_nv_delay. */
    I_I2C*  p_me;		/* -> context for this object. */
	int 	usec;		/* =  amount of time to delay in microseconds. */
{
    int   t_delay;   /* Amount of time to delay. */
    int   t_delta;   /* Difference between now and sampling start. */
    u_int t_start;   /* Value of timer when we started sampling. */

    t_delay = (usec + 19)/10;
    PV_RREG(p_me->p_xv, t_start, TmrCnt.l);
    for (;;)
    {
        t_delta = i_I2C_get_timer(p_me) - t_start;
        if ( t_delta < 0 ) 					/* Wrapped. */
			t_start = 0;
		else if ( t_delta >= t_delay )
            return;
    }
}


/*---------------------------------------------------------------------------*
 |"m_I2C_end_of_msg"
 |
 |  Send an end of message sequence to I2C bus.
 |
 *---------------------------------------------------------------------------*/
static void m_I2C_end_of_msg( p_me ) 	/* Formerly i2ceom. */
    I_I2C*  p_me;		/* -> context for this object. */
{
	XV_struct*	p_xv;	/* Fast access to p_me->p_xv, our board context. */

	p_xv = p_me->p_xv;

    PV_WREG(p_xv, I2C.b.enbl, 1); /* enable output */ 
    m_I2C_delay(p_me, 10);    
    PV_WREG(p_xv, I2C.b.data,0);
    m_I2C_delay(p_me, 1);     
    PV_WREG(p_xv, I2C.b.clk,1);
    m_I2C_delay(p_me, 5);     
    PV_WREG(p_xv, I2C.b.data,1);  /* set data high */
    m_I2C_delay(p_me, 5);         /* Wait for SCL Low Time=5us    */
}


/*---------------------------------------------------------------------------*
 |"m_I2C_nv_read"
 |
 |  Read an area of nvram. The value of unit indicates which nvram device:
 |       0 - main board
 |       1 - option card
 |       2 - tuner box
 |
 |    = 0 if success,  non-zero if error
 *---------------------------------------------------------------------------*/
m_I2C_nv_read(p_me, unit, p_buf, nv_addr, count)
    I_I2C*  p_me;		/* -> context for this object. */
    int     unit;		/* =  # of nvram device to be read. */
    char*   p_buf;		/* -> where to place data read. */
    int     nv_addr;	/* =  offset into nvram to start read at. */
    int     count;		/* =  # of bytes of nvram to read. */
{
	int   data;		/* Data byte read. */
    int   i_byte;	/* # of byte (0 to count-1) now reading. */
	int   paddr;	/* Address of nvram device. */
    char* p_byte;	/* Where to place next byte read. */
	int   rc;		/* Return code from method calls. */

    paddr = nv_address[unit];
    p_byte = p_buf;
    for (i_byte = 0; i_byte < count; i_byte++)
    {
        rc = m_I2C_read_data(p_me, paddr, nv_addr++, &data);
        *p_byte++ = data;
        if (rc) 
        {
           ERRORF((fd, "m_I2C_nv_read: rc = %d\n", rc));
           return(rc);
        }
    }
    return(0);
}


/*---------------------------------------------------------------------------*
 |"m_I2C_nv_write"
 |
 |  Write to an area of nvram. The value of unit indicates the nvram device:
 |       0 - main board
 |       1 - option card
 |       2 - tuner box
 |
 |    = 0 if success,  non-zero if error
 *---------------------------------------------------------------------------*/
m_I2C_nv_write(p_me, unit, p_buf, nv_addr, count)
    I_I2C*  p_me;		/* -> context for this object. */
    int     unit;		/* =  # of nvram device to be written. */
    char*   p_buf;		/* -> where to get data to write. */
    int     nv_addr;	/* =  offset into nvram to start write at. */
    int     count;		/* =  # of bytes of nvram to write. */
{
    int   i_byte;	/* # of byte (0 to count-1) now reading. */
	int   paddr;	/* Address of nvram device. */
    char* p_byte;	/* Where to place next byte read. */
	int   readback;	/* Status register for I2C. */
	int   rc;		/* Return code from method calls. */

    paddr = nv_address[unit];
    p_byte = p_buf;
    for (i_byte =0; i_byte < count; i_byte++)
    {
        rc = m_I2C_write(p_me, paddr, nv_addr++, *p_byte++);
        if (rc) return(rc);
		m_I2C_delay(p_me, 10000); 
    }
    return(0);
}


/*---------------------------------------------------------------------------*
 |"m_I2C_ping"
 |
 |  Verify an I2C bus device is present. We send a start of message, slave 
 |  address and sub-address over I2C bus. Then terminate with an end of
 |  message.
 |
 |    = 0 on success, = 1 if no address ack, = 2 if no sub-address ack.
 *---------------------------------------------------------------------------*/
static int m_I2C_ping( p_me, addr, saddr )
    I_I2C*  p_me;       /* -> context for this object. */
    int     addr;       /* =  slave address of device. */
    int     saddr;      /* =  sub-address on device. */
{
    int         ack;    /* Return code from m_I2C_acknowledge. */

    m_I2C_start_of_msg(p_me);
    m_I2C_slave_address(p_me, addr, 0);     /* 0 indicates we are writing. */
    if ( ack = m_I2C_acknowledge(p_me, addr, 4) )
    {
        m_I2C_end_of_msg(p_me);
        return 1;
    }
    m_I2C_sub_address(p_me, saddr);
    if ( ack = m_I2C_acknowledge(p_me, addr, 4) )
    {
        m_I2C_end_of_msg(p_me);
        return 2;
    }
    return 0;
}


/*---------------------------------------------------------------------------*
 |"m_I2C_read_data"
 |
 |  Read a data byte from an I2C bus device. 
 |
 |  Notes:
 |    (1) We do this by sending the slave and sub addresses for the 
 |        device, then send a second start of message and read the data byte.
 |
 |    (2) i2c devices with more that 1 readable sub-address require a write 
 |        cycle to pre-set the sub-address pointer prior to reading a data 
 |        byte.
 |
 |    = 0 if success, -1 if error
 *---------------------------------------------------------------------------*/
static int m_I2C_read_data( p_me, addr, saddr, p_data ) /* Formerly i2cr. */
    I_I2C*  p_me;		/* -> context for this object. */
	int		addr;		/* =  slave address for the device to be read. */
	int		saddr;		/* =  sub address on the device to be read. */
	int*	p_data;		/* -> where to place data byte read. */
{
	int			byte;		/* Data byte read assembling. */
    int 		i_bit;		/* = # of bit of sub-address sending. */
	XV_struct*	p_xv;		/* Fast access to p_me->p_xv, our board context. */
	int			reg_data;	/* Data register value. */

	p_xv = p_me->p_xv;

	/* SELECT THE DEVICE: */

    m_I2C_set_address(p_me, addr,saddr);
    m_I2C_start_of_msg(p_me);
	m_I2C_slave_address(p_me, addr, 1);	/* 1 indicates we are reading. */
	if ( m_I2C_acknowledge(p_me, addr, 1) )
    {
        m_I2C_end_of_msg(p_me);
        return -1;  				/* was 0 */
    }

	/*  READ BACK THE DATA: */
    
    PV_WREG(p_xv, I2C.b.enbl, 0);       /* Set the data direction for read */
    m_I2C_delay(p_me, 5);             	/* Wait for Tsu (SDA>SCL)=250ns */
    byte=0;
    for ( i_bit = 0; i_bit < 8; i_bit++ ) 
    {
        byte <<= 1;                		/* this does nothing on first iter */
        PV_WREG(p_xv, I2C.b.clk,1);     /* Low-to-high SCL transition   */
        m_I2C_delay(p_me, 5);         	/* Let the readback data settle */
        PV_RREG(p_xv, reg_data, I2CStat.b.data);        /* Jon */
        byte |= (reg_data & 1);
        PV_WREG(p_xv, I2C.b.clk,0);      
        m_I2C_delay(p_me, 5);         /* Wait for SCL Low Time=5us    */
    }
    m_I2C_end_of_msg(p_me);
    *p_data = byte;
    return 0; 
}


/*---------------------------------------------------------------------------*
 |"m_I2C_read_no_sub"
 |
 |  Read a data byte from an I2C bus device without sending a sub-address. 
 |  Use this method when a device does not require a sub-address on read
 |  or if the sub-address pointer has been set previously.
 |  We do this by sending the slave address and then reading the data byte.
 |
 |    = 0 if success, -1 if error
 *---------------------------------------------------------------------------*/
static int m_I2C_read_no_sub( p_me, addr, p_data ) /* Formerly i2crdir. */
    I_I2C*  p_me;		/* -> context for this object. */
	int		addr;		/* =  slave address for the device to be read. */
	int*	p_data;		/* -> where to place data byte read. */
{
	int			byte;		/* Data byte read assembling. */
    int 		i_bit;		/* = # of bit of sub-address sending. */
	XV_struct*	p_xv;		/* Fast access to p_me->p_xv, our board context. */
	int			reg_data;	/* Data register value. */

	p_xv = p_me->p_xv;

	/*  ADDRESS SETUP: */

    m_I2C_start_of_msg(p_me);
	m_I2C_slave_address(p_me, addr, 1);	/* 1 indicates read. */
	if ( m_I2C_acknowledge(p_me, addr, 1) )
    {
        m_I2C_end_of_msg(p_me);
        return -1;  
    }


	/* READBACK OF DATA BYTE: */
    
    PV_WREG(p_xv, I2C.b.enbl, 0);       /* Set the data direction for read */
    m_I2C_delay(p_me, 10);            	/* Wait for Tsu (SDA>SCL)=250ns */
    byte = 0;
    for ( i_bit =0; i_bit < 8 ;i_bit++ ) 
    {
        byte <<= 1;                		/* this does nothing on first iter */
        PV_WREG(p_xv, I2C.b.clk,1);     /* Low-to-high SCL transition   */
        m_I2C_delay(p_me, 5);           /* Let the readback data settle */
        PV_RREG(p_xv, reg_data, I2CStat.b.data); 
		if (i_bit == 7) 
		{
			PV_WREG(p_xv, I2C.b.enbl,0);
		}
        byte |= (reg_data & 1);
        PV_WREG(p_xv, I2C.b.clk,0);      
        m_I2C_delay(p_me, 5);          /* Wait for SCL Low Time=5us    */
    }
	m_I2C_end_of_msg(p_me);
    *p_data = byte;
    return 0; 
}


/*---------------------------------------------------------------------------*
 |"m_I2C_set_address"
 |
 |  Select an I2C bus device and sub-address on that device. We send a start
 |  of message, slave address and sub-address over I2C bus.
 |
 |    = 0 on success, = 1 if no address ack, = 2 if no sub-address ack.
 *---------------------------------------------------------------------------*/
static int m_I2C_set_address( p_me, addr, saddr ) 	/* Formerly i2_set_addr. */
    I_I2C*  p_me;		/* -> context for this object. */
	int		addr;		/* =  slave address of device. */
	int		saddr;		/* =  sub-address on device. */
{
    int 		ack;	/* Return code from m_I2C_acknowledge. */

    m_I2C_start_of_msg(p_me);
	m_I2C_slave_address(p_me, addr, 0);		/* 0 indicates we are writing. */
	if ( ack = m_I2C_acknowledge(p_me, addr, 1) )
    {
        m_I2C_end_of_msg(p_me);
        return ack;
    }
    m_I2C_sub_address(p_me, saddr);
	if ( ack = m_I2C_acknowledge(p_me, addr, 2) )
    {
        m_I2C_end_of_msg(p_me);
        return ack;
    }
    return 0;
}


/*---------------------------------------------------------------------------*
 |"m_I2C_slave_address"
 |
 |  Send a slave byte to currently addressed device on I2C bus.
 |
 *---------------------------------------------------------------------------*/
static void m_I2C_slave_address(p_me, addr, read_flag)	/* Formerly i2cpa. */
    I_I2C* p_me;		/* -> context for this object. */
	int	   addr;		/* =  slave address to be sent. */
	int	   read_flag;	/* =  1 if this is for a read operation. */
{
	/* address is a 7-bit value */

	int 		i_bit;	/* # of bit in address now sending. */
	XV_struct*	p_xv;	/* Fast access to p_me->p_xv, our board context. */

	p_xv = p_me->p_xv;

	for (i_bit = 0; i_bit < 7; i_bit++) 
	{
		PV_WREG(p_xv, I2C.b.data, (addr & 0x40)>>6);  /* MSB */
		m_I2C_delay(p_me, 1);     	/* Wait for Tsu (SDA>SCL)=250ns */
		PV_WREG(p_xv, I2C.b.clk,1); /* Low-to-high SCL transition   */
		m_I2C_delay(p_me, 5);     	/* SCL high time = 5us          */
		PV_WREG(p_xv, I2C.b.clk,0); /* High-to-low SCL transition   */
		m_I2C_delay(p_me, 5);     	/* Wait for Th (SCL<SDA)=5us    */
		addr <<= 1;
	}
	PV_WREG(p_xv, I2C.b.data, read_flag);    
	m_I2C_delay(p_me, 1);
	PV_WREG(p_xv,I2C.b.clk, 1);
	m_I2C_delay(p_me, 5);
	PV_WREG(p_xv,I2C.b.clk, 0);
	PV_WREG(p_xv,I2C.b.enbl,0);
	m_I2C_delay(p_me, 5);
}


/*---------------------------------------------------------------------------*
 |"m_I2C_start_of_msg"
 |
 |  Send out a start of message sequence over the I2C bus.
 |
 *---------------------------------------------------------------------------*/
static void m_I2C_start_of_msg( p_me ) 	/* Formerly i2csom. */
    I_I2C* p_me;		/* -> context for this object. */
{
	XV_struct*	p_xv;	/* Fast access to p_me->p_xv (board context). */

	p_xv = p_me->p_xv;
#ifdef BITI
	PV_WREG(p_xv,I2C.l, 0x07);	/* Set cen: 0 data: 1 enbl:1 clk:1 */
#else
	PV_WREG(p_xv,I2C.l,0xe0000000); /* Set clk: 1 enbl: 1 data:1 cen:0 */
#endif
	m_I2C_delay(p_me, 5);           /* SCL high time = 5us           */
	PV_WREG(p_xv,I2C.b.data, 0);    /* High-to-low transition on SDA */
	m_I2C_delay(p_me, 5);           /* Wait for Tsu (SCL>SDA)= 5us   */
	PV_WREG(p_xv,I2C.b.clk, 0);     /* High-to-low transition on SCL */
	m_I2C_delay(p_me, 4);           /* Wait for Th (SCL<SDA) = 4us   */
}


/*---------------------------------------------------------------------------*
 |"m_I2C_sub_address"
 |
 |  Send an 8-bit subaddress for an I2C bus device.
 |
 *---------------------------------------------------------------------------*/
static void m_I2C_sub_address( p_me, saddr ) 	/* Formerly i2csa. */
    I_I2C*  p_me;		/* -> context for this object. */
	int		saddr;		/* =  subaddress to be sent. */
{
    int 		i_bit;	/* = # of bit of sub-address sending. */
	XV_struct*	p_xv;	/* Fast access to p_me->p_xv, our board context. */

	p_xv = p_me->p_xv;

	PV_WREG(p_xv, I2C.b.enbl, 1);
    for ( i_bit = 0;i_bit < 8;i_bit++ )
    {
        PV_WREG(p_xv,I2C.b.data, (saddr & 0x80)>>7);  /* MSB            */
        m_I2C_delay(p_me, 1); 		/* Wait for Tsu (SDA>SCL)=250ns */
        PV_WREG(p_xv,I2C.b.clk,1); 	/* Low-to-high SCL transition   */
        m_I2C_delay(p_me, 5);       /* SCL high time = 5us          */
        PV_WREG(p_xv,I2C.b.clk,0);  /* High-to-low SCL transition   */
		if (i_bit == 7) 
		{
			PV_WREG(p_xv,I2C.b.enbl,0);
		}
        m_I2C_delay(p_me, 5); 		/* Wait for Th (SCL<SDA)=5us    */
        saddr<<=1;
    }
}


/*---------------------------------------------------------------------------*
 |"m_I2C_terminate"
 |
 |  Destructor for this object: free its dynamically allocated storage.
 |
 *---------------------------------------------------------------------------*/
static void m_I2C_terminate( p_me )
    I_I2C*  p_me;       /* -> object context. */
{
    free((char*)p_me);
}
 

/*---------------------------------------------------------------------------*
 |"m_I2C_write"
 |
 |  Send a slave address and sub-address then write a data byte to the 
 |  selected I2C bus device.
 |
 |    = 0 on success, error code on failure (see m_I2C_acknowledge).
 *---------------------------------------------------------------------------*/
static int m_I2C_write( p_me, addr, saddr, data ) 	/* Formerly i2cw. */
    I_I2C*  p_me;		/* -> context for this object. */
	int		addr;		/* =  slave address of device to write byte to. */
	int		saddr;		/* =  sub-address of device to write byte to. */
	int		data;		/* =  data byte to be written. */
{
	XV_struct*	p_xv;	/* Fast access to p_me->p_xv, our board context. */
	int			rc;		/* Return code from routines we call. */

	p_xv = p_me->p_xv;

    if ( (rc = m_I2C_set_address(p_me, addr,saddr)) ) return rc;
    m_I2C_write_data(p_me, data);
    rc = m_I2C_acknowledge(p_me, addr, 3);
    m_I2C_end_of_msg(p_me);
    return rc;
}


/*---------------------------------------------------------------------------*
 |"m_I2C_write_data"
 |
 |  Write a data byte to an I2C bus device.
 |
 *---------------------------------------------------------------------------*/
static void m_I2C_write_data( p_me, data ) 	/* Formerly i2cwd. */
    I_I2C*  p_me;		/* -> context for this object. */
	int		data;		/* =  data byte to be written. */
{
    int 		i_bit;	/* = # of bit of sub-address sending. */
	XV_struct*	p_xv;	/* Fast access to p_me->p_xv, our board context. */

	p_xv = p_me->p_xv;

	PV_WREG(p_xv, I2C.b.enbl, 1);
    for ( i_bit = 0 ;i_bit < 8; i_bit++ )  
    {
        PV_WREG(p_xv, I2C.b.data, (data & 0x80)>>7);  /* MSB             */
        m_I2C_delay(p_me, 1); 		 /* Wait for Tsu (SDA>SCL)=250ns */
        PV_WREG(p_xv, I2C.b.clk, 1); /* Low-to-high SCL transition   */
        m_I2C_delay(p_me, 5); 		 /* SCL high time = 5us          */
        PV_WREG(p_xv, I2C.b.clk, 0); /* High-to-low SCL transition   */
#ifdef NOTDEF
		if (i_bit == 7) 
		{
			PV_WREG(p_xv, I2C.b.enbl, 0);
		}
#endif
        m_I2C_delay(p_me, 5); 		 /* Wait for Th (SCL<SDA)=5us    */
        data<<=1;
    }
}


/*---------------------------------------------------------------------------*
 |"m_I2C_write_indexed"
 |
 |  Send a slave address, sub-address and index, then write a data byte to 
 |  the selected I2C bus device's register.
 |
 |    = 0 on success, error code on failure (see m_I2C_acknowledge).
 *---------------------------------------------------------------------------*/
static int m_I2C_write_indexed( p_me, addr, saddr, index, data ) 
    I_I2C*  p_me;		/* -> context for this object. */
	int		addr;		/* =  slave address of device to write byte to. */
	int		saddr;		/* =  sub-address of device to write byte to. */
	int		index;		/* =  # of register at sub-address to write byte to. */
	int		data;		/* =  data byte to be written. */
{
	XV_struct*	p_xv;	/* Fast access to p_me->p_xv, our board context. */
	int			rc;		/* Return code from routines we call. */

	p_xv = p_me->p_xv;

    if ( (rc = m_I2C_set_address(p_me, addr,saddr)) ) return rc;

    m_I2C_write_data(p_me, index);
    rc = m_I2C_acknowledge(p_me, addr, 3);

    m_I2C_write_data(p_me, data);
    rc = m_I2C_acknowledge(p_me, addr, 3);

    m_I2C_end_of_msg(p_me);
    return rc;
}


/*---------------------------------------------------------------------------*
 |"m_I2C_write_indexed3"
 |
 |  Send a slave address, sub-address and index, then write 3 data bytes to 
 |  the selected I2C bus device's register.
 |
 |    = 0 on success, error code on failure (see m_I2C_acknowledge).
 *---------------------------------------------------------------------------*/
static int m_I2C_write_indexed3( p_me, addr, saddr, index, data1, data2, data3 )
    I_I2C*  p_me;		/* -> context for this object. */
	int		addr;		/* =  slave address of device to write byte to. */
	int		saddr;		/* =  sub-address of device to write byte to. */
	int		index;		/* =  # of register at sub-address to write byte to. */
	int		data1;		/* =  first data byte to be written. */
	int		data2;		/* =  second data byte to be written. */
	int		data3;		/* =  third data byte to be written. */
{
	XV_struct*	p_xv;	/* Fast access to p_me->p_xv, our board context. */
	int			rc;		/* Return code from routines we call. */

	p_xv = p_me->p_xv;

    if ( (rc = m_I2C_set_address(p_me, addr,saddr)) ) return rc;

    m_I2C_write_data(p_me, index);
    rc = m_I2C_acknowledge(p_me, addr, 3);

    m_I2C_write_data(p_me, data1);
    rc = m_I2C_acknowledge(p_me, addr, 3);

    m_I2C_write_data(p_me, data2);
    rc = m_I2C_acknowledge(p_me, addr, 3);

    m_I2C_write_data(p_me, data3);
    rc = m_I2C_acknowledge(p_me, addr, 3);

    m_I2C_end_of_msg(p_me);
    return rc;
}


static TV_I2C methods_tv = {
	m_I2C_acknowledge,
	m_I2C_delay,
	m_I2C_end_of_msg,
	m_I2C_nv_read,
	m_I2C_nv_write,
	m_I2C_ping,
	m_I2C_read_data,
	m_I2C_read_no_sub,
	m_I2C_set_address,
	m_I2C_slave_address,
	m_I2C_start_of_msg,
	m_I2C_sub_address,
	m_I2C_terminate,
	m_I2C_write,
	m_I2C_write_data,
	m_I2C_write_indexed,
	m_I2C_write_indexed3
};


/*---------------------------------------------------------------------------*
 |"O_I2C_O_I2C"
 |
 |  Create and return an I2C bus access object.
 |
 |    -> object on success, 0 on failure
 *---------------------------------------------------------------------------*/
O_I2C* O_I2C_O_I2C(p_xv)
	XV_struct*	p_xv;		/* -> Parallax access object. */
{
    I_I2C*  p_me;		/* -> context for this object. */

	if ( !(p_me = (I_I2C*)malloc(sizeof(I_I2C))) )
	{
		ERRORF((fd, "O_I2C_O_I2C: Unable to allocate storage for O_I2C\n"));
		return 0;
	}

	p_me->atv = &methods_tv;
	p_me->p_xv = p_xv;
	PV_WREG(p_me->p_xv, TmrSpd.l, 0x171);	/* Make sure ticks are correct. */
	return (O_I2C*)p_me;
}


#ifdef WHAT_TO_DO
nv_clock(p_me)
    XV_struct* p_me;
{
    /* routine to generate test clock for timing setup */
    printf(" running clock test\n");
PV_WREG(p_me,I2C.b.cen,0);              /* Jon */
    while(1)
    {
        plxv_nv_delay(p_me, 5);
        PV_WREG(p_me,I2C.b.clk,1);
        plxv_nv_delay(p_me, 5);
        PV_WREG(p_me,I2C.b.clk,0);      /* High-to-low SCL transition   */
    }
}

/****************************************************************/

#endif /* WHAT_TO_DO */
