/*
 *  rsrc_generic.c:	Implementation of generic portion of a resource object.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     12 Nov 93 - Andy Goodrich
 *       - Initial coding
 *
 * $Log:   /plxhp/dave/pvcsrc/hwlib/archives/generic_rsrc.c_v  $
 * 
 *    Rev 1.1   09 Sep 1996 10:11:54   dave
 * Initial revision
 * Revision 1.1  1995/12/08 10:28:10  dist
 * Initial revision
 *
 * Revision 1.4  95/08/24  14:03:12  14:03:12  dist ()
 * Andy Goodrich: Modified initialization code to include generic deletion
 *                notify routine in symbol table creation. This will close
 *                a memory leak when the hardware library is initialized and
 *                terminated multiple times.
 * 
 * Revision 1.3  95/05/25  17:08:48  17:08:48  dist ()
 * Andy Goodrich - Combined HP/AIX version for HP 9.1 code freeze.
 * 
 * Revision 1.2  95/04/17  12:28:44  12:28:44  dist ()
 * Andy Goodrich - added RCS header.
 * 
 *  Copyright(c) 1994, Parallax Graphics, Inc.
 */
#include <sys/types.h>			/* Standard types for this C environment. */
#include <stdio.h>				/* Stream input/output declarations. */
#include "parse_value.h"		/* Pick up definitions for p_value routine. */
#ifdef NOTDEF
#include "symtab.h"				/* Pick up symbol table object. */
#include "rsrc.h"				/* Pick up public version of this object. */
#endif
#include "objects.h"			/* Pick up public object definitions. */
#include "xvdebug.h"			/* Pick up ERRORF and DEBUGF. */

typedef struct I_Rsrc {
    O_Rsrc      base;			/* Public symbols for object. */
	O_SST*		p_st;			/* Symbol table to use. */
} I_Rsrc;


/*---------------------------------------------------------------------------*
 |"m_Rsrc_empty"
 |
 |  Empty a resource object.
 *---------------------------------------------------------------------------*/
static void m_Rsrc_empty(p_me)
    I_Rsrc *p_me;   /* -> resource object to empty. */
{
	O_Rsrc_empty( p_me->p_st );
}
 
/*---------------------------------------------------------------------------*
 |"m_Rsrc_exists"
 |
 |  Check for the existance of a resource.
 |
 |    = type if symbol exists, 0 if not.
 *---------------------------------------------------------------------------*/
static int m_Rsrc_exists( p_me, name )
    I_Rsrc*      p_me;       /* -> resource object's context. */
    char*       name;        /* -> name of resource to check existance of. */
{
    SST_Entry*    p_entry;     /* Entry for this resource. */
 
    p_entry  = O_SST_get(p_me->p_st, name);

	return (p_entry) ? p_entry->type : 0;
}

/*---------------------------------------------------------------------------*
 |"m_Rsrc_get"
 |
 |  Get the value of a resource. 
 |
 |    = value of resource, exit on error.
 *---------------------------------------------------------------------------*/
static int m_Rsrc_get( p_me, name )
    I_Rsrc*      p_me;        /* -> resource object to get resource from. */
    char*       name;        /* -> name of resource to get value of. */
{
    SST_Entry*    p_entry;     /* Entry for this resource. */
 
    if ( !(p_entry = O_SST_get(p_me->p_st, name)) ) 
	{
		ERRORF((fd, "*** %s does not exist!!!\n", name));
		exit(0);
	}
 
	switch ( p_entry->type )
	{
	  case STE_Integer:	        return p_entry->value.i; 
	  case STE_String_Constant:	return (int)p_entry->value.s.p_string;
	  case STE_String:	        return (int)p_entry->value.s.p_string;
	}
}


/*---------------------------------------------------------------------------*
 |"m_Rsrc_get_long"
 |
 |  Get the long value of a resource. 
 |
 |    = length of resource, exit on error.
 *---------------------------------------------------------------------------*/
static int m_Rsrc_get_long( p_me, name, p_buf )
    I_Rsrc*     p_me;       /* -> resource object to get resource from. */
    char*       name;       /* -> name of resource to get value of. */
	char*		p_buf;		/* -> where to place value. */
{
	int			i_value;	/* # of character now moving. */
	int			n_value;	/* # of characters in value. */
    SST_Entry*  p_entry;    /* Entry for this resource. */
	char*		p_from;		/* Next byte to move. */
	char*		p_to;		/* Where to place next byte. */
 
    if ( !(p_entry = O_SST_get(p_me->p_st, name)) ) 
	{
		ERRORF((fd, "*** %s does not exist!!!\n", name));
		exit(0);
	}
 
	switch ( p_entry->type )
	{
	  case STE_Integer:	
		n_value = 4; 
		p_from = (char*)&p_entry->value.i; 
		break;
	  case STE_String_Constant:	
	  case STE_String:	
		n_value = p_entry->value.s.n_string; 
		p_from = (char*)p_entry->value.s.p_string;
		break;
	}
	p_to = p_buf;
	for ( i_value = 0; i_value < n_value; i_value++ )
	{
		*p_to++ = *p_from++;
	}
	return n_value;
}

/*--------------------------------------------------------------------------*
 |"m_Rsrc_load_from_file"
 |
 |  Load resources from a file.
 |
 |    = # of resources loaded on success, 0 on failure
 *--------------------------------------------------------------------------*/
int m_Rsrc_load_from_file( p_me, fd )
	I_Rsrc*		p_me;	/* -> resource object to be loaded.  */
	FILE*		fd;		/* -> file descriptor to read from.  */
{
	return O_SST_load(p_me->p_st, fd);
}


/*---------------------------------------------------------------------------*
 |"m_Rsrc_remove"
 |
 |  Remove a resource given its name.
 *---------------------------------------------------------------------------*/
static int m_Rsrc_remove(p_me, name ) 
    I_Rsrc*     p_me; 	/* -> resource object to remove resource from. */
    char*       name;   /* -> name of resource to remove. */
{
	return O_SST_remove(p_me->p_st, name);
}

/*---------------------------------------------------------------------------*
 |"m_Rsrc_set"
 |
 |  Set the value of a resource.
 |
 |    != 0 on success, 0 on failure
 *---------------------------------------------------------------------------*/
static int m_Rsrc_set(p_me, name, value ) 
    I_Rsrc*     p_me; 	/* -> resource object to set resource in. */
    char*       name;  	/* -> name of resource to remove. */
    int   	    value;  /* = value to set. */
{
	return O_SST_set(p_me->p_st, name, STE_Integer, value);
}

/*---------------------------------------------------------------------------*
 |"m_Rsrc_set_long"
 |
 |  Set the long value of a resource.
 |
 |    != 0 on success, 0 on failure
 *---------------------------------------------------------------------------*/
static int m_Rsrc_set_long(p_me, name, length, value ) 
    I_Rsrc*     p_me; 	/* -> resource object to set resource in. */
    char*       name;  	/* -> name of resource to remove. */
	int			length;	/* =  type of value. */
    void*  	    value;  /* -> value to set. */
{
	ST_String	string;	/* Temporary to hold value to set. */

	string.n_string = length;
	string.p_string = value;

	/* #### Should we make a distinction between constant and non-constants. */
	return O_SST_set(p_me->p_st, name, STE_String_Constant, &string);
}


/*--------------------------------------------------------------------------*
 |"m_Rsrc_store_to_file"
 |
 |  Store the contents of a resource object in a file. 
 |
 |    = 1 on success, 0 on failure
 *--------------------------------------------------------------------------*/
int m_Rsrc_store_to_file( p_me, fd )
	I_Rsrc*		p_me;	/* -> resource object to be stored.  */
	FILE*		fd;		/* -> file descriptor to read from.  */
{
	int		rc;	/* Return code from symbol table store. */

	if ( fd )
	{
		return O_SST_store(p_me->p_st, fd);
	}
	else if ( !(fd = fopen("/opt/parallax/etc/plxv0", "r")) )
	{
		return 0;
	}
	else
	{
		rc = O_SST_store(p_me->p_st, fd);
		fclose(fd);
		return rc;
	}
}


/*---------------------------------------------------------------------------*
 |"m_Rsrc_terminate"
 |
 |  Terminate use of a resource object, freeing any necessary storage.
 |
 *---------------------------------------------------------------------------*/
static void m_Rsrc_terminate(p_me) 
    I_Rsrc      *p_me;  /* -> resource object to free up. */
{
	O_SST_terminate(p_me->p_st);
	free(p_me);
}

static void no_defaults(p_me)
    I_Rsrc      *p_me;  /* -> resource object to free up. */
{
	ERRORF((fd, "O_Rsrc: no dd function provided for defaults\n"));
}

static void no_load_from_context(p_me)
    I_Rsrc      *p_me;  /* -> resource object to free up. */
{
	ERRORF((fd, "O_Rsrc: no dd function provided for load_from_context\n"));
}

static void no_store_to_context(p_me)
    I_Rsrc      *p_me;  /* -> resource object to free up. */
{
	ERRORF((fd, "O_Rsrc: no dd function provided for store_to_context\n"));
}

static Tv_Rsrc rsrc_methods =
{
	no_defaults,				/* defaults */
	m_Rsrc_empty,
	m_Rsrc_exists,
	m_Rsrc_get,
	m_Rsrc_get_long,
	no_load_from_context,		/* load_from_context */
	m_Rsrc_load_from_file,
	m_Rsrc_remove,
	m_Rsrc_set,
	m_Rsrc_set_long,
	no_store_to_context,		/* store_to_context */
	m_Rsrc_store_to_file,
	m_Rsrc_terminate
};


/*---------------------------------------------------------------------------*
 |"O_Rsrc_Generic"
 |
 |  Create and return a resource object.
 |
 |    -> resource object on success, 0 on failure
 *---------------------------------------------------------------------------*/
O_Rsrc *O_Rsrc_Generic(average_symbols)
	int		average_symbols;		/* = average # of symbols expected. */
{
	void    O_SST_Generic_Delete();	/* Default deletion routine. */
    I_Rsrc* p_me;  					/* Symbol table creating. */

	p_me = (I_Rsrc *)malloc(sizeof(I_Rsrc));

    /*
     *  INITIALIZE DATA STRUCTURE:
     */
	p_me->base.atv = &rsrc_methods;
	p_me->p_st = O_SST_O_SST(256, O_SST_Generic_Delete);

	return (O_Rsrc *)p_me;
}

