/*
 *  arg_parse.h: Command argument parsing routine.
 *
 *    Copyright(c) 1989, Andrew C. Goodrich
 *
 *    Permission to use granted Parallax Graphics, Inc.
 *    Changes copyright Parallax Graphics, Inc.
 */
#ifndef arg_parse_h_INCLUDED
#define arg_parse_h_INCLUDED

typedef struct Pos_Arg {    /* Positional argument definition: */
	int		type;			/* ... positional argument type (see enum below.) */
	int*	p_storage;		/* ... where to store argument. */
	int		value;			/* ... value field (used with Pos_Callback). */
} Pos_Arg;

enum {				/* Values for type field of Pos_Arg. */
	Pos_Fence = 0,	/* End of list indicator. */
	Pos_Backup,	 	/* Back up position pointer one (re-execute previous). */
	Pos_Callback,   /* Call (*p_storage)(value, argv[i_argv]) */
	Pos_Callback1,  /* Call (*p_storage)(value, argv[i_argv], argv[i_argv+1]) */
	Pos_Float,		/* Parse a floating point value. */
	Pos_Integer,	/* Parse an integer value. */
	Pos_String,		/* Parse a string value. */
};

typedef struct Opt_Arg {	/* Optional argument definition: */
	int		type;			/* ... optional argument type (see enum below.) */
	char*	p_text;			/* ... text for optional argument. */
	int*	p_storage;		/* ... where to store argument. */
	int		value;			/* ... value field (used with Opt_No_Value). */
} Opt_Arg;

enum {				/* Values for type field of Opt_Arg. */
	Opt_Fence = 0,	/* End of list indicator. */
	Opt_Float,		/* Floating point value follows argument. */
	Opt_Integer,	/* Integer value follows argument. */
	Opt_String,		/* String value follows argument. */
	Opt_No_Value,   /* No value follows, set storage to value field. */
	Opt_Callback,   /* Call (*p_storage)(value, argv[i_argv]) */
	Opt_Callback1,  /* Call (*p_storage)(value, argv[i_argv], argv[i_argv+1]) */
};

typedef void (*CB_Rtn)();	/* Typedef for routine calling. */

#endif  /* arg_parse_h_INCLUDED */
/*
 *  FORWARD FUNCTION REFERENCES:
 */


/*---------------------------------------------------------------------------*
 |"arg_parse"
 |
 |  Parse a set of positional and optional arguments which were entered
 |  on a command line. 
 |  (1) Note the positional arguments are processed in the order given, 
 |      but optional arguments may appear in any order.
 |  (2) The first element in argv[] is assumed to be the program name.
 |
 |    = 1 on success, 0 on failure.
 *---------------------------------------------------------------------------*/
arg_parse( p_poss, p_opts, argc, argv )
	Pos_Arg*	p_poss;	/* -> table of positional arguments. */
	Opt_Arg*	p_opts;	/* -> table of optional arguments. */
	int			argc;	/* =  # of command line arguments parsed. */
	char*		argv[];	/* =  vector of pointers to arguments parsed. */
{
	int			i_argv;	/* Entry in argv now processing. */
	Pos_Arg*	p_pos;	/* Positional argument now processing. */
	Opt_Arg*	p_opt;	/* Optional argument now processing. */

	p_pos = p_poss;

	for ( i_argv = 1; i_argv < argc; i_argv++ )
	{
		for ( p_opt = p_opts; p_opt && p_opt->type != Opt_Fence; p_opt++ )
		{
			if ( !strcmp(p_opt->p_text, argv[i_argv]) )
			{
				switch( p_opt->type )
				{
				  case Opt_Callback:
					(*(CB_Rtn)(p_opt->p_storage))(p_opt->value, argv[i_argv]);
					break;

				  case Opt_Callback1:
					i_argv++;
					if ( i_argv >= argc )
					{
						fprintf(stderr, "*** %s requires an argument\n",
							argv[i_argv-1]);
						return 0;
					}
					(*(CB_Rtn)(p_opt->p_storage))(p_opt->value, argv[i_argv-1],
						argv[i_argv]);
					break;

				  case Opt_Float:
					i_argv++;
					if ( i_argv >= argc )
					{
						fprintf(stderr, "*** %s requires an argument\n",
							argv[i_argv-1]);
						return 0;
					}
					if ( sscanf(argv[i_argv], "%f", p_opt->p_storage) != 1 )
					{
						fprintf(stderr, "*** %s is an illegal float value\n",
							argv[i_argv]);
						return 0;
					}
					break;

				  case Opt_Integer:
					i_argv++;
					if ( i_argv >= argc )
					{
						fprintf(stderr, "*** %s requires an argument\n",
							argv[i_argv-1]);
						return 0;
					}
					if ( sscanf(argv[i_argv], "%d", p_opt->p_storage) != 1 )
					{
						fprintf(stderr, "*** %s is an illegal integer\n",
							argv[i_argv]);
						return 0;
					}
					break;

				  case Opt_No_Value: 
					*p_opt->p_storage = p_opt->value; 
					break;

				  case Opt_String:
					i_argv++;
					if ( i_argv >= argc )
					{
						fprintf(stderr, "*** %s requires an argument\n",
							argv[i_argv-1]);
						return 0;
					}
					p_opt->p_storage = argv[i_argv];
					break;
				}
				break;
			}
		}
		if ( !p_opt || p_opt->type == Opt_Fence )
		{
			if ( !p_pos || p_pos->type == Pos_Fence ) return 1;
			if ( p_pos->type == Pos_Backup ) --p_pos;
			switch( p_pos->type )
			{
			  case Pos_Callback:
				(*(CB_Rtn)(p_pos->p_storage))(p_pos->value, argv[i_argv]);
				break;

			  case Pos_Callback1:
				i_argv++;
				if ( i_argv >= argc )
				{
					fprintf(stderr, "*** %s requires an argument\n",
						argv[i_argv-1]);
					return 0;
				}
				(*(CB_Rtn)(p_pos->p_storage))(p_pos->value, argv[i_argv-1],
					argv[i_argv]);
				break;

			  case Pos_Float:
				if ( sscanf(argv[i_argv], "%f", p_pos->p_storage) != 1 )
				{
					fprintf(stderr, "*** %s is an illegal float value\n",
						argv[i_argv]);
					return 0;
				}
				break;

			  case Pos_Integer:
				if ( sscanf(argv[i_argv], "%d", p_pos->p_storage) != 1 )
				{
					fprintf(stderr, "*** %s is an illegal integer\n",
						argv[i_argv]);
					return 0;
				}
				break;

			  case Pos_String:
				p_pos->p_storage =  argv[i_argv];
				break;

			}
			p_pos++;
		}
	}
}
