
/*
 * Copyright (c) 1993 Parallax Graphics Inc.
 */

#pragma ident "@(#)tvtwo_nvram.c	1.5 Copyright (c) 1993 Parallax Graphics Inc"

#include <sys/types.h>
#include <sys/cmn_err.h>
#include <sys/ddi.h>
#include <sys/sunddi.h>

#include "tvtwo.h"

int
tvtwo_nvram_init(XV_regs *regs, XV_nvram *nvram);

int
tvtwo_nv_write(XV_regs *regs, dev_t dev, paddr_t prt, int nv_addr, int count);

int
tvtwo_nv_read(XV_regs *regs, dev_t dev, paddr_t ptr, int nv_addr, int count);

static int
nvram_read(XV_regs *regs, int paddr, caddr_t ptr, int addr, int count);

static int
nvram_write(XV_regs *regs, int paddr, caddr_t ptr, int addr, int count);

static void
i2csom(XV_regs *regs);

static void
i2cpa(XV_regs *regs, int addr, int read_flag);

static int
i2cack(XV_regs *regs, int context);

static void
i2cwd(XV_regs *regs, int data);

static void
i2ceom(XV_regs *regs);

static int
i2c_set_addr(XV_regs *regs, int addr, int saddr);

static int
plxv_i2cr(XV_regs *regs, int addr, int saddr, int *data);

int
plxv_i2cw(XV_regs *regs, u_int addr, u_int saddr, u_int data);

#define PLXV_DELAY(usec) drv_usecwait(usec)

/*
 * nvram unit  assignement:
 *	50 - main board
 *	51 - option card
 *	52 - tuner box
 */

#define NVRAM_MAIN 0x50
#define NVRAM_OPT 0x51
#define NVRAM_TUNER 0x52
#define DEF_RAS_MODE ~0x10
#define DMSDREV_2 2

int
tvtwo_nvram_init(XV_regs *regs, XV_nvram *nvram)
{
	/*
	 * Attempt to read the configuration information from
	 * the nvram.  It's fatal if we cannot or if the contents
	 * are corrupted.
	 */
	if (nvram_read(regs, NVRAM_MAIN, (caddr_t)&nvram->main,
		0, sizeof(XV_controller_nvram)) != 0) {
		cmn_err(CE_WARN, "XVIDEO: cannot read controller nvram\n");
		return (1);
	} else if (nvram->main.hlib_rev != HLIB_REV) {
		cmn_err(CE_WARN, "XVIDEO: corrupted controller nvram\n");
		return (1);
	}

	/*
	 * Attempt to read the option configuration information from
	 * the nvram.  It's not fatal if an option board is not installed,
	 * however, if the nvram on the option board is corrupted we
	 * fail.
	 */
	if (nvram_read(regs, NVRAM_OPT, (caddr_t)&nvram->opt,
		0, sizeof(XV_option_nvram)) == 0) {
		DEBUGF((CE_CONT, "Option card installed\n"));
		if (nvram->opt.hlib_rev != HLIB_REV) {
			cmn_err(CE_WARN, "XVIDEO: corrupted option nvram\n");
			return (1);
		}
	}

	return (0);
}

#ifdef NOTDEF

/*
 * What follows are various defaults for the NVRAM.
 * These would be used in an emergency to reset the nvram
 * from the driver.
 *
 * However, it makes more sense to have an application 
 * program the nvram through the driver, since its structure
 * may change (although that would break the driver).
 */

static XV_disp_format def_h_disp[] = {
	{287,12,31,42,31,287}, /*horiz disp 1152/66hz */
	{287,9,31,37,0xbe,0x8000}, /* horiz disp 1152/76hz */
	{287,13,31,35,31,287} /*horiz disp 1152/60hz */
};

static XV_disp_format def_v_disp[] = {
	{899,1, 3,30,0,0},    /* vert disp 900/66hz */
	{899,1, 7,32,0,0},    /* vert disp 900/76hz */
	{899,1, 3,29,0,0}    /* vert disp 900/60hz */
};

static XV_io_port def_input_port = {
	IO_DMSD,
	{128,128,128,128},    /* initial video knobs */
	{128,128,128,128},    /* initial video knobs */
	{128,128,128,128},    /* initial video knobs */
	{{640,482,43,10,20}, {768,574,51,7,23}}, /* input video format dmsd */
	{DMSDREV_2,0, /* dmsd_setup */
	 0x50,0x30,0,0xe8,0xb6,0xe9,0x7,0,
	 0xf8,0xf8,0x90,0x90,0,0x80,0x7a,0x59,
	 0,0x2c,0x34,0xa,0xf4,0xce,0xe7,
	 0x0,0x0,0}
};

static XV_rgbin_config def_rgb_config = {
	{(HSYNC_HIGH | VSYNC_HIGH),
	 {0x9c,0x87,0xf6,0x7f,
	  0xd2,0x276,31,40,0x39,0x42,0x60,
	  0,0,0},
	  {{780,640,0x78,0,0,0}, {944,768,0x9c,0,0,0}},
	  {{0x98,0,0x98,0,0x98,0,0,0},
	   {0x98,0x4c,0x98,0,0x98,0x4c,0,0}}
	}
};

static XV_io_port def_output_port = {
	IO_COMP_OUT,
	{128,128,128,128},    /* initial video knobs */
	{128,128,128,128},    /* initial video knobs */
	{128,128,128,128},    /* initial video knobs */
	{{640,482,43,7,21}, {768,574,51,7,34}}, /* output video format dmsd */
	{DMSDREV_2,0, /* dmsd_setup */
	 0x50,0x30,0,0xe8,0xb6,0xe9,0x7,0,
	 0xf8,0xf8,0x90,0x90,0,0x80,0x7a,0x59,
	 0,0x2c,0x34,0xa,0xf4,0xce,0xe7,
	 0x0,0x0,0}
};

static u_short def_dispflgs[]	= {0x1e2a,0x1f2a,0x1e2a};
static u_char def_c3rev[]	= {0, C3REV_1, 0};
static u_long def_startup[]	= {0x08000608, 0x0800060a, 0x08000608};

static nvram_opttype
nvram_code2opttype(u_int code)
{
	switch(code) {
		case OPT_1:
			return (OPT1);
		case OPT_2:
			return (OPT2);
		case OPT_3:
			return (OPT3);
		case OPT_4:
			return (OPT4);
	{
	return (DEFAULT_OPT);
}

static nvram_bdtype
nvram_code2bdtype(u_int code)
{
	switch (code) {
		case SBUS_1:
			return (SBUS1);
		case SBUS_2:
			return (SBUS2);
		case SBUS_3:
			return (SBUS3);
		case SBUS_4:
			return (SBUS4);
		case SBUS_5:
			return (SBUS5);
	}
	return(DEFAULT_BD);
}

static u_int type2bdcode[]	= {SBUS_1, SBUS_2, SBUS_3, SBUS_4, SBUS_5};
static u_int type2optcode[]	= {OPT_1, OPT_2, OPT_3, OPT_4};
#define nvram_bdtype2code(type) (type2bdcode[type])
#define nvram_opttype2code(type) (type2optcode[type])

static void
nvram_set_default(XV_controller_nvram *nvram, nvram_bdtype type)
{
	switch(type) {
		case SBUS1: /* 1152 x 900 66 hz uses 92.9405 mhz clk */
		case SBUS2: /* 1152 x 900 76 hz uses 105.561 mHz clk */
		case SBUS3: /* 1152 x 900 60 hz uses 83.0204 mhz clk */
		case SBUS4: /* MULTI/POWER VIDEO */
		case SBUS5: /* 640 x 480 60 hz (Purvis) */
		{
			nvram->board_code = nvram_type2bdcode(type);
			nvram->ga_rev = GAREV_2;
			nvram->c3_rev = def_c3rev[type];
			nvram->clt_type = BT463;
			nvram->hlib_rev = HLIB_REV;
			nvram->power_hours = 0;
			nvram->power_cycles = 1;
			nvram->xsize = 1152;
			nvram->ysize = 900;
			nvram->Attr = 0x80c0;
			nvram->DispKludge = 0xc41f;
			nvram->DispFlgs = def_dispflgs[type];
			nvram->StartUp = def_startup[type];
			nvram->TmrInit = 0x171;
			nvram->RasMode = DEF_RAS_MODE;
			nvram->OLDI2C = 0;
			nvram->h = def_h_disp[type];
			nvram->v = def_v_disp[type];
			nvram->d = def_input_port;
			break;
		}
		default:
	}
}

static void
nvram_set_default_opt(XV_option_nvram *nvram, nvram_opttype type)
{
	switch (type) {
		case OPT_1: {	/* the VIO option card */
			nvram->board_code = nvram_type2optcode(type);
			nvram->clt_type = BT473;
			nvram->hlib_rev = HLIB_REV;
			nvram->power_hours = 0;
			nvram->power_cycles = 1;

			/* b port is input (the DMSD chipset) */
			nvram->b = def_input_port;
			nvram->b.config.dmsd.unit = 1;

			/* a port is output */
			nvram->a = def_output_port;
			nvram->a.config.gen.xpoint_bits =
				A_DATA_OUTPUT_W | OUTPUT_EN_DATA |
				CE_HIGH | HSYNC_HIGH | VSYNC_HIGH ;
			break;
		}
		case OPT_2:	/* the RGB card */
		case OPT_3:	/* the SONYRGB card */
		case OPT_4: {	/* the SONYRGB2 card */
			nvram->board_code = nvram_type2optcode(type);
			nvram->clt_type = BT473;
			nvram->hlib_rev = HLIB_REV;
			nvram->power_hours = 0;
			nvram->power_cycles = 1;
			/* b port is rgb output */
			nvram->b = def_output_port;
			nvram->b.flags = IO_RGB_OUT;
			/* stuff of xpoint_b */
			nvram->b.config.gen.xpoint_bits = 0x52535;
			/* a port is input */
			nvram->a = def_input_port;
			switch (type) {
				case OPT_2:
					nvram->a.flags = IO_RGB_IN;
					break;
				case OPT_3:
				case OPT_4:
					nvram->a.flags = IO_SONYRGB_IN;
					break;
			}	
			nvram->a.config = def_rgb_config;
			break;
		}
	}
}
#endif /* NOTDEF */

/*
 * i2c start of message
 */

static void
i2csom(XV_regs *regs)
{
/*
	XV_WREG(regs, MiscMode.b.blu, 7);
	XV_WREG(regs, MiscMode.b.blu, 0);
*/
	XV_WREG(regs, I2C.b.cen, 0);	/* Jon enable clock output */
	XV_WREG(regs, I2C.b.data, 1);	/* Make sure data high before */
	XV_WREG(regs, I2C.b.enbl, 1);	/* Set Data direction write */
	XV_WREG(regs, I2C.b.clk, 1);	/* Make sure clock is high */
	PLXV_DELAY(5);			/* SCL high time = 5us */
	XV_WREG(regs, I2C.b.data, 0);	/* High-to-low transition SDA */
	PLXV_DELAY(5);			/* Wait for Tsu (SCL>SDA)=5us */
	XV_WREG(regs, I2C.b.clk, 0);	/* High-to-low transition SCL */
	PLXV_DELAY(4);			/* Wait for Th (SCL<SDA) =4us */
}

/*
 * send i2c slave address
 */

static void
i2cpa(XV_regs *regs, int addr, int read_flag)
{
        /* address is a 7-bit value */
	int i;
	for (i=0;i<7;i++) {
		XV_WREG(regs, I2C.b.data, (addr & 0x40)>>6);  /* MSB */
		PLXV_DELAY(1);		/* Wait for Tsu (SDA>SCL)=250ns	*/
		XV_WREG(regs, I2C.b.clk, 1);	/* Low-to-high SCL transition */
		PLXV_DELAY(5);			/* SCL high time = 5us */
		XV_WREG(regs, I2C.b.clk, 0);	/* High-to-low SCL transition */
		PLXV_DELAY(5);		/* Wait for Th (SCL<SDA)=5us	*/
		addr <<= 1;
	}
	XV_WREG(regs, I2C.b.data, read_flag);	
	PLXV_DELAY(1);
	XV_WREG(regs, I2C.b.clk, 1);
	PLXV_DELAY(5);
	XV_WREG(regs, I2C.b.clk, 0);
	XV_WREG(regs, I2C.b.enbl, 0);
	PLXV_DELAY(5);
}

/*
 * i2c acknowledge
 */

static int
i2cack(XV_regs *regs, int context)
{
	int readback;	

	/* context is used to tell this routine what is trying to be ack'ed
	 * so that a more informative error message can be generated
	 *
	 * context		meaning
	 * 1		primary-address
	 * 2		sub-address
	 * 3		data-write
	 *
	 * this routine will return the value of context iff there is and
	 * error.  otherwise it will return 0.
	 */ 

	XV_WREG(regs, I2C.b.enbl, 0); /* Set data direction for read */
	PLXV_DELAY(10);
	XV_WREG(regs, I2C.b.clk, 1); /* Low-to-high SCL transition */
	PLXV_DELAY(5);		/* Wait for SCL Low Time=5us */
	readback = XV_RREG(regs, I2CStat.b.data);	/* Jon */

#ifdef notdef
	if (readback & 0x1) {
		switch (context) {
			case 1:
				DEBUGF((CE_CONT,
					"nvram: error writing slave addr\n"));
				break;
			case 2:
				DEBUGF((CE_CONT,
					"nvram: error writing sub-address\n"));
				break;
			case 3:
				DEBUGF((CE_CONT,
					"nvram: error writing data\n"));
				break;
		}
	}
#endif

	XV_WREG(regs, I2C.b.clk,0); /* High-to-low SCL transition */
	PLXV_DELAY(5);
	/*   XV_WREG(regs, I2C.b.enbl, 1); */
	PLXV_DELAY(5);		/* Wait for SCL Low Time=5us */

	if (readback & 0x1)
		return(context);

	return(0);
}
 
/*
 * i2c write data
 */

static void
i2cwd(XV_regs *regs, int data)
{
	int i;

	XV_WREG(regs, I2C.b.enbl, 1);
	for (i=0;i<8;i++) {
		XV_WREG(regs, I2C.b.data, (data & 0x80)>>7);  /* MSB */
		PLXV_DELAY(1);		/* Wait for Tsu (SDA>SCL)=250ns	*/
		XV_WREG(regs, I2C.b.clk, 1);	/* Low-to-high SCL transition */
		PLXV_DELAY(5);			/* SCL high time = 5us */
		XV_WREG(regs, I2C.b.clk, 0);	/* High-to-low SCL transition */
		if (i == 7)
			XV_WREG(regs, I2C.b.enbl, 0);
		PLXV_DELAY(5);			/* Wait for Th (SCL<SDA)=5us */
		data<<=1;
	}
}

/*
 * i2c end of message
 */

static void
i2ceom(XV_regs *regs)
{
	XV_WREG(regs, I2C.b.enbl, 1);	/* enable output */ 
	PLXV_DELAY(10);	
	XV_WREG(regs, I2C.b.data, 0);
	PLXV_DELAY(1);	
	XV_WREG(regs, I2C.b.clk, 1);
	PLXV_DELAY(5);	
	XV_WREG(regs, I2C.b.data, 1);	/* set data high */
	PLXV_DELAY(5);			/* Wait for SCL Low Time=5us	*/
}



/*
 * send start, i2c slave address, and i2c sub-address
 * error returns: 0 good, 1= no addr ack, 2 = no sub-addr ack
 */

static int
i2c_set_addr(XV_regs *regs, int addr, int saddr)
{
	int i;

	i2csom(regs);
	i2cpa(regs, addr, 0);  /* writing */
	if ((i=i2cack(regs, 1))) {
		i2ceom(regs);
		return(i);
	}
	i2cwd(regs, saddr);
	if ((i=i2cack(regs, 2))) {
		i2ceom(regs);
		return(i);
	}
	return(0);
}

/*
 * send i2c slave and sub-address then send second som and read byte
 */

static int
plxv_i2cr(XV_regs *regs, int addr, int saddr, int *data)
{
	/*
	 *i2c devices with more that 1 readable sub-address require
	 *a write cycle to pre-set the sub-address pointer prior to
	 *reading a data byte
	 */

	int i,j,rb;

	i2c_set_addr(regs, addr, saddr);
	i2csom(regs);
	i2cpa(regs, addr, 1); /* reading */
	if (i2cack(regs, 1)) {
		i2ceom(regs);
		return(-1);  /* was 0 */
	}
	/* readback */
	XV_WREG(regs, I2C.b.enbl, 0);	/* Set the data direction for read */
	PLXV_DELAY(5);		/* Wait for Tsu (SDA>SCL)=250ns	*/
	j=0;
	for (i=0;i<8;i++) {
		j <<= 1;		/* this does nothing on first iter */
		XV_WREG(regs, I2C.b.clk, 1);	/* Low-to-high SCL transition	*/
		PLXV_DELAY(5);		/* Let the readback data settle	*/
		rb = XV_RREG(regs, I2CStat.b.data);	/* Jon */
		j |= (rb & 1);
		XV_WREG(regs, I2C.b.clk, 0);	
		PLXV_DELAY(5);		/* Wait for SCL Low Time=5us	*/
	}
	i2ceom(regs);
	*data = j;
	return(0);		/* added */
}

/*
 * send slave address, sub-address and data byte
 */

int
plxv_i2cw(XV_regs *regs, u_int addr, u_int saddr, u_int data)
{
	int ret;

	if (ret = i2c_set_addr(regs, addr, saddr))
		return(ret);
	i2cwd(regs, data);
	ret = i2cack(regs, 3);
	i2ceom(regs);
	return(ret);
}

/****************************************************************/

int
tvtwo_nv_write(XV_regs *regs, dev_t dev, paddr_t ptr, int nv_addr, int count)
{
	int paddr;

	switch (TVTWO_DEV2MINOR(dev)) {
		case TVTWO_NVRAM_MINOR:
			paddr = NVRAM_MAIN;
			break;
		case TVTWO_OPT_MINOR:
			paddr = NVRAM_OPT;
			break;
		case TVTWO_TUNER_MINOR:
			paddr = NVRAM_TUNER;
			break;
		default:
			return (1);
	}

	return nvram_write(regs, paddr, (caddr_t)ptr, nv_addr, count);
}

int
tvtwo_nv_read(XV_regs *regs, dev_t dev, paddr_t ptr, int nv_addr, int count)
{
	int paddr;

	switch (TVTWO_DEV2MINOR(dev)) {
		case TVTWO_NVRAM_MINOR:
			paddr = NVRAM_MAIN;
			break;
		case TVTWO_OPT_MINOR:
			paddr = NVRAM_OPT;
			break;
		case TVTWO_TUNER_MINOR:
			paddr = NVRAM_TUNER;
			break;
		default:
			return (1);
	}

	return nvram_read(regs, paddr, (caddr_t)ptr, nv_addr, count);
}

#ifndef TV3
static int
nvram_read(XV_regs *regs, int paddr, caddr_t ptr, int addr, int count)
{
	char *j;
	int i, k, data;

	j = ptr;
	for (i=0; i<count; i++) {
		k = plxv_i2cr(regs, paddr, addr++, &data);
		*j++ = data;
		if (k)
			return(k);
	}
	return(0);
}

static int
nvram_write(XV_regs *regs, int paddr, caddr_t ptr, int addr, int count)
{
	char *j;
	int i, k;

	j = ptr;
	for (i=0; i<count; i++) {
		k = plxv_i2cw(regs, paddr, addr++, *j++);
		if (k)
			return(k);
		PLXV_DELAY(27000);
	}
	return(0);
}
#else /* TV3 */

/*  TV3 CARD NVRAM SUPPORT
 *
 *    Since the TV3 overlay card (hedge) does not contain an NVRAM we
 *    simulate one here. This minimizes the differences between the TV3
 *    driver and that for the TV2 family.
 */
struct barf {
	short a;
	u_short :16;
	XV_disp_format b;
	} c = { 0, {287,12,31,42,31,287} };
XV_controller_nvram nvram_main = {
    0,                      /* u_char  board_code */
    GAREV_2,                /* u_char  ga_rev */
    C3REV_1,                /* u_char  c3_rev */
    0/*BT495*/,             /* u_char  clt_type */
    0,                      /* u_long  serial_number */
    HLIB_REV,               /* u_short hlib_rev */
    /*0,*/                      /* u_short :16 */
    0,                      /* u_long  power_hours */
    1,                      /* u_short power_cycles */
    1280,                   /* u_short xsize */
    1024,                   /* u_short ysize */
    0x80c0,                 /* u_short Attr */
    0xc41f,                 /* u_short DispKludge */
    0xa48,                  /* u_short DispFlgs */
    0x604 | 0x8000000,      /* u_long  StartUp */
    0x171,                  /* u_short TmrInit */
    DEF_RAS_MODE,           /* u_short RasMode */
    0,                      /* u_char  OLDI2C */
    /*0,*/                      /* u_long  :24 */
	{287,12,31,42,31,287},  /* XV_disp_format  h */
	{899,1, 3,30,0,0},      /* XV_disp_format  v */
	{                       /* XV_io_port d */
		IO_DMSD,
		{128,128,128,128},    /* initial video knobs */
		{128,128,128,128},    /* initial video knobs */
		{128,128,128,128},    /* initial video knobs */
		{{640,482,43,10,20}, {768,574,51,7,23}}, /* input video format dmsd */
		{
			DMSDREV_2,0, /* dmsd_setup */
			0x50,0x30,0,0xe8,0xb6,0xe9,0x7,0,
			0xf8,0xf8,0x90,0x90,0,0x80,0x7a,0x59,
			0,0x2c,0x34,0xa,0xf4,0xce,0xe7,
			0x0,0x0,0
		}
	},
	0, 						/* u_char  unused_2[10] */
};

XV_option_nvram nvram_opt = {
    OPT_1,                  /* u_char  board_code */
    0,                      /* u_char  board_rev */
    0,                      /* u_char  clt_type */
    /*0,*/                      /* u_char  :8 */
    0,                      /* u_long  serial_number */
    HLIB_REV,               /* u_short hlib_rev */
    /*0,*/                      /* u_short :16 */
    0,                      /* u_long  power_hours */
    1,                      /* u_short power_cycles */
    /*0,*/                      /* u_short :16 */
    {                       /* XV_io_port a */
		IO_COMP_OUT,
		{128,128,128,128},    /* initial video knobs */
		{128,128,128,128},    /* initial video knobs */
		{128,128,128,128},    /* initial video knobs */
		{{640,482,43,7,21}, {768,574,51,7,34}}, /* output video format dmsd */
		{DMSDREV_2,1, /* dmsd_setup */
		 0x50,0x30,0,0xe8,0xb6,0xe9,0x7,0,
		 0xf8,0xf8,0x90,0x90,0,0x80,0x7a,0x59,
		 0,0x2c,0x34,0xa,0xf4,0xce,0xe7,
		 0x0,0x0,0}
	},
    {                       /* XV_io_port b */
        IO_DMSD,
        {128,128,128,128},    /* initial video knobs */
        {128,128,128,128},    /* initial video knobs */
        {128,128,128,128},    /* initial video knobs */
        {{640,482,43,10,20}, {768,574,51,7,23}}, /* input video format dmsd */
        {
            DMSDREV_2,0, /* dmsd_setup */
            0x50,0x30,0,0xe8,0xb6,0xe9,0x7,0,
            0xf8,0xf8,0x90,0x90,0,0x80,0x7a,0x59,
            0,0x2c,0x34,0xa,0xf4,0xce,0xe7,
            0x0,0x0,0
        }
    },
    0    					/* u_char  unused[10] */
};


static int
nvram_read(XV_regs *regs, int paddr, caddr_t ptr, int addr, int count)
{
    char *j;
    int i, k, data;
 
	char*	p_nvram;	/* Alias for nvram storage area. */

	switch( paddr )
	{
	  case  NVRAM_MAIN: p_nvram = (char*)&nvram_main; break;
	  case  NVRAM_OPT:  p_nvram = (char*)&nvram_opt;  break;
	  default:          return 0;
	}
	bcopy(&p_nvram[addr], ptr, count);
	return 0;
}
 
static int
nvram_write(XV_regs *regs, int paddr, caddr_t ptr, int addr, int count)
{
	return 0;
}
#endif /* !TV3 */
