/*
 *  O_XV_Select.c:	Implementation of Xv adaptor selection object.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  Copyright(c) 1995, Parallax Graphics, Inc.
 *
 *  REVISION HISTORY:
 *
 * $Log:   /plxhp/dave/pvcsrc/O_XV/archives/O_XV_Select.c_v  $
 * 
 *    Rev 1.1   09 Sep 1996 09:52:14   dave
 * Initial revision
 * Revision 1.1  1995/08/29 15:33:37  dist
 * Initial revision
 *
 */
#include <sys/types.h>			/* Standard types for this C environment. */
#include <stdio.h>				/* Stream input/output declarations. */
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <Xvlib.h>
#include "Xvddx.h"
#include "O_XV.h"


/*---------------------------------------------------------------------------*
 |"m_XV_Select_dump"
 |
 |  Dump the contents of an O_XV_Select object.
 *---------------------------------------------------------------------------*/
void m_XV_Select_dump( p_me, fd )
	O_XV_Select*	p_me;	/* -> object context. */
	FILE*			fd;		/* -> file descriptor to dump contents to. */
{
	int  			i_adaptor;	/* # of adaptor record we are now accessing. */
	XvAdaptorInfo*  p_adaptor;	/* Information about adaptor to dump. */

	fprintf(fd, "\nO_XV_Select object at address %08x\n", p_me);
	fprintf(fd, "\n    Transfer vector at %08x contents are:\n", p_me->tv);
	fprintf(fd, "        dump      %08x\n", p_me->tv->dump);
	fprintf(fd, "        terminate %08x\n", p_me->tv->terminate);
	fprintf(fd, "\n    Display structure at %08x, root window id is %d\n", 
		p_me->p_display, p_me->w_root
	);
	fprintf(fd, "\n    Xv Version information:\n");
	fprintf(fd, "       Version: %d  revision: %d\n", 
		p_me->xv_version, p_me->xv_revision
	);
	fprintf(fd, "       Major opcode: %d (0x%x)  event base: %d	(0x%x)\n",
		p_me->xv_major_op, p_me->xv_major_op,
        p_me->xv_event_base, p_me->xv_event_base
	);
	fprintf(fd, "       Error base:   %d (0x%x)\n",
		p_me->xv_error_base, p_me->xv_error_base
	);

	switch ( p_me->n_adaptors )
	{
	  case 0:
		fprintf(fd, "\n    There are no adaptors present\n");
		break;
	  case 1:
		fprintf(fd, "\n    There is 1 adaptor present:\n");
		break;
	  default:
		fprintf(fd, "\n    There are %d adaptors present:\n", p_me->n_adaptors);
		break;
	}
	for ( i_adaptor = 0; i_adaptor < p_me->n_adaptors; i_adaptor++ )
	{
		p_adaptor = &p_me->p_adaptors[i_adaptor];
		fprintf(fd, "       Adaptor %d:\n", i_adaptor);
		fprintf(fd, "         type: %s  # of ports: %d\n", 
			p_adaptor->name, p_adaptor->num_ports
		);
	}
}


/*---------------------------------------------------------------------------*
 |"m_XV_Select_query_type"
 |
 |  Return number of adaptors found of the specified type.
 |
 |    =  number of adaptors found.
 *---------------------------------------------------------------------------*/
int m_XV_Select_query_type( p_me, type )
	O_XV_Select*	p_me;	 /* -> object context. */
	char*			type;	 /* -> type of adaptor, or 0 to total count. */
{
	int		i_adaptor;	/* # of adaptor whose info we are looking at. */
	int		n_type;		/* # of adaptors of type 'type' encountered. */

	if ( !type ) return p_me->n_adaptors;

	for ( n_type = i_adaptor = 0; i_adaptor < p_me->n_adaptors; i_adaptor++ )
	{
		if ( !strcmp(type, p_me->p_adaptors[i_adaptor].name) ) n_type++;
	}
	return n_type;

}


/*---------------------------------------------------------------------------*
 |"m_XV_Select_select_port"
 |
 |  Select a port given an adaptor type, instance and port number. 
 |
 |    -> O_XV object on success, 0 if error occurs.
 *---------------------------------------------------------------------------*/
O_XV* m_XV_Select_select_port( p_me, type, i_adapt, i_port )
	O_XV_Select*	p_me;	 /* -> object context. */
	char*			type;	 /* -> type of adaptor, or 0 to indicate linear. */
	int				i_adapt; /* =  instance of adaptor type, or # if type = 0.*/
	int				i_port;	 /* =  # of port on adaptor. */
{
	int		i_adaptor;  /* # of adaptor in linear. */
	O_XV*	p_oxv;		/* O_XV object returning. */


	/*  NO TYPE GIVEN, USE i_adapt AS AN INDEX INTO THE LIST OF ADAPTORS. */

	if ( !type )
	{
		if ( i_adapt >= p_me->n_adaptors )
		{
			fprintf(stderr, "There are only %d adaptors and you want %d\n",
				p_me->n_adaptors, i_adapt
			);
			return 0;
		}
		else if ( i_port >= p_me->p_adaptors[i_adapt].num_ports )
		{
			fprintf(stderr, "There are only %d ports and you want %d\n",
				p_me->p_adaptors[i_adapt].num_ports, i_port
			);
			return 0;
		}
		else
		{
			if ( !(p_oxv = p_me->p_oxv_adaptors[i_adapt].p_ports[i_port])  )
			{
				p_oxv = p_me->p_oxv_adaptors[i_adapt].p_ports[i_port] =
				    O_XV_O_XV(0, p_me->p_display, i_adapt, i_port);
			}
		}
		return p_oxv;
	}

	
	/*  A TYPE WAS GIVEN, USE i_adapt AS THE # OF THE INSTANCE OF ADAPTOR
	 *  OF TYPE type.
	 */
	else
	{
		for ( i_adaptor = 0; i_adaptor < p_me->n_adaptors; i_adaptor++ )
		{
			if ( !strcmp( p_me->p_adaptors[i_adaptor].name, type) )
			{
				if ( --i_adapt <= 0 ) 
				{
					if ( i_port >= p_me->p_adaptors[i_adaptor].num_ports )
					{
						fprintf(stderr, 
							"There are only %d ports and you want %d\n",
							p_me->p_adaptors[i_adaptor].num_ports, i_port
						);
						return 0;
					}
					else if ( !(p_oxv = 
						p_me->p_oxv_adaptors[i_adaptor].p_ports[i_port])
					)
					{
						p_oxv = 
							p_me->p_oxv_adaptors[i_adaptor].p_ports[i_port] =
							O_XV_O_XV(0, p_me->p_display, i_adaptor, i_port);
					}
					return p_oxv;
				}
			}
		}

		return 0;
	}
}


/*---------------------------------------------------------------------------*
 |"m_XV_Select_terminate"
 |
 |  Terminate use of an object, freeing its dynamic storage.
 *---------------------------------------------------------------------------*/
void m_XV_Select_terminate( p_me, fd )
	O_XV_Select*	p_me;	/* -> object context. */
{
	int		i_adaptor;	/* Entry in p_oxv_adaptors now accessing. */
	int		i_port;		/* Entry in p_oxv_adaptors.p_ports now accessing. */

	for ( i_adaptor = 0; i_adaptor < p_me->n_adaptors; i_adaptor++ )
	{
		for ( i_port=0; i_port<p_me->p_adaptors[i_adaptor].num_ports; i_port++ )
		{
			if ( p_me->p_oxv_adaptors[i_adaptor].p_ports[i_port]  )
			{
			    O_XV_Terminate(p_me->p_oxv_adaptors[i_adaptor].p_ports[i_port]);
			}
		}
		free( (char*)p_me->p_oxv_adaptors[i_adaptor].p_ports);
	}

	free( (char*)p_me->p_oxv_adaptors);
	free((char*)p_me->p_adaptors);
	free((char*)p_me);
}


static Tv_XV_Select O_XV_Select_methods = {
	m_XV_Select_dump,
	m_XV_Select_query_type,
	m_XV_Select_select_port,
	m_XV_Select_terminate
};


/*---------------------------------------------------------------------------*
 |"O_XV_Select_O_XV_Select"
 |
 |  Create and return an XV adaptor selection object.
 |
 |    -> object on success, 0 if error detected
 *---------------------------------------------------------------------------*/
O_XV_Select* O_XV_Select_O_XV_Select( user_p_me, p_display )
	O_XV_Select*	user_p_me;	/* -> object context to fill in, or 0. */
	Display*		p_display;	/* -> display to acquire Xv adpator info on. */
{
	int		        i_adaptor;	/* Entry in p_oxv_adaptors now accessing. */
	O_XV_Select*	p_me;		/* Object context to fill in. */
	int				status;		/* Return code from function calls. */

	if ( user_p_me )
	{
		p_me = user_p_me;
	}
	else
	{
		if ( !(p_me = (O_XV_Select*)malloc(sizeof(O_XV_Select))) )
		{
			return 0;
		}
	}

	p_me->tv = &O_XV_Select_methods;


    /*  MAKE SURE Xv VIDEO EXTENSION IS PRESENT, AND GET THE ADAPTOR LIST. */
     
    p_me->screen = XDefaultScreen(p_display);
	p_me->p_display = p_display;
    p_me->w_root = XDefaultRootWindow(p_display);
    status = XvQueryExtension(p_me->p_display,
        &p_me->xv_version, &p_me->xv_revision, &p_me->xv_major_op,
        &p_me->xv_event_base, &p_me->xv_error_base
	);
    if (status != Success)
    {
        fprintf(stderr, "O_XV_Select_O_XV_Select: no Xv extension\n");
        if ( !user_p_me ) free(p_me);
        return 0;
    }


    XvQueryAdaptors(p_me->p_display, p_me->w_root, &p_me->n_adaptors,
        &p_me->p_adaptors);

	p_me->p_oxv_adaptors = (O_XV_Sel_Adapt*)
		malloc(sizeof(O_XV_Sel_Adapt)*p_me->n_adaptors);

	for ( i_adaptor = 0; i_adaptor < p_me->n_adaptors; i_adaptor++ )
	{
		p_me->p_oxv_adaptors[i_adaptor].p_ports = 
			(O_XV**)malloc(sizeof(O_XV*)*p_me->p_adaptors[i_adaptor].num_ports);
		memset(p_me->p_oxv_adaptors[i_adaptor].p_ports, 0, 
			sizeof(O_XV*)*p_me->p_adaptors[i_adaptor].num_ports); 
	}
	return p_me;
}
