/*
 * xvtiming.c:    Test timing of various XV operations.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     26 May 93 - Andy Goodrich
 *       - Initial coding
 *
 * $Log:   /plxhp/dave/source/O_XV/archives/xvtiming.c_v  $
 * 
 *    Rev 1.0   12 Sep 1996 11:54:58   dave
 * init
 *
 *  Copyright(c) 1995, Parallax Graphics
 */
#include <sys/types.h>          /* Standard types for this C environment. */
#include <stdio.h>              /* Stream input/output declarations. */
#include <sys/time.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <Xvlib.h>
#include "Xvddx.h"
#include "O_XV.h"
#include "rasterfile.h"

/*  FORWARD REFERENCES: */

void timer_off();
void timer_on();


enum {
	CMD_Help = 0,
	CMD_XGetImage,
	CMD_XvGetImage_JPEG,
	CMD_XvGetImage_XImage,
	CMD_XPutImage,
	CMD_XvPutImage_JPEG,
	CMD_XvPutImage_XImage
};

typedef struct _Timer_Cb {
	struct timeval  end_time;	/* Ending time of day for test. */
	int			 	n_loops;	/* # of loops performed. */
	struct timeval  start_time; /* Starting time of day for test. */
} Timer_Cb;

/*----------------------------------------------------------------------------*
 |"build_image"
 |
 |  Build an image to use for Put Image tests.
 |
 |    -> image on success
 |    =  0 if error detected
 *----------------------------------------------------------------------------*/
XImage* build_image( p_xv, width, height )
	O_XV*	p_xv;	/* -> xv access object using. */
    int     width;  /* =  width of XImage to create. */
    int     height; /* =  height of XImage to create. */
{
    int     i_col;      /* Column now processing. */
    uint*   im_data;   /* Data for image. */
    uint*   im_pixel;  /* Pixel within im_data accessing. */
    int     i_pixel;    /* Number of pixels in im_data left to process. */
    int     i_row;      /* Row now processing. */


    /* ALLOCATE STORAGE FOR THE DATA AND CREATE THE GRID
     *  (1) Clear the background.
     *  (2) Generate the horizontal lines (including one at height-1)
     *  (3) Generate the vertical lines (including one at width-1)
     */
    if ( (im_data = (u_int)malloc(width*height*4)) == 0 ) return 0;
     
    for ( i_pixel = width*height, im_pixel = im_data; i_pixel; i_pixel-- )
    {
        *im_pixel++ = 0x00000000;
    }
       
    for ( i_row = 0; i_row < height; i_row += 40 )
    {  
        for ( i_col = 0; i_col < width; i_col++ )
        {
            im_data[i_row*width+i_col] = 0x00ffffff;
        }
    }
 
    for ( i_col = 0; i_col < width; i_col += 40 )
    {
        for ( i_row = 0; i_row < height; i_row++ )
        {
            im_data[i_row*width+i_col] = 0x00ffffff;
        }
    }
 
    /*
     *  CREATE THE XIMAGE AND RETURN IT TO OUR CALLER
     */
    return XCreateImage(p_xv->display, 
        XDefaultVisual(p_xv->display, XDefaultScreen(p_xv->display)),
		32, ZPixmap, 
		0, im_data, width, height, 32, 0);
}


/*----------------------------------------------------------------------------*
 |"main"
 |
 |  Entry point for system for this application.
 |
 *----------------------------------------------------------------------------*/
main( argc, argv )
    int     argc;   /* = # of command arguments parsed by shell. */
    char*   argv[]; /* = vector of pointers to command arguments parsed. */
{
    Atom        a_image;    /* Atom designating an image type we will get. */
	int         cmd;		/* Command to execute. */
    XEvent      event;      /* Information about X event we intercepted. */
    FILE*       fd;         /* File descriptor to read image from. */
    int         height;     /* Height of image to display. */
    int         i_argv;     /* # of element in 'argv' accessing. */
    int         i_file;     /* Index into argv of name of file to open. */
    int         i_loop;     /* # of this time around timing loop. */
	int			n_image;	/* # of bytes in image returned by XvGetImage. */
	int			n_loops;	/* # of timing loops to do. */
    int         output;     /* = 1 if should output to tv. */
    Display*    p_display;  /* Display accessing. */
    XImage*     p_image;    /* X image to display. */
    O_XV*       p_xv;       /* Xv access object. */ 
    O_XV*       p_xv1;      /* Xv access object. */ 
	Timer_Cb	timer_cb;	/* Timer control block using. */
    int         width;      /* Width of image to display. */
    

    /*  PROCESS OUR COMMAND PARAMETERS: 
     *
     *    An unrecognized keyword is assumed to be the file to be displayed.
     */
	cmd = CMD_Help;
	n_loops = 1000;
    output = 0;
    i_file = 0;
	width = 640;
	height = 480;
    for ( i_argv = 1; i_argv < argc; i_argv++ )
    {
        if ( !strcmp(argv[i_argv], "xgetimage") )
        {
            cmd = CMD_XGetImage;
        }
        else if ( !strcmp(argv[i_argv], "xvgetimage") )
        {
            cmd = CMD_XvGetImage_XImage;
        }
        else if ( !strcmp(argv[i_argv], "xvgetjpeg") )
        {
            cmd = CMD_XvGetImage_JPEG;
        }
        else if ( !strcmp(argv[i_argv], "xputimage") )
        {
            cmd = CMD_XPutImage;
        }
        else if ( !strcmp(argv[i_argv], "xvputimage") )
        {
            cmd = CMD_XvPutImage_XImage;
        }
        else if ( !strcmp(argv[i_argv], "xvputjpeg") )
        {
            cmd = CMD_XvPutImage_JPEG;
        }
        else if ( !strcmp(argv[i_argv], "-geom") )
        {
			if ( ++i_argv >= argc || sscanf(argv[i_argv], "%d", &width) != 1 )
			{
				fprintf(stderr, "You must specify a width with -geom\n");
				return 0;
			}
			if ( ++i_argv >= argc || sscanf(argv[i_argv], "%d", &height) != 1 )
			{
				fprintf(stderr, "You must specify a height with -geom\n");
				return 0;
			}
        }
        else if ( !strcmp(argv[i_argv], "-loops") )
        {
			if ( ++i_argv >= argc || sscanf(argv[i_argv], "%d", &n_loops) != 1 )
			{
				fprintf(stderr, "You must specify a loop count with -loops\n");
				return 0;
			}
		}
        else if ( !strcmp(argv[i_argv], "-output") )
        {
            output = 1;
        }
        else
        {
            i_file = i_argv;
        }
    }

	if ( cmd == CMD_Help )
    {
        fprintf(stderr, "Usage: xvshow CMD [-geom W H] [-loops N]\n");
		fprintf(stderr, "  where CMD is one of the following: \n");
		fprintf(stderr, "     xgetimage xputimage xvgetimage xvputimage\n");
		fprintf(stderr, "     xvgetjpeg xvputjpeg\n");
        return 0;
    }


    /*  OPEN THE X DISPLAY AND CREATE AN XV ACCESS OBJECT FOR PORT 0. */

    if ( !(p_display = XOpenDisplay(0)) ) 
    {
        fprintf(stderr, "\nCould not open display\n");
        return 0;
    }

    if ( !(p_xv = O_XV_O_XV( 0, p_display, 0, 0 )) )
    {
        fprintf(stderr, "\nCould create O_XV object\n");
        return 0;
    }


    /*  SET UP WINDOW WE WILL BE USING. */

    O_XV_Window( p_xv, -1, -1 );
    p_xv->win_width = width;
    p_xv->win_height = height;
	O_XV_Video_On(p_xv);
	O_XV_Video_Off(p_xv);
	XFlush(p_display);


    /*  PROCESS BASED ON THE TYPE OF TIMING REQUESTED. */

	timer_cb.n_loops = n_loops;
    switch( cmd )
	{
	  case CMD_XGetImage:
		timer_on(&timer_cb);
		printf("\nPerforming %d XGetImage calls with XImage size %d x %d\n",
			n_loops, width, height);
		for ( i_loop = 0; i_loop < n_loops; i_loop++ )
		{
         	p_image = XGetImage(p_xv->display, p_xv->w_main, 
				0, 0, width, height, -1, ZPixmap);
			XDestroyImage(p_image);
		}
		timer_off(&timer_cb);
		break;

#ifndef AIX
	  case CMD_XvGetImage_JPEG:
		a_image = XInternAtom(p_xv->display,"XV_IMAGE_JPEG",False);
		timer_on(&timer_cb);
		printf("\nPerforming %d XvGetImage calls with JPEG size %d x %d\n",
			n_loops, width, height);
		for ( i_loop = 0; i_loop < n_loops; i_loop++ )
		{
			XvGetImage(p_xv->display, p_xv->port_id, p_xv->w_main, p_xv->gc, 
			   a_image, &n_image, &p_image, 0, 0, width, height, 
			   0, 0, width, height);
			free(p_image);
		}
		timer_off(&timer_cb);
		break;

	  case CMD_XvGetImage_XImage:
		a_image = XInternAtom(p_xv->display,"XV_IMAGE_XIMAGE",False);
		timer_on(&timer_cb);
		printf("\nPerforming %d XvGetImage calls with XImage size %d x %d\n",
			n_loops, width, height);
		for ( i_loop = 0; i_loop < n_loops; i_loop++ )
		{
			XvGetImage(p_xv->display, p_xv->port_id, p_xv->w_main, p_xv->gc, 
			   a_image, &n_image, &p_image, 0, 0, width, height, 
			   0, 0, width, height);
			XDestroyImage(p_image);
		}
		timer_off(&timer_cb);
		break;
#endif /* !AIX */

	  case CMD_XPutImage:
		printf("\nPerforming %d XPutImage calls with XImage size %d x %d\n",
			n_loops, width, height);
		p_image = build_image(p_xv, width, height);
		timer_on(&timer_cb);
		for ( i_loop = 0; i_loop < n_loops; i_loop++ )
		{
			XPutImage(p_xv->display, p_xv->w_main, p_xv->gc, p_image, 0, 0, 
				0, 0, width, height);
		}
		timer_off(&timer_cb);
		break;

#ifndef AIX
	  case CMD_XvPutImage_XImage:
		a_image = XInternAtom(p_xv->display,"XV_IMAGE_XIMAGE",False);
		printf("\nPerforming %d XvPutImage calls with XImage size %d x %d\n",
			n_loops, width, height);
		p_image = build_image(p_xv, width, height);
		timer_on(&timer_cb);
		for ( i_loop = 0; i_loop < n_loops; i_loop++ )
		{
			XvPutImage(p_xv->display, p_xv->port_id, p_xv->w_main, p_xv->gc, 
				a_image, 0, p_image, 0, 0, width, height, 0, 0, width, height);
		}
		timer_off(&timer_cb);
		break;
#endif /* !AIX */
	}
}


/*----------------------------------------------------------------------------*
 |"timer_off"
 |
 |  Timing complete, print out delta, etc. 
 |
 *----------------------------------------------------------------------------*/
void timer_off( p_timer_cb )
	Timer_Cb*	p_timer_cb;	/* -> timer information. */
{
	int		d_sec;
	int		d_usec;
	int		time_per_op;

	gettimeofday( &p_timer_cb->end_time, 0 );
	d_sec =	p_timer_cb->end_time.tv_sec - p_timer_cb->start_time.tv_sec,
	d_usec = p_timer_cb->end_time.tv_usec - p_timer_cb->start_time.tv_usec;
	if ( d_usec < 0 )
	{
		d_usec += 1000000;
		d_sec--;
	}
	printf("  Total time %d.%06d seconds\n", d_sec, d_usec);
	time_per_op = ((d_sec*1000000) + d_usec) / p_timer_cb->n_loops;
	printf(" or %d.%06d seconds per operation, or %f operations per second\n", 
		time_per_op / 1000000, time_per_op % 1000000, 
		1000000.0 / (double) time_per_op);
}


/*----------------------------------------------------------------------------*
 |"timer_on"
 |
 |  Initiate timing.
 |
 *----------------------------------------------------------------------------*/
void timer_on( p_timer_cb )
	Timer_Cb*	p_timer_cb;	/* -> timer information. */
{
	gettimeofday( &p_timer_cb->start_time, 0 );
}
