/*
 *  bt463_lut.c: Support for Brooktree 463 Lookup Table Object.
 *
 *   The Bt463 is a RAMDAC providing 528 lookup table locations and window
 *   identifier support. The lookup table is 24 bits wide providing 8-bits
 *   of intensity information for red, green, and blue. This objects supports
 *   three visual types based on the two low-order tag bits provided to the
 *   RAMDAC (the upper two tag bits are used for cursor support):
 *   
 *      Bits  Definition
 *      ----  ----------
 *       00 - 8-bit pseudo color, the lookup table operates with 8-in 24-out
 *            semantics.
 *       01 - 24-bit direct color, the lookup table operates as three 8-in
 *            8-out tables, one for each color component (rgb).
 *       10 - 24-bit true color, the lookup table entries are bypassed.
 *       11 - 24-bit true color, the lookup table entries are bypassed.
 *
 *  These bits are provided in the bottom byte of the type indicator for
 *  get_entries and set_entries.
 *
 * (c) Copyright 1995 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 * $Log:   /plxhp/dave/pvcsrc/hwlib/archives/bt463_lut.c_v  $
 * 
 *    Rev 1.1   09 Sep 1996 10:08:40   dave
 * Initial revision
 * Revision 1.1  1995/12/11 20:42:44  dist
 * Initial revision
 *
 */
#include <stdio.h>			/* Pickup stdout and friends. */
#include <sys/types.h>		/* Pick up u_int and friends. */
#include "objects.h"		/* Pick up object definitions. */
#include "video_stream.h"	/* Pick up Parallax hardware registers defs. */
#include "hw_context.h"		/* Pick up Parallax hardware library context. */
#include "xvdebug.h"		/* Pick up XV debugging macros. */


/*  INTERNAL OBJECT DEFINITION FOR BROOKTREE 463 OBJECT: */

typedef struct _Bt463_Wid {
	unsigned int unused:8;		/* 24-31: unused. */
	unsigned int bypass:1;		/* 23:    bypass lut if = 1. */
	unsigned int startaddr:6;	/* 17-22: lookup table starting address. */
	unsigned int ovlymsk:4;		/* 13-16: overlay mask bits. */
	unsigned int ovlyloc:1;		/* 12:    overlay location. */
	unsigned int mode:3;		/* 9-11:  mode of operation. */
	unsigned int planes:4;		/* 5-8:   # of planes. */
	unsigned int shift:5;		/* 0-4:   pixel shift. */
} Bt463_Wid;

typedef struct _O_Lut_Bt463 {
	TV_Lut*		atv;		  /* Methods: first entry in all derived objects. */
	XV_struct*	p_xv;		  /* Parallax hardware library context structure. */
	int			cmd_regs[3];  /* Command register values to use. */
	Bt463_Wid	wid_tab[16];  /* Current window id table entries in use. */
} O_Lut_Bt463;


enum {					/* Bt463 register addresses: */
	Bt463_CR0 = 0x312,	/* Command register 0. */
	Bt463_CR1 = 0x313	/* Command register 0. */
};


#define Bt463_Set_Address(p_xv, addr) \
{ \
	PV_WREG(p_xv, Bt.red.reg0, (addr) & 0xff ); \
	PV_WREG(p_xv, Bt.red.reg1, ((addr) >> 8) & 0xff); \
}

#define Bt463_Wid_Entry( shft, planes, mode, ovlyloc, ovlymsk, strtad, bypass) \
	(shft & 0x1f) | ((planes  & 0xf)  << 5) ((mode    & 0x7) << 9)  | \
    ((ovlyloc & 0x1)  << 12) ((ovlymsk & 0xf) << 13) | \
	(((strtad >> 4)  & 0x3f) << 17) | ((bypass & 0x1) << 23) 


/* FORWARD FUNCTION REFERENCES: */

static int  m_Lut_Bt463_control(/* p_me, op, arg */);
static void m_Lut_Bt463_dump(/* p_me, fd */);
static int  m_Lut_Bt463_get_entries(/* p_me, start, count, p_value */);
static int  m_Lut_Bt463_init_hw(/* p_me */);
static int  m_Lut_Bt463_set_entries(/* p_me, start, count, p_value */);
static void m_Lut_Bt463_terminate(/* p_me */);

/*  SOME STANDARD COMMAND REGISTER SETTINGS: */

static int cmd_reg_normal[] = {
	0x40, /* 4:1 mux, blink xx */
	0x40, /* overlay common, 24 planes, 4 over 16 tables, no under, no cursor */
	0x40  /* no sync ,pedistal */
};

static int cmd_reg_8_over_24[] = {
	0x40, /* 4:1 mux, blink xx */
	0x10, /* #### */
	0x40  /* no sync ,pedistal */
};


/* SOME STANDARD WINDOW ID TABLES: */

/* bypass lookup table for all window ids. */
static Bt463_Wid wid_table_bypass[]={   /*  #  visual:       lookup table: */ 
	{0, 1, 0>>4, 0, 0, 0, 8, 0},        /*  0: true color    bypass. */
	{0, 1, 0>>4, 0, 0, 0, 8, 0},        /*  1: true color    bypass. */
	{0, 1, 0>>4, 0, 0, 0, 8, 0},        /*  2: true color    bypass. */
	{0, 1, 0>>4, 0, 0, 0, 8, 0},        /*  3: true color    bypass. */
	{0, 1, 0>>4, 0, 0, 0, 8, 0},        /*  4: true color    bypass. */
	{0, 1, 0>>4, 0, 0, 0, 8, 0},        /*  5: true color    bypass. */
	{0, 1, 0>>4, 0, 0, 0, 8, 0},        /*  6: true color    bypass. */
	{0, 1, 0>>4, 0, 0, 0, 8, 0},        /*  7: true color    bypass. */
	{0, 1, 0>>4, 0, 0, 0, 8, 0},        /*  8: true color    bypass. */
	{0, 1, 0>>4, 0, 0, 0, 8, 0},        /*  9: true color    bypass. */
	{0, 1, 0>>4, 0, 0, 0, 8, 0},        /* 10: true color    bypass. */
	{0, 1, 0>>4, 0, 0, 0, 8, 0},        /* 11: true color    bypass. */
	{0, 1, 0>>4, 0, 0, 0, 8, 0},        /* 12: true color    bypass. */
	{0, 1, 0>>4, 0, 0, 0, 8, 0},        /* 13: true color    bypass. */
	{0, 1, 0>>4, 0, 0, 0, 8, 0},        /* 14: true color    bypass. */
	{0, 1, 0>>4, 0, 0, 0, 8, 0},        /* 15: true color    bypass. */
};

static Bt463_Wid wid_table_normal[]={  /*  #  visual:       lookup table: */ 
	{0, 0, 0>>4, 0, 0, 0, 8, 0},       /*  0: pseudo color  0-ff. */
	{0, 1, 0>>4, 0, 0, 0, 8, 0},       /*  1: true color    bypass. */
	{0, 1, 0>>4, 0, 0, 0, 8, 0},       /*  2: true color    bypass. */
	{0, 1, 0>>4, 0, 0, 0, 8, 0},       /*  3: true color    bypass. */

	{0, 0, 0>>4, 0, 0, 0, 8, 0},       /*  4: pseudo color  0-ff. */
	{0, 1, 0>>4, 0, 0, 0, 8, 0},       /*  5: true color    bypass. */
	{0, 1, 0>>4, 0, 0, 0, 8, 0},       /*  6: true color    bypass. */
	{0, 1, 0>>4, 0, 0, 0, 8, 0},       /*  7: true color    bypass. */

	{0, 0, 0x100>>4, 0, 0, 0, 1, 27},  /*  8: cursor        foreground. */
	{0, 0, 0x100>>4, 0, 0, 0, 1, 27},  /*  9: cursor        foreground. */
	{0, 0, 0x100>>4, 0, 0, 0, 1, 27},  /* 10: cursor        foreground. */
	{0, 0, 0x100>>4, 0, 0, 0, 1, 27},  /* 11: cursor        foreground. */

	{0, 0, 0x110>>4, 0, 0, 0, 1, 27},  /* 12: cursor        foreground. */
	{0, 0, 0x110>>4, 0, 0, 0, 1, 27},  /* 13: cursor        foreground. */
	{0, 0, 0x110>>4, 0, 0, 0, 1, 27},  /* 14: cursor        foreground. */
	{0, 0, 0x110>>4, 0, 0, 0, 1, 27},  /* 15: cursor        foreground. */
};

/* 8 over 24 no shift, 8 planes, true color, all overlay enab, std overlay. */
static Bt463_Wid wid_table_8_over_24[]={/*  #  visual:       lookup table: */ 
	{0, 0, 0x100>>4, 0xf, 0, 0, 8, 0},  /*  0: true color    0x100-0x1ff. */
	{0, 0, 0x100>>4, 0xf, 0, 0, 8, 0},  /*  1: true color    0x100-0x1ff. */
	{0, 0, 0x100>>4, 0xf, 0, 0, 8, 0},  /*  2: true color    0x100-0x1ff. */
	{0, 0, 0x100>>4, 0xf, 0, 0, 8, 0},  /*  3: true color    0x100-0x1ff. */
	{0, 0, 0x100>>4, 0xf, 0, 0, 8, 0},  /*  4: true color    0x100-0x1ff. */
	{0, 0, 0x100>>4, 0xf, 0, 0, 8, 0},  /*  5: true color    0x100-0x1ff. */
	{0, 0, 0x100>>4, 0xf, 0, 0, 8, 0},  /*  6: true color    0x100-0x1ff. */
	{0, 0, 0x100>>4, 0xf, 0, 0, 8, 0},  /*  7: true color    0x100-0x1ff. */
	{0, 0, 0x100>>4, 0xf, 0, 0, 8, 0},  /*  8: true color    0x100-0x1ff. */
	{0, 0, 0x100>>4, 0xf, 0, 0, 8, 0},  /*  9: true color    0x100-0x1ff. */
	{0, 0, 0x100>>4, 0xf, 0, 0, 8, 0},  /* 10: true color    0x100-0x1ff. */
	{0, 0, 0x100>>4, 0xf, 0, 0, 8, 0},  /* 11: true color    0x100-0x1ff. */
	{0, 0, 0x100>>4, 0xf, 0, 0, 8, 0},  /* 12: true color    0x100-0x1ff. */
	{0, 0, 0x100>>4, 0xf, 0, 0, 8, 0},  /* 13: true color    0x100-0x1ff. */
	{0, 0, 0x100>>4, 0xf, 0, 0, 8, 0},  /* 14: true color    0x100-0x1ff. */
	{0, 0, 0x100>>4, 0xf, 0, 0, 8, 0},  /* 15: true color    0x100-0x1ff. */
};


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt463_control"
 |
 |  Perform a device-dependent control operation.
 |
 |    = 1 if success, 0 if error detected.
 *---------------------------------------------------------------------------*/
static int m_Lut_Bt463_control( p_me, op, arg )
	O_Lut_Bt463* p_me;	/* -> context for this object. */
	char*		 op;	/* -> string specifying operation to perform. */
	int			 arg;	/* =  optional argument for operation. */
{
	if ( !strcmp(op, "Normal_Mode") )
	{
		memcpy(p_me->cmd_regs, &cmd_reg_normal, sizeof(cmd_reg_normal));
		memcpy(p_me->wid_tab, &wid_table_normal, sizeof(wid_table_normal));
		m_Lut_Bt463_init_hw(p_me);
	}

	else if ( !strcmp(op, "Bypass_Mode") )
	{
		memcpy(p_me->cmd_regs, &cmd_reg_normal, sizeof(cmd_reg_normal));
		memcpy(p_me->wid_tab, &wid_table_bypass, sizeof(wid_table_bypass));
		m_Lut_Bt463_init_hw(p_me);
	}

	else if ( !strcmp(op, "8_Over_24_Mode") )
	{
		memcpy(p_me->cmd_regs, &cmd_reg_8_over_24, sizeof(cmd_reg_8_over_24));
		memcpy(p_me->wid_tab, &wid_table_8_over_24,sizeof(wid_table_8_over_24));
		m_Lut_Bt463_init_hw(p_me);
	}

	else
	{
		ERRORF((fd, "O_Lut_Bt463_control: unrecognized option %d\n", op));
		return 0;
	}
	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt463_dump"
 |
 |  Dump the contents of this object to a file descriptor.
 *---------------------------------------------------------------------------*/
static void m_Lut_Bt463_dump( p_me, fd )
	O_Lut_Bt463* p_me;	/* -> context for this object. */
	FILE*		 fd;	/* -> file descriptor to write object contents to. */
{
	int		i_wid;	/* Element in wid table now accessing. */
	union {
		int			i;
		Bt463_Wid	wid;
	}		wid_value;

	fprintf(fd, "\nBrooktree 463 object @ 0x%08x: \n", p_me);
	fprintf(fd, "   Hardware library context at 0x%08x\n", p_me->p_xv);
	fprintf(fd, "   Command register values:  0     1     2\n");
	fprintf(fd, "                            0x%02x  0x%02x  0x%02x\n",
		p_me->cmd_regs[0], p_me->cmd_regs[1], p_me->cmd_regs[2]);
	fprintf(fd,  "  Window id values: \n");
	fprintf(fd,  
		"    Entry     value   bypass startaddr ovlymsk ovlyloc mode planes shift\n");
	for ( i_wid=0; i_wid < sizeof(wid_table_normal)/sizeof(Bt463_Wid); i_wid++ )
	{
		wid_value.wid = p_me->wid_tab[i_wid];
		fprintf(fd,  
			"     %3d    %08x    %s    0x%04x    0x%x      %d    0x%x   %2d     %2d \n",
			i_wid, wid_value.i, p_me->wid_tab[i_wid].bypass ? "Yes" : "No ",
			p_me->wid_tab[i_wid].startaddr<<4, p_me->wid_tab[i_wid].ovlymsk, 
			p_me->wid_tab[i_wid].ovlyloc, p_me->wid_tab[i_wid].mode, 
			p_me->wid_tab[i_wid].planes, p_me->wid_tab[i_wid].shift
		);
	}
		
	/*  VERIFY TRANSFER VECTOR IS CORRECT. */

	if ( p_me->atv->control != m_Lut_Bt463_control ) 
	{
		fprintf(fd, 
			"  *** control method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->control, m_Lut_Bt463_control);
	}
	else if ( p_me->atv->dump != m_Lut_Bt463_dump ) 
	{
		fprintf(fd, 
			"  *** dump method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->dump, m_Lut_Bt463_dump);
	}
	if ( p_me->atv->get_entries != m_Lut_Bt463_get_entries ) 
	{
		fprintf(fd, 
			"  *** get_entries method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->get_entries, m_Lut_Bt463_get_entries);
	}
	if ( p_me->atv->init_hw != m_Lut_Bt463_init_hw ) 
	{
		fprintf(fd, 
			"  *** init_hw method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->init_hw, m_Lut_Bt463_init_hw);
	}
	if ( p_me->atv->set_entries != m_Lut_Bt463_set_entries ) 
	{
		fprintf(fd, 
			"  *** set_entries method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->set_entries, m_Lut_Bt463_set_entries);
	}
	if ( p_me->atv->terminate != m_Lut_Bt463_terminate ) 
	{
		fprintf(fd, 
			"  *** terminate method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->terminate, m_Lut_Bt463_terminate);
	}
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt463_get_entries"
 |
 |  Return the values of a range of lookup table entries. There are three 
 |  8-in 8-out lookup tables, one for each color component. The values 
 |  pointed to by p_values are integers of the form XBGR.
 |
 |    = 1 on success, 0 if error detected.
 *---------------------------------------------------------------------------*/
static int m_Lut_Bt463_get_entries( p_me, type, start, count, p_values )
	O_Lut_Bt463*	p_me;		/* -> context for this object. */
	int				type;		/* =  type of entries (see objects.h). */
	int				start;		/* =  first location in table to return. */
	int				count;		/* =  # of entries to return. */
	int*			p_values;	/* -> where to place entries being returned. */
{
	int			blue;		/* Blue component of value. */
	int			green;		/* Green component of value. */
	int			i_entry;	/* # of entry in lut now getting. */
	int*		p_entry;	/* Data for entry in lut now getting. */
	XV_struct*	p_xv;		/* Fast access to hardware context: p_me->p_xv. */
	int			red;		/* Red component of value. */

	p_xv = p_me->p_xv;
	switch ( type & O_Lut_Entry_Type_Mask )
	{
	  case O_Lut_Entry_Lut:
		p_entry = p_values;
        Bt463_Set_Address(p_xv, start);
		for ( i_entry = start; i_entry < start+count; i_entry++ )
		{
            PV_RREG(p_xv, red, Bt.red.reg3);
            PV_RREG(p_xv, green, Bt.red.reg3);
            PV_RREG(p_xv, blue, Bt.red.reg3);
            *p_entry++ = red | ( green << 8 ) | ( blue << 16 );
        }
		break;
	  case O_Lut_Entry_Wid:
	    PV_WREG(p_xv, DispKludge.b.disp_blank, 1);
		p_entry = p_values;
		Bt463_Set_Address(p_xv, 0x300 | (start & 0xf));
		for ( i_entry = start; i_entry < start+count; i_entry++, p_entry++ )
		{
			PV_RREG(p_xv, red, Bt.red.reg2);
			PV_RREG(p_xv, green, Bt.red.reg2);
			PV_RREG(p_xv, blue, Bt.red.reg2);
			*p_entry = (blue << 16) | (green << 8) | red;
		}
	    PV_WREG(p_xv, DispKludge.b.disp_blank, 0);
		break;
	  default:
		return 0;
	}
	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt463_init_hw"
 |
 |  Initialize hardware to its default state.
 |
 |    = 1 on success, 0 if error detected.
 *---------------------------------------------------------------------------*/
static int m_Lut_Bt463_init_hw( p_me )
	O_Lut_Bt463*	p_me;		/* -> context for this object. */
{
	int			i_entry;	/* # of entry in lut now setting. */
	XV_struct*	p_xv;	/* Fast access to hardware context: p_me->p_xv. */

	p_xv = p_me->p_xv;


	/*  SET UP THE CONTROL REGISTER VALUES. */

	Bt463_Set_Address(p_xv, 0x201);
	PV_WREG(p_xv,Bt.red.reg2, 0x40);    /* 4:1 mux , blink xx */
	PV_WREG(p_xv,Bt.red.reg2, 0x40);    /* overlay common, 24 planes, 4 over */
                                        /* 16 tables, no under, no cursor */
	PV_WREG(p_xv,Bt.red.reg2, 0x40);    /* no sync ,pedistal */


    /* LOAD READ MASK REGISTER TO RETURN ALL PLANES AND TURN OFF BLINKING. */

    Bt463_Set_Address(p_xv, 0x205);
    PV_WREG(p_xv,Bt.red.reg2, 0xff);
    PV_WREG(p_xv,Bt.red.reg2, 0xff);
    PV_WREG(p_xv,Bt.red.reg2, 0xff);
    PV_WREG(p_xv,Bt.red.reg2, 0xff);
 
    PV_WREG(p_xv,Bt.red.reg2, 0);
    PV_WREG(p_xv,Bt.red.reg2, 0);
    PV_WREG(p_xv,Bt.red.reg2, 0);
    PV_WREG(p_xv,Bt.red.reg2, 0);

 
	/* LOAD THE WINDOW IDENTIFIER FUNCTION TABLE ENTRIES. */

	O_Lut_set_entries(p_me, O_Lut_Entry_Wid, 0, 16, p_me->wid_tab);


	/* LOAD LINEAR RAMP INTO LOOKUP TABLE. */

	Bt463_Set_Address(p_xv, 0x000);
	for ( i_entry = 0; i_entry < 512; i_entry++ )
	{
		PV_WREG(p_xv,Bt.red.reg3, i_entry & 0xff);
		PV_WREG(p_xv,Bt.red.reg3, i_entry & 0xff);
		PV_WREG(p_xv,Bt.red.reg3, i_entry & 0xff);
	}
	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt463_set_entries"
 |
 |  Set the values of a range of lookup table entries. There are three 8-in
 |  8-out lookup tables, one for each color component. The values pointed to
 |  by p_values are integers of the form XBGR.
 |
 |    = 1 on success, 0 if error detected.
 *---------------------------------------------------------------------------*/
static int m_Lut_Bt463_set_entries( p_me, type, start, count, p_values )
	O_Lut_Bt463*	p_me;		/* -> context for this object. */
	int				type;		/* =  type of entries (see objects.h). */
	int				start;		/* =  first location in table to set. */
	int				count;		/* =  # of entries to set. */
	int*			p_values;	/* -> where to get values for entries. */
{
	int			i_entry;	/* # of entry in lut now setting. */
	int*		p_entry;	/* Data for entry in lut now setting. */
	XV_struct*	p_xv;		/* Fast access to h/w context: p_me->p_xv. */

	p_xv = p_me->p_xv;
	switch ( type & O_Lut_Entry_Type_Mask )
	{
	  case O_Lut_Entry_Cursor:
		p_entry = p_values;
		for ( i_entry = start; i_entry < start+count; i_entry++, p_entry++ )
		{
			Bt463_Set_Address(p_xv, 0x202 | (i_entry & 0xf));
			PV_WREG(p_xv,Bt.red.reg3, *p_entry & 0xff);
			PV_WREG(p_xv,Bt.red.reg3, (*p_entry >> 8)  & 0xff);
			PV_WREG(p_xv,Bt.red.reg3, (*p_entry >> 16) & 0xff);
		}
		break;
	  case O_Lut_Entry_Lut:
		p_entry = p_values;
		for ( i_entry = start; i_entry < start+count; i_entry++, p_entry++ )
		{
			Bt463_Set_Address(p_xv, 0x000 | (i_entry & 0xf));
			PV_WREG(p_xv,Bt.red.reg3, *p_entry & 0xff);
			PV_WREG(p_xv,Bt.red.reg3, (*p_entry >> 8)  & 0xff);
			PV_WREG(p_xv,Bt.red.reg3, (*p_entry >> 16) & 0xff);
		}
		break;
	  case O_Lut_Entry_Wid:
	    PV_WREG(p_xv, DispKludge.b.disp_blank, 1);
		p_entry = p_values;
		for ( i_entry = start; i_entry < start+count; i_entry++, p_entry++ )
		{
			Bt463_Set_Address(p_xv, 0x300 | (i_entry & 0xf));
			PV_WREG(p_xv,Bt.red.reg2, *p_entry & 0xff);
			PV_WREG(p_xv,Bt.red.reg2, (*p_entry >> 8)  & 0xff);
			PV_WREG(p_xv,Bt.red.reg2, (*p_entry >> 16) & 0xff);
		}
	    PV_WREG(p_xv, DispKludge.b.disp_blank, 0);
	  default:
		return 0;
	}
	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt463_terminate"
 |
 |  Terminate use of this object.
 *---------------------------------------------------------------------------*/
static void m_Lut_Bt463_terminate( p_me )
	O_Lut_Bt463*	p_me;		/* -> context for this object. */
{
	free((char*)p_me);
}


static TV_Lut methods_Lut_Bt463 = {
	m_Lut_Bt463_control,
	m_Lut_Bt463_dump,
	m_Lut_Bt463_get_entries,
	m_Lut_Bt463_init_hw,
	m_Lut_Bt463_set_entries,
	m_Lut_Bt463_terminate
};


/*---------------------------------------------------------------------------*
 |"O_Lut_Bt463_O_Lut_Bt463"
 |
 |  Create an instance of a Brooktree 463 lookup table object.
 |
 |    -> object's context on success, or 0 if error occurred.
 *---------------------------------------------------------------------------*/
O_Lut* O_Lut_Bt463_O_Lut_Bt463( p_me, p_xv )
	O_Lut_Bt463* p_me;		/* -> context to fill in, or 0 for dynamic alloc. */
	XV_struct*	 p_xv;		/* -> context for hardware library. */
{
	if ( !p_me )
	{
		if ( !(p_me = (O_Lut_Bt463*)malloc(sizeof(O_Lut_Bt463))) )
		{
			ERRORF((fd, 
				"O_Lut_Bt463_O_Lut_Bt463: Unable to allocate storage\n"));
		}
	}
	p_me->atv = &methods_Lut_Bt463;
	memcpy(p_me->cmd_regs, &cmd_reg_normal, sizeof(cmd_reg_normal));
	p_me->p_xv = p_xv;
	memcpy(p_me->wid_tab, &wid_table_normal, sizeof(wid_table_normal));

	return (O_Lut*)p_me;
}
