/*
 *  dmse_output.c: DMSE Output Object Implementation.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     25 Sep 94 - Andy Goodrich
 *       - Initial coding
 *
 *  Copyright(c) 1994, Parallax Graphics, Inc.
 *
 * $Log:   /plxhp/dave/pvcsrc/hwlib/archives/dmse_output.c_v  $
 * 
 *    Rev 1.2   09 Sep 1996 10:10:52   dave
 * shut off output on terminate
 * Revision 1.2  1996/04/20 00:11:32  dist
 * shut off output on terminate
 *
 * Revision 1.1  95/12/08  11:53:47  11:53:47  dist (Software Distribution Account)
 * Initial revision
 * 
 * Revision 1.4  95/05/25  17:11:53  17:11:53  dist ()
 * Andy Goodrich - HP 9.1 code freeze version.
 * 
 * Revision 1.2  95/04/17  12:29:06  12:29:06  dist ()
 * Andy Goodrich - added RCS header.
 * 
 */
#include <sys/types.h>		/* Standard types for this C environment. */
#include <stdio.h>			/* Stream input/output declarations. */
#include "objects.h"		/* PowerVideo 700 object defintions. */
#include "hw_context.h"		/* Pick up hardware context. */
#include "plxv_xpoint.h"	/* Cross point switch definitions. */

/* XV_DIAG VALUES:
 *
 *  0x20 - Device polling
 *  1    - Normal one-shot debugging
 */
#define XV_DIAG 1
#include "xvdebug.h"

/*
 *  DMSE OUTPUT OBJECT INTERNAL DEFINITION:
 */
typedef struct _I_Output_Dmse {
    TV_Output*  atv;            /* Methods: must be the first entry!! */
 
    int         dmse_address;       /* Address of dmse for this output. */
    int         port;               /* Unit number of this output. */
    XV_struct*  p_xv;               /* Hardware information. */
    char*       values_now;         /* Current dmse values in use. */
    char        values_ntsc[0x10];  /* Values for ntsc video. */
    char        values_pal[0x10];   /* Values for pal video. */
    int         xpoint_value;       /* Value for crosspoint switch. */
} I_Output_Dmse;


/*
 *  DMSE VALUES:
 */
static char dmse_addresses[2] = {0x58,0x58};

static XV_video_format output_format[2] = {
	{640,482,43,7,21},
	{768,574,51,7,34}
};


/*---------------------------------------------------------------------------*
 |"m_Output_Dmse_init_hw"
 |
 |  Disable an output.
 |
 *---------------------------------------------------------------------------*/
void m_Output_Dmse_init_hw( p_me )
	I_Output_Dmse*	p_me;		/* -> object context. */
{
	O_Rsrc_get_long(p_me->p_xv->p_rsrc, "a.dmse.ntsc", p_me->values_ntsc);
	O_Rsrc_get_long(p_me->p_xv->p_rsrc, "a.dmse.pal", p_me->values_pal);
	p_me->values_now = p_me->values_ntsc;

	p_me->p_xv->plxv_io_format[0] = &output_format[0];

	O_Output_set_type(p_me, "NTSC", "COMPOSITE", 0);
	O_Output_set_levels(p_me, 0x7f, 0x7f, 0x7f, 0x7f);
	O_Output_off(p_me);
}


/*---------------------------------------------------------------------------*
 |"m_Output_Dmse_off"
 |
 |  Disable an output.
 |
 *---------------------------------------------------------------------------*/
void m_Output_Dmse_off( p_me )
	I_Output_Dmse*	p_me;		/* -> object context. */
{
    int     bits;       /* Shadow value for register. */

    bits = p_me->p_xv->xv_shdw->b.Bt.grn.reg1;
    bits = bits & 0xfe;
    PV_WREG(p_me->p_xv,Bt.grn.reg1, bits);
    bits = p_me->p_xv->xv_shdw->b.Bt.grn.reg2;
	bits = (bits & 0xf9) | 0x4;
    PV_WREG(p_me->p_xv,Bt.grn.reg2, bits);
	plxv_xpoint_b(p_me->p_xv, CE_HIGH | HSYNC_HIGH | VSYNC_HIGH
		| ODD_HIGH  | ODD_INMODE_A | SHUTTLE_EN);
}


/*---------------------------------------------------------------------------*
 |"m_Output_Dmse_on"
 |
 |  Enable an output.
 |
 *---------------------------------------------------------------------------*/
void m_Output_Dmse_on( p_me )
	I_Output_Dmse*	p_me;		/* -> object context. */
{
    int     bits;       /* Shadow value for register. */

	bits = p_me->p_xv->xv_shdw->b.Bt.grn.reg1;
	bits = bits | 0x1;
	PV_WREG(p_me->p_xv,Bt.grn.reg1, bits);
	bits = p_me->p_xv->xv_shdw->b.Bt.grn.reg2;
	bits = (bits & 0xf9) | 0x2;
	PV_WREG(p_me->p_xv,Bt.grn.reg2, bits);
	plxv_xpoint_b(p_me->p_xv, p_me->xpoint_value);
}


/*---------------------------------------------------------------------------*
 |"m_Output_Dmse_set_levels"
 |
 |  Set color characteristics of video output.
 |
 *---------------------------------------------------------------------------*/
void m_Output_Dmse_set_levels( p_me, brightness, contrast, hue, saturation )
	I_Output_Dmse*	p_me;		/* -> object context. */
	int				brightness;	/* =  brightness level to set. */
	int				contrast;	/* =  contrast level to set. */
	int				hue;		/* =  hue level to set. */
	int				saturation;	/* =  satruation level to set. */
{
	int		i_entry;		/* Entry in dmse clt setting. */

    for( i_entry = 0; i_entry < 256; i_entry++ )
    {
		if ( O_I2C_write_indexed3(p_me->p_xv->p_i2c, p_me->dmse_address, 0x0,
			i_entry, i_entry, i_entry, i_entry) )
#ifdef NOTDEF
        if (plxv_i2cwi3(p_me->p_xv, p_me->dmse_address, 0x0, i_entry, i_entry, 
			i_entry, i_entry))
#endif
        { 
            ERRORF((fd, "m_Output_Dmse_set_levels: i2c write error on %d\n",
				i_entry));
            return;
        }
    }   
    O_I2C_write_indexed(p_me->p_xv->p_i2c, p_me->dmse_address, 0x02, 0x00, 
		p_me->values_now[0]);
#ifdef NOTDEF
    plxv_i2cwi(p_me->p_xv, p_me->dmse_address, 0x02, 0x00, p_me->values_now[0]);
#endif
}


/*---------------------------------------------------------------------------*
 |"m_Output_Dmse_set_register"
 |
 |  Set the value of a DMSE register.
 |
 |    = 1 if success, 0 if error detected
 *---------------------------------------------------------------------------*/
static int m_Output_Dmse_set_register( p_me, reg, value)
    I_Output_Dmse*  p_me;       /* -> object context. */
    int             reg;        /* =  register number to set value of. */
    int             value;      /* =  value to set register to. */
{
    DEBUGF(1, (fd, "O_Output_set_regsiter: %x = %x\n", reg, value));
    if ( O_I2C_write_indexed(p_me->p_xv->p_i2c, p_me->dmse_address, 0x02,
       reg, value) )
    {
        return 0;
    }
    return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Output_Dmse_set_type"
 |
 |  Set type of video output. 
 |
 |    = 1 on success, = 0 if error detected
 *---------------------------------------------------------------------------*/
int m_Output_Dmse_set_type( p_me, timing, format, instance )
	I_Output_Dmse*	p_me;		/* -> object context. */
	char*			timing;		/* =  timing to use (NTSC, PAL, SECAM). */
	char*			format;		/* =  format to use (COMPOSITE, YC). */
	int				instance;	/* =  instance of format. */
{
	int		i_value;	/* Value in dmse now setting. */

    /*  PROCESS TIMING INFORMATION: */
     
    if ( !strcmp(timing, "NTSC") ) 
    {
		p_me->values_now = p_me->values_ntsc;
		p_me->xpoint_value = 0x52575;
		plxv_xpoint_b(p_me->p_xv, 0x52575);
		p_me->p_xv->plxv_io_format[p_me->port] = &(output_format[0]);
    } 
    else if ( !strcmp(timing, "PAL") )
    { 
		p_me->values_now = p_me->values_pal;
		p_me->xpoint_value = 0x52535;
		plxv_xpoint_b(p_me->p_xv, 0x52535);
		p_me->p_xv->plxv_io_format[p_me->port] = &(output_format[1]);
    } 
    else if ( !strcmp(timing, "SECAM") )
    { 
		p_me->values_now = p_me->values_pal;
		p_me->xpoint_value = 0x52535;
		plxv_xpoint_b(p_me->p_xv, 0x52535);
		p_me->p_xv->plxv_io_format[p_me->port] = &(output_format[1]);
    } 
    else
    { 
        fprintf(stderr, "Unrecognized timing type %s\n", timing); 
        return 0; 
    }    

	for ( i_value = 0; i_value < sizeof(p_me->values_ntsc); i_value++ )
	{
        if ( O_I2C_write_indexed(p_me->p_xv->p_i2c, p_me->dmse_address, 0x02,
			 i_value, p_me->values_now[i_value]) )
#ifdef NOTDEF
	    if ( plxv_i2cwi(p_me->p_xv, p_me->dmse_address, 0x02, 
			 i_value, p_me->values_now[i_value]) )
#endif
		{
			ERRORF((fd, "O_Output_set_type: i2c write error index %d\n", 
				i_value));
			return 0;
		}
	}
     
     
    /*  PROCESS FORMAT INFORMATION: */ 
     
    return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Output_Dmse_terminate"
 |
 |  Destructor for this object: free its dynamically allocated storage.
 |
 *---------------------------------------------------------------------------*/
static void m_Output_Dmse_terminate( p_me )
    I_Output_Dmse*  p_me;       /* -> object context. */
{
    m_Output_Dmse_off( p_me );
    free((char*)p_me);
}
 

/*
 *  I_Output_Dmse Object Methods:
 */
static TV_Output O_Output_methods= {
	m_Output_Dmse_init_hw,
	m_Output_Dmse_off,
	m_Output_Dmse_on,
	m_Output_Dmse_set_levels,
	m_Output_Dmse_set_register,
	m_Output_Dmse_set_type,
	m_Output_Dmse_terminate
};

/*---------------------------------------------------------------------------*
 |"O_Output_Dmse_O_Output_Dmse"
 |
 |  Create and return an output access object. Note we do not initialize the
 |  hardware here, that is done by the init_hw method.
 |
 *---------------------------------------------------------------------------*/
O_Output* O_Output_Dmse_O_Output_Dmse( p_me, unit, p_xv )
	I_Output_Dmse*	p_me;	/* -> object storage, or 0 for dynamic alloc. */
	int				unit;	/* =  # of output creating object for. */
	XV_struct*		p_xv;	/* -> hardware definition. */
{
	int				i_poll;	/* # of time around existence loop. */
	int				status;	/* Status byte to verify existence. */


   /*  READ THE STATUS REGISTER TO VERIFY THE DMSE IS THERE. */

    for ( i_poll = 0; i_poll < 5; i_poll++ )
    {
        if ( !O_I2C_read_no_sub(p_xv->p_i2c, dmse_addresses[unit], &status) )
            break;
    }
    if ( i_poll >= 5 )
    {
        return 0;
    }

	if ( !p_me )
	{
		if ( !(p_me = (I_Output_Dmse*)malloc(sizeof(I_Output_Dmse))) )
		{
			ERRORF((fd,"*** Unable to allocate output object for unit %d\n", 
				unit));
			return 0;
		}
	}

	p_me->atv = &O_Output_methods;
	p_me->dmse_address = dmse_addresses[unit];
	p_me->port = PORT_B;
	p_me->p_xv = p_xv;

	return (O_Output*)p_me;
}

