/*
 *  input_7110.c: Phillips 7110 One Chip Front End input object implementation.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     09 Jan 95 - Andy Goodrich
 *       - Initial coding
 *
 *  Copyright(c) 1994, Parallax Graphics, Inc.
 */
#include <sys/types.h>		/* Standard types for this C environment. */
#include <stdio.h>			/* Stream input/output declarations. */
#include "hw_context.h"		/* Hardware context for card. */
#include "objects.h"		/* Generic object defintions. */
#include "plxv_xpoint.h"	/* Cross point switch definitions. */


/* XV_DIAG FLAGS:
 *
 *   0x20 - Device polling diagnostics.
 *   1    - Normal one-shot debugging.
 */
/* #define XV_DIAG 0x21   */
#include "xvdebug.h"


/*  INTERNAL OBJECT DEFINITION FOR PHILLIPS 7110 INPUT OBJECT: */

typedef struct _O_Input_7110 {
    TV_Input*   atv;                /* Methods: must be the first entry!! */
    int         dcsc_address;       /* Address of dcsc for this input. */
    int         inverse_video;      /* = 1 if we have inverse video. */
    int         ofc1_address;       /* Address of ofc1 for this input. */
    int         port;               /* Unit number of this input. */
    XV_struct*  p_xv;               /* Hardware information. */
    char        values_now[0x1a];   /* Current DMSD values in use. */
} O_Input_7110;


/*  I2C SLAVE ADDRESS VALUES: */

static char dcsc_addresses[2] = {0x70,0x71};
static char ofc1_addresses[2] = {0x9C >> 1, 0x9E >> 1}; /* */


/*  DUAD REGISTER VALUES: */

static u_char duad_60_hertz[] = {
/*   20    21    22    23    24    25    26    27  */
	0xd9, 0x10, 0x40, 0x41, 0x80, 0x41, 0x80, 0x4f, 
/*   28    29    2A    2B    2C    2D    2E    2F  */
    0xfe, 0x01, 0xcf, 0x0f, 0x03, 0x01, 0x81, 0x03,
/*   30    31    32    33    34                    */
	0x44, 0x75, 0x01, 0x8c, 0x03
};
	
static XV_video_format input_format[2] = {
	{640,482,43,10,20},
	{768,574,51,7,23}
};

static int clip(val)
int val;
{
    if (val < 0) return(0);
    if (val > 0xff) return(0xff);
    return(val);
}

static void i_Input_7110_dcsc_init(p_me)
	O_Input_7110 *p_me;
{
	int temp, i_entry;
    O_I2C*  p_i2c;

	p_i2c = p_me->p_xv->p_i2c;


	/*  PLACE 7110 INTO BYPASS MODE (BIT 3 OF MODE REGISTER 2 ) TO HIDE THE 
     *  LOADING OF THE TABLE. THEN PREPARE THE DCSC (7192) TO RECEIVE THE TABLE.
	 */
	 /* #### NON-GENERIC !!! */

    temp = p_me->values_now[0xd];
    temp &= ~2;
    if(O_I2C_write(p_i2c, p_me->ofc1_address, 0xd, temp))

	O_I2C_write(p_i2c, p_me->dcsc_address, 0, 0x3a); /* enable write to vluts */
	O_I2C_set_address(p_i2c, p_me->dcsc_address, 4);
	O_I2C_write_data(p_i2c, 0);
	O_I2C_acknowledge(p_i2c, p_me->dcsc_address, 3);


	/*  LOOP LOADING A RAMP INTO THE DCSC. */

    for ( i_entry=0; i_entry < 256; i_entry++)
    {
		O_I2C_write_data(p_i2c, i_entry);
        O_I2C_acknowledge(p_i2c, p_me->dcsc_address, 3);
    }
	O_I2C_write(p_i2c, p_me->dcsc_address, 0, 0xfa); /* enable read to vluts */


	/*  SET 7110 INTO NORMAL MODE BY TURNING OFF BIT 3 OF MODE REGISTER 2. */

	 /* #### NON-GENERIC !!! */
    O_I2C_write(p_me->p_xv->p_i2c, p_me->ofc1_address, 0xd,
		p_me->values_now[0xd]);
}


/*---------------------------------------------------------------------------*
 |"m_Input_7110_init_hw"
 |
 |  Initialize hardware settings for this channel.
 |
 *---------------------------------------------------------------------------*/
static void m_Input_7110_init_hw( p_me )
	O_Input_7110*	p_me;		/* -> object context. */
{
	O_I2C*	p_i2c;		/* Fast access to I2C bus object: p_me->p_xv->p_i2c. */
	int i_value, bits;

	TRACEF(0x80, (fd, "m_Input_7110_init_hw: entry\n"));
	p_i2c = p_me->p_xv->p_i2c;
	p_me->inverse_video = 0;

	switch( p_me->port )
	{
	  case PORT_D:
		O_Rsrc_get_long(p_me->p_xv->p_rsrc, "d.dmsd.ntsc", p_me->values_now);
		plxv_xpoint_a(p_me->p_xv, (CE_HIGH | HSYNC_HIGH | VSYNC_HIGH
			| ODD_HIGH  | ODD_INMODE_A | SHUTTLE_EN));
		m_Input_7110_set_mode(p_me, 3); /* set input to comp 1 */
		p_me->p_xv->plxv_io_format[0] = &input_format[0];
		break;
	  case PORT_B:
		O_Rsrc_get_long(p_me->p_xv->p_rsrc, "b.dmsd.ntsc", p_me->values_now);
		plxv_xpoint_b(p_me->p_xv, (CE_HIGH | HSYNC_HIGH | VSYNC_HIGH
	        | ODD_HIGH  | ODD_INMODE_A | SHUTTLE_EN));
		m_Input_7110_set_mode(p_me, 3); /* set input to comp 1 */
		p_me->p_xv->plxv_io_format[1] = &input_format[0];
		break;
	}

	/* LOAD UP STANDARD SET OF VALUES, AND THEN SET DEFAULT INPUT. */

    for ( i_value = 0; i_value < sizeof(p_me->values_now); i_value++ )
    {
        O_I2C_write(p_i2c, p_me->ofc1_address,
            i_value, p_me->values_now[i_value]);
	}
    for ( i_value = 0; i_value < sizeof(duad_60_hertz); i_value++ )
    {
        O_I2C_write(p_i2c, p_me->ofc1_address,
            0x20+i_value, duad_60_hertz[i_value]);
    }

#if 0
	O_I2C_write(p_i2c, p_me->ofc1_address, 0x21, 0x10); /* REFS off */
#endif

	i_Input_7110_dcsc_init(p_me);
	O_Input_set_type(p_me, "NTSC", "COMPOSITE", 0);
	O_Input_set_levels(p_me, 0x7f, 0x7f, 0x7f, 0x7f);

	TRACEF(0x80, (fd, "m_Input_7110_init_hw: exit\n"));
}


/*---------------------------------------------------------------------------*
 |"m_Input_7110_off"
 |
 |  Disable an input.
 |
 *---------------------------------------------------------------------------*/
static void m_Input_7110_off( p_me )
	O_Input_7110*	p_me;		/* -> object context. */
{
    int     bits;       /* Shadow value for register. */

    if ( p_me->port == PORT_B ) 
    {   
		bits = p_me->p_xv->xv_shdw->b.Bt.grn.reg1;
		bits = bits | 0x1;
		PV_WREG(p_me->p_xv,Bt.grn.reg1, bits);
		bits = p_me->p_xv->xv_shdw->b.Bt.grn.reg2;
		bits = (bits & 0xf9) | 0x2;
		PV_WREG(p_me->p_xv,Bt.grn.reg2, bits);
	}
}


/*---------------------------------------------------------------------------*
 |"m_Input_7110_on"
 |
 |  Enable an input.
 |
 *---------------------------------------------------------------------------*/
static void m_Input_7110_on( p_me )
	O_Input_7110*	p_me;		/* -> object context. */
{
    int     bits;       /* Shadow value for register. */
 
    if ( p_me->port == PORT_B ) 
    {   
        bits = p_me->p_xv->xv_shdw->b.Bt.grn.reg1;
        bits = bits & 0xfe;
        PV_WREG(p_me->p_xv,Bt.grn.reg1, bits);
        bits = p_me->p_xv->xv_shdw->b.Bt.grn.reg2;
		bits = (bits & 0xf9) | 0x4;
        PV_WREG(p_me->p_xv,Bt.grn.reg2, bits);
		plxv_xpoint_b(p_me->p_xv, CE_HIGH | HSYNC_HIGH | VSYNC_HIGH
			| ODD_HIGH  | ODD_INMODE_A | SHUTTLE_EN);
    } 
	else
	{
		plxv_xpoint_a(p_me->p_xv, CE_HIGH | HSYNC_HIGH | VSYNC_HIGH
			| ODD_HIGH  | ODD_INMODE_A | SHUTTLE_EN);
	}
}


/*---------------------------------------------------------------------------*
 |"m_Input_7110_set_levels"
 |
 |  Set color characteristics of video input.
 |
 *---------------------------------------------------------------------------*/
static void 
	m_Input_7110_set_levels( p_me, brightness, contrast, hue, saturation )
	O_Input_7110*	p_me;		/* -> object context. */
	int				brightness;	/* =  brightness level to set. */
	int				contrast;	/* =  contrast level to set. */
	int				hue;		/* =  hue level to set. */
	int				saturation;	/* =  satruation level to set. */
{
    float   a,b,c;		/* Coefficients for calculating clt entries. */
    int 	i_entry;	/*	Entry in clt now filling in. */
	O_I2C*	p_i2c;		/* Fast access to I2C bus object: p_me->p_xv->p_i2c. */
	int		temp;		/* Temporary for register values. */

	p_i2c = p_me->p_xv->p_i2c;

 
	/* SET SATURATION (DMSD VALUE 18 0x12): */

	p_me->values_now[18] = (saturation / 2) & 0x7f;
	if ( O_I2C_write(p_i2c, p_me->ofc1_address, 18, p_me->values_now[18]) )
	{
		ERRORF((fd, "plxv_input_adjust: error writing to 7110\n"));
		return;
	}


	/* SET HUE (DMSD VALUE 7): */

	p_me->values_now[7] = hue - 0x80;
	if ( O_I2C_write(p_i2c, p_me->ofc1_address, 7, p_me->values_now[7]) )
	{
		ERRORF((fd, "plxv_input_adjust: error writing to 7110\n"));
		return;
	}

	/* SET BRIGHTNES (DMSD VALUE 25 0x19): */

	p_me->values_now[0x19] = (brightness & 255);
	if(O_I2C_write(p_i2c, p_me->ofc1_address, 0x19, p_me->values_now[0x19]))
	{
		ERRORF((fd, "plxv_input_adjust: error writing to 7110\n"));
		return;
	}

	/* SET CONTRAST (DMSD VALUE 19 0x13): */

    p_me->values_now[0x13] = (contrast / 2) & 0x7f;
	if(O_I2C_write(p_i2c, p_me->ofc1_address, 0x13, p_me->values_now[0x13]))
	{
		ERRORF((fd, "plxv_input_adjust: error writing to 7110\n"));
		return;
	}
}


/*---------------------------------------------------------------------------*
 |"m_Input_7110_set_register"
 |
 |  Set the value of a 7110 register.
 |
 |    = 1 if success, 0 if error detected
 *---------------------------------------------------------------------------*/
static int m_Input_7110_set_register( p_me, reg, value)
	O_Input_7110*	p_me;		/* -> object context. */
	int				reg;		/* =  register number to set value of. */
	int				value;		/* =  value to set register to. */
{
	if( O_I2C_write(p_me->p_xv->p_i2c, p_me->ofc1_address, reg, value) )
	{
		return 0;
	}
	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Input_7110_set_type"
 |
 |  Set type of video input.
 |
 |    Notes:
 |      (1) For PORT_B we need to disable output and enable input since the
 |          port is shared.
 |
 |    = 1 if success, 0 if error detected
 *---------------------------------------------------------------------------*/
static int m_Input_7110_set_type( p_me, timing, format, instance )
	O_Input_7110*	p_me;		/* -> object context. */
	char*			timing;		/* =  timing to use (NTSC, PAL, SECAM). */
	char*			format;		/* =  input format to use (COMPOSITE, YC). */
	int				instance;	/* =  instance of format. */
{
	int		bits;		/* Temporary to manipulate register bits in. */
	int 	i_value;    /* # of value now loading. */
	O_I2C*	p_i2c;		/* Fast access to I2C bus object: p_me->p_xv->p_i2c. */
	int		sub2eval;	/* Value for 7110 subaddress 2e. */

	p_i2c = p_me->p_xv->p_i2c;


	/*  PROCESS TIMING INFORMATION: */

	if ( !strcmp(timing, "NTSC") )
	{
		p_me->p_xv->plxv_io_format[p_me->port] = &(input_format[0]);
		p_me->values_now[0xd] = 0x86;
		p_me->values_now[0x11] = 0x2c;
		sub2eval = 0x81;
	}
	else if ( !strcmp(timing, "PAL") )
	{
		p_me->p_xv->plxv_io_format[p_me->port] = &(input_format[1]);
		p_me->values_now[0xd] = 0x86;
		p_me->values_now[0x11] = 0x59;
		sub2eval = 0x9a;
	}
	else if ( !strcmp(timing, "SECAM") )
	{
		p_me->p_xv->plxv_io_format[p_me->port] = &(input_format[1]);
		p_me->values_now[0xd] = 0x87;
		p_me->values_now[0x11] = 0x59;
		sub2eval = 0x9a;
	}
	else
	{
		ERRORF((fd, "Unrecognized timing type %s\n", timing));
		return 0;
	}
	for ( i_value = 0; i_value < sizeof(p_me->values_now); i_value++ )
	{
		O_I2C_write(p_i2c, p_me->ofc1_address, 
			i_value, p_me->values_now[i_value]);
	}

    O_I2C_write(p_i2c, p_me->ofc1_address, 0x2e, sub2eval);


	/*  PROCESS CONNECTION INFORMATION: */

	if ( !strcmp(format, "COMPOSITE") )
	{
		if ( p_me->port == PORT_B ) /* Enable input port, disable output port.*/
		{
			bits = p_me->p_xv->xv_shdw->b.Bt.grn.reg2;
			bits = (bits & 0xf9) | 0x4;
			PV_WREG(p_me->p_xv, Bt.grn.reg2, bits);
			plxv_xpoint_b(p_me->p_xv, CE_HIGH | HSYNC_HIGH | VSYNC_HIGH
					| ODD_HIGH  | ODD_INMODE_A | SHUTTLE_EN);
		}
		else
		{
			plxv_xpoint_a(p_me->p_xv, (CE_HIGH | HSYNC_HIGH | VSYNC_HIGH
						  | ODD_HIGH  | ODD_INMODE_A | SHUTTLE_EN));
		}
		m_Input_7110_set_mode(p_me, instance + 3);/* set input to comp 1 or 2*/
		O_I2C_write(p_i2c, p_me->ofc1_address, 0xe, 
			p_me->values_now[0x0e]|instance);
	    /* now set bpf & chroma trap, etc */
		O_I2C_write(p_i2c, p_me->ofc1_address,0x6,p_me->values_now[6]&~0x80);
	}
	else if ( !strcmp(format, "YC") )
	{
		if ( p_me->port == PORT_D )
		{
			plxv_xpoint_a(p_me->p_xv, (CE_HIGH | HSYNC_HIGH | VSYNC_HIGH
						  | ODD_HIGH  | ODD_INMODE_A | SHUTTLE_EN));
		}
		else
		{
			bits = p_me->p_xv->xv_shdw->b.Bt.grn.reg2;
			bits = (bits & 0xf9) | 0x4;
			PV_WREG(p_me->p_xv, Bt.grn.reg2, bits);
			plxv_xpoint_b(p_me->p_xv, CE_HIGH | HSYNC_HIGH | VSYNC_HIGH
				| ODD_HIGH  | ODD_INMODE_A | SHUTTLE_EN);
		}
		m_Input_7110_set_mode(p_me, instance + 6);/* set input to yc 1 or 2 */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0xe, 
			p_me->values_now[0xe] | 2 | instance);
		O_I2C_write(p_i2c, p_me->ofc1_address,0x6,p_me->values_now[6]|0x80);
	}
	else
	{
		ERRORF((fd, "Unrecognized format type %s\n", format));
		return 0;
	}

	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Input_7110_set_mode"
 |
 |  Set duad mode to select a specific wiring combination for input.
 |
 |    = 1 if success, 0 if error detected
 *---------------------------------------------------------------------------*/
static int m_Input_7110_set_mode( p_me, mode )
	O_Input_7110*	p_me;		/* -> object context. */
	int				mode;		/* Wiring mode: 0 - 8. */
{
	O_I2C*	p_i2c;		/* Fast access to I2C bus object: p_me->p_xv->p_i2c. */

	p_i2c = p_me->p_xv->p_i2c;

	switch( mode )
	{
	  case 0: 
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x06, 0x00); 
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x20, 0xd9); /* AI21 active. */ 
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x21, 0x11); /* REFS on */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x22, 0x40); /* AD2 luma/chroma*/
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x2c, 0x03); /* Clamp select. */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x30, 0x44); /* AD2 gain active*/
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x31, 0x75); /* AOSL = 01b. */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x21, 0x10); /* REFS off */
		break;
	  case 1: 
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x06, 0x00); 
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x20, 0xd8); /* AI22 active. */ 
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x21, 0x11); /* REFS on */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x22, 0x40); /* AD2 luma/chroma*/
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x2c, 0x03); /* Clamp select. */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x30, 0x44); /* AD2 gain active*/
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x31, 0x75); /* AOSL = 01b. */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x21, 0x10); /* REFS off */
		break;
	  case 2: 
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x06, 0x00); 
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x20, 0xba); /* AI31 active. */ 
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x21, 0x07); /* REFS on */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x22, 0x91); /* AD3 luma/chroma*/
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x2c, 0x03); /* Clamp select. */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x30, 0x60); /* AD3 gain active*/
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x31, 0xB5); /* AOSL = 10b. */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x21, 0x05); /* REFS off */
		break;
	  case 3: 
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x06, 0x00); 
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x20, 0xb8); /* AI32 active. */ 
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x21, 0x07); /* REFS on */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x22, 0x91); /* AD3 luma/chroma*/
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x2c, 0x03); /* Clamp select. */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x30, 0x60); /* AD3 gain active*/
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x31, 0xB5); /* AOSL = 10b. */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x21, 0x05); /* REFS off */
		break;
	  case 4: 
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x06, 0x00); 
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x20, 0x7c); /* AI41 active. */ 
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x21, 0x07); /* REFS on */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x22, 0xd2); /* AD3 luma/chroma*/
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x2c, 0x83); /* Clamp select. */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x30, 0x60); /* AD3 gain active*/
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x31, 0xB5); /* AOSL = 10b. */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x21, 0x03); /* REFS off */
		break;
	  case 5: 
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x06, 0x00); 
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x20, 0x78); /* AI41 active. */ 
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x21, 0x07); /* REFS on */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x22, 0xd2); /* AD3 luma/chroma*/
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x2c, 0x83); /* Clamp select. */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x30, 0x60); /* AD3 gain active*/
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x31, 0xB5); /* AOSL = 10b. */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x21, 0x03); /* REFS off */
		break;
	  case 6: 
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x06, 0x80); /* YC mode. */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x20, 0x59); /* AI21=y, AI42=c */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x21, 0x17); /* REFS on */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x22, 0x42); /* AD2 lum,AD3 chr*/
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x2c, 0xa3); /* Clamp select. */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x30, 0x44); /* AD2 gain active*/
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x31, 0x75); /* AOSL = 01b. */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x21, 0x12); /* REFS off */
		break;
	  case 7: 
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x06, 0x80); /* YC mode. */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x20, 0x9a); /* AI31=y, AI22=c */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x21, 0x17); /* REFS on */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x22, 0xB1); /* AD3 lum,AD2 chr*/
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x2c, 0x13); /* Clamp select. */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x30, 0x60); /* AD3 gain active*/
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x31, 0xB5); /* AOSL = 10b. */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x21, 0x14); /* REFS off */
		break;
	  case 8: 
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x06, 0x80); /* YC mode. */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x20, 0x3C); /* AI41=y, AI32=c */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x21, 0x27); /* REFS on */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x22, 0xC1); /* AD2 lum,AD3 chr*/
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x2c, 0x23); /* Clamp select. */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x30, 0x44); /* AD2 gain active*/
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x31, 0x75); /* AOSL = 01b. */
		O_I2C_write(p_i2c, p_me->ofc1_address, 0x21, 0x21); /* REFS off */
		break;

	}
}


/*---------------------------------------------------------------------------*
 |"m_Input_7110_status"
 |
 |  Return the status associated with an input device.
 |
 |    = status, 0 if error detected.
 *---------------------------------------------------------------------------*/
static int m_Input_7110_status(p_me, unit)
	O_Input_7110*	p_me;		/* -> object context. */
	int unit;
{
    int data;
    if( O_I2C_write(p_me->p_xv->p_i2c, p_me->ofc1_address, 0xd, 
			p_me->values_now[0xd])
	)
	{
	    printf("unable to write to addr 0xd prior to read\n");
	    return 0;
    }
	if( O_I2C_read_no_sub(p_me->p_xv->p_i2c, p_me->ofc1_address, &data) )
    {
	    ERRORF((fd, "unable to read status reg on 7110\n"));
	    return 0;
    }
    return data;
}

/*---------------------------------------------------------------------------*
 |"m_Input_7110_terminate"
 |
 |  Terminate use of an object, freeing its storage.
 *---------------------------------------------------------------------------*/
static void m_Input_7110_terminate( p_me )
	O_Input_7110*	p_me;		/* -> object context. */
{
	free((char*)p_me);
}


/*
 *  O_Input_7110 Object Methods:
 */
static TV_Input O_Input_7110_methods = {
	m_Input_7110_init_hw,
	m_Input_7110_off,
	m_Input_7110_on,
	m_Input_7110_set_levels,
	m_Input_7110_set_register,
	m_Input_7110_set_type,
	m_Input_7110_status,
	m_Input_7110_terminate
};

/*---------------------------------------------------------------------------*
 |"O_Input_7110_O_Input_7110"
 |
 |  Entry point for system for this application.
 |
 *---------------------------------------------------------------------------*/
O_Input* O_Input_7110_O_Input_7110( p_me, unit, p_xv )
	O_Input_7110*	p_me;	/* Object creating. */
	int				unit;	/* =  # of input creating object for. */
	XV_struct*		p_xv;	/* -> hardware definition. */
{
	int data;

	if ( !p_me )
	{
		if ( !(p_me = (O_Input_7110*)malloc(sizeof(O_Input_7110))) )
		{
			ERRORF((fd, 
				"*** Unable to allocate input object for unit %d\n", unit));
			return 0;
		}
	}

	p_me->atv = &O_Input_7110_methods;
	p_me->dcsc_address = dcsc_addresses[unit];
	p_me->ofc1_address = ofc1_addresses[unit];
	p_me->port = unit ? PORT_B : PORT_D;
	p_me->p_xv = p_xv;

    if(O_I2C_write(p_xv->p_i2c, ofc1_addresses[unit], 0x0d, 4))
    {
		DEBUGF(0x20, (fd, "Unable to write to device @ %x\n", 
			ofc1_addresses[unit] << 1));
        return 0;
    }

	if(O_I2C_read_no_sub(p_xv->p_i2c, ofc1_addresses[unit], &data))
    {
		DEBUGF(0x20, (fd, "Unable to read id register of device @ %x\n", 
			ofc1_addresses[unit] << 1));
	    return (O_Input*)0;
    }

    DEBUGF(0x20, (fd, "7110 @ %x id is %x\n", ofc1_addresses[unit] << 1, data));

	return (O_Input*)p_me;
}

