/*
 *  bt281_lut.c: Support for Brooktree 281 Lookup Table Object.
 *
 *   The Bt281 is a color space converter, as well as a set of color lookup 
 *   tables individually for red, green, and blue. The color space converter 
 *   performs normalized operations, the color tables are used to set offset 
 *   and gain.
 *
 * (c) Copyright 1995 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 * $Log:   /plxhp/dave/pvcsrc/hwlib/archives/bt281_lut.c_v  $
 * 
 *    Rev 1.1   09 Sep 1996 10:08:28   dave
 * Initial revision
 * Revision 1.1  1995/12/11 20:42:26  dist
 * Initial revision
 *
 */
#include <stdio.h>			/* Pickup stdout and friends. */
#include <sys/types.h>		/* Pick up u_int and friends. */
#include "objects_sun.h"	/* Pick up object definitions. */
#include "xvdebug.h"		/* Pick up XV debugging macros. */

/*  INTERNAL OBJECT DEFINITION FOR BROOKTREE 281 OBJECT: */

typedef struct _O_Lut_Bt281 {
	TV_Lut*		atv;		/* Methods: first entry in all derived objects. */
	XV_struct*	p_xv;		/* Parallax hardware library context structure. */
	int			sel_yuv;	/* = 1 if operating in yuv mode. */
} O_Lut_Bt281;


enum {					/* Bt281 register addresses: */
	Bt281_CR0 = 0x312,	/* Command register 0. */
	Bt281_CR1 = 0x313	/* Command register 0. */
};


/* FORWARD FUNCTION REFERENCES: */

static void i_Lut_Bt281_load(/* p_me, addr,data */);
static void i_Lut_Bt281_set_addr(/* p_me, addr */);
static void i_Lut_Bt281_set_option_rsaddr(/* p_me, device_code,rsaddr */);
static int  m_Lut_Bt281_control(/* p_me, op, arg */);
static void m_Lut_Bt281_dump(/* p_me, fd */);
static int  m_Lut_Bt281_get_entries(/* p_me, start, count, p_value */);
static int  m_Lut_Bt281_init_hw(/* p_me */);
static int  m_Lut_Bt281_set_entries(/* p_me, start, count, p_value */);
static void m_Lut_Bt281_terminate(/* p_me */);


static void i_Lut_Bt281_load(p_me, addr,data)
	O_Lut_Bt281* p_me;	/* -> context for this object. */
	int 		 addr;	/* =  address to be loaded. */
	int			 data;	/* =  data value to be loaded. */
{
    i_Lut_Bt281_set_addr(p_me, addr);
    i_Lut_Bt281_set_option_rsaddr(p_me, 2, 2);
    PV_WREG(p_me->p_xv, Ext.reg1, data);
}


static void i_Lut_Bt281_set_addr(p_me, addr)
	O_Lut_Bt281* p_me;	/* -> context for this object. */
	int 		addr;	/* =  address to be setup. */
{
    int a,b;
 
    a = 0xff & addr;
    b = 0xff & (addr >> 8);
    i_Lut_Bt281_set_option_rsaddr(p_me, 2, 0);
    PV_WREG(p_me->p_xv,Ext.reg1,a);
    i_Lut_Bt281_set_option_rsaddr(p_me, 2, 1);
    PV_WREG(p_me->p_xv,Ext.reg1,b);
}

static void i_Lut_Bt281_set_option_rsaddr(p_me, device_code,rsaddr)
	O_Lut_Bt281* p_me;			/* -> context for this object. */
	int			 device_code;	/* =  device code for option. */
	int			 rsaddr;		/* = */
{
    int i,j,k;
 
    i = (device_code & 3) << 3;
    j = rsaddr & 7;
    k = i | j;
    PV_WREG(p_me->p_xv, Ext.reg0,k);
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt281_control"
 |
 |  Perform a device-dependent control operation.
 |
 |    = 1 if success, 0 if error detected.
 *---------------------------------------------------------------------------*/
static int m_Lut_Bt281_control( p_me, op, arg )
	O_Lut_Bt281* p_me;	/* -> context for this object. */
	char*		 op;	/* -> string specifying operation to perform. */
	int			 arg;	/* =  optional argument for operation. */
{
	if ( !strcmp(op, "SELECT_YUV") )
	{
		p_me->sel_yuv = 1;
	}
	else if ( !strcmp(op, "SELECT_RGB") )
	{
		p_me->sel_yuv = 0;
	}
	else
	{
		ERRORF((fd, "O_Lut_Bt281_control: unknown option: %s\n", op));
		return 0;
	}
	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt281_dump"
 |
 |  Dump the contents of this object to a file descriptor.
 *---------------------------------------------------------------------------*/
static void m_Lut_Bt281_dump( p_me, fd )
	O_Lut_Bt281* p_me;	/* -> context for this object. */
	FILE*		 fd;	/* -> file descriptor to write object contents to. */
{
	fprintf(fd, "\nBrooktree 281 object @ 0x%08x: \n");
	fprintf(fd, "   Hardware library context at 0x%08x\n", p_me->p_xv);
	fprintf(fd, "   Operating mode is %s\n", p_me->sel_yuv ? "YUV" : "RGB");
	if ( p_me->atv->control != m_Lut_Bt281_control ) 
	{
		fprintf(fd, 
			"  *** control method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->control, m_Lut_Bt281_control);
	}
	else if ( p_me->atv->dump != m_Lut_Bt281_dump ) 
	{
		fprintf(fd, 
			"  *** dump method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->dump, m_Lut_Bt281_dump);
	}
	if ( p_me->atv->get_entries != m_Lut_Bt281_get_entries ) 
	{
		fprintf(fd, 
			"  *** get_entries method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->get_entries, m_Lut_Bt281_get_entries);
	}
	if ( p_me->atv->init_hw != m_Lut_Bt281_init_hw ) 
	{
		fprintf(fd, 
			"  *** init_hw method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->init_hw, m_Lut_Bt281_init_hw);
	}
	if ( p_me->atv->set_entries != m_Lut_Bt281_set_entries ) 
	{
		fprintf(fd, 
			"  *** set_entries method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->set_entries, m_Lut_Bt281_set_entries);
	}
	if ( p_me->atv->terminate != m_Lut_Bt281_terminate ) 
	{
		fprintf(fd, 
			"  *** terminate method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->terminate, m_Lut_Bt281_terminate);
	}
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt281_get_entries"
 |
 |  Return the values of a range of lookup table entries. There are three 
 |  8-in 8-out lookup tables, one for each color component. The values 
 |  pointed to by p_values are integers of the form XBGR.
 |
 |    = 1 on success, 0 if error detected.
 *---------------------------------------------------------------------------*/
static int m_Lut_Bt281_get_entries( p_me, type, start, count, p_values )
	O_Lut_Bt281*	p_me;		/* -> context for this object. */
	int				type;		/* =  type of entries (see objects.h). */
	int				start;		/* =  first location in table to return. */
	int				count;		/* =  # of entries to return. */
	int*			p_values;	/* -> where to place entries being returned. */
{
	int		blue;		/* Blue component of value. */
	int		green;		/* Green component of value. */
	int		i_entry;	/* # of entry in lut now getting. */
	int*	p_entry;	/* Data for entry in lut now getting. */
	int		red;		/* Red component of value. */

	switch ( type & O_Lut_Entry_Type_Mask )
	{
	  case O_Lut_Entry_Lut:
		p_entry = p_values;
		for ( i_entry = start; i_entry < start+count; i_entry++ )
		{
			i_Lut_Bt281_set_addr(p_me, 0x000 + i_entry);
			i_Lut_Bt281_set_option_rsaddr(p_me, 2, 2);
			PV_RREG(p_me->p_xv, red, Ext.reg1);
			i_Lut_Bt281_set_addr(p_me, 0x100 + i_entry);
			i_Lut_Bt281_set_option_rsaddr(p_me, 2, 2);
			PV_RREG(p_me->p_xv, green, Ext.reg1);
			i_Lut_Bt281_set_addr(p_me, 0x200 + i_entry);
			i_Lut_Bt281_set_option_rsaddr(p_me, 2, 2);
			PV_RREG(p_me->p_xv, blue, Ext.reg1);
			*p_entry++ = (blue << 16) | (green << 8) | red;
		}
		break;
	  default:
		return 0;
	}
	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt281_init_hw"
 |
 |  Initialize hardware to its default state.
 |
 |    = 1 on success, 0 if error detected.
 *---------------------------------------------------------------------------*/
static int m_Lut_Bt281_init_hw( p_me )
	O_Lut_Bt281*	p_me;		/* -> context for this object. */
{
	if ( p_me->sel_yuv )	/* Use color space convertor. */
	{
		i_Lut_Bt281_load(p_me, Bt281_CR0, 0x20);
		i_Lut_Bt281_load(p_me, Bt281_CR1, 0);
	}
	else					/* Use RGB. */
	{
		i_Lut_Bt281_load(p_me, Bt281_CR0, 0);
		i_Lut_Bt281_load(p_me, Bt281_CR1, 0);
	}
	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt281_set_entries"
 |
 |  Set the values of a range of lookup table entries. There are three 8-in
 |  8-out lookup tables, one for each color component. The values pointed to
 |  by p_values are integers of the form XBGR.
 |
 |    = 1 on success, 0 if error detected.
 *---------------------------------------------------------------------------*/
static int m_Lut_Bt281_set_entries( p_me, type, start, count, p_values )
	O_Lut_Bt281*	p_me;		/* -> context for this object. */
	int				type;		/* =  type of entries (see objects.h). */
	int				start;		/* =  first location in table to set. */
	int				count;		/* =  # of entries to set. */
	int*			p_values;	/* -> where to get values for entries. */
{
	int		i_entry;	/* # of entry in lut now setting. */
	int*	p_entry;	/* Data for entry in lut now setting. */

	switch ( type & O_Lut_Entry_Type_Mask )
	{
	  case O_Lut_Entry_Lut:
		p_entry = p_values;
		for ( i_entry = start; i_entry < start+count; i_entry++, p_entry++ )
		{
			i_Lut_Bt281_load(p_me, 0x000 + i_entry, (*p_entry) >> 16);
			i_Lut_Bt281_load(p_me, 0x100 + i_entry, (*p_entry) >> 8);
			i_Lut_Bt281_load(p_me, 0x200 + i_entry, (*p_entry) );
		}
		break;
	  default:
		return 0;
	}
	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt281_terminate"
 |
 |  Terminate use of this object.
 *---------------------------------------------------------------------------*/
static void m_Lut_Bt281_terminate( p_me )
	O_Lut_Bt281*	p_me;		/* -> context for this object. */
{
	free((char*)p_me);
}


static TV_Lut methods_Lut_Bt281 = {
	m_Lut_Bt281_control,
	m_Lut_Bt281_dump,
	m_Lut_Bt281_get_entries,
	m_Lut_Bt281_init_hw,
	m_Lut_Bt281_set_entries,
	m_Lut_Bt281_terminate
};


/*---------------------------------------------------------------------------*
 |"O_Lut_Bt281_O_Lut_Bt281"
 |
 |  Create an instance of a Brooktree 281 lookup table object.
 |
 |    -> object's context on success, or 0 if error occurred.
 *---------------------------------------------------------------------------*/
O_Lut* O_Lut_Bt281_O_Lut_Bt281( p_me, p_xv )
	O_Lut_Bt281* p_me;		/* -> context to fill in, or 0 for dynamic alloc. */
	XV_struct*	 p_xv;		/* -> context for hardware library. */
{
	if ( !p_me )
	{
		if ( !(p_me = (O_Lut_Bt281*)malloc(sizeof(O_Lut_Bt281))) )
		{
			ERRORF((fd, 
				"O_Lut_Bt281_O_Lut_Bt281: Unable to allocate storage\n"));
		}
	}
	p_me->atv = &methods_Lut_Bt281;
	p_me->p_xv = p_xv;
	p_me->sel_yuv = 0;
	
	return (O_Lut*)p_me;
}
