/*
 *  bt495_lut.c: Support for Brooktree 495 Lookup Table Object.
 *
 *   The Bt495 is a RAMDAC providing pseudo color support via a set of
 *   three 8-in 8-out lookup tables, one for red, green, and blue.
 *   This object supports 2 visual types based on low-order tag bit
 *   0 = pseudo color, and 1 = true color.
 *
 * (c) Copyright 1995 Parallax Graphics Inc, All Rights Reserved
 *
 * Parallax Graphics, Inc.
 * 2500 Condensa Street
 * Santa Clara, California 95051
 *
 * $Log:   /plxhp/dave/pvcsrc/hwlib/archives/bt495_lut.c_v  $
 * 
 *    Rev 1.2   09 Sep 1996 10:09:04   dave
 * added tag functions
 * Revision 1.2  1996/07/22 20:05:24  dist
 * added tag functions
 *
 * Revision 1.1  95/12/11  12:42:33  12:42:33  dist ()
 * Initial revision
 * 
 */
#include <stdio.h>			/* Pickup stdout and friends. */
#include <sys/types.h>		/* Pick up u_int and friends. */
#include "objects.h"		/* Pick up object definitions. */
#include "hw_context.h"		/* Pick up Parallax hardware library context. */
#include "xvdebug.h"		/* Pick up XV debugging macros. */

#define NUM_VIDEO_TAGS 128       /* 128 video tags */
#define PSEUDO_TAG 0            /* pseudo color tag */
#define TRUE_TAG 0x1           /* true color tag */
#define VIDEO_TAG 0x3          /* starting video tag */

/*  INTERNAL OBJECT DEFINITION FOR BROOKTREE 495 OBJECT: */

typedef struct _Bt495_Tag {
    int     tag;
    int     used;
} Bt495_Tag;


typedef struct _O_Lut_Bt495 {
	TV_Lut*		atv;		/* Methods: first entry in all derived objects. */
	XV_struct*	p_xv;		/* Parallax hardware library context structure. */
	int			sel_yuv;	/* = 1 if operating in yuv mode. */
    Bt495_Tag   vid_tags[NUM_VIDEO_TAGS]; /* video tag aray */
    int         next_tag;     /* next free tag */
} O_Lut_Bt495;


enum {					/* Bt495 register addresses: */
	Bt495_CR0 = 0x312,	/* Command register 0. */
	Bt495_CR1 = 0x313	/* Command register 0. */
};


/* FORWARD FUNCTION REFERENCES: */

static int  m_Lut_Bt495_control(/* p_me, op, arg */);
static void m_Lut_Bt495_dump(/* p_me, fd */);
static int  m_Lut_Bt495_get_entries(/* p_me, start, count, p_value */);
static int  m_Lut_Bt495_init_hw(/* p_me */);
static int  m_Lut_Bt495_set_entries(/* p_me, start, count, p_value */);
static void m_Lut_Bt495_terminate(/* p_me */);

/* added to get/free tag values */

static int  m_Lut_Bt495_pseudo_tag(/* p_me */);
static int  m_Lut_Bt495_true_tag(/* p_me */);
static int  m_Lut_Bt495_direct_tag(/* p_me */);
static int  m_Lut_Bt495_video_tag(/* p_me */);
static void m_Lut_Bt495_free_video_tag(/* p_me, tag */);



/*  Bt495 REGISTER ACCESS MACROS. */

#define Bt495_Lut_Write(p_xv, addr, value) \
{ \
    PV_WREG(p_xv, Bt.red.reg0, (addr) & 0xff); \
    PV_WREG(p_xv, Bt.red.reg1, 0); \
    PV_WREG(p_xv, Bt.red.reg3, (value) & 0xff); \
    PV_WREG(p_xv, Bt.red.reg3, ((value) >> 8) & 0xff); \
    PV_WREG(p_xv, Bt.red.reg3, ((value) >> 16) & 0xff); \
}

#define Bt495_Reg_Read(p_xv, addr, value) \
{ \
    PV_WREG(p_xv, Bt.red.reg0, (addr) & 0xff); \
    PV_WREG(p_xv, Bt.red.reg1, ((addr) >> 8) & 0x7); \
    PV_RREG(p_xv, value, Bt.red.reg2); \
}

#define Bt495_Reg_Write(p_xv, addr, value) \
{ \
    PV_WREG(p_xv, Bt.red.reg0, (addr) & 0xff); \
    PV_WREG(p_xv, Bt.red.reg1, ((addr) >> 8) & 0x7); \
    PV_WREG(p_xv, Bt.red.reg2, value); \
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt495_control"
 |
 |  Perform a device-dependent control operation.
 |
 |    = 1 if success, 0 if error detected.
 *---------------------------------------------------------------------------*/
static int m_Lut_Bt495_control( p_me, op, arg )
	O_Lut_Bt495* p_me;	/* -> context for this object. */
	char*		 op;	/* -> string specifying operation to perform. */
	int			 arg;	/* =  optional argument for operation. */
{
	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt495_dump"
 |
 |  Dump the contents of this object to a file descriptor.
 *---------------------------------------------------------------------------*/
static void m_Lut_Bt495_dump( p_me, fd )
	O_Lut_Bt495* p_me;	/* -> context for this object. */
	FILE*		 fd;	/* -> file descriptor to write object contents to. */
{
	fprintf(fd, "\nBrooktree 495 object @ 0x%08x: \n");
	fprintf(fd, "   Hardware library context at 0x%08x\n", p_me->p_xv);
	if ( p_me->atv->control != m_Lut_Bt495_control ) 
	{
		fprintf(fd, 
			"  *** control method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->control, m_Lut_Bt495_control);
	}
	else if ( p_me->atv->dump != m_Lut_Bt495_dump ) 
	{
		fprintf(fd, 
			"  *** dump method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->dump, m_Lut_Bt495_dump);
	}
	if ( p_me->atv->get_entries != m_Lut_Bt495_get_entries ) 
	{
		fprintf(fd, 
			"  *** get_entries method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->get_entries, m_Lut_Bt495_get_entries);
	}
	if ( p_me->atv->init_hw != m_Lut_Bt495_init_hw ) 
	{
		fprintf(fd, 
			"  *** init_hw method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->init_hw, m_Lut_Bt495_init_hw);
	}
	if ( p_me->atv->set_entries != m_Lut_Bt495_set_entries ) 
	{
		fprintf(fd, 
			"  *** set_entries method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->set_entries, m_Lut_Bt495_set_entries);
	}
	if ( p_me->atv->terminate != m_Lut_Bt495_terminate ) 
	{
		fprintf(fd, 
			"  *** terminate method address is 0x%08x and should be 0x%08x\n",
			p_me->atv->terminate, m_Lut_Bt495_terminate);
	}
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt495_get_entries"
 |
 |  Return the values of a range of lookup table entries. There are three 
 |  8-in 8-out lookup tables, one for each color component. The values 
 |  pointed to by p_values are integers of the form XBGR.
 |
 |    = 1 on success, 0 if error detected.
 *---------------------------------------------------------------------------*/
static int m_Lut_Bt495_get_entries( p_me, type, start, count, p_values )
	O_Lut_Bt495*	p_me;		/* -> context for this object. */
	int				type;		/* =  type of entries (see objects.h). */
	int				start;		/* =  first location in table to return. */
	int				count;		/* =  # of entries to return. */
	int*			p_values;	/* -> where to place entries being returned. */
{
	int			blue;		/* Blue component of value. */
	int			green;		/* Green component of value. */
	int			i_entry;	/* # of entry in lut now getting. */
	int*		p_entry;	/* Data for entry in lut now getting. */
	XV_struct*	p_xv;		/* Fast access to p_me->p_xv. */
	int			red;		/* Red component of value. */

	p_xv = p_me->p_xv;
	switch ( type & O_Lut_Entry_Type_Mask )
	{
	  case O_Lut_Entry_Lut:
		p_entry = p_values;
		PV_WREG(p_xv, Bt.red.reg0, start & 0xff); 
		PV_WREG(p_xv, Bt.red.reg1, 0); 
		for ( i_entry = start; i_entry < start+count; i_entry++, p_entry++ )
		{
			PV_RREG(p_xv, red, Bt.red.reg3);
			PV_RREG(p_xv, green, Bt.red.reg3);
			PV_RREG(p_xv, blue, Bt.red.reg3);
			*p_entry = red | ( green << 8 ) | ( blue << 16 );
		}
		break;
	  default:
		return 0;
	}
	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt495_init_hw"
 |
 |  Initialize hardware to its default state.
 |
 |    = 1 on success, 0 if error detected.
 *---------------------------------------------------------------------------*/
static int m_Lut_Bt495_init_hw( p_me )
	O_Lut_Bt495*	p_me;		/* -> context for this object. */
{
    int 		i_entry;	/* # of entry in cursor ram or lut accessing. */
    XV_struct*	p_xv;		/* Fast access to p_me->p_xv. */

	p_xv = p_me->p_xv;
 
    Bt495_Reg_Write(p_xv, 0x201,0xc0); /* Cmd Register_0: 5:1 mux, 8 bit mode */
    Bt495_Reg_Write(p_xv, 0x203,0x00); /* Cmd Register_2: sync&pedestal off */
#ifdef NOTDEF /* dual monitor mode.  */
    Bt495_Reg_Write(p_xv, 0x203,0x80); /* Cmd Register_2: sync on pedestal off*/
#endif /* NOTDEF */
    Bt495_Reg_Write(p_xv, 0x204,0x00); /* Interleave Register: display config */
    Bt495_Reg_Write(p_xv, 0x205,0xff); /* Red Lookup Bypass disabled */
    Bt495_Reg_Write(p_xv, 0x206,0xff); /* Green Lookup Bypass disabled */
    Bt495_Reg_Write(p_xv, 0x207,0xff); /* Blue Lookup Bypass disabled */
    Bt495_Reg_Write(p_xv, 0x208,0x01); /* LB Lookup Bypass disabled */
    Bt495_Reg_Write(p_xv, 0x209,0x00); /* Red Blink disabled */
    Bt495_Reg_Write(p_xv, 0x20a,0x00); /* Green Blink disabled */
    Bt495_Reg_Write(p_xv, 0x20b,0x00); /* Blue Blink disabled */
    Bt495_Reg_Write(p_xv, 0x20c,0x00); /* LB Blink disabled */
    Bt495_Reg_Write(p_xv, 0x210,0x00); /* Cursor disabled */
 
    for (i_entry=0x400; i_entry<=0x7ff; i_entry++)
        Bt495_Reg_Write(p_xv, i_entry, 0); /* Clear Cursor Ram */
 
    for (i_entry=0x0; i_entry<=0xff; i_entry++)
        Bt495_Lut_Write(p_xv, i_entry, 0x00000000); /* Black entire LUT */

	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt495_set_entries"
 |
 |  Set the values of a range of lookup table entries. There are three 8-in
 |  8-out lookup tables, one for each color component. The values pointed to
 |  by p_values are integers of the form XBGR.
 |
 |    = 1 on success, 0 if error detected.
 *---------------------------------------------------------------------------*/
static int m_Lut_Bt495_set_entries( p_me, type, start, count, p_values )
	O_Lut_Bt495*	p_me;		/* -> context for this object. */
	int				type;		/* =  type of entries (see objects.h). */
	int				start;		/* =  first location in table to set. */
	int				count;		/* =  # of entries to set. */
	int*			p_values;	/* -> where to get values for entries. */
{
	int			i_entry;	/* # of entry in lut now setting. */
	int*		p_entry;	/* Data for entry in lut now setting. */
	XV_struct*	p_xv;		/* Fast access to p_me->p_xv. */

	p_xv = p_me->p_xv;
	switch ( type )
	{
	  case O_Lut_Entry_Lut:
		p_entry = p_values;
		for ( i_entry = start; i_entry < start+count; i_entry++, p_entry++ )
		{
			Bt495_Lut_Write(p_xv, i_entry, *p_entry); 
		}
		break;
	  default:
		return 0;
	}
	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_Lut_Bt495_terminate"
 |
 |  Terminate use of this object.
 *---------------------------------------------------------------------------*/
static void m_Lut_Bt495_terminate( p_me )
	O_Lut_Bt495*	p_me;		/* -> context for this object. */
{
	free((char*)p_me);
}


static TV_Lut methods_Lut_Bt495 = {
	m_Lut_Bt495_control,
	m_Lut_Bt495_dump,
	m_Lut_Bt495_get_entries,
	m_Lut_Bt495_init_hw,
	m_Lut_Bt495_set_entries,
	m_Lut_Bt495_terminate,
    m_Lut_Bt495_pseudo_tag,
    m_Lut_Bt495_true_tag,
    m_Lut_Bt495_direct_tag,
    m_Lut_Bt495_video_tag,
    m_Lut_Bt495_free_video_tag,
};


/*---------------------------------------------------------------------------*
 |"O_Lut_Bt495_O_Lut_Bt495"
 |
 |  Create an instance of a Brooktree 495 lookup table object.
 |
 |    -> object's context on success, or 0 if error occurred.
 *---------------------------------------------------------------------------*/
O_Lut* O_Lut_Bt495_O_Lut_Bt495( p_me, p_xv )
	O_Lut_Bt495* p_me;		/* -> context to fill in, or 0 for dynamic alloc. */
	XV_struct*	 p_xv;		/* -> context for hardware library. */
{
	int		id;		/* Identifier byte returned by Brooktree RAMDAC. */
    int i, j;
	
	Bt495_Reg_Read(p_xv, 0x200, id);
	if ( (id & 0xff) != 0x2d )
	{
		ERRORF((fd, "O_Lut_Bt495_O_Lut_Bt495: expecting id 0x2d got id 0x%x\n",
			id & 0xff));
		return 0;
	}

	if ( !p_me )
	{
		if ( !(p_me = (O_Lut_Bt495*)malloc(sizeof(O_Lut_Bt495))) )
		{
			ERRORF((fd, 
				"O_Lut_Bt495_O_Lut_Bt495: Unable to allocate storage\n"));
		}
	}
	p_me->atv = &methods_Lut_Bt495;
	p_me->p_xv = p_xv;
	
/* init the video tag table */

	for(j = VIDEO_TAG, i = 0; i < NUM_VIDEO_TAGS; ++i, j += 2)
    {
	    p_me->vid_tags[i].used = 0;
	    p_me->vid_tags[i].tag = j;
    }
    p_me->next_tag = 0;

	return (O_Lut*)p_me;
}

static int  m_Lut_Bt495_pseudo_tag(p_me)
O_Lut_Bt495* p_me;
{
    return PSEUDO_TAG;
}

static int  m_Lut_Bt495_true_tag(p_me)
O_Lut_Bt495* p_me;
{
    return TRUE_TAG;
}

static int  m_Lut_Bt495_direct_tag(p_me)
O_Lut_Bt495* p_me;
{
    return -1; /* not supported */
}

static int  m_Lut_Bt495_video_tag(p_me, tag)
O_Lut_Bt495* p_me;
int tag;
{
	int i, j;
    if(tag)
	{
	    for(i = 0; i < NUM_VIDEO_TAGS; ++i)
	    {
		    if(p_me->vid_tags[p_me->next_tag].tag == tag)
			{
				p_me->vid_tags[p_me->next_tag].used += 1;
				break;
			}
	    }
	}
	else
	{
	  i = p_me->next_tag++;
	  while(p_me->next_tag != i)
	  {
		if(p_me->vid_tags[p_me->next_tag].used == 0)
			break;
		++p_me->next_tag;
		if(p_me->next_tag == NUM_VIDEO_TAGS)
			p_me->next_tag = 0;
	  }
	  if(p_me->next_tag == i)
	  {
		++p_me->next_tag;
		if(p_me->next_tag == NUM_VIDEO_TAGS)
			p_me->next_tag = 0;
	  }
	}
	return p_me->vid_tags[i].tag;
}

static void m_Lut_Bt495_free_video_tag(p_me, tag)
O_Lut_Bt495* p_me;
int tag;
{
	int i;
	for(i = 0; i < NUM_VIDEO_TAGS; ++i)
	{
		if(p_me->vid_tags[i].tag == tag)
		{
			if(p_me->vid_tags[i].used)
				p_me->vid_tags[i].used -= 1;
			if((p_me->vid_tags[i].used == 0) && (i < p_me->next_tag))
				p_me->next_tag = i;
		}
    }
}
