/*
 *  main_sun.c: Parallax Sun platform specific routines.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     29 Dec 94 - Andy Goodrich
 *       - Initial coding
 *
 *  Copyright(c) 1994, Parallax Graphics, Inc.
 */
#include <sys/types.h>			/* Standard types for this C environment. */
#include <stdio.h>				/* Stream input/output declarations. */
#include <fcntl.h>				/* Pick up O_RDWR and friends. */
#include "objects_sun.h"		/* Parallax Sun platform object defintions. */
#include "plxv_xpoint.h"		/* Cross point switch definitions. */
#define XV_DIAG 1
#include "xvdebug.h"

XV_struct*	pvsun_open();


/*---------------------------------------------------------------------------*
 |"i_PVsun_init_bus"
 |
 |  Internal routine to initialize bus, including interrupts.
 |
 |    = 1 if success, 0 if error detected.
 *---------------------------------------------------------------------------*/
static int i_PVsun_init_bus( p_me )
	XV_struct*	p_me;	/* -> hardware context. */
{
	int i_write;	/* # of times we have written register. */

	PV_WREG(p_me,StartUp.l, 0x8000618);
	for(i_write=0;i_write<10000;i_write++);
        PV_WREG(p_me,StartUp.l,0x8000608);
	for(i_write=0;i_write<10000;i_write++);
        PV_WREG(p_me,DispFlgs.l,0xa28);

    PV_WREG(p_me,Attr.l,p_me->main.Attr);
    PV_WREG(p_me,BIntMask.l,0);
    PV_WREG(p_me,IntMask.l,0x8000);
    PV_WREG(p_me,RasMode.l,p_me->main.RasMode);

	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_PVsun_ccube_present"
 |
 |  Return whether a ccube chip is present on this board or not.
 |
 |    = 1 if ccube present, 0 if not.
 *---------------------------------------------------------------------------*/
static int m_PVsun_ccube_present( p_me )
	XV_struct*	p_me;	/* -> hardware context. */
{
    return p_me->main.c3_rev ? 1 : 0;
}

/*---------------------------------------------------------------------------*
 |"m_PVsun_dump"
 |
 |  Dump the contents of a Parallax access object. 
 |
 *---------------------------------------------------------------------------*/
static void m_PVsun_dump( p_me, fd )
	XV_struct*	p_me;	/* -> hardware context. */
	FILE*		fd;		/* -> descriptor for file to dump to. */
{
	int		i_unit;		/* Unit # of object pointer now dumping. */
	int		n_units;	/* # of objects to dump. */

	fprintf(fd, "\n\nDump of XV_struct object at 0x%08x:\n", p_me);
	fprintf(fd, "   atv          = 0x%08x \n", p_me->atv);
    fprintf(fd, "   c3base       = 0x%08x \n", p_me->c3base);
    fprintf(fd, "   c3_rreg      = 0x%08x \n", p_me->c3_rreg);
    fprintf(fd, "   c3_wreg      = 0x%08x \n", p_me->c3_wreg);
    fprintf(fd, "   eprombase    = 0x%08x \n", p_me->eprombase);
	fprintf(fd, "   fd_ccube     = 0x%08x \n", p_me->fd_ccube);
    fprintf(fd, "   maps         = 0x%08x \n", p_me->maps);
    fprintf(fd, "   m1base       = 0x%08x \n", p_me->m1base);
    fprintf(fd, "   m8base       = 0x%08x \n", p_me->m8base);
    fprintf(fd, "   m24base      = 0x%08x \n", p_me->m24base);
	n_units = sizeof(p_me->p_inputs) / sizeof(p_me->p_inputs[0]);
	for( i_unit = 0; i_unit < n_units; i_unit++ )
	{
		fprintf(fd, "   p_inputs[%d]  = 0x%08x \n", i_unit, 
			p_me->p_inputs[i_unit]);
	}
	for( i_unit = 0; i_unit < n_units; i_unit++ )
	{
		fprintf(fd, "   p_outputs[%d] = 0x%08x \n", i_unit, 
			p_me->p_outputs[i_unit]);
	}
	fprintf(fd, "   p_draw       = 0x%08x \n", p_me->p_draw);
	fprintf(fd, "   p_i2c        = 0x%08x \n", p_me->p_i2c);
	fprintf(fd, "   p_lut        = 0x%08x \n", p_me->p_lut);
	fprintf(fd, "   p_mmap       = 0x%08x \n", p_me->p_mmap);
	fprintf(fd, "   p_rsrc       = 0x%08x \n", p_me->p_rsrc);
    fprintf(fd, "   regbase      = 0x%08x \n", p_me->regbase);
	fprintf(fd, "   xvm          = 0x%08x \n", p_me->xvm);
    fprintf(fd, "   xv_rreg      = 0x%08x \n", p_me->xv_rreg);
    fprintf(fd, "   xv_shdw      = 0x%08x \n", p_me->xv_shdw);
    fprintf(fd, "   xv_wreg      = 0x%08x \n", p_me->xv_wreg);
}

/*---------------------------------------------------------------------------*
 |"m_PVsun_get_zoom"
 |
 |  Return current zoom factor.
 |
 |    = current zoom factor.
 *---------------------------------------------------------------------------*/
static int m_PVsun_get_zoom( p_me )
	XV_struct*	p_me;	/* -> hardware context. */
{
	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_PVsun_init_context"
 |
 |  Initialize context from current resource object values.
 *---------------------------------------------------------------------------*/
static void m_PVsun_init_context( p_me )
	XV_struct*	p_me;	/* -> hardware context. */
{
    XV_addr_maps*   ap;
	int				i_row;
    int 			Xres;
	int				Yres;


	if (O_I2C_nv_read(p_me->p_i2c, 0,&p_me->main,0,sizeof(XV_controller_nvram)))
	{
		ERRORF((fd, "Unable to read main nvram!!! Using defaults\n"));
		O_Rsrc_store_to_context(p_me->p_rsrc, p_me);
	}
	if (O_I2C_nv_read(p_me->p_i2c, 1, &p_me->opt, 0, sizeof(XV_option_nvram)))
	{
		p_me->opt.board_code = 0;
		p_me->opt.clt_type = 0;
	}

	ap = p_me->maps;

    switch ( Xres = p_me->main.xsize )
	{
		case 1152:	p_me->addr_wrap = 0x100000;	break;
		case 1280:	p_me->addr_wrap = 0x140000;	break;
	}
    Yres = p_me->main.ysize;
 
    for ( i_row=0; i_row<Yres; i_row++) 
		ap->reg[i_row] =  (Xres *  i_row);
    for ( i_row=0; i_row<Yres; i_row++) 
		ap->m24[i_row] = (int *)( (int) p_me->m24base + ((Xres*4) * i_row));
    for ( i_row=0; i_row<Yres; i_row++) 
		ap->m1[i_row] = (int *)( (int) p_me->m1base + ((Xres/8) * i_row));
}


/*---------------------------------------------------------------------------*
 |"m_PVsun_init_disp"
 |
 |  Initialize display hardware from current context values. 
 *---------------------------------------------------------------------------*/
static void m_PVsun_init_disp( p_me )
	XV_struct*	p_me;	/* -> hardware context. */
{
    short i;
    PV_WREG(p_me,StartUp.l,p_me->main.StartUp);
    PV_WREG(p_me,DispFlgs.l,p_me->main.DispFlgs);
    PV_WREG(p_me,DispKludge.l,p_me->main.DispKludge);
    PV_WREG(p_me,HFP.l,p_me->main.h.fp);
    PV_WREG(p_me,HSync.l,p_me->main.h.sy);
    PV_WREG(p_me,HBP.l,p_me->main.h.bp);
    PV_WREG(p_me,HDisp.l,p_me->main.h.disp);
    PV_WREG(p_me,VFP.l,p_me->main.v.fp);
    PV_WREG(p_me,VBP.l,p_me->main.v.bp);
    PV_WREG(p_me,VDisp.l,p_me->main.v.disp);
    PV_WREG(p_me,VSync.l,p_me->main.v.sy);
    i = p_me->main.h.hsync_ser;
    if(i & 0x80) i += 0x80;
    PV_WREG(p_me,HSyncS.l,i);
    PV_WREG(p_me,HNSyncS.l,p_me->main.h.hsync_nser);
}


/*---------------------------------------------------------------------------*
 |"m_PVsun_init_hw"
 |
 |  Initialize hardware from current context values. 
 *---------------------------------------------------------------------------*/
static void m_PVsun_init_hw( p_me )
	XV_struct*	p_me;	/* -> hardware context. */
{
    int             		ioctl_data;		/* Value to pass in ioctl. */
	static XV_video_format  loop_format = { /* Default video format. */
		640,482,43,7,21
	};

    i_PVsun_init_bus(p_me);
    m_PVsun_init_disp(p_me);
	O_Draw_init_hw(p_me->p_draw);
	plxv_init_video(p_me);
#ifdef NOTDEF
    plxv_init_bt495(p_me);
#endif
	O_Lut_init_hw(p_me->p_lut);


	/* INITIALIZE INPUT/OUTPUT PORTS AND COMPRESSION CHIP: */

	if ( p_me->p_outputs[PORT_A] ) O_Output_init_hw(p_me->p_outputs[PORT_A]);
	if ( p_me->p_inputs[PORT_B] ) O_Input_init_hw(p_me->p_inputs[PORT_B]);
	if ( p_me->p_inputs[PORT_D] ) O_Input_init_hw(p_me->p_inputs[PORT_D]);
	if ( m_PVsun_ccube_present(p_me) ) C3Init(p_me);
	p_me->plxv_io_format[LOOPBACK] = &loop_format;
	p_me ->format_v_ptr = &(p_me->plxv_io_format[LOOPBACK]);
	p_me ->format_w_ptr = &(p_me->plxv_io_format[LOOPBACK]);
}


/*---------------------------------------------------------------------------*
 |"m_PVsun_init_pll"
 |
 |  Set zoom factor.
 *---------------------------------------------------------------------------*/
static void m_PVsun_init_pll( p_me, factor )
	XV_struct*	p_me;	/* -> hardware context. */
{
}


/*---------------------------------------------------------------------------*
 |"m_PVsun_set_delay"
 |
 |  Set delay factor.
 *---------------------------------------------------------------------------*/
static void m_PVsun_set_delay( p_me, factor )
	XV_struct*	p_me;	/* -> hardware context. */
	int			factor;	/* =  delay factor. */
{
}


/*---------------------------------------------------------------------------*
 |"m_PVsun_set_zoom"
 |
 |  Set zoom factor.
 *---------------------------------------------------------------------------*/
static int m_PVsun_set_zoom( p_me, factor )
	XV_struct*	p_me;	/* -> hardware context. */
	int			factor;	/* =  zoom factor. */
{
	return 1;
}


/*---------------------------------------------------------------------------*
 |"m_PVsun_terminate"
 |
 |  Set zoom factor.
 *---------------------------------------------------------------------------*/
static void m_PVsun_terminate( p_me )
	XV_struct*	p_me;	/* -> hardware context. */
{
}


/*---------------------------------------------------------------------------*
 |"m_PVsun_vertical"
 |
 |  Turn on or off vertical interrupt processing in the device driver.
 *---------------------------------------------------------------------------*/
static int m_PVsun_vertical( p_me, on_off )
	XV_struct*	p_me;	/* -> hardware context. */
	int			on_off;	/* =  1 if enabled, 0 if not. */
{
    int             ioctl_data;

#ifdef NOTDEF
    ioctl_data = on_off ? 1 : 0;
    if ( ioctl(p_me->fd_ccube, GCTEST, &ioctl_data) )
        fprintf(stderr, "**** ioctl error on off\n");
#endif
}


/*---------------------------------------------------------------------------*
 |"m_PVsun_visibility"
 |
 |  Turn on or off visibility.
 *---------------------------------------------------------------------------*/
static void m_PVsun_visibility( p_me, on_off )
    XV_struct*  p_me;   /* -> hardware context. */
    int         on_off; /* =  1 if visible, 0 if blanked. */
{
    int vis;    /* Binary version of on_off. */
 
    vis = (on_off) ? 1 : 0;
    PV_WREG(p_me, DispKludge.b.disp_blank, vis);
}
 

static TV_PVsun main_methods_tv = {
	m_PVsun_ccube_present,
	m_PVsun_dump,
	m_PVsun_get_zoom,
	m_PVsun_init_context,
	m_PVsun_init_hw,
	m_PVsun_init_pll,
	m_PVsun_set_delay,
	m_PVsun_set_zoom,
	m_PVsun_terminate,
	m_PVsun_vertical,
	m_PVsun_visibility
};


/*---------------------------------------------------------------------------*
 |"O_HWL_O_HWL"
 |
 |  Generic open command for hardware library access which uses Sun platform.
 |
 |   -> hardware library context on success, = 0 on failure
 *---------------------------------------------------------------------------*/
XV_struct* O_HWL_O_HWL(level, device, rsrc_file)
	char* level;		/* -> level of initialization desired. */
    char* device;		/* -> name of device to open. */
	char* rsrc_file;	/* -> resource name or 0. */
{
	return pvsun_open(level, device, rsrc_file);
}


/*---------------------------------------------------------------------------*
 |"O_HWL_device"
 |
 |  Return name of Parallax device to use. This will either be the value of
 |  the environment variable PLX_DEVICE or a default of "/dev/plxv0".
 *---------------------------------------------------------------------------*/
char* O_HWL_device()
{
	char*	device;

	device = (char*)getenv("PLX_DEVICE");

	return device ? device : 0;
}


/*---------------------------------------------------------------------------*
 |"O_HWL_rsrc"
 |
 |  Return name of resource file to use. This will either be the value of the
 |  environment variable PLX_RSRC or a default of "/opt/parallax/etc/plxv0.
 *---------------------------------------------------------------------------*/
char* O_HWL_rsrc()
{
	char*	rsrc;

	rsrc = (char*)getenv("PLX_RSRC");

	return rsrc ? rsrc : "/opt/parallax/etc/plxv0";
}


/*---------------------------------------------------------------------------*
 |"O_Input_O_Input"
 |
 |  Return an input device given a hardware context and unit number for the
 |  device.
 |
 |    -> object on success, 0 if error detected or device not present.
 *---------------------------------------------------------------------------*/
O_Input* O_Input_O_Input(p_me, unit, p_xv)
	O_Input*	p_me;	/* -> object storage or 0. */
	int			unit;	/* =  unit number of device. */
	XV_struct*	p_xv;	/* -> hardware library context to use. */
{
	return (O_Input*)O_Input_Dmsd_O_Input_Dmsd(p_me, unit, p_xv);
}


/*---------------------------------------------------------------------------*
 |"O_Output_O_Output"
 |
 |  Return an output device given a hardware context and unit number for the
 |  device.
 |
 |    -> object on success, 0 if error detected or device not present.
 *---------------------------------------------------------------------------*/
O_Output* O_Output_O_Output(p_me, unit, p_xv)
	O_Output*	p_me;	/* -> object storage or 0. */
	int			unit;	/* =  unit number of device. */
	XV_struct*	p_xv;	/* -> hardware library context to use. */
{
	return (O_Output*)O_Output_Dmse_O_Output_Dmse(p_me, unit, p_xv);
}


/*---------------------------------------------------------------------------*
 |"pvsun_open"
 |
 |  Open a Parallax Sun platform device to a specified initialization level:
 |     "minimum"    - create only its I2C and resource objects.
 |     "no_hw_init" - create all hardware access objects, but don't 
 |                    initialize the hardware.
 |     "full"       - create a fully initialized access object.
 *---------------------------------------------------------------------------*/
XV_struct* pvsun_open(level, device, rsrc_file)
	char* level;		/* -> object level: "minimum", "no_hw_init", "full". */
    char* device;		/* -> name of device to open. */
	char* rsrc_file;	/* -> resource name or 0. */
{
    XV_addr_maps*       ap;			/* Address maps structure. */
	FILE*				fd_rsrc;	/* File to load resource values from. */
	int					i_shadow;	/* # of word in shadow registers zeroing. */
	int					n_shadow;	/* # of words in shadow registers. */
	O_Mmap*				p_mmap;		/* Memory map of PVsun device. */
	int*				p_shadow;	/* Word in shadow registers zeroing. */
    XV_struct*         	p_xv;		/* Context we are filling in. */
    XV_shdw_regs*      	sp;			/* Shadow register area. */
    XV_mem_map*        	xvm;		/* Map for card. */


	DEBUGF(0x80, (fd, "Entry to pvsun_open(%s, %s, %s) stack at %08x\n", 
		level, device, rsrc_file, &ap));

	/*  OPEN DEVICE AND ACQUIRE AN ADDRESS MAP FOR IT. */

	if ( !(p_mmap = (O_Mmap*)O_Mmap_O_Mmap(device)) )
	{
		fprintf(stderr, "*** Unable to memory map %s\n", device);
	}


	/*  ALLOCATE STRUCTURES WE NEED, INCLUDING OUR CONTEXT: */
 
    if ((p_xv = (XV_struct *)malloc(sizeof(XV_struct))) == NULL)
    {   /* malloc failed */
        printf("malloc failed on plxv_init_struct:main struct\n");
        exit(-1);
    }
    if ((sp = (XV_shdw_regs *)malloc(sizeof(XV_shdw_regs))) == NULL)
    {   /* malloc failed */
        printf("malloc failed on plxv_init_struct:xv_shdw\n");
        exit(-1);
    }
    if ((ap = (XV_addr_maps *)malloc(sizeof(XV_addr_maps))) == NULL)
    {   /* malloc failed */
        printf("malloc failed on plxv_init_struct:addr_maps\n");
        exit(-1);
    }


	/* FILL IN CONTEXT WITH MEMORY MAP ADDRESSES: */

	p_xv->p_mmap = p_mmap;
	xvm = (XV_mem_map*)((u_int)O_Mmap_virtual(p_mmap, 0x10000) - 0x10000);
	p_xv->p_mmap = p_mmap;
	p_xv->atv = (void*)&main_methods_tv;
    p_xv->c3base = (int*)O_Mmap_virtual(p_mmap, 0x50000);
    p_xv->c3_rreg = (C3_RREGS *)p_xv->c3base;
    p_xv->c3_wreg = (C3_WREGS *)p_xv->c3base;
    p_xv->eprombase = (int*)&xvm->Prom0;
	p_xv->fd_ccube = p_mmap->fd_ccube;
    p_xv->maps = ap;
    p_xv->m1base = (int*)O_Mmap_virtual(p_mmap, 0x10000);
    p_xv->m8base = (char*)O_Mmap_virtual(p_mmap, 0x80000);
    p_xv->m24base = (int*)O_Mmap_virtual(p_mmap, 0x200000);
    p_xv->regbase = (int*)O_Mmap_virtual(p_mmap, 0x40000);
	p_xv->xvm = xvm;
    p_xv->xv_rreg = (XV_read_regs*)p_xv->regbase;
    p_xv->xv_shdw = (XV_shdw_regs*)sp;
    p_xv->xv_wreg = (XV_write_regs*)p_xv->regbase;


    /* ZERO REGISTER FILE SHADOW STRUCTURE. */

    n_shadow = sizeof(XV_shdw_regs) >> 2;
    for (p_shadow = (int*)sp, i_shadow = 0; i_shadow < n_shadow; i_shadow++ ) 
	{
		*p_shadow++ = 0;
	}


	/* ALLOCATE MINIMUM SET OF OBJECTS AND INITIALIZE CONTEXT FROM RESOURCES. */

	p_xv->p_draw = O_Draw_O_Draw(0, p_xv);
    p_xv->p_i2c = (O_I2C*)O_I2C_O_I2C(p_xv);
	p_xv->p_lut = O_Lut_Bt463_O_Lut_Bt463(0, p_xv);
    p_xv->p_rsrc = (O_Rsrc*)O_Rsrc_O_Rsrc_pvsun();
	if ( rsrc_file )
	{	
		if ( fd_rsrc = fopen(rsrc_file, "r") )
		{
			O_Rsrc_load_from_file(p_xv->p_rsrc, fd_rsrc);
			fclose(fd_rsrc);
		}
	}
	m_PVsun_init_context(p_xv);


	/*  IF A MINIMUMAL OBJECT WAS REQUESTED WE ARE DONE. */

    if ( !strcmp(level, "minimum") ) return p_xv;


	/* INITIALIZE OBJECTS FOR INPUT OUTPUT PORTS: */

#ifdef FIX_THIS
	p_xv->p_outputs[PORT_A] = O_Output_O_Output(0, p_xv);
	p_xv->p_inputs[PORT_B]  = O_Input_O_Input( 1, p_xv );
	p_xv->p_inputs[PORT_D] = O_Input_O_Input( 0, p_xv ); 
#endif FIX_THIS


	/*  IF AN OBJECT WITHOUT HARDWARE INITIALIZATION IS WANTED WE ARE DONE. */

	if ( !strcmp(level, "no_hw_init") ) return p_xv;

		
	/*  INITIALIZE ALL THE HARDWARE... */

	m_PVsun_init_hw( p_xv );

	DEBUGF(0x80, (fd, "Exit from pvsun_open(%s, %s, %s) stack at %08x\n", 
		level, device, rsrc_file, &ap));
	return p_xv;
}
