/*
 *  objects.h: Object definitions for object-oriented hardware library.
 *
 *  CONVENTIONS:
 *    (1) Tab stops every 4 characters.
 *    (2) Routines are arranged in alphabetical order within file.
 *    (3) Variables are declared in alphabetical order within routines.
 *
 *  REVISION HISTORY:
 *
 *     25 Sep 94 - Andy Goodrich
 *       - Initial coding
 *
 *  Copyright(c) 1994, Parallax Graphics, Inc.
 */
#ifndef  objects_h_INCLUDED
#define objects_h_INCLUDED

#include <sys/types.h>		/* Pick up u_int and friends. */


/*--------------------------------------------------------------------------*
 |  HARDWARE LIBRARY ACCESS OBJECT DEFINITION:
 |
 |    (This is just XV_struct in sheep's clothing.)
 *--------------------------------------------------------------------------*/

#define O_HWL_init_hw(obj) (*(obj)->atv->init_hw)(obj)


/*---------------------------------------------------------------------------*
 *  DRAWING OBJECT DEFINITION:
 *
 *    Notes:
 *      (1) The GC field f_transparency, when 1, places the hardware in
 *          1-bit transparency mode. Foreground color must have bit-24 a 1 
 *          to write, background color must have bit-24 a 0 to not write.
 *          (1's write fgcolor, 0's don't write at all).
 *---------------------------------------------------------------------------*/
#define O_Draw_copy_area(obj, src_x, src_y, dst_x, dst_y, w, h) \
	(*(obj)->atv->copy_area)(obj, src_x, src_y, dst_x, dst_y, w, h)

#define O_Draw_dump(obj, fd) (*(obj)->atv->dump)(obj, fd)

#define O_Draw_fill_box(obj, x, y, w, h) (*(obj)->atv->fill_box)(obj,x, y, w, h)

#define O_Draw_get_gc(obj, p_gc) (*(obj)->atv->get_gc)(obj, p_gc)

#define O_Draw_init_hw(obj) (*(obj)->atv->init_hw)(obj)

#define O_Draw_set_gc(obj, p_gc) (*(obj)->atv->set_gc)(obj, p_gc)

#define O_Draw_terminate(obj) (*(obj)->atv->terminate)(obj)

#define O_Draw_vector(obj, x_bgn, y_bgn, x_end, y_end) \
	(*(obj)->atv->vector)(obj, x_bgn, y_bgn, x_end, y_end)

#define O_Draw_vector_uncook(obj,x0,y0,num_pix,i1,i2,e,left_flag,up_flag,x_major_flag) \
	(*(obj)->atv->vector_uncook)(obj,x0,y0,num_pix,i1,i2,e,left_flag, up_flag,x_major_flag)

#define O_Draw_wait_for_done(obj) (*(obj)->atv->wait_for_done)(obj)

typedef struct _O_Draw_Gc {	/* Drawing controls which may be set: */
	u_int	color;			/* Color for accelerator operations. */
	u_int	color1_bg;		/* Background color to use with 1-bit ops. */
	u_int	color1_fg;		/* Foreground color to use with 1-bit ops. */
	int		f_transparency;	/* = 1 if using transparency mode, 0 if not. */
	u_int	readback;		/* Byte to read 8-bit ops (0=r,1=g,2=b,3=tags)*/
	u_int	rop;			/* Raster operation bits. */
	u_int	stencil_mask;	/* Read mask for comparing wid bits w/value. */
	u_int	stencil_value;	/* Value to compare w/wid bits in word. */
	u_int	wid_mask;		/* Write mask for setting wid values. */
	u_int	wid_value;		/* Value to write in to wid bits of word. */
	u_int	write_mask;		/* 32-bit write mask for data written. */
} O_Draw_Gc;

typedef struct _TV_Draw { 
	int     (*copy_area)();		/* Copy a rectangular area on screen. */
	void	(*dump)();			/* Dump contents of this object to a file. */
	int 	(*fill_box)();		/* Hardware fill a rectangular area. */
	void	(*get_gc)();		/* Get current drawing control values. */
	void	(*init_hw)();		/* Init hardware from current context values */
	int 	(*set_gc)();		/* Set current drawing control values. */
	void	(*terminate)();		/* Terminate use of this object. */
	int     (*vector)();		/* Draw a vector. */
	void	(*wait_for_done)();	/* Wait for accelerator to complete. */
	int     (*vector_uncook)();	/* Draw a vector. uncooked */
} TV_Draw;

typedef struct _O_Draw {
	TV_Draw*	atv;		/* Methods: first entry in all derived objects. */
} O_Draw;

O_Draw* O_Draw_O_Draw();	/* Object constructor. */


/*--------------------------------------------------------------------------*
 |  INPUT OBJECT DEFINITION:
 *--------------------------------------------------------------------------*/

#define O_Input_init_hw(obj) (*(obj)->atv->init_hw)(obj)

#define O_Input_off(obj) (*(obj)->atv->off)(obj)

#define O_Input_on(obj) (*(obj)->atv->on)(obj)

#define O_Input_set_levels(obj, bright, contrast, hue, satur) \
	(*(obj)->atv->set_levels)(obj, bright, contrast, hue, satur)

#define O_Input_set_register(obj, reg, value) \
	(*(obj)->atv->set_register)(obj, reg, value)

#define O_Input_set_type(obj, timing, format, instance) \
	(*(obj)->atv->set_type)(obj, timing, format, instance)

#define O_Input_status(obj) (*(obj)->atv->status)(obj)

#define O_Input_terminate(obj) (*(obj)->atv->terminate)(obj)

#define O_Input_parameter_string(obj, s) (*(obj)->atv->set_parameter)(obj, s)

typedef struct _TV_Input { 
	void	(*init_hw)();		/* Init hardware from current context values. */
	void	(*off)();			/* Turn off input. */
	void	(*on)();			/* Turn on input. */
	void	(*set_levels)();	/* Set color levels of input. */
	int		(*set_register)();	/* Set a register value. */
	int		(*set_type)();		/* Set type of input to use. */
	int		(*status)();		/* Return status of video signal. */
	void    (*terminate)();		/* Object destructor. */
    int		(*set_parameter)(); /* use string to set parameter */
} TV_Input;

typedef struct _O_Input {
	TV_Input*	atv;		/* Methods: first entry in all derived objects. */
} O_Input;


/*--------------------------------------------------------------------------*
 *  I2C OBJECT DEFINITION:
 *--------------------------------------------------------------------------*/

#define O_I2C_acknowledge(obj, addr, context) \
	(*(obj)->atv->acknowledge)(obj, addr, context)

#define O_I2C_delay(obj, usec) (*(obj)->atv->delay)(obj, usec)

#define O_I2C_end_of_msg(obj) (*(obj)->atv->end_of_msg)(obj)

#define O_I2C_ping(obj, addr, saddr) \
	(*(obj)->atv->ping)(obj, addr, saddr) 

#define O_I2C_nv_read(obj, unit, p_data, addr, count) \
	(*(obj)->atv->nv_read)(obj, unit, p_data, addr, count) 

#define O_I2C_nv_write(obj, unit, p_data, addr, count) \
	(*(obj)->atv->nv_write)(obj, unit, p_data, addr, count) 

#define O_I2C_read_data(obj, addr, saddr, p_data) \
	(*(obj)->atv->read_data)(obj, addr, saddr, p_data)

#define O_I2C_read_no_sub(obj, addr, p_data) \
	(*(obj)->atv->read_no_sub)(obj, addr, p_data)

#define O_I2C_set_address(obj, addr, saddr) \
	(*(obj)->atv->set_address)(obj, addr, saddr)

#define O_I2C_slave_address(obj, addr, dir) \
	(*(obj)->atv->slave_address)(obj, addr, dir)

#define O_I2C_start_of_msg(obj) (*(obj)->atv->start_of_msg)(obj)

#define O_I2C_sub_address(obj, addr) \
	(*(obj)->atv->sub_address)(obj, addr)

#define O_I2C_terminate(obj) (*(obj)->atv->terminate)(obj)

#define O_I2C_write(obj, addr, saddr, data) \
	(*(obj)->atv->write)(obj, addr, saddr, data)

#define O_I2C_write_data(obj, data) \
	(*(obj)->atv->write_data)(obj, data)

#define O_I2C_write_indexed(obj, addr, saddr, index, data) \
	(*(obj)->atv->write_indexed)(obj, addr, saddr, index, data)

#define O_I2C_write_indexed3(obj, addr, saddr, index, data1, data2, data3) \
	(*(obj)->atv->write_indexed3)(obj, addr, saddr, index, \
		data1, data2, data3)

typedef struct _TV_I2C {
	int		(*acknowledge)();	/* Read ack for last operation. */
	void	(*delay)();			/* Delay a certain # of microseconds. */
	void	(*end_of_msg)();	/* Send end of message. */
	int		(*nv_read)();		/* Read a portion of nvram. */
	int		(*nv_write)();		/* Write a portion of nvram. */
	int		(*ping)();			/* Check if a device is present or not. */
	int 	(*read_data)();		/* Read a data byte from a device. */
	int 	(*read_no_sub)();	/* Read a data byte using no sub-address. */
	int 	(*set_address)();	/* Set device address and sub-address. */
	void	(*slave_address)();	/* Send slave address for a device. */
	void	(*start_of_msg)();	/* Send start of message. */
	void	(*sub_address)();	/* Send subaddress for a device. */
	void    (*terminate)();		/* Object destructor. */
	int 	(*write)();			/* Send slave, sub, data byte to a device. */
	void	(*write_data)();	/* Send data byte to a device. */
	int 	(*write_indexed)();	/* Send slave, sub, data byte to a device. */
	int 	(*write_indexed3)();/* Send slave, sub, 3 data bytes to a device. */
} TV_I2C;

typedef struct _O_I2C {
	TV_I2C*	atv;			/* Methods transfer vector. */
} O_I2C;


/*--------------------------------------------------------------------------*
 |  LOOKUP TABLE OBJECT:
 *--------------------------------------------------------------------------*/
#define O_Lut_control(obj, op, arg) (*(obj)->atv->control)(obj, op, arg)

#define O_Lut_dump(obj, fd) (*(obj)->atv->dump)(obj, fd)

#define O_Lut_get_entries(obj, type, start, count, p_values) \
	(*(obj)->atv->get_entries)(obj, type, start, count, p_values)

#define O_Lut_get_entry(obj, type, start, p_value)  \
	(*(obj)->atv->get_entries)(obj, type, start, 1, p_value)

#define O_Lut_init_hw(obj) (*(obj)->atv->init_hw)(obj)

#define O_Lut_set_entries(obj, type, start, count, p_values) \
	(*(obj)->atv->set_entries)(obj, type, start, count, p_values)

#define O_Lut_set_entry(obj, type, start, value) \
{ \
	int	value_loc = value; \
	(*(obj)->atv->set_entries)(obj, type, start, 1, &value_loc); \
}

#define O_Lut_terminate(obj) (*(obj)->atv->terminate)(obj)
#define O_Lut_pseudo_tag(obj) (*(obj)->atv->pseudo_tag)(obj)
#define O_Lut_true_tag(obj) (*(obj)->atv->true_tag)(obj)
#define O_Lut_direct_tag(obj) (*(obj)->atv->direct_tag)(obj)
#define O_Lut_video_tag(obj, tag) (*(obj)->atv->video_tag)(obj, tag)
#define O_Lut_free_video_tag(obj, tag) (*(obj)->atv->free_video_tag)(obj, tag)

typedef struct _TV_Lut { 
	int 	(*control)();		/* Perform an device-dependent operation. */
	void	(*dump)();			/* Dump contents of this object. */
	int 	(*get_entries)();	/* Get the values of lookup table entries. */
	int 	(*init_hw)();		/* Init hardware from current context values. */
	int 	(*set_entries)();	/* Set the values of lookup table entries. */
	void	(*terminate)();		/* Terminate use of this object. */
	int		(*pseudo_tag)();	/* Get pseudo color tag. */
	int		(*true_tag)();		/* Get true color tag. */
	int		(*direct_tag)();	/* Get direct color tag. */
	int		(*video_tag)();		/* Get video tag. */
	void	(*free_video_tag)();/* Free video color tag. */
} TV_Lut;

typedef struct _O_Lut {
	TV_Lut*	atv;		/* Methods: first entry in all derived objects. */
} O_Lut;

/* Types of get_entries/set_entries: */

#define O_Lut_Entry_Cursor      0x01000000  /* Window identifer entries. */
#define O_Lut_Entry_Lut         0x02000000  /* Lookup table entries. */
#define O_Lut_Entry_Overlay     0x03000000  /* Window identifer entries. */
#define O_Lut_Entry_Wid         0x04000000  /* Window identifer entries. */
#define O_Lut_Pseudo		0x0	    /* pseudo color class */
#define O_Lut_Direct		0x1	    /* direct color class */
 
#define O_Lut_Entry_Type_Mask   0xff000000  /* Mask for entry type. */
#define O_Lut_Entry_Visual_Mask 0x000000ff  /* Mask for visual being set. */
  

/*--------------------------------------------------------------------------*
 |  MMAP OBJECT:
 *--------------------------------------------------------------------------*/
 
#define	O_Mmap_dump(obj, fd) (*((O_Mmap*)(obj))->atv->dump)(obj, fd)

#define	O_Mmap_real(obj, addr) (*((O_Mmap*)(obj))->atv->real)(obj, addr)

#define	O_Mmap_terminate(obj) (*((O_Mmap*)(obj))->atv->terminate)(obj)

#define	O_Mmap_virtual(obj, addr) (*((O_Mmap*)(obj))->atv->virtual)(obj, addr)

typedef struct _Mmap_Region {
	u_int	length;		/* Length of region. */
	u_int	sbus;		/* Sbus address for base of region. */
	u_int	virtual;	/* Virtual memory address for base of region. */
} Mmap_Region;

typedef struct _Tv_Mmap {
	void    (*dump)();		/* Dump the contents of an object to a fd. */
	u_int	(*real)();	    /* Map virtual address to real (sbus) address. */
	void	(*terminate)();	/* Destructor for this object. */
	u_int	(*virtual)();	/* Map real (sbus) address to virtual address. */
} Tv_Mmap;

enum {	/* Indices to entries in maps array. */
	MM_Ccube = 0,
	MM_Eprom,
	MM_M1,
	MM_M8,
	MM_M24,
	MM_Regs,
	nMM
};

typedef struct _O_Mmap {
	Tv_Mmap		*atv;		/* Memory map methods. */
	int   		fd_ccube;	/* File descriptor: ccube register area. */
	int 		fd_eprom;	/* File descriptor: eprom. */
	int 		fd_m1;		/* File descriptor: 1-bit memory area. */
	int 		fd_m8;		/* File descriptor: 8-bit memory area. */
	int 		fd_m24;		/* File descriptor: 24-bit memory area. */
	int 		fd_regs;	/* File descriptor: register area. */
	Mmap_Region maps[nMM];	/* Regions mapped in. */
	char		name[256];	/* Name of device being mapped. */
} O_Mmap;


O_Mmap	*O_Mmap_Create(); /* Create and return a memory map object. */


/*--------------------------------------------------------------------------*
 |  OUTPUT OBJECT DEFINITION: 
 *--------------------------------------------------------------------------*/

#define O_Output_init_hw(obj) (*(obj)->atv->init_hw)(obj)

#define O_Output_off(obj) (*(obj)->atv->off)(obj)

#define O_Output_on(obj) (*(obj)->atv->on)(obj)

#define O_Output_set_levels(obj, bright, contrast, hue, satur) \
	(*(obj)->atv->set_levels)(obj, bright, contrast, hue, satur)

#define O_Output_set_register(obj, reg, value) \
	(*(obj)->atv->set_register)(obj, reg, value)

#define O_Output_set_type(obj, timing, format, instance) \
	(*(obj)->atv->set_type)(obj, timing, format, instance)

#define O_Output_terminate(obj) (*(obj)->atv->terminate)(obj)

#define O_Output_parameter_string(obj, s) (*(obj)->atv->set_parameter)(obj, s)

typedef struct _TV_Output { 
	void	(*init_hw)();		/* Init hardware from current context values. */
	void	(*off)();			/* Turn off input. */
	void	(*on)();			/* Turn on input. */
	void	(*set_levels)();	/* Set color levels of input. */
	int		(*set_register)();	/* Set register value. */
	int		(*set_type)();		/* Set type of input to use. */
	void    (*terminate)();		/* Object destructor. */
    int		(*set_parameter)(); /* use string to set parameter */
} TV_Output;

typedef struct _O_Output {
	TV_Output*	atv;		/* Methods: first entry in all derived objects. */
} O_Output;

	
/*--------------------------------------------------------------------------*
 |  RESOURCE OBJECT DEFINITION:
 *--------------------------------------------------------------------------*/

#define O_Rsrc_defaults(rsrc, bd)  (*(rsrc)->atv->defaults)(rsrc, bd)

#define O_Rsrc_empty(rsrc)         (*(rsrc)->atv->empty)(rsrc)

#define O_Rsrc_exists(rsrc, name)  (*(rsrc)->atv->exists)(rsrc, name)

#define O_Rsrc_get(rsrc, name)     (*(rsrc)->atv->get)(rsrc, name)

#define O_Rsrc_get_long(rsrc, name, p_buf) \
	(*(rsrc)->atv->get_long)(rsrc, name, p_buf)

#define O_Rsrc_load_from_context(rsrc, p_xv) \
	(*(rsrc)->atv->load_from_context)(rsrc, p_xv)

#define O_Rsrc_load_from_file(rsrc, fd) \
	(*(rsrc)->atv->load_from_file)(rsrc, fd)

#define O_Rsrc_remove(rsrc, name)  (*(rsrc)->atv->remove)(rsrc, name)

#define O_Rsrc_set(rsrc, name, value) \
	(*(rsrc)->atv->set)(rsrc, name, value)

#define O_Rsrc_set_long(rsrc, name, length, value) \
	(*(rsrc)->atv->set)(rsrc, name, length, value)

#define O_Rsrc_store_to_context(rsrc, p_xv) \
	(*(rsrc)->atv->store_to_context)(rsrc, p_xv)

#define O_Rsrc_store_to_file(rsrc, fd) \
	(*(rsrc)->atv->store_to_file)(rsrc, fd)

#define O_Rsrc_terminate(rsrc)        (*(rsrc)->atv->terminate)(rsrc)

typedef struct Tv_Rsrc{
	void		(*defaults)();		    /* Set resource values to defaults. */
	void		(*empty)();	 			/* Empty a symbol table. */
    int  		(*exists)();   			/* See if a resource exists. */
    int  		(*get)();   			/* Get a resource value. */
    int  		(*get_long)(); 			/* Get a resource long value. */
	void		(*load_from_context)();	/* Initialize resource from context. */
	int			(*load_from_file)();	/* Load resources from a file. */
    int         (*remove)(); 			/* Remove a symbol table value. */
    int         (*set)();    			/* Set a resource value. */
    int         (*set_long)(); 			/* Set a long resource value. */
	void		(*store_to_context)();	/* Initialize context from resource. */
	int			(*store_to_file)();		/* Store resources to a file. */
    void        (*terminate)(); 		/* Terminate use of resource object. */
} Tv_Rsrc;

typedef struct _O_Rsrc{
	Tv_Rsrc*	atv;				/* Symbol table transfer vector. */
} O_Rsrc;


O_Rsrc* O_Rsrc_Create();            /* Resource object creation routine. */


/*--------------------------------------------------------------------------*
 |  SYMBOL TABLE OBJECT DEFINITION:
 *--------------------------------------------------------------------------*/
 
#define O_SST_create_cookie(sto)	(*(sto)->atv->create_cookie)(sto)

#define O_SST_dump(sto, fd)         (*(sto)->atv->dump)(sto, fd)

#define O_SST_empty(sto)            (*(sto)->atv->empty)(sto)

#define O_SST_free_cookie(sto,cookie)	\
	(*(sto)->atv->free_cookie)(sto, cookie)

#define O_SST_get(sto, name)        (*(sto)->atv->get)(sto, name)

#define O_SST_load(sto, fd) \
	(*(sto)->atv->load)(sto, fd)

#define O_SST_next(sto, cookie) \
	(*(sto)->atv->next)(sto, cookie)

#define O_SST_present(sto, cookie)  (*(sto)->atv->present)(sto,cookie)

#define O_SST_remove(sto, name)     (*(sto)->atv->remove)(sto, name)

#define O_SST_set(sto, name, type, value) \
	(*(sto)->atv->set)(sto, name, type, value)

#define O_SST_store(sto, fd) \
	(*(sto)->atv->store)(sto, fd)

#define O_SST_terminate(sto)        (*(sto)->atv->terminate)(sto)

typedef struct {        /* String descriptor: */
    int     n_string;   /* ... # of characters in string. */
    u_char* p_string;   /*... pointer to string value. */
} ST_String;

typedef union {         /* Value of symbol. */
    char*       c;      /* ... string value. */
    double      d;      /* ... floating point value. */
    int         i;      /* ... integer value. */
    void*       p;      /* ... generic pointer. */
    ST_String   s;      /* ... string descriptor. */
} ST_Value;

typedef struct SST_Entry {
    struct SST_Entry* p_next;   /* Next symbol table entry in bucket. */
    int               type;     /* Type of value. */ 
    ST_Value          value;    /* Pointer to value of symbol (word aligned). */
	char              name[1];  /* Name of symbol. */
} SST_Entry;

enum STE_Type {         /* Symbol table entry value types: */
    STE_No_Value = 0,   /* No value has been assigned. */
    STE_Character,      /* Character string value has been assigned. */
	STE_Double,         /* Floating point value has been assigned. */
    STE_Generic,        /* Generic pointer value has been assigned. */
    STE_Integer,        /* Integer value has been assigned. */
    STE_String,         /* Length, value pointer pair. */
	STE_String_Constant /* Statically allocated length, value pointer pair. */
};

typedef struct Tv_SST {
	void*		(*create_cookie)();	/* Return a presentation cookie. */
	void		(*dump)();			/* Dump the contents of table to a fd. */
	void		(*empty)();	 		/* Empty a symbol table. */
	void		(*free_cookie)();	/* Free a presentation cookie. */
    SST_Entry*  (*get)();   		/* Get a symbol table value. */
	int			(*load)();			/* Load symbols from a file. */
    SST_Entry*  (*next)();  		/* Get next symbol in symtab present. */
    int         (*present)();  		/* Set up to list out symbols in table. */
    int         (*remove)(); 		/* Remove a symbol table value. */
    int         (*set)();    		/* Set a symbol table value. */
	int			(*store)();			/* Store symbols to a file. */
    void        (*terminate)(); 	/* Terminate use of a symbol table. */
} Tv_SST;

typedef struct _O_SST {
	Tv_SST	*atv;				/* Symbol table transfer vector. */
	void	(*delete_notify)();	/* Symbol deletion notify routine. */
} O_SST;


O_SST* O_SST_Create(                /* Symbol table allocation routine.
	int     average_symbols;        // = average # of symbols expected. 
	void    (*delete_rtn)(          // -> delete rtn to call or zero.  
		char	*name,				// -> name of symbol deleting.
		void	*value				// =  value of symbol deleting. 
	); */
);

/*---------------------------------------------------------------------------*
 *  VIDEO OBJECT DEFINITION:
 *
 *    Notes:
 *      (1) The GC field f_transparency, when 1, places the hardware in
 *          1-bit transparency mode. Foreground color must have bit-24 a 1
 *          to write, background color must have bit-24 a 0 to not write.
 *          (1's write fgcolor, 0's don't write at all).
 *---------------------------------------------------------------------------*/
#define O_Video_dump(obj, fd) (*(obj)->atv->dump)(obj, fd)
 
#define O_Video_get_attributes(obj, p_attr) (*(obj)->atv->get_attr)(obj,p_attr)
 
#define O_Video_init_hw(obj) (*(obj)->atv->init_hw)(obj)
 
#define O_Video_set_attributes(obj, p_attr) (*(obj)->atv->set_attr)(obj,p_attr)
 
#define O_Video_terminate(obj) (*(obj)->atv->terminate)(obj)
 
#define O_Video_video_in(obj, chan, single) \
	(*(obj)->atv->video_in)(obj, chan, single)
 
#define O_Video_video_off(obj, chan, now) \
	(*(obj)->atv->video_off)(obj, chan, now)
 
#define O_Video_video_out(obj, chan, single) \
	(*(obj)->atv->video_out)(obj, chan, single)
 
#define O_Video_wait(obj, chan) (*(obj)->atv->wait)(obj, chan)
 
typedef struct _O_Video_Attr {  /* Drawing controls which may be set: */
	int		draw_w;   	 	/* Size of drawable area in X direction. */
	int		draw_h;    	 	/* Size of drawable area in Y direction. */
	int		draw_x;	  	 	/* X coordinate of start of drawable area. */
	int		draw_y;	   		/* Y coordinate of start of drawable area. */
	int		non_interlaced;	/* = 1 if non-interlaced, 0 if interlaced. */
    u_int   stencil_mask;  	/* Read mask for comparing wid bits w/value. */
    u_int   stencil_value; 	/* Value to compare w/wid bits in word. */
	int		video_w;  		/* Size of video signal in X direction. */
	int		video_h;  		/* Size of video signal in Y direction. */
	int		video_x;	   	/* X coordinate of start of video signal. */
	int		video_y;	   	/* Y coordinate of start of video signal. */
    u_int   write_mask;    	/* 24-bit write mask for data written. */
} O_Video_Attr;

typedef struct _TV_Video {
    void    (*dump)();          /* Dump contents of this object to a file. */
    void    (*get_attr)();      /* Get current video channel attributes. */
    void    (*init_hw)();       /* Init hardware from current context values */
    int     (*set_attr)();      /* Set current video channel attributes. */
    void    (*terminate)();     /* Terminate use of this object. */
    int     (*video_in)();      /* Turn on video input on a channel. */
    void    (*video_off)();     /* Turn off video on a channel. */
    int     (*video_out)();     /* Turn on video output on a channel. */
	int 	(*wait_for_done)();	/* Wait for video to start or stop. */
} TV_Video;
 
typedef struct _O_Video {
    TV_Video*    atv;        /* Methods: first entry in all derived objects. */
} O_Video;
 
O_Video* O_Video_O_Video();    /* Object constructor. */
 

#endif  /* objects_h_INCLUDED */
