/***********************************************************
Copyright 1991 by RasterOps, Inc., Santa Clara, California,
and the Massachusetts Institute of Technology, Cambridge, Massachusetts.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the names of Digital or MIT not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

******************************************************************/
/*
** File: 
**
**   xvhw.h ---  Xv video extension hardware dependent layer 
**               interface. This defines a standard object for 
**               accessing hardware-dependent functions. The layer
**               may be replaced with a vendor's specific 
**               implementation by utilizing a different xvhw_xxx.c and
**               xvhw.h.
**            
**
** Author: 
**
**   Andy Goodrich (RasterOps)
**
** Revisions:
**
**   11.04.93 Andy Goodrich
**     - Moved from original pip_obj.h and prefixed with xvhw instead of
**       pip.
**
**   08.08.92 Andy Goodrich
**     - Initial definition.
**
** $Log:   /plxhp/dave/pvcsrc/xv/include/archives/xvhw.h_v  $
 * 
 *    Rev 1.3   09 Sep 1996 10:39:56   dave
 * Andy Goodrich: New image support.
** Revision 1.3  1995/12/11 15:42:01  dist
** Andy Goodrich: New image support.
**
 * Revision 1.2  95/05/30  15:55:07  15:55:07  dist ()
 * Andy Goodrich - split Xvhw_Get_Image x,y into xw, yw and xo, yo.
 * 
 * Revision 1.1  95/05/02  11:55:59  11:55:59  dist ()
 * Initial revision
 * 
**
** Notes:
**   (1) Tab stops every four characters.
*/

#ifndef xvhw_INCLUDED
#define xvhw_INCLUDED 

/*----------------------------------------------------------------------------*
 |  Data Structures Associated With Calls To XvhwRec Functions:
 *----------------------------------------------------------------------------*/
typedef struct Xvhw_Flags {		/* Public flags: */
	u_int	stop_for_update:1;	/* = 1 if stop video during screen updates. */
	u_int	separate_fb:1;		/* = 1 if video fb not same as screen fb. */
	u_int   debug:1;			/* = 1 if producing debugging messages. */
	u_int   unused:29;			/* For later use. */
} Xvhw_Flags;

typedef struct Xvhw_Drw_Area {	/* Output area specification: */
	u_char	horizontal_flip;	/* = 1 if image flipped horizontally. */
	u_char	vertical_flip;		/* = 1 if image flipped vertically. */
	u_char	fields_inverted;	/* = 1 if display even fields before odd. */
	u_char	direction;			/* Direction of video (see enum below). */
	u_short	left;				/* Left of upper left corner. */
	u_short	top;				/* Top of upper left corner. */
	u_short	width;				/* Width of output area. */
	u_short	height;				/* Height of output area. */
}	Xvhw_Drw_Area;

enum {							/* Direction field values. */
	Xvhw_Signal_To_Drawable = 0,
	Xvhw_Drawable_To_Signal = 1
};

typedef struct Xvhw_Fb_Type {	/* Type of frame buffer: */
	int		pitch;				/* # of pixels between scan lines. */
	int		frequency;			/* Frequency of monitor. */
}	Xvhw_Fb_Type;

typedef struct Xvhw_Image {     /* Image location information: */
	int	drawable_x;				/* X origin of drawable image is in. */
	int drawable_y;				/* Y origin of drawable image is in. */
	int offset_x;				/* X offset w/in drawable of image upper left */
	int offset_y;				/* Y offset w/in drawable of image upper left */
	int image_x;				/* X offset w/in image storage of upper left. */
	int image_y;				/* Y offset w/in image storage of upper left. */
	int image_w;				/* Width of image in pixels. */
	int image_h;				/* Height of image in lines. */
	int image_depth;			/* Depth of pixel in data bits. */
	int image_data_depth;		/* # of valid data bits in a pixel. */
	int image_stride;			/* # of pixels in image data between lines. */
} Xvhw_Image;

typedef struct Xvhw_Levels {	/* Picture level settings: */
	int	brightness;				/* Picture brightness: -1000 to 1000. */
	int	contrast;				/* picture contrast: -1000 to 1000. */
	int	hue;      				/* Picture hue level : -1000 to 1000. */
	int	saturation;				/* Picture saturation: -1000 to 1000. */
}	Xvhw_Levels;

typedef struct Xvhw_Signal {	/* Video signal characteristics: */
	u_char	gray_scale;			/* = 1 if to present gray scale video. */
	u_char	vtr_mode;			/* Video Tape Recorder mode (bad signal). */
	u_char	unused[2];			/* Unused. */
	int		timing;				/* Timing: (XV_NTSC, etc.). */
	int		type;				/* Type: (XV_COMPOSITE, etc. ) */
}	Xvhw_Signal;

typedef struct Xvhw_Sig_Area {	/* Input signal area specification: */
	u_short	left;				/* Left side (0 is normal signal start). */
	u_short	top;				/* Top side (0 is normal signal start. */
	u_short	width;				/* Width of output area. */
	u_short	height;				/* Height of output area. */
}	Xvhw_Sig_Area;


/*============================================================================*
 ||
 ||	Public Interface To Xv Device Dependent Execution (Hardware) Layer:
 ||
 *===========================================================================*/
typedef struct _XvhwRec {
	Xvhw_Flags  attr;						/* Publicly available flags. */
	int			adaptor_port_number;		/* Port # this object is for. */
	int         video_height;				/* Height of video fb (not host!) */
	int         video_pitch;				/* Pitch of video fb (not host!) */
	int     	(*control)();				/* Perform a control function. */
	void		(*dump)();					/* Dump object contents. */
	int			(*fill_true_color)();		/* Fill true color memory. */
	int			(*fill_video_enable)();		/* Fill video enable memory. */
	void		(*get_levels)();			/* Get picture levels. */
	void		(*get_drawable_area)();		/* Get drawable area. */
	int         (*get_image)();             /* Get image data. */
	void		(*get_signal_area)();		/* Get video signal area. */
	void		(*get_signal_type)();		/* Get video input type. */
	int			(*initialize)();			/* Initialize video. */
	int			(*off)();					/* Turn video off. */
	int			(*on)();					/* Turn video on. */
	int			(*one_shot)();				/* Turn video on for one frame. */
	int         (*put_image)();             /* Put image data. */
	int			(*resume)();				/* Pop video live stack on. */
	void		(*set_levels)();			/* Set picture levels. */
	void		(*set_drawable_area)();		/* Set drawable area. */
	void		(*set_signal_area)();		/* Set video signal area. */
	void		(*set_signal_type)();		/* Set video input type. */
	int 		(*status)();				/* Get video status. */
	int			(*suspend)();				/* Push video live stack off. */
	void		(*terminate)();				/* Terminate object. */
} XvhwRec, *XvhwPtr;

/*
 *	Macros to access fields in video object.
 *
 *     Xvhw_Max_Height - height of video frame buffer in lines (as opposed
 *                        to host fb, which may be the same)
 *     Xvhw_Max_Width  - width of video frame buffer in pixels (as opposed
 *                        to host fb, which may be the same)
 */
#define Xvhw_Attr_Separate_Fb(obj)     ((XvhwPtr)(obj))->attr.separate_fb

#define Xvhw_Attr_Stop_For_Update(obj) ((XvhwPtr)(obj))->attr.stop_for_update

#define Xvhw_Max_Height(obj) ((XvhwPtr)(obj))->video_height

#define Xvhw_Max_Width(obj) ((XvhwPtr)(obj))->video_pitch

/*----------------------------------------------------------------------------*
 |"Xvhw_Control"
 |
 |  Perform a control function on a video object.
 |
 |  Arguments:
 |   XvhwPtr     obj    -> control block (object) for this instance.
 |   char        *cmd   -> command to perform.
 |   caddr_t     arg    =  optional argument. 
 |
 |      = 1 if success
 |      = 0 if failure
 *----------------------------------------------------------------------------*/
#define Xvhw_Control(obj, cmd, arg) \
	(*((XvhwPtr)(obj))->control)(obj, cmd, arg)


/*----------------------------------------------------------------------------*
 |"Xvhw_Dump"
 |
 |  Dump the contents of a video object to a file
 |
 |  Arguments:
 |    XvhwPtr     obj   -> control block (object) for this instance.
 |    FILE        *fd   -> file descriptor to dump to. 
 *----------------------------------------------------------------------------*/
#define Xvhw_Dump(obj, fd) \
	(*((XvhwPtr)(obj))->dump)(obj, fd)

/*----------------------------------------------------------------------------*
 |"Xvhw_Fill_True_Color"
 |
 |  Fill a rectangular region in the true color memory.
 |
 |  Arguments:
 |     XvhwPtr     obj     -> control block (object) for this instance.
 |     int         left    =  left side of region to fill in. 
 |     int         top     =  top side of region to fill in. 
 |     int         width   =  width of region to fill in. 
 |     int         height  =  height of region to fill in. 
 |     u_int       color   =  color to fill pixels with. 
 *----------------------------------------------------------------------------*/
#define Xvhw_Fill_True_Color(obj, left, top, width, height, color) \
	(*((XvhwPtr)(obj))->fill_true_color)(obj, left, top, width, height, color)

/*----------------------------------------------------------------------------*
 |"Xvhw_Fill_Video_Enable"
 |
 |  Fill a rectangular region in the video enable memory. This routine wraps 
 |  tv_biased_fill_video_enable for the |outside world. All we do here is to 
 |  bias the rectangle to be filled by the current location of the video on the
 |  screen, and call tv_biased_fill_video_enable.
 |
 |  Arguments: 
 |    XvhwPtr     obj     -> control block (object) for this instance.
 |    int         left    =  left side of region to fill in. 
 |    int         top     =  top side of region to fill in. 
 |    int         width   =  width of region to fill in. 
 |    int         height  =  height of region to fill in. 
 |    u_int       color   =  color to fill pixels with. 
 |
 |      = 1 if success
 |      = 0 if rectangle is not within boundaries of video enable memory
 *----------------------------------------------------------------------------*/
#define Xvhw_Fill_Video_Enable(obj, left, top, width, height, color) \
	(*((XvhwPtr)(obj))->fill_video_enable)(obj, left,top,width,height,color)

/*----------------------------------------------------------------------------*
 |"Xvhw_Get_Drawable_Area"
 |
 |  Return the current output area information.
 |
 |  Arguments:
 |    XvhwPtr        obj    -> control block (object) for this instance.
 |    Xvhw_Drw_Area  *area  -> where to put current output area. 
 *----------------------------------------------------------------------------*/
#define Xvhw_Get_Drawable_Area(obj, area) \
	(*((XvhwPtr)(obj))->get_drawable_area)(obj, area)

/*----------------------------------------------------------------------------*
 |"Xvhw_Get_Image"
 |
 |  Return the data from the true color buffer.
 |
 | Arguments:
 |   XvhwPtr     obj    -> control block (object) for this instance.
 |   Xvhw_Image* p_geom -> geometry information about image.
 |   u_int*      buf    -> buffer to place data into. 
 *----------------------------------------------------------------------------*/
#define Xvhw_Get_Image(obj, p_geom, buf) \
	(*((XvhwPtr)(obj))->get_image)(obj, p_geom, buf)

/*----------------------------------------------------------------------------*
 |"Xvhw_Get_Levels"
 |
 |  Return the picture levels (brightness, contrast, hue, saturation).
 |
 |  Arguments:
 |    XvhwPtr      obj      -> control block (object) for this instance.
 |    Xvhw_Levels  *levels  -> where to put current levels. 
 *----------------------------------------------------------------------------*/
#define Xvhw_Get_Levels(obj, levels) \
	(*((XvhwPtr)(obj))->get_levels)(obj, levels)

/*----------------------------------------------------------------------------*
 |"Xvhw_Get_Signal_Area"
 |
 |  Return the current output area information.
 |
 |  Arguments:
 |    XvhwPtr        obj      -> control block (object) for this instance.
 |    Xvhw_Sig_Area  *signal  -> where to put signal area info. 
 *----------------------------------------------------------------------------*/
#define Xvhw_Get_Signal_Area(obj, area) \
	(*((XvhwPtr)(obj))->get_signal_area)(obj, area)

/*----------------------------------------------------------------------------*
 |"Xvhw_Get_Signal_Type"
 |
 |  Get the type and timing of signal being used. 
 |  
 |  Arguments:
 |    XvhwPtr      obj      -> control block (object) for this instance.
 |    Xvhw_Signal  *signal  -> signal specification. 
 *----------------------------------------------------------------------------*/
#define Xvhw_Get_Signal_Type(obj, signal) \
	(*((XvhwPtr)(obj))->get_signal_type)(obj, signal)

/*----------------------------------------------------------------------------*
 |"Xvhw_Get_Status"
 |
 |  Return current status of video operation.
 |  
 |  Arguments:
 |    XvhwPtr      obj      -> control block (object) for this instance.
 |
 |  Return value:
 |    status (see Xvddx.h for bit assignments)
 *----------------------------------------------------------------------------*/
#define Xvhw_Get_Status(obj) \
	(*((XvhwPtr)(obj))->status)(obj)

/*----------------------------------------------------------------------------*
 |"Xvhw_Initialize"
 |
 |  Initialize video hardware to an operable state. After this
 |  call the video will be off.  
 |
 |  Arguments: 
 |    XvhwPtr      obj      -> control block (object) for this instance.
 |    Xvhw_Signal  *signal  -> signal type information. 
 *----------------------------------------------------------------------------*/
#define Xvhw_Initialize(obj, signal) \
	(*((XvhwPtr)(obj))->initialize)(obj, signal)

/*----------------------------------------------------------------------------*
 |"Xvhw_Off"
 |
 |  Turn off video. Return its previous operational state.
 |
 |  Arguments:
 |    XvhwPtr     obj                -> control block for instance.
 |    int         wait_for_inactive  =  1 wait for video to actually turn off 
 |
 |      = 1 if video was operating at time of call
 |      = 0 if video was halted at time of call
 *----------------------------------------------------------------------------*/
#define Xvhw_Off(obj, wait) \
	(*((XvhwPtr)(obj))->off)(obj, wait)

/*----------------------------------------------------------------------------*
 |"Xvhw_On"
 |
 |  Turn on video.  
 |
 |  Arguments:
 |     XvhwPtr     obj              -> control block for instance.
 |     int         wait_for_active  = 1 wait for video to go active. 
 |
 |    = 1 if wait_for_active is zero, or video is now running
 |    = 0 if video did not turn on (hardware error)
 *----------------------------------------------------------------------------*/
#define Xvhw_On(obj, wait) \
	(*((XvhwPtr)(obj))->on)(obj, wait)

/*----------------------------------------------------------------------------*
 |"Xvhw_One_Shot"
 |
 |  Turn video on for a frame. Upon return the video will be off,
 |  with the "off" semaphore incremented appropriately.
 |
 |  Arguments:
 |     XvhwPtr  obj              -> control block for instance.
 |
 |    = 1 if success
 |    = 0 if error detected
 *----------------------------------------------------------------------------*/
#define Xvhw_One_Shot(obj) \
	(*((XvhwPtr)(obj))->one_shot)(obj)

/*----------------------------------------------------------------------------*
 |"Xvhw_Put_Image"
 |
 |  Place data into the true color buffer.
 |
 |  Arguments:
 |    XvhwPtr     obj    -> control block (object) for this instance.
 |    Xvhw_Image* p_geom -> geometry information about image.
 |    u_int       *buf   -> buffer to get data from. 
 *----------------------------------------------------------------------------*/
#define Xvhw_Put_Image(obj, p_geom, buf) \
	(*((XvhwPtr)(obj))->put_image)(obj, p_geom, buf)

/*----------------------------------------------------------------------------*
 |"Xvhw_Resume"
 |  Pop the video "off" semaphore, and if it goes to zero and video_frozen
 |  is not set, start up the video again. Providing a pop_stack parameter of 1 
 |  will clear the semaphore to zero (i.e., pop all suspends and start the 
 |  video if video_frozen is not set).
 |
 |  Arguments:
 |    XvhwPtr     obj        -> control block (object) for this instance.
 |    int         pop_stack  =  1 if should pop semaphore stack to active. 
 |
 |      = 1 if video now operating
 |      = 0 if video still halted
 *----------------------------------------------------------------------------*/
#define Xvhw_Resume(obj, pop_stack) \
	(*((XvhwPtr)(obj))->resume)(obj, pop_stack)

/*----------------------------------------------------------------------------*
 |"Xvhw_Set_Drawable_Area"
 |
 |  Set the size of the rectangle for the video output area.
 |
 |  Arguments:
 |    XvhwPtr        obj    -> control block (object) for this instance.
 |    Xvhw_Drw_Area  *area  -> output area specification. 
 *----------------------------------------------------------------------------*/
#define Xvhw_Set_Drawable_Area(obj, area) \
	(*((XvhwPtr)(obj))->set_drawable_area)(obj, area)

/*----------------------------------------------------------------------------*
 |"Xvhw_Set_Levels"
 |
 |  Set the picture levels (brightness, contrast, hue, saturation).
 |
 |  Arguments:
 |    XvhwPtr      obj      -> control block (object) for this instance.
 |    Xvhw_Levels  *levels  -> where to get new levels. 
 *----------------------------------------------------------------------------*/
#define Xvhw_Set_Levels(obj, levels) \
	(*((XvhwPtr)(obj))->set_levels)(obj, levels)

/*----------------------------------------------------------------------------*
 |"Xvhw_Set_Signal_Area"
 |
 |  Set the size of the rectangle for the signal input area.
 |
 |  Arguments:
 |    XvhwPtr        obj      -> control block (object) for this instance.
 |    Xvhw_Sig_Area  *signal  -> new signal input area. 
 *----------------------------------------------------------------------------*/
#define Xvhw_Set_Signal_Area(obj, area) \
	(*((XvhwPtr)(obj))->set_signal_area)(obj, area)

/*----------------------------------------------------------------------------*
 |"Xvhw_Set_Signal_Type"
 |
 |  Set the type and timing of signal being used. A value of -1 in any of 
 |  the fields of "signal" indicates the current setting should be used.
 |
 |  Arguments:
 |    XvhwPtr      obj      -> control block (object) for this instance.
 |    Xvhw_Signal  *signal  -> signal specification. 
 *----------------------------------------------------------------------------*/
#define Xvhw_Set_Signal_Type(obj, signal) \
	(*((XvhwPtr)(obj))->set_signal_type)(obj, signal)

/*----------------------------------------------------------------------------*
 |"Xvhw_Suspend"
 |
 |  Push the video "off" semaphore by one. If it is running stop it and
 |  optionally wait for the video to actually stop running.
 |
 |  Arguments:
 |    XvhwPtr     obj   -> control block (object) for this instance.
 |    int         wait  =  1 if should wait for video to stop.
 |
 |      = 1 if video was operating at time of call
 |      = 0 if video was halted at time of call
 *----------------------------------------------------------------------------*/
#define Xvhw_Suspend(obj, wait) \
	(*((XvhwPtr)(obj))->suspend)(obj, wait)

/*----------------------------------------------------------------------------*
 |"Xvhw_Terminate"
 |
 |  Terminate operations of Pip object: turn off video and free dynamically 
 |  allocated storage. Note the pointer to this object can no longer be 
 |  used!
 |
 |  Arguments:
 |    XvhwPtr  obj   -> control block (object) for this instance.
 *----------------------------------------------------------------------------*/
#define Xvhw_Terminate(obj) \
	(*((XvhwPtr)(obj))->terminate)(obj)

#endif /* xvhw_INCLUDED */
